@@ -278,7 +278,7 @@ class OVSBridge(BaseOVS):
     def set_igmp_snooping_state(self, state):
         state = bool(state)
         other_config = {
-            'mcast-snooping-disable-flood-unregistered': str(state)}
+            'mcast-snooping-disable-flood-unregistered': 'false'}
         with self.ovsdb.transaction() as txn:
             txn.add(
                 self.ovsdb.db_set('Bridge', self.br_name,
@@ -287,6 +287,16 @@ class OVSBridge(BaseOVS):
                 self.ovsdb.db_set('Bridge', self.br_name,
                                   ('other_config', other_config)))
 
+    def set_igmp_snooping_flood(self, port_name, state):
+        state = str(state)
+        other_config = {
+            'mcast-snooping-flood-reports': state,
+            'mcast-snooping-flood': state}
+        self.ovsdb.db_set(
+            'Port', port_name,
+            ('other_config', other_config)).execute(
+                check_error=True, log_errors=True)
+
     def create(self, secure_mode=False):
         other_config = {
             'mac-table-size': str(cfg.CONF.OVS.bridge_mac_table_size)}
@@ -528,7 +538,7 @@ class OVSBridge(BaseOVS):
         if tunnel_type == TYPE_GRE_IP6:
             # NOTE(slaweq) According to the OVS documentation L3 GRE tunnels
             # over IPv6 are not supported.
-            options['packet_type'] = 'legacy'
+            options['packet_type'] = 'legacy_l2'
         attrs.append(('options', options))
 
         return self.add_port(port_name, *attrs)
