@@ -15,35 +15,26 @@
 
 import sys
 
+from neutron_lib.agent import l2_extension
 from neutron_lib import constants
+from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from neutron._i18n import _, _LE, _LW
-from neutron.agent.l2 import l2_agent_extension
 from neutron.agent.linux import bridge_lib
-from neutron.common import utils as n_utils
+from neutron.conf.agent import l2_ext_fdb_population
 from neutron.plugins.ml2.drivers.linuxbridge.agent.common import (
      constants as linux_bridge_constants)
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
      constants as ovs_constants)
 
-# if shared_physical_device_mappings is not configured KeyError will be thrown
-fdb_population_opt = [
-    cfg.ListOpt('shared_physical_device_mappings', default=[],
-                help=_("Comma-separated list of "
-                       "<physical_network>:<network_device> tuples mapping "
-                       "physical network names to the agent's node-specific "
-                       "shared physical network device between "
-                       "SR-IOV and OVS or SR-IOV and linux bridge"))
-]
-cfg.CONF.register_opts(fdb_population_opt, 'FDB')
+l2_ext_fdb_population.register_fdb_population_opts()
 
 LOG = logging.getLogger(__name__)
 
 
 class FdbPopulationAgentExtension(
-        l2_agent_extension.L2AgentExtension):
+        l2_extension.L2AgentExtension):
     """The FDB population is an agent extension to OVS or linux bridge
     who's objective is to update the FDB table for existing instance
     using normal port, thus enabling communication between SR-IOV instances
@@ -81,9 +72,9 @@ class FdbPopulationAgentExtension(
                 try:
                     _stdout = bridge_lib.FdbInterface.show(device)
                 except RuntimeError as e:
-                    LOG.warning(_LW(
+                    LOG.warning(
                         'Unable to find FDB Interface %(device)s. '
-                        'Exception: %(e)s'), {'device': device, 'e': e})
+                        'Exception: %(e)s', {'device': device, 'e': e})
                     continue
                 self.device_to_macs[device] = _stdout.split()[::3]
 
@@ -102,10 +93,10 @@ class FdbPopulationAgentExtension(
             try:
                 bridge_lib.FdbInterface.add(mac, device)
             except RuntimeError as e:
-                LOG.warning(_LW(
+                LOG.warning(
                     'Unable to add mac %(mac)s '
                     'to FDB Interface %(device)s. '
-                    'Exception: %(e)s'),
+                    'Exception: %(e)s',
                     {'mac': mac, 'device': device, 'e': e})
                 return
             self.device_to_macs[device].append(mac)
@@ -113,19 +104,19 @@ class FdbPopulationAgentExtension(
         def delete_port(self, devices, port_id):
             mac = self.portid_to_mac.get(port_id)
             if mac is None:
-                LOG.warning(_LW('Port Id %(port_id)s does not have a rule for '
-                    'devices %(devices)s in FDB table'),
-                    {'port_id': port_id, 'devices': devices})
+                LOG.warning('Port Id %(port_id)s does not have a rule for '
+                            'devices %(devices)s in FDB table',
+                            {'port_id': port_id, 'devices': devices})
                 return
             for device in devices:
                 if mac in self.device_to_macs[device]:
                     try:
                         bridge_lib.FdbInterface.delete(mac, device)
                     except RuntimeError as e:
-                        LOG.warning(_LW(
+                        LOG.warning(
                             'Unable to delete mac %(mac)s '
                             'from FDB Interface %(device)s. '
-                            'Exception: %(e)s'),
+                            'Exception: %(e)s',
                             {'mac': mac, 'device': device, 'e': e})
                         return
                     self.device_to_macs[device].remove(mac)
@@ -137,17 +128,17 @@ class FdbPopulationAgentExtension(
         valid_driver_types = (linux_bridge_constants.EXTENSION_DRIVER_TYPE,
                               ovs_constants.EXTENSION_DRIVER_TYPE)
         if driver_type not in valid_driver_types:
-            LOG.error(_LE('FDB extension is only supported for OVS and '
-                          'linux bridge agent, currently uses '
-                          '%(driver_type)s'), {'driver_type': driver_type})
+            LOG.error('FDB extension is only supported for OVS and '
+                      'linux bridge agent, currently uses '
+                      '%(driver_type)s', {'driver_type': driver_type})
             sys.exit(1)
 
-        self.device_mappings = n_utils.parse_mappings(
+        self.device_mappings = helpers.parse_mappings(
             cfg.CONF.FDB.shared_physical_device_mappings, unique_keys=False)
         devices = self._get_devices()
         if not devices:
-            LOG.error(_LE('Invalid configuration provided for FDB extension: '
-                          'no physical devices'))
+            LOG.error('Invalid configuration provided for FDB extension: '
+                      'no physical devices')
             sys.exit(1)
         self.fdb_tracker = self.FdbTableTracker(devices)
 
@@ -169,8 +160,8 @@ class FdbPopulationAgentExtension(
         self.fdb_tracker.delete_port(devices, port_id)
 
     def _get_devices(self):
-        def _flatten_list(l):
-            return [item for sublist in l for item in sublist]
+        def _flatten_list(li):
+            return [item for sublist in li for item in sublist]
 
         return _flatten_list(self.device_mappings.values())
 
