@@ -16,7 +16,6 @@
 from neutron_lib import constants
 
 from neutron.agent.l3 import dvr_edge_router
-from neutron.agent.l3 import dvr_snat_ns
 from neutron.agent.l3 import ha_router
 from neutron.agent.l3 import router_info
 
@@ -27,8 +26,8 @@ class DvrEdgeHaRouter(dvr_edge_router.DvrEdgeRouter,
        DVR router with HA capabilities.
     """
 
-    def __init__(self, agent, host, *args, **kwargs):
-        super(DvrEdgeHaRouter, self).__init__(agent, host,
+    def __init__(self, host, *args, **kwargs):
+        super(DvrEdgeHaRouter, self).__init__(host,
                                               *args, **kwargs)
         self.enable_snat = None
 
@@ -56,7 +55,34 @@ class DvrEdgeHaRouter(dvr_edge_router.DvrEdgeRouter,
         self._plug_ha_router_port(
             sn_port,
             self._get_snat_int_device_name,
-            dvr_snat_ns.SNAT_INT_DEV_PREFIX)
+            constants.SNAT_INT_DEV_PREFIX)
+
+    def add_centralized_floatingip(self, fip, fip_cidr):
+        interface_name = self.get_snat_external_device_interface_name(
+                self.get_ex_gw_port())
+        self._add_vip(fip_cidr, interface_name)
+
+        self.set_ha_port()
+        if (self.is_router_primary() and self.ha_port and
+                self.ha_port['status'] == constants.PORT_STATUS_ACTIVE):
+            return super(DvrEdgeHaRouter, self).add_centralized_floatingip(
+                fip, fip_cidr)
+        else:
+            return constants.FLOATINGIP_STATUS_ACTIVE
+
+    def remove_centralized_floatingip(self, fip_cidr):
+        self._remove_vip(fip_cidr)
+        if self.is_router_primary():
+            super(DvrEdgeHaRouter, self).remove_centralized_floatingip(
+                fip_cidr)
+
+    def get_centralized_fip_cidr_set(self):
+        ex_gw_port = self.get_ex_gw_port()
+        if not ex_gw_port:
+            return set()
+        interface_name = self.get_snat_external_device_interface_name(
+            ex_gw_port)
+        return set(self._get_cidrs_from_keepalived(interface_name))
 
     def external_gateway_added(self, ex_gw_port, interface_name):
         super(DvrEdgeHaRouter, self).external_gateway_added(
@@ -86,20 +112,14 @@ class DvrEdgeHaRouter(dvr_edge_router.DvrEdgeRouter,
         ha_router.HaRouter.external_gateway_updated(self, ex_gw_port,
                                                     interface_name)
 
-    def initialize(self, process_monitor):
-        self._create_snat_namespace()
-        super(DvrEdgeHaRouter, self).initialize(process_monitor)
-
-    def get_router_cidrs(self, device):
-        return router_info.RouterInfo.get_router_cidrs(self, device)
-
     def _external_gateway_added(self, ex_gw_port, interface_name,
                                 ns_name, preserve_ips):
-        self._plug_external_gateway(ex_gw_port, interface_name, ns_name)
+        link_up = self.external_gateway_link_up()
+        self._plug_external_gateway(ex_gw_port, interface_name, ns_name,
+                                    link_up=link_up)
 
     def _is_this_snat_host(self):
-        return (self.agent_conf.agent_mode
-                == constants.L3_AGENT_MODE_DVR_SNAT)
+        return self.agent_conf.agent_mode == constants.L3_AGENT_MODE_DVR_SNAT
 
     def _dvr_internal_network_removed(self, port):
         super(DvrEdgeHaRouter, self)._dvr_internal_network_removed(port)
@@ -114,5 +134,5 @@ class DvrEdgeHaRouter(dvr_edge_router.DvrEdgeRouter,
         self.driver.plug(port['network_id'], port['id'],
                          interface_name, port['mac_address'],
                          namespace=self.snat_namespace.name,
-                         prefix=dvr_snat_ns.SNAT_INT_DEV_PREFIX,
+                         prefix=constants.SNAT_INT_DEV_PREFIX,
                          mtu=port.get('mtu'))
