@@ -17,16 +17,14 @@ import collections
 
 import netaddr
 from neutron_lib import constants as lib_constants
-from neutron_lib import exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
-import six
 
 from neutron.agent.l3 import dvr_fip_ns
 from neutron.agent.l3 import dvr_router_base
 from neutron.agent.linux import ip_lib
-from neutron.common import constants as n_const
 from neutron.common import utils as common_utils
+from neutron.privileged.agent.linux import ip_lib as priv_ip_lib
 
 LOG = logging.getLogger(__name__)
 # xor-folding mask used for IPv6 rule index
@@ -42,29 +40,52 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         super(DvrLocalRouter, self).__init__(host, *args, **kwargs)
 
         self.floating_ips_dict = {}
-        self.centralized_floatingips_set = set()
         # Linklocal subnet for router and floating IP namespace link
         self.rtr_fip_subnet = None
         self.rtr_fip_connect = False
         self.fip_ns = None
         self._pending_arp_set = set()
 
-    def get_centralized_router_cidrs(self):
-        return self.centralized_floatingips_set
+        self.load_used_fip_information()
+
+    def load_used_fip_information(self):
+        """Some information needed to remove a floating ip e.g. it's associated
+        ip rule priorities, are stored in memory to avoid extra db lookups.
+        Since this is lost on agent restart we need to reload them.
+        """
+        ex_gw_port = self.get_ex_gw_port()
+        if ex_gw_port:
+            fip_ns = self.agent.get_fip_ns(ex_gw_port['network_id'])
+
+            for fip in self.get_floating_ips():
+                floating_ip = fip['floating_ip_address']
+                fixed_ip = fip['fixed_ip_address']
+                rule_pr = fip_ns.lookup_rule_priority(floating_ip)
+                if rule_pr:
+                    self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
+                else:
+                    LOG.error("Rule priority not found for floating ip %s",
+                              floating_ip)
 
     def migrate_centralized_floating_ip(self, fip, interface_name, device):
         # Remove the centralized fip first and then add fip to the host
         ip_cidr = common_utils.ip_to_cidr(fip['floating_ip_address'])
         self.floating_ip_removed_dist(ip_cidr)
         # Now add the floating_ip to the current host
-        self.floating_ip_added_dist(fip, ip_cidr)
+        return self.floating_ip_added_dist(fip, ip_cidr)
 
     def floating_forward_rules(self, fip):
         """Override this function defined in router_info for dvr routers."""
         if not self.fip_ns:
             return []
 
-        if fip.get(n_const.DVR_SNAT_BOUND):
+        if fip.get(lib_constants.DVR_SNAT_BOUND):
+            return []
+
+        # For dvr_no_external node should not process any floating IP
+        # iptables rules.
+        if (self.agent_conf.agent_mode ==
+                lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL):
             return []
 
         fixed_ip = fip['fixed_ip_address']
@@ -73,9 +94,10 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         dnat_from_floatingip_to_fixedip = (
             'PREROUTING', '-d %s/32 -i %s -j DNAT --to-destination %s' % (
                 floating_ip, rtr_2_fip_name, fixed_ip))
-        snat_from_fixedip_to_floatingip = (
-            'float-snat', '-s %s/32 -j SNAT --to-source %s' % (
-                fixed_ip, floating_ip))
+        to_source = '-s %s/32 -j SNAT --to-source %s' % (fixed_ip, floating_ip)
+        if self.iptables_manager.random_fully:
+            to_source += ' --random-fully'
+        snat_from_fixedip_to_floatingip = ('float-snat', to_source)
         return [dnat_from_floatingip_to_fixedip,
                 snat_from_fixedip_to_floatingip]
 
@@ -105,11 +127,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def floating_ip_added_dist(self, fip, fip_cidr):
         """Add floating IP to respective namespace based on agent mode."""
-        if fip.get(n_const.DVR_SNAT_BOUND):
-            floating_ip_status = self.add_centralized_floatingip(fip, fip_cidr)
-            if floating_ip_status == lib_constants.FLOATINGIP_STATUS_ACTIVE:
-                self.centralized_floatingips_set.add(fip_cidr)
-            return floating_ip_status
+        if fip.get(lib_constants.DVR_SNAT_BOUND):
+            return self.add_centralized_floatingip(fip, fip_cidr)
         if not self._check_if_floatingip_bound_to_host(fip):
             # TODO(Swami): Need to figure out what status
             # should be returned when the floating IP is
@@ -120,11 +139,17 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
             # the floating IP is intended for this host should
             # be done.
             return
+
+        # dvr_no_external host should not process any floating IP route rules.
+        if (self.agent_conf.agent_mode ==
+                lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL):
+            return
+
         floating_ip = fip['floating_ip_address']
         fixed_ip = fip['fixed_ip_address']
         self._add_floating_ip_rule(floating_ip, fixed_ip)
         fip_2_rtr_name = self.fip_ns.get_int_device_name(self.router_id)
-        #Add routing rule in fip namespace
+        # Add routing rule in fip namespace
         fip_ns_name = self.fip_ns.get_name()
         if self.rtr_fip_subnet is None:
             self.rtr_fip_subnet = self.fip_ns.local_subnets.allocate(
@@ -143,26 +168,29 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
     def _add_floating_ip_rule(self, floating_ip, fixed_ip):
         rule_pr = self.fip_ns.allocate_rule_priority(floating_ip)
         self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
-        ip_rule = ip_lib.IPRule(namespace=self.ns_name)
-        ip_rule.rule.add(ip=fixed_ip,
-                         table=dvr_fip_ns.FIP_RT_TBL,
-                         priority=rule_pr)
+
+        ip_lib.add_ip_rule(namespace=self.ns_name, ip=fixed_ip,
+                           table=dvr_fip_ns.FIP_RT_TBL,
+                           priority=int(str(rule_pr)))
 
     def _remove_floating_ip_rule(self, floating_ip):
         if floating_ip in self.floating_ips_dict:
             fixed_ip, rule_pr = self.floating_ips_dict[floating_ip]
-            ip_rule = ip_lib.IPRule(namespace=self.ns_name)
-            ip_rule.rule.delete(ip=fixed_ip,
-                                table=dvr_fip_ns.FIP_RT_TBL,
-                                priority=rule_pr)
+            ip_lib.delete_ip_rule(self.ns_name, ip=fixed_ip,
+                                  table=dvr_fip_ns.FIP_RT_TBL,
+                                  priority=int(str(rule_pr)))
             self.fip_ns.deallocate_rule_priority(floating_ip)
-            #TODO(rajeev): Handle else case - exception/log?
+        else:
+            LOG.error("Unable to find necessary information to complete "
+                      "removal of floating ip rules for %s - will require "
+                      "manual cleanup (see LP 1891673 for details).",
+                      floating_ip)
 
     def floating_ip_removed_dist(self, fip_cidr):
         """Remove floating IP from FIP namespace."""
-        if fip_cidr in self.centralized_floatingips_set:
+        centralized_fip_cidrs = self.get_centralized_fip_cidr_set()
+        if fip_cidr in centralized_fip_cidrs:
             self.remove_centralized_floatingip(fip_cidr)
-            self.centralized_floatingips_set.remove(fip_cidr)
             return
         floating_ip = fip_cidr.split('/')[0]
         fip_2_rtr_name = self.fip_ns.get_int_device_name(self.router_id)
@@ -176,7 +204,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
             device = ip_lib.IPDevice(fip_2_rtr_name, namespace=fip_ns_name)
 
-            device.route.delete_route(fip_cidr, str(rtr_2_fip.ip))
+            device.route.delete_route(fip_cidr, via=str(rtr_2_fip.ip))
+            return device
 
     def floating_ip_moved_dist(self, fip):
         """Handle floating IP move between fixed IPs."""
@@ -190,7 +219,9 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         return self.floating_ip_added_dist(fip, ip_cidr)
 
     def remove_floating_ip(self, device, ip_cidr):
-        self.floating_ip_removed_dist(ip_cidr)
+        fip_2_rtr_device = self.floating_ip_removed_dist(ip_cidr)
+        if fip_2_rtr_device:
+            fip_2_rtr_device.delete_conntrack_state(ip_cidr)
 
     def move_floating_ip(self, fip):
         self.floating_ip_moved_dist(fip)
@@ -216,12 +247,15 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
     def _process_arp_cache_for_internal_port(self, subnet_id):
         """Function to process the cached arp entries."""
         arp_remove = set()
+        device, device_exists = self.get_arp_related_dev(subnet_id)
         for arp_entry in self._pending_arp_set:
             if subnet_id == arp_entry.subnet_id:
                 try:
                     state = self._update_arp_entry(
                         arp_entry.ip, arp_entry.mac,
-                        arp_entry.subnet_id, arp_entry.operation)
+                        arp_entry.subnet_id, arp_entry.operation,
+                        device=device,
+                        device_exists=device_exists)
                 except Exception:
                     state = False
                 if state:
@@ -240,20 +274,14 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         self._pending_arp_set -= arp_delete
 
     def _update_arp_entry(
-        self, ip, mac, subnet_id, operation, nud_state='permanent'):
+            self, ip, mac, subnet_id, operation, device,
+            device_exists=True):
         """Add or delete arp entry into router namespace for the subnet."""
-        port = self._get_internal_port(subnet_id)
-        # update arp entry only if the subnet is attached to the router
-        if not port:
-            return False
 
         try:
-            # TODO(mrsmith): optimize the calls below for bulk calls
-            interface_name = self.get_internal_device_name(port['id'])
-            device = ip_lib.IPDevice(interface_name, namespace=self.ns_name)
-            if device.exists():
+            if device_exists:
                 if operation == 'add':
-                    device.neigh.add(ip, mac, nud_state=nud_state)
+                    device.neigh.add(ip, mac)
                 elif operation == 'delete':
                     device.neigh.delete(ip, mac)
                 return True
@@ -270,6 +298,16 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
             with excutils.save_and_reraise_exception():
                 LOG.exception("DVR: Failed updating arp entry")
 
+    def get_arp_related_dev(self, subnet_id):
+        port = self._get_internal_port(subnet_id)
+        # update arp entry only if the subnet is attached to the router
+        if not port:
+            return None, False
+        interface_name = self.get_internal_device_name(port['id'])
+        device = ip_lib.IPDevice(interface_name, namespace=self.ns_name)
+        device_exists = device.exists()
+        return device, device_exists
+
     def _set_subnet_arp_info(self, subnet_id):
         """Set ARP info retrieved from Plugin for existing ports."""
         # TODO(Carl) Can we eliminate the need to make this RPC while
@@ -278,16 +316,37 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         ignored_device_owners = (
             lib_constants.ROUTER_INTERFACE_OWNERS +
             tuple(common_utils.get_dvr_allowed_address_pair_device_owners()))
+        device, device_exists = self.get_arp_related_dev(subnet_id)
 
         for p in subnet_ports:
-            nud_state = 'permanent' if p.get('device_owner') else 'reachable'
             if p['device_owner'] not in ignored_device_owners:
                 for fixed_ip in p['fixed_ips']:
                     self._update_arp_entry(fixed_ip['ip_address'],
                                            p['mac_address'],
                                            subnet_id,
                                            'add',
-                                           nud_state=nud_state)
+                                           device=device,
+                                           device_exists=device_exists)
+                for allowed_address_pair in p.get('allowed_address_pairs', []):
+                    self._update_arp_entry(allowed_address_pair['ip_address'],
+                                           allowed_address_pair['mac_address'],
+                                           subnet_id,
+                                           'add',
+                                           device=device,
+                                           device_exists=device_exists)
+
+        # subnet_ports does not have snat port if the port is still unbound
+        # by the time this function is called. So ensure to add arp entry
+        # for snat port if port details are updated in router info.
+        for p in self.get_snat_interfaces():
+            for fixed_ip in p['fixed_ips']:
+                if fixed_ip['subnet_id'] == subnet_id:
+                    self._update_arp_entry(fixed_ip['ip_address'],
+                                           p['mac_address'],
+                                           subnet_id,
+                                           'add',
+                                           device=device,
+                                           device_exists=device_exists)
         self._process_arp_cache_for_internal_port(subnet_id)
 
     @staticmethod
@@ -302,7 +361,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         """
         net = netaddr.IPNetwork(ip_cidr)
         if net.version == 6:
-            if isinstance(ip_cidr, six.text_type):
+            if isinstance(ip_cidr, str):
                 ip_cidr = ip_cidr.encode()  # Needed for Python 3.x
             # the crc32 & 0xffffffff is for Python 2.6 and 3.0 compatibility
             snat_idx = binascii.crc32(ip_cidr) & 0xffffffff
@@ -317,13 +376,12 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
     def _delete_gateway_device_if_exists(self, ns_ip_device, gw_ip_addr,
                                          snat_idx):
         try:
-            ns_ip_device.route.delete_gateway(gw_ip_addr,
-                                        table=snat_idx)
-        except exceptions.DeviceNotFoundError:
+            ns_ip_device.route.delete_gateway(gw_ip_addr, table=snat_idx)
+        except priv_ip_lib.NetworkInterfaceNotFound:
             pass
 
-    def _stale_ip_rule_cleanup(self, ns_ipr, ns_ipd, ip_version):
-        ip_rules_list = ns_ipr.rule.list_rules(ip_version)
+    def _stale_ip_rule_cleanup(self, namespace, ns_ipd, ip_version):
+        ip_rules_list = ip_lib.list_ip_rules(namespace, ip_version)
         snat_table_list = []
         for ip_rule in ip_rules_list:
             snat_table = ip_rule['table']
@@ -335,24 +393,23 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                                     dvr_fip_ns.FIP_PR_END)):
                 continue
             gateway_cidr = ip_rule['from']
-            ns_ipr.rule.delete(ip=gateway_cidr,
-                               table=snat_table,
-                               priority=priority)
+            ip_lib.delete_ip_rule(namespace, ip=gateway_cidr, table=snat_table,
+                                  priority=priority)
             snat_table_list.append(snat_table)
         for tb in snat_table_list:
             ns_ipd.route.flush(ip_version, table=tb)
 
     def gateway_redirect_cleanup(self, rtr_interface):
-        ns_ipr = ip_lib.IPRule(namespace=self.ns_name)
         ns_ipd = ip_lib.IPDevice(rtr_interface, namespace=self.ns_name)
-        self._stale_ip_rule_cleanup(ns_ipr, ns_ipd, lib_constants.IP_VERSION_4)
-        self._stale_ip_rule_cleanup(ns_ipr, ns_ipd, lib_constants.IP_VERSION_6)
+        self._stale_ip_rule_cleanup(self.ns_name, ns_ipd,
+                                    lib_constants.IP_VERSION_4)
+        self._stale_ip_rule_cleanup(self.ns_name, ns_ipd,
+                                    lib_constants.IP_VERSION_6)
 
     def _snat_redirect_modify(self, gateway, sn_port, sn_int, is_add):
         """Adds or removes rules and routes for SNAT redirection."""
         cmd = ['net.ipv4.conf.%s.send_redirects=0' % sn_int]
         try:
-            ns_ipr = ip_lib.IPRule(namespace=self.ns_name)
             ns_ipd = ip_lib.IPDevice(sn_int, namespace=self.ns_name)
             for port_fixed_ip in sn_port['fixed_ips']:
                 # Iterate and find the gateway IP address matching
@@ -368,17 +425,19 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                         if is_add:
                             ns_ipd.route.add_gateway(gw_ip_addr,
                                                      table=snat_idx)
-                            ns_ipr.rule.add(ip=sn_port_cidr,
-                                            table=snat_idx,
-                                            priority=snat_idx)
+                            ip_lib.add_ip_rule(namespace=self.ns_name,
+                                               ip=sn_port_cidr,
+                                               table=snat_idx,
+                                               priority=snat_idx)
                             ip_lib.sysctl(cmd, namespace=self.ns_name)
                         else:
                             self._delete_gateway_device_if_exists(ns_ipd,
                                                                   gw_ip_addr,
                                                                   snat_idx)
-                            ns_ipr.rule.delete(ip=sn_port_cidr,
-                                               table=snat_idx,
-                                               priority=snat_idx)
+                            ip_lib.delete_ip_rule(self.ns_name,
+                                                  ip=sn_port_cidr,
+                                                  table=snat_idx,
+                                                  priority=snat_idx)
         except Exception:
             if is_add:
                 exc = 'DVR: error adding redirection logic'
@@ -408,7 +467,12 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
             self._set_subnet_arp_info(subnet['id'])
             if ex_gw_port:
                 # Check for address_scopes here if gateway exists.
-                if self._check_if_address_scopes_match(port, ex_gw_port):
+                address_scopes_match = self._check_if_address_scopes_match(
+                    port, ex_gw_port)
+                if (address_scopes_match and
+                    (self.agent_conf.agent_mode in
+                        [lib_constants.L3_AGENT_MODE_DVR,
+                         lib_constants.L3_AGENT_MODE_DVR_SNAT])):
                     self._add_interface_routing_rule_to_router_ns(port)
                     self._add_interface_route_to_fip_ns(port)
         self._snat_redirect_add_from_port(port)
@@ -417,9 +481,12 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         ex_gw_port = self.get_ex_gw_port()
         if not ex_gw_port:
             return
-        if self._check_if_address_scopes_match(port, ex_gw_port):
-            # If address scopes match there is no need to cleanup the
-            # snat redirect rules, hence return here.
+        address_scopes_match = self._check_if_address_scopes_match(
+            port, ex_gw_port)
+        if (address_scopes_match and
+            (self.agent_conf.agent_mode in
+                [lib_constants.L3_AGENT_MODE_DVR,
+                 lib_constants.L3_AGENT_MODE_DVR_SNAT])):
             return
         sn_port = self.get_snat_port_for_internal_port(port)
         if not sn_port:
@@ -438,7 +505,12 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
         # Delete DVR address_scope static route for the removed interface
         # Check for address_scopes here.
-        if self._check_if_address_scopes_match(port, self.ex_gw_port):
+        address_scopes_match = self._check_if_address_scopes_match(
+            port, self.ex_gw_port)
+        if (address_scopes_match and
+            (self.agent_conf.agent_mode in
+                [lib_constants.L3_AGENT_MODE_DVR,
+                 lib_constants.L3_AGENT_MODE_DVR_SNAT])):
             self._delete_interface_route_in_fip_ns(port)
             self._delete_interface_routing_rule_in_router_ns(port)
             # If address scopes match there is no need to cleanup the
@@ -458,7 +530,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def get_floating_agent_gw_interface(self, ext_net_id):
         """Filter Floating Agent GW port for the external network."""
-        fip_ports = self.router.get(n_const.FLOATINGIP_AGENT_INTF_KEY, [])
+        fip_ports = self.router.get(
+            lib_constants.FLOATINGIP_AGENT_INTF_KEY, [])
         return next(
             (p for p in fip_ports if p['network_id'] == ext_net_id), None)
 
@@ -472,20 +545,28 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def enable_snat_redirect_rules(self, ex_gw_port):
         for p in self.internal_ports:
-            if not self._check_if_address_scopes_match(p, ex_gw_port):
-                gateway = self.get_snat_port_for_internal_port(p)
-                if not gateway:
-                    continue
+            gateway = self.get_snat_port_for_internal_port(p)
+            if not gateway:
+                continue
+            address_scopes_match = self._check_if_address_scopes_match(
+                p, ex_gw_port)
+            if (not address_scopes_match or
+                (self.agent_conf.agent_mode ==
+                    lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL)):
                 internal_dev = self.get_internal_device_name(p['id'])
                 self._snat_redirect_add(gateway, p, internal_dev)
 
     def disable_snat_redirect_rules(self, ex_gw_port):
         for p in self.internal_ports:
-            if not self._check_if_address_scopes_match(p, ex_gw_port):
-                gateway = self.get_snat_port_for_internal_port(
-                    p, self.snat_ports)
-                if not gateway:
-                    continue
+            gateway = self.get_snat_port_for_internal_port(
+                p, self.snat_ports)
+            if not gateway:
+                continue
+            address_scopes_match = self._check_if_address_scopes_match(
+                p, ex_gw_port)
+            if (not address_scopes_match or
+                (self.agent_conf.agent_mode ==
+                    lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL)):
                 internal_dev = self.get_internal_device_name(p['id'])
                 self._snat_redirect_remove(gateway, p, internal_dev)
 
@@ -499,14 +580,41 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         self.enable_snat_redirect_rules(ex_gw_port)
         for port in self.get_snat_interfaces():
             for ip in port['fixed_ips']:
+                subnet_id = ip['subnet_id']
+                device, device_exists = self.get_arp_related_dev(subnet_id)
                 self._update_arp_entry(ip['ip_address'],
                                        port['mac_address'],
-                                       ip['subnet_id'],
-                                       'add')
+                                       subnet_id,
+                                       'add',
+                                       device=device,
+                                       device_exists=device_exists)
 
     def external_gateway_updated(self, ex_gw_port, interface_name):
         pass
 
+    def process_floating_ip_nat_rules(self):
+        """Configure NAT rules for the router's floating IPs.
+
+        Configures iptables rules for the floating ips of the given router
+        """
+        # Clear out all iptables rules for floating ips
+        self.iptables_manager.ipv4['nat'].clear_rules_by_tag('floating_ip')
+
+        floating_ips = self.get_floating_ips()
+        # Loop once to ensure that floating ips are configured.
+        for fip in floating_ips:
+            # If floating IP is snat_bound, then the iptables rule should
+            # not be installed to qrouter namespace, since the mixed snat
+            # namespace may already install it.
+            if fip.get(lib_constants.DVR_SNAT_BOUND):
+                continue
+            # Rebuild iptables rules for the floating ip.
+            for chain, rule in self.floating_forward_rules(fip):
+                self.iptables_manager.ipv4['nat'].add_rule(
+                    chain, rule, tag='floating_ip')
+
+        self.iptables_manager.apply()
+
     def external_gateway_removed(self, ex_gw_port, interface_name):
         # TODO(Carl) Should this be calling process_snat_dnat_for_fip?
         self.process_floating_ip_nat_rules()
@@ -573,14 +681,23 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def process_external(self):
         if self.agent_conf.agent_mode != (
-            n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL):
+                lib_constants.L3_AGENT_MODE_DVR_NO_EXTERNAL):
             ex_gw_port = self.get_ex_gw_port()
             if ex_gw_port:
                 self.create_dvr_external_gateway_on_agent(ex_gw_port)
                 self.connect_rtr_2_fip()
         super(DvrLocalRouter, self).process_external()
 
+    def _check_rtr_2_fip_connect(self):
+        """Checks if the rtr to fip connect exists, if not sets to false."""
+        fip_ns_name = self.fip_ns.get_name()
+        if ip_lib.network_namespace_exists(fip_ns_name):
+            fip_2_rtr_name = self.fip_ns.get_int_device_name(self.router_id)
+            if not ip_lib.device_exists(fip_2_rtr_name, namespace=fip_ns_name):
+                self.rtr_fip_connect = False
+
     def connect_rtr_2_fip(self):
+        self._check_rtr_2_fip_connect()
         if self.fip_ns.agent_gateway_port and not self.rtr_fip_connect:
             ex_gw_port = self.get_ex_gw_port()
             self.fip_ns.create_rtr_2_fip_link(self)
@@ -604,8 +721,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         if int_port_addr_value is None:
             return False
         if ((key != lib_constants.IP_VERSION_6) and
-            int_port_addr_scopes.get(str(key)) in
-            ext_port_addr_scopes.values()):
+                int_port_addr_scopes.get(str(key)) in
+                ext_port_addr_scopes.values()):
             return True
         return False
 
@@ -618,7 +735,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                 return
             for subnet in router_port['subnets']:
                 rtr_port_cidr = subnet['cidr']
-                device.route.delete_route(rtr_port_cidr, str(rtr_2_fip_ip))
+                device.route.delete_route(rtr_port_cidr, via=str(rtr_2_fip_ip))
 
     def _add_interface_route_to_fip_ns(self, router_port):
         rtr_2_fip_ip, fip_2_rtr_name = self.get_rtr_fip_ip_and_interface_name()
@@ -632,20 +749,18 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                 device.route.add_route(rtr_port_cidr, str(rtr_2_fip_ip))
 
     def _add_interface_routing_rule_to_router_ns(self, router_port):
-        ip_rule = ip_lib.IPRule(namespace=self.ns_name)
         for subnet in router_port['subnets']:
             rtr_port_cidr = subnet['cidr']
-            ip_rule.rule.add(ip=rtr_port_cidr,
-                             table=dvr_fip_ns.FIP_RT_TBL,
-                             priority=dvr_fip_ns.FAST_PATH_EXIT_PR)
+            ip_lib.add_ip_rule(namespace=self.ns_name, ip=rtr_port_cidr,
+                               table=dvr_fip_ns.FIP_RT_TBL,
+                               priority=dvr_fip_ns.FAST_PATH_EXIT_PR)
 
     def _delete_interface_routing_rule_in_router_ns(self, router_port):
-        ip_rule = ip_lib.IPRule(namespace=self.ns_name)
         for subnet in router_port['subnets']:
             rtr_port_cidr = subnet['cidr']
-            ip_rule.rule.delete(ip=rtr_port_cidr,
-                                table=dvr_fip_ns.FIP_RT_TBL,
-                                priority=dvr_fip_ns.FAST_PATH_EXIT_PR)
+            ip_lib.delete_ip_rule(self.ns_name, ip=rtr_port_cidr,
+                                  table=dvr_fip_ns.FIP_RT_TBL,
+                                  priority=dvr_fip_ns.FAST_PATH_EXIT_PR)
 
     def get_rtr_fip_ip_and_interface_name(self):
         """Function that returns the router to fip interface name and ip."""
@@ -700,8 +815,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                     operation, route, fip_ns_name, tbl_index)
         super(DvrLocalRouter, self).update_routing_table(operation, route)
 
-    def _update_fip_route_table_with_next_hop_routes(
-        self, operation, route, fip_ns_name, tbl_index):
+    def _update_fip_route_table_with_next_hop_routes(self, operation, route,
+                                                     fip_ns_name, tbl_index):
         cmd = ['ip', 'route', operation, 'to', route['destination'],
                'via', route['nexthop'], 'table', tbl_index]
         ip_wrapper = ip_lib.IPWrapper(namespace=fip_ns_name)
@@ -712,8 +827,8 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                       "router %(id)s",
                       {'ns': fip_ns_name, 'id': self.router_id})
 
-    def _check_if_route_applicable_to_fip_namespace(
-        self, route, agent_gateway_port):
+    def _check_if_route_applicable_to_fip_namespace(self, route,
+                                                    agent_gateway_port):
         ip_cidrs = common_utils.fixed_ip_cidrs(agent_gateway_port['fixed_ips'])
         nexthop_cidr = netaddr.IPAddress(route['nexthop'])
         for gw_cidr in ip_cidrs:
