@@ -12,6 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import abc
 import collections
 
 import netaddr
@@ -19,13 +20,15 @@ from neutron_lib import constants as lib_constants
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.utils import helpers
 from oslo_log import log as logging
+from oslo_utils import netutils
+from pyroute2.netlink import exceptions as pyroute2_exc
 
 from neutron._i18n import _
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_manager
 from neutron.agent.linux import ra
-from neutron.common import constants as n_const
+from neutron.common import coordination
 from neutron.common import ipv6_utils
 from neutron.common import utils as common_utils
 from neutron.ipam import utils as ipam_utils
@@ -41,7 +44,7 @@ ADDRESS_SCOPE_MARK_ID_MAX = 2048
 DEFAULT_ADDRESS_SCOPE = "noscope"
 
 
-class RouterInfo(object):
+class BaseRouterInfo(object, metaclass=abc.ABCMeta):
 
     def __init__(self,
                  agent,
@@ -52,16 +55,88 @@ class RouterInfo(object):
                  use_ipv6=False):
         self.agent = agent
         self.router_id = router_id
-        self.agent_conf = agent_conf
-        self.ex_gw_port = None
+        # Invoke the setter for establishing initial SNAT action
         self._snat_enabled = None
-        self.fip_map = {}
+        self.router = router
+        self.agent_conf = agent_conf
+        self.driver = interface_driver
+        self.use_ipv6 = use_ipv6
+
         self.internal_ports = []
+        self.ns_name = None
+        self.process_monitor = None
+
+    def initialize(self, process_monitor):
+        """Initialize the router on the system.
+
+        This differs from __init__ in that this method actually affects the
+        system creating namespaces, starting processes, etc.  The other merely
+        initializes the python object.  This separates in-memory object
+        initialization from methods that actually go do stuff to the system.
+
+        :param process_monitor: The agent's process monitor instance.
+        """
+        self.process_monitor = process_monitor
+
+    @property
+    def router(self):
+        return self._router
+
+    @router.setter
+    def router(self, value):
+        self._router = value
+        if not self._router:
+            return
+        # enable_snat by default if it wasn't specified by plugin
+        self._snat_enabled = self._router.get('enable_snat', True)
+
+    @abc.abstractmethod
+    def delete(self, agent):
+        pass
+
+    @abc.abstractmethod
+    def process(self, agent):
+        """Process updates to this router
+
+        This method is the point where the agent requests that updates be
+        applied to this router.
+
+        :param agent: Passes the agent in order to send RPC messages.
+        """
+        pass
+
+    def get_ex_gw_port(self):
+        return self.router.get('gw_port')
+
+    def get_gw_ns_name(self):
+        return self.ns_name
+
+    def get_internal_device_name(self, port_id):
+        return (INTERNAL_DEV_PREFIX + port_id)[:self.driver.DEV_NAME_LEN]
+
+    def get_external_device_name(self, port_id):
+        return (EXTERNAL_DEV_PREFIX + port_id)[:self.driver.DEV_NAME_LEN]
+
+    def get_external_device_interface_name(self, ex_gw_port):
+        return self.get_external_device_name(ex_gw_port['id'])
+
+
+class RouterInfo(BaseRouterInfo):
+
+    def __init__(self,
+                 agent,
+                 router_id,
+                 router,
+                 agent_conf,
+                 interface_driver,
+                 use_ipv6=False):
+        super(RouterInfo, self).__init__(agent, router_id, router, agent_conf,
+                                         interface_driver, use_ipv6)
+
+        self.ex_gw_port = None
+        self.fip_map = {}
         self.pd_subnets = {}
         self.floating_ips = set()
-        # Invoke the setter for establishing initial SNAT action
-        self.router = router
-        self.use_ipv6 = use_ipv6
         ns = self.create_router_namespace_object(
             router_id, agent_conf, interface_driver, use_ipv6)
         self.router_namespace = ns
@@ -76,8 +151,6 @@ class RouterInfo(object):
         self.initialize_address_scope_iptables()
         self.initialize_metadata_iptables()
         self.routes = []
-        self.driver = interface_driver
-        self.process_monitor = None
         # radvd is a neutron.agent.linux.ra.DaemonMonitor
         self.radvd = None
         self.centralized_port_forwarding_fip_set = set()
@@ -85,16 +158,7 @@ class RouterInfo(object):
         self.qos_gateway_ips = set()
 
     def initialize(self, process_monitor):
-        """Initialize the router on the system.
-
-        This differs from __init__ in that this method actually affects the
-        system creating namespaces, starting processes, etc.  The other merely
-        initializes the python object.  This separates in-memory object
-        initialization from methods that actually go do stuff to the system.
-
-        :param process_monitor: The agent's process monitor instance.
-        """
-        self.process_monitor = process_monitor
+        super(RouterInfo, self).initialize(process_monitor)
         self.radvd = ra.DaemonMonitor(self.router_id,
                                       self.ns_name,
                                       process_monitor,
@@ -108,38 +172,16 @@ class RouterInfo(object):
         return namespaces.RouterNamespace(
             router_id, agent_conf, iface_driver, use_ipv6)
 
-    @property
-    def router(self):
-        return self._router
-
-    @router.setter
-    def router(self, value):
-        self._router = value
-        if not self._router:
-            return
-        # enable_snat by default if it wasn't specified by plugin
-        self._snat_enabled = self._router.get('enable_snat', True)
-
-    def is_router_master(self):
+    def is_router_primary(self):
         return True
 
-    def get_internal_device_name(self, port_id):
-        return (INTERNAL_DEV_PREFIX + port_id)[:self.driver.DEV_NAME_LEN]
-
-    def get_external_device_name(self, port_id):
-        return (EXTERNAL_DEV_PREFIX + port_id)[:self.driver.DEV_NAME_LEN]
-
-    def get_external_device_interface_name(self, ex_gw_port):
-        return self.get_external_device_name(ex_gw_port['id'])
-
-    def get_gw_ns_name(self):
-        return self.ns_name
-
     def _update_routing_table(self, operation, route, namespace):
-        cmd = ['ip', 'route', operation, 'to', route['destination'],
-               'via', route['nexthop']]
-        ip_wrapper = ip_lib.IPWrapper(namespace=namespace)
-        ip_wrapper.netns.execute(cmd, check_exit_code=False)
+        method = (ip_lib.add_ip_route if operation == 'replace' else
+                  ip_lib.delete_ip_route)
+        try:
+            method(namespace, route['destination'], via=route['nexthop'])
+        except (RuntimeError, OSError, pyroute2_exc.NetlinkError):
+            pass
 
     def update_routing_table(self, operation, route):
         self._update_routing_table(operation, route, self.ns_name)
@@ -159,9 +201,6 @@ class RouterInfo(object):
             LOG.debug("Removed route entry is '%s'", route)
             self.update_routing_table('delete', route)
 
-    def get_ex_gw_port(self):
-        return self.router.get('gw_port')
-
     def get_floating_ips(self):
         """Filter Floating IPs to be hosted on this agent."""
         return self.router.get(lib_constants.FLOATINGIP_KEY, [])
@@ -580,6 +619,16 @@ class RouterInfo(object):
                                                 internal_ports)
 
         enable_ra = False
+        for p in old_ports:
+            self.internal_network_removed(p)
+            LOG.debug("removing port %s from internal_ports cache", p)
+            self.internal_ports.remove(p)
+            enable_ra = enable_ra or self._port_has_ipv6_subnet(p)
+            for subnet in p['subnets']:
+                if ipv6_utils.is_ipv6_pd_enabled(subnet):
+                    self.agent.pd.disable_subnet(self.router_id, subnet['id'])
+                    self.pd_subnets.pop(subnet['id'], None)
+
         for p in new_ports:
             self.internal_network_added(p)
             LOG.debug("appending port %s to internal_ports cache", p)
@@ -596,16 +645,6 @@ class RouterInfo(object):
                             lib_constants.PROVISIONAL_IPV6_PD_PREFIX):
                         self.pd_subnets[subnet['id']] = subnet['cidr']
 
-        for p in old_ports:
-            self.internal_network_removed(p)
-            LOG.debug("removing port %s from internal_ports cache", p)
-            self.internal_ports.remove(p)
-            enable_ra = enable_ra or self._port_has_ipv6_subnet(p)
-            for subnet in p['subnets']:
-                if ipv6_utils.is_ipv6_pd_enabled(subnet):
-                    self.agent.pd.disable_subnet(self.router_id, subnet['id'])
-                    del self.pd_subnets[subnet['id']]
-
         updated_cidrs = []
         for p in updated_ports:
             self._update_internal_ports_cache(p)
@@ -660,14 +699,16 @@ class RouterInfo(object):
         return [common_utils.ip_to_cidr(ip['floating_ip_address'])
                 for ip in floating_ips]
 
-    def _plug_external_gateway(self, ex_gw_port, interface_name, ns_name):
+    def _plug_external_gateway(self, ex_gw_port, interface_name, ns_name,
+                               link_up=True):
         self.driver.plug(ex_gw_port['network_id'],
                          ex_gw_port['id'],
                          interface_name,
                          ex_gw_port['mac_address'],
                          namespace=ns_name,
                          prefix=EXTERNAL_DEV_PREFIX,
-                         mtu=ex_gw_port.get('mtu'))
+                         mtu=ex_gw_port.get('mtu'),
+                         link_up=link_up)
 
     def _get_external_gw_ips(self, ex_gw_port):
         gateway_ips = []
@@ -707,13 +748,14 @@ class RouterInfo(object):
             if not self.is_v6_gateway_set(gateway_ips):
                 # There is no IPv6 gw_ip, use RouterAdvt for default route.
                 self.driver.configure_ipv6_ra(
-                    ns_name, interface_name, n_const.ACCEPT_RA_WITH_FORWARDING)
+                    ns_name, interface_name,
+                    lib_constants.ACCEPT_RA_WITH_FORWARDING)
             else:
                 # Otherwise, disable it
                 disable_ra = True
         if disable_ra:
             self.driver.configure_ipv6_ra(ns_name, interface_name,
-                                          n_const.ACCEPT_RA_DISABLED)
+                                          lib_constants.ACCEPT_RA_DISABLED)
         self.driver.configure_ipv6_forwarding(ns_name, interface_name, enabled)
         # This will make sure the 'all' setting is the same as the interface,
         # which is needed for forwarding to work.  Don't disable once it's
@@ -726,7 +768,11 @@ class RouterInfo(object):
         LOG.debug("External gateway added: port(%s), interface(%s), ns(%s)",
                   ex_gw_port, interface_name, ns_name)
         self._plug_external_gateway(ex_gw_port, interface_name, ns_name)
+        self._external_gateway_settings(ex_gw_port, interface_name,
+                                        ns_name, preserve_ips)
 
+    def _external_gateway_settings(self, ex_gw_port, interface_name,
+                                   ns_name, preserve_ips):
         # Build up the interface and gateway IP addresses that
         # will be added to the interface.
         ip_cidrs = common_utils.fixed_ip_cidrs(ex_gw_port['fixed_ips'])
@@ -748,8 +794,8 @@ class RouterInfo(object):
         for ip_version in (lib_constants.IP_VERSION_4,
                            lib_constants.IP_VERSION_6):
             gateway = device.route.get_gateway(ip_version=ip_version)
-            if gateway and gateway.get('gateway'):
-                current_gateways.add(gateway.get('gateway'))
+            if gateway and gateway.get('via'):
+                current_gateways.add(gateway.get('via'))
         for ip in current_gateways - set(gateway_ips):
             device.route.delete_gateway(ip)
         for ip in gateway_ips:
@@ -771,17 +817,19 @@ class RouterInfo(object):
         return any(netaddr.IPAddress(gw_ip).version == 6
                    for gw_ip in gateway_ips)
 
-    def external_gateway_added(self, ex_gw_port, interface_name):
+    def get_router_preserve_ips(self):
         preserve_ips = self._list_floating_ip_cidrs() + list(
             self.centralized_port_forwarding_fip_set)
         preserve_ips.extend(self.agent.pd.get_preserve_ips(self.router_id))
+        return preserve_ips
+
+    def external_gateway_added(self, ex_gw_port, interface_name):
+        preserve_ips = self.get_router_preserve_ips()
         self._external_gateway_added(
             ex_gw_port, interface_name, self.ns_name, preserve_ips)
 
     def external_gateway_updated(self, ex_gw_port, interface_name):
-        preserve_ips = self._list_floating_ip_cidrs() + list(
-            self.centralized_port_forwarding_fip_set)
-        preserve_ips.extend(self.agent.pd.get_preserve_ips(self.router_id))
+        preserve_ips = self.get_router_preserve_ips()
         self._external_gateway_added(
             ex_gw_port, interface_name, self.ns_name, preserve_ips)
 
@@ -846,9 +894,8 @@ class RouterInfo(object):
 
     def _prevent_snat_for_internal_traffic_rule(self, interface_name):
         return (
-            'POSTROUTING', '! -i %(interface_name)s '
-                           '! -o %(interface_name)s -m conntrack ! '
-                           '--ctstate DNAT -j ACCEPT' %
+            'POSTROUTING', '! -o %(interface_name)s -m conntrack '
+                           '! --ctstate DNAT -j ACCEPT' %
                            {'interface_name': interface_name})
 
     def external_gateway_nat_fip_rules(self, ex_gw_ip, interface_name):
@@ -876,7 +923,7 @@ class RouterInfo(object):
         mark = self.agent_conf.external_ingress_mark
         mark_packets_entering_external_gateway_port = (
             'mark', '-i %s -j MARK --set-xmark %s/%s' %
-                    (interface_name, mark, n_const.ROUTER_MARK_MASK))
+                    (interface_name, mark, lib_constants.ROUTER_MARK_MASK))
         return [mark_packets_entering_external_gateway_port]
 
     def _empty_snat_chains(self, iptables_manager):
@@ -939,6 +986,7 @@ class RouterInfo(object):
         finally:
             self.update_fip_statuses(fip_statuses)
 
+    @coordination.synchronized('router-lock-ns-{self.ns_name}')
     def process_external(self):
         fip_statuses = {}
         try:
@@ -1033,16 +1081,30 @@ class RouterInfo(object):
         # requests that arrive before the filter metadata redirect
         # rule is installed will be dropped.
         mark_metadata_for_internal_interfaces = (
-            '-d 169.254.169.254/32 '
+            '-d %(metadata_cidr)s '
             '-i %(interface_name)s '
             '-p tcp -m tcp --dport 80 '
             '-j MARK --set-xmark %(value)s/%(mask)s' %
-            {'interface_name': INTERNAL_DEV_PREFIX + '+',
+            {'metadata_cidr': lib_constants.METADATA_V4_CIDR,
+             'interface_name': INTERNAL_DEV_PREFIX + '+',
              'value': self.agent_conf.metadata_access_mark,
-             'mask': n_const.ROUTER_MARK_MASK})
+             'mask': lib_constants.ROUTER_MARK_MASK})
         self.iptables_manager.ipv4['mangle'].add_rule(
             'PREROUTING', mark_metadata_for_internal_interfaces)
 
+        if netutils.is_ipv6_enabled():
+            mark_metadata_v6_for_internal_interfaces = (
+                '-d %(metadata_v6_ip)s/128 '
+                '-i %(interface_name)s '
+                '-p tcp -m tcp --dport 80 '
+                '-j MARK --set-xmark %(value)s/%(mask)s' %
+                {'metadata_v6_ip': lib_constants.METADATA_V6_IP,
+                 'interface_name': INTERNAL_DEV_PREFIX + '+',
+                 'value': self.agent_conf.metadata_access_mark,
+                 'mask': lib_constants.ROUTER_MARK_MASK})
+            self.iptables_manager.ipv6['mangle'].add_rule(
+                'PREROUTING', mark_metadata_v6_for_internal_interfaces)
+
     def _get_port_devicename_scopemark(self, ports, name_generator):
         devicename_scopemark = {lib_constants.IP_VERSION_4: dict(),
                                 lib_constants.IP_VERSION_6: dict()}
@@ -1147,6 +1209,7 @@ class RouterInfo(object):
             self.get_address_scope_mark_mask(address_scope))
         iptables_manager.ipv4['nat'].add_rule('snat', rule)
 
+    @coordination.synchronized('router-lock-ns-{self.ns_name}')
     def process_address_scope(self):
         with self.iptables_manager.defer_apply():
             self.process_ports_address_scope_iptables()
@@ -1174,13 +1237,6 @@ class RouterInfo(object):
 
     @common_utils.exception_logger()
     def process(self):
-        """Process updates to this router
-
-        This method is the point where the agent requests that updates be
-        applied to this router.
-
-        :param agent: Passes the agent in order to send RPC messages.
-        """
         LOG.debug("Process updates, router %s", self.router['id'])
         self.centralized_port_forwarding_fip_set = set(self.router.get(
             'port_forwardings_fip_set', set()))
