@@ -18,42 +18,35 @@ import time
 
 import netaddr
 from neutron_lib import constants
-from oslo_config import cfg
+from neutron_lib import exceptions
 from oslo_log import log as logging
-from oslo_log import versionutils
-import six
+from oslo_utils import excutils
+from pyroute2.netlink import exceptions as pyroute2_exc
 
-from neutron._i18n import _, _LE, _LI, _LW
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import ip_lib
-from neutron.agent.linux import utils
-from neutron.common import exceptions
-
+from neutron.common import utils
+from neutron.conf.plugins.ml2.drivers import ovs_conf
+from neutron.plugins.ml2.drivers.openvswitch.agent.common \
+    import constants as ovs_const
+from neutron.privileged.agent.linux import ethtool
 
 LOG = logging.getLogger(__name__)
 
-OPTS = [
-    cfg.StrOpt('ovs_integration_bridge',
-               default='br-int',
-               help=_('Name of Open vSwitch bridge to use')),
-    cfg.BoolOpt('ovs_use_veth',
-                default=False,
-                help=_('Uses veth for an OVS interface or not. '
-                       'Support kernels with limited namespace support '
-                       '(e.g. RHEL 6.5) so long as ovs_use_veth is set to '
-                       'True.')),
-]
+
+def _get_veth(name1, name2, namespace2):
+    return (ip_lib.IPDevice(name1),
+            ip_lib.IPDevice(name2, namespace=namespace2))
 
 
-@six.add_metaclass(abc.ABCMeta)
-class LinuxInterfaceDriver(object):
+class LinuxInterfaceDriver(object, metaclass=abc.ABCMeta):
 
-    # from linux IF_NAMESIZE
-    DEV_NAME_LEN = 14
+    DEV_NAME_LEN = constants.LINUX_DEV_LEN
     DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
 
-    def __init__(self, conf):
+    def __init__(self, conf, **kwargs):
         self.conf = conf
+        self._mtu_update_warn_logged = False
 
     @property
     def use_gateway_ips(self):
@@ -111,30 +104,52 @@ class LinuxInterfaceDriver(object):
         # Neutron, so it would be deleted if we added it to the 'previous'
         # list here
         default_ipv6_lla = ip_lib.get_ipv6_lladdr(device.link.address)
-        previous = {addr['cidr'] for addr in device.addr.list(
-            filters=['permanent'])} - {default_ipv6_lla}
 
-        # add new addresses
+        cidrs = set()
+        remove_ips = set()
+
+        # normalize all the IP addresses first
         for ip_cidr in ip_cidrs:
-
             net = netaddr.IPNetwork(ip_cidr)
             # Convert to compact IPv6 address because the return values of
             # "ip addr list" are compact.
             if net.version == 6:
                 ip_cidr = str(net)
-            if ip_cidr in previous:
-                previous.remove(ip_cidr)
+            cidrs.add(ip_cidr)
+
+        # Determine the addresses that must be added and removed
+        for address in device.addr.list():
+            cidr = address['cidr']
+            dynamic = address['dynamic']
+
+            # skip the IPv6 link-local
+            if cidr == default_ipv6_lla:
+                # it's already configured, leave it alone
+                cidrs.discard(cidr)
                 continue
 
-            device.addr.add(ip_cidr)
+            if cidr in preserve_ips:
+                continue
 
-        # clean up any old addresses
-        for ip_cidr in previous:
-            if ip_cidr not in preserve_ips:
-                if clean_connections:
-                    device.delete_addr_and_conntrack_state(ip_cidr)
-                else:
-                    device.addr.delete(ip_cidr)
+            # Statically created addresses are OK, dynamically created
+            # addresses must be removed and replaced
+            if cidr in cidrs and not dynamic:
+                cidrs.remove(cidr)
+                continue
+
+            remove_ips.add(cidr)
+
+        # Clean up any old addresses.  This must be done first since there
+        # could be a dynamic address being replaced with a static one.
+        for ip_cidr in remove_ips:
+            if clean_connections:
+                device.delete_addr_and_conntrack_state(ip_cidr)
+            else:
+                device.addr.delete(ip_cidr)
+
+        # add any new addresses
+        for ip_cidr in cidrs:
+            device.addr.add(ip_cidr)
 
     def init_router_port(self,
                          device_name,
@@ -158,22 +173,34 @@ class LinuxInterfaceDriver(object):
                      namespace=namespace,
                      preserve_ips=preserve_ips or [],
                      clean_connections=clean_connections)
+        self.set_onlink_routes(device_name, namespace, extra_subnets,
+                               preserve_ips)
 
+    def set_onlink_routes(self, device_name, namespace, extra_subnets,
+                          preserve_ips=None, is_ipv6=True):
+        """Manage on-link routes (routes without an associate address)
+
+        :param device_name: interface name
+        :param namespace: namespace name
+        :param extra_subnets: subnets attached to this interface without an IP
+                              address set in this interface
+        :param preserve_ips: IPs or CIDRs not to be deleted from the device
+                             on-link route list
+        """
         device = ip_lib.IPDevice(device_name, namespace=namespace)
-
-        # Manage on-link routes (routes without an associated address)
         new_onlink_cidrs = set(s['cidr'] for s in extra_subnets or [])
+        preserve_ips = set(preserve_ips if preserve_ips else [])
 
-        v4_onlink = device.route.list_onlink_routes(constants.IP_VERSION_4)
-        v6_onlink = device.route.list_onlink_routes(constants.IP_VERSION_6)
-        existing_onlink_cidrs = set(r['cidr'] for r in v4_onlink + v6_onlink)
+        onlink = device.route.list_onlink_routes(constants.IP_VERSION_4)
+        if is_ipv6:
+            onlink += device.route.list_onlink_routes(constants.IP_VERSION_6)
+        existing_onlink_cidrs = set(r['cidr'] for r in onlink)
 
         for route in new_onlink_cidrs - existing_onlink_cidrs:
-            LOG.debug("adding onlink route(%s)", route)
+            LOG.debug('Adding onlink route (%s)', route)
             device.route.add_onlink_route(route)
-        for route in (existing_onlink_cidrs - new_onlink_cidrs -
-                      set(preserve_ips or [])):
-            LOG.debug("deleting onlink route(%s)", route)
+        for route in existing_onlink_cidrs - new_onlink_cidrs - preserve_ips:
+            LOG.debug('Deleting onlink route (%s)', route)
             device.route.delete_onlink_route(route)
 
     def add_ipv6_addr(self, device_name, v6addr, namespace, scope='global'):
@@ -200,10 +227,11 @@ class LinuxInterfaceDriver(object):
                 break
 
     def get_ipv6_llas(self, device_name, namespace):
-        device = ip_lib.IPDevice(device_name,
-                                 namespace=namespace)
-
-        return device.addr.list(scope='link', ip_version=6)
+        kwargs = {'family': utils.get_socket_address_family(
+                                constants.IP_VERSION_6),
+                  'scope': 'link'}
+        return ip_lib.get_devices_with_ip(namespace, name=device_name,
+                                          **kwargs)
 
     def check_bridge_exists(self, bridge):
         if not ip_lib.device_exists(bridge):
@@ -213,33 +241,57 @@ class LinuxInterfaceDriver(object):
         return (self.DEV_NAME_PREFIX + port.id)[:self.DEV_NAME_LEN]
 
     @staticmethod
-    def configure_ipv6_ra(namespace, dev_name):
-        """Configure acceptance of IPv6 route advertisements on an intf."""
-        # Learn the default router's IP address via RAs
-        ip_lib.IPWrapper(namespace=namespace).netns.execute(
-            ['sysctl', '-w', 'net.ipv6.conf.%s.accept_ra=2' % dev_name])
+    def configure_ipv6_ra(namespace, dev_name, value):
+        """Configure handling of IPv6 Router Advertisements on an
+        interface. See common/constants.py for possible values.
+        """
+        cmd = ['net.ipv6.conf.%(dev)s.accept_ra=%(value)s' % {'dev': dev_name,
+                                                              'value': value}]
+        ip_lib.sysctl(cmd, namespace=namespace)
+
+    @staticmethod
+    def configure_ipv6_forwarding(namespace, dev_name, enabled):
+        """Configure IPv6 forwarding on an interface."""
+        cmd = ['net.ipv6.conf.%(dev)s.forwarding=%(enabled)s' %
+               {'dev': dev_name, 'enabled': int(enabled)}]
+        ip_lib.sysctl(cmd, namespace=namespace)
 
     @abc.abstractmethod
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plug in the interface only for new devices that don't exist yet."""
 
     def plug(self, network_id, port_id, device_name, mac_address,
-             bridge=None, namespace=None, prefix=None, mtu=None):
+             bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
         if not ip_lib.device_exists(device_name,
                                     namespace=namespace):
-            try:
-                self.plug_new(network_id, port_id, device_name, mac_address,
-                              bridge, namespace, prefix, mtu)
-            except TypeError:
-                versionutils.report_deprecated_feature(
-                    LOG,
-                    _LW('Interface driver does not support MTU parameter. '
-                        'This may not work in future releases.'))
-                self.plug_new(network_id, port_id, device_name, mac_address,
-                              bridge, namespace, prefix)
+            self._safe_plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu, link_up)
         else:
-            LOG.info(_LI("Device %s already exists"), device_name)
+            LOG.info("Device %s already exists", device_name)
+            if mtu:
+                self.set_mtu(
+                    device_name, mtu, namespace=namespace, prefix=prefix)
+            else:
+                LOG.warning("No MTU configured for port %s", port_id)
+
+    def _safe_plug_new(self, network_id, port_id, device_name, mac_address,
+            bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
+        try:
+            self.plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu, link_up)
+        except TypeError:
+            LOG.warning("Interface driver's plug_new() method should now "
+                        "accept additional optional parameter 'link_up'. "
+                        "Usage of plug_new() method which takes from 5 to 9 "
+                        "positional arguments is now deprecated and will not "
+                        "be possible in W release.")
+            self.plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu)
 
     @abc.abstractmethod
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
@@ -262,10 +314,33 @@ class LinuxInterfaceDriver(object):
         """
         return True
 
+    def set_mtu(self, device_name, mtu, namespace=None, prefix=None):
+        """Set MTU on the interface."""
+        if not self._mtu_update_warn_logged:
+            LOG.warning("Interface driver cannot update MTU for ports")
+            self._mtu_update_warn_logged = True
+
+    def set_link_status(self, device_name, namespace=None, link_up=True):
+        ns_dev = ip_lib.IPWrapper(namespace=namespace).device(device_name)
+        try:
+            utils.wait_until_true(ns_dev.exists, timeout=3)
+        except utils.WaitTimeout:
+            LOG.debug('Device %s may have been deleted concurrently',
+                      device_name)
+            return False
+
+        if link_up:
+            ns_dev.link.set_up()
+        else:
+            ns_dev.link.set_down()
+
+        return True
+
 
 class NullDriver(LinuxInterfaceDriver):
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         pass
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
@@ -277,8 +352,9 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
 
     DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
 
-    def __init__(self, conf):
-        super(OVSInterfaceDriver, self).__init__(conf)
+    def __init__(self, conf, **kwargs):
+        super(OVSInterfaceDriver, self).__init__(conf, **kwargs)
+        ovs_conf.register_ovs_agent_opts(self.conf)
         if self.conf.ovs_use_veth:
             self.DEV_NAME_PREFIX = 'ns-'
 
@@ -300,10 +376,11 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         ovs.replace_port(device_name, *attrs)
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plug in the interface."""
         if not bridge:
-            bridge = self.conf.ovs_integration_bridge
+            bridge = self.conf.OVS.integration_bridge
 
         self.check_bridge_exists(bridge)
 
@@ -330,7 +407,7 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
                 ns_dev.link.set_address(mac_address)
                 break
             except RuntimeError as e:
-                LOG.warning(_LW("Got error trying to set mac, retrying: %s"),
+                LOG.warning("Got error trying to set mac, retrying: %s",
                             str(e))
                 time.sleep(1)
         else:
@@ -339,28 +416,43 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
 
         # Add an interface created by ovs to the namespace.
         if not self.conf.ovs_use_veth and namespace:
-            namespace_obj = ip.ensure_namespace(namespace)
-            namespace_obj.add_device_to_namespace(ns_dev)
+            try:
+                namespace_obj = ip.ensure_namespace(namespace)
+                namespace_obj.add_device_to_namespace(ns_dev)
+            except (pyroute2_exc.NetlinkError, OSError):
+                # To prevent the namespace failure from blasting OVS, the OVS
+                # port creation should be reverted. Possible exceptions:
+                # - NetlinkError in case of duplicated interface
+                # - OSError in case of corrupted namespace
+                LOG.warning("Failed to plug interface %s into bridge %s, "
+                            "cleaning up", device_name, bridge)
+                with excutils.save_and_reraise_exception():
+                    ovs = ovs_lib.OVSBridge(bridge)
+                    ovs.delete_port(tap_name)
 
         # NOTE(ihrachys): the order here is significant: we must set MTU after
         # the device is moved into a namespace, otherwise OVS bridge does not
         # allow to set MTU that is higher than the least of all device MTUs on
         # the bridge
         if mtu:
-            ns_dev.link.set_mtu(mtu)
-            if self.conf.ovs_use_veth:
-                root_dev.link.set_mtu(mtu)
+            self.set_mtu(device_name, mtu, namespace=namespace, prefix=prefix)
         else:
-            LOG.warning(_LW("No MTU configured for port %s"), port_id)
+            LOG.warning("No MTU configured for port %s", port_id)
 
-        ns_dev.link.set_up()
+        if link_up:
+            ns_dev.link.set_up()
         if self.conf.ovs_use_veth:
+            # ovs-dpdk does not do checksum calculations for veth interface
+            # (bug 1832021)
+            if self.conf.OVS.datapath_type == ovs_const.OVS_DATAPATH_NETDEV:
+                ethtool.offload(ns_dev.name, rx=False, tx=False,
+                                namespace=namespace)
             root_dev.link.set_up()
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
         if not bridge:
-            bridge = self.conf.ovs_integration_bridge
+            bridge = self.conf.OVS.integration_bridge
 
         tap_name = self._get_tap_name(device_name, prefix)
         self.check_bridge_exists(bridge)
@@ -373,67 +465,18 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
                 device.link.delete()
                 LOG.debug("Unplugged interface '%s'", device_name)
         except RuntimeError:
-            LOG.error(_LE("Failed unplugging interface '%s'"),
+            LOG.error("Failed unplugging interface '%s'",
                       device_name)
 
-
-class IVSInterfaceDriver(LinuxInterfaceDriver):
-    """Driver for creating an internal interface on an IVS bridge."""
-
-    DEV_NAME_PREFIX = constants.TAP_DEVICE_PREFIX
-
-    def __init__(self, conf):
-        super(IVSInterfaceDriver, self).__init__(conf)
-        self.DEV_NAME_PREFIX = 'ns-'
-
-    def _get_tap_name(self, dev_name, prefix=None):
-        dev_name = dev_name.replace(prefix or self.DEV_NAME_PREFIX,
-                                    constants.TAP_DEVICE_PREFIX)
-        return dev_name
-
-    def _ivs_add_port(self, device_name, port_id, mac_address):
-        cmd = ['ivs-ctl', 'add-port', device_name]
-        utils.execute(cmd, run_as_root=True)
-
-    def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
-        """Plug in the interface."""
-        ip = ip_lib.IPWrapper()
-        tap_name = self._get_tap_name(device_name, prefix)
-
-        root_dev, ns_dev = ip.add_veth(tap_name, device_name)
-        root_dev.disable_ipv6()
-
-        self._ivs_add_port(tap_name, port_id, mac_address)
-
-        ns_dev = ip.device(device_name)
-        ns_dev.link.set_address(mac_address)
-
-        if mtu:
-            ns_dev.link.set_mtu(mtu)
+    def set_mtu(self, device_name, mtu, namespace=None, prefix=None):
+        if self.conf.ovs_use_veth:
+            tap_name = self._get_tap_name(device_name, prefix)
+            root_dev, ns_dev = _get_veth(
+                tap_name, device_name, namespace2=namespace)
             root_dev.link.set_mtu(mtu)
         else:
-            LOG.warning(_LW("No MTU configured for port %s"), port_id)
-
-        if namespace:
-            namespace_obj = ip.ensure_namespace(namespace)
-            namespace_obj.add_device_to_namespace(ns_dev)
-
-        ns_dev.link.set_up()
-        root_dev.link.set_up()
-
-    def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
-        """Unplug the interface."""
-        tap_name = self._get_tap_name(device_name, prefix)
-        try:
-            cmd = ['ivs-ctl', 'del-port', tap_name]
-            utils.execute(cmd, run_as_root=True)
-            device = ip_lib.IPDevice(device_name, namespace=namespace)
-            device.link.delete()
-            LOG.debug("Unplugged interface '%s'", device_name)
-        except RuntimeError:
-            LOG.error(_LE("Failed unplugging interface '%s'"),
-                      device_name)
+            ns_dev = ip_lib.IPWrapper(namespace=namespace).device(device_name)
+        ns_dev.link.set_mtu(mtu)
 
 
 class BridgeInterfaceDriver(LinuxInterfaceDriver):
@@ -442,7 +485,8 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
     DEV_NAME_PREFIX = 'ns-'
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plugin the interface."""
         ip = ip_lib.IPWrapper()
 
@@ -456,13 +500,13 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
         ns_veth.link.set_address(mac_address)
 
         if mtu:
-            root_veth.link.set_mtu(mtu)
-            ns_veth.link.set_mtu(mtu)
+            self.set_mtu(device_name, mtu, namespace=namespace, prefix=prefix)
         else:
-            LOG.warning(_LW("No MTU configured for port %s"), port_id)
+            LOG.warning("No MTU configured for port %s", port_id)
 
         root_veth.link.set_up()
-        ns_veth.link.set_up()
+        if link_up:
+            ns_veth.link.set_up()
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
@@ -471,5 +515,13 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
             device.link.delete()
             LOG.debug("Unplugged interface '%s'", device_name)
         except RuntimeError:
-            LOG.error(_LE("Failed unplugging interface '%s'"),
+            LOG.error("Failed unplugging interface '%s'",
                       device_name)
+
+    def set_mtu(self, device_name, mtu, namespace=None, prefix=None):
+        tap_name = device_name.replace(prefix or self.DEV_NAME_PREFIX,
+                                       constants.TAP_DEVICE_PREFIX)
+        root_dev, ns_dev = _get_veth(
+            tap_name, device_name, namespace2=namespace)
+        root_dev.link.set_mtu(mtu)
+        ns_dev.link.set_mtu(mtu)
