@@ -14,9 +14,13 @@
 #    under the License.
 
 import collections
+import ctypes
+from ctypes import util
+import sys
 
 import netaddr
 from neutron_lib import constants
+from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
@@ -26,24 +30,25 @@ from neutron.agent.linux import ip_conntrack
 from neutron.agent.linux import ipset_manager
 from neutron.agent.linux import iptables_comments as ic
 from neutron.agent.linux import iptables_manager
-from neutron.common import constants as n_const
-from neutron.common import ipv6_utils
+from neutron.agent.linux import utils as a_utils
+from neutron.common import _constants as const
 from neutron.common import utils as c_utils
 
 
 LOG = logging.getLogger(__name__)
 SG_CHAIN = 'sg-chain'
 SPOOF_FILTER = 'spoof-filter'
-CHAIN_NAME_PREFIX = {firewall.INGRESS_DIRECTION: 'i',
-                     firewall.EGRESS_DIRECTION: 'o',
+CHAIN_NAME_PREFIX = {constants.INGRESS_DIRECTION: 'i',
+                     constants.EGRESS_DIRECTION: 'o',
                      SPOOF_FILTER: 's'}
-IPSET_DIRECTION = {firewall.INGRESS_DIRECTION: 'src',
-                   firewall.EGRESS_DIRECTION: 'dst'}
+IPSET_DIRECTION = {constants.INGRESS_DIRECTION: 'src',
+                   constants.EGRESS_DIRECTION: 'dst'}
 comment_rule = iptables_manager.comment_rule
+libc = ctypes.CDLL(util.find_library('libc.so.6'))
 
 
 def get_hybrid_port_name(port_name):
-    return (constants.TAP_DEVICE_PREFIX + port_name)[:n_const.LINUX_DEV_LEN]
+    return (constants.TAP_DEVICE_PREFIX + port_name)[:constants.LINUX_DEV_LEN]
 
 
 class mac_iptables(netaddr.mac_eui48):
@@ -53,13 +58,14 @@ class mac_iptables(netaddr.mac_eui48):
 
 class IptablesFirewallDriver(firewall.FirewallDriver):
     """Driver which enforces security groups through iptables rules."""
-    IPTABLES_DIRECTION = {firewall.INGRESS_DIRECTION: 'physdev-out',
-                          firewall.EGRESS_DIRECTION: 'physdev-in'}
+    IPTABLES_DIRECTION = {constants.INGRESS_DIRECTION: 'physdev-out',
+                          constants.EGRESS_DIRECTION: 'physdev-in'}
     CONNTRACK_ZONE_PER_PORT = False
 
     def __init__(self, namespace=None):
-        self.iptables = iptables_manager.IptablesManager(state_less=True,
-            use_ipv6=ipv6_utils.is_enabled_and_bind_by_default(),
+        self.iptables = iptables_manager.IptablesManager(
+            state_less=True,
+            use_ipv6=netutils.is_ipv6_enabled(),
             namespace=namespace)
         # TODO(majopela, shihanzhang): refactor out ipset to a separate
         # driver composed over this one
@@ -67,6 +73,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         # list of port which has security group
         self.filtered_ports = {}
         self.unfiltered_ports = {}
+        self.trusted_ports = []
         self.ipconntrack = ip_conntrack.get_conntrack(
             self.iptables.get_rules_for_table, self.filtered_ports,
             self.unfiltered_ports, namespace=namespace,
@@ -86,6 +93,37 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         self.updated_rule_sg_ids = set()
         self.updated_sg_members = set()
         self.devices_with_updated_sg_members = collections.defaultdict(list)
+        self._iptables_protocol_name_map = {}
+        self._check_netfilter_for_bridges()
+
+    @staticmethod
+    def _check_netfilter_for_bridges():
+        """Check if br_netfilter is loaded and the needed flags for IPtables"""
+        log_warning = False
+        if not a_utils.execute(
+                ['sysctl', '-N', 'net.bridge'], run_as_root=True,
+                log_fail_as_error=False, check_exit_code=False):
+            LOG.warning('Kernel module br_netfilter is not loaded.')
+            log_warning = True
+        if not log_warning:
+            for proto in ('arp', 'ip', 'ip6'):
+                key = 'net.bridge.bridge-nf-call-%stables' % proto
+                enabled = a_utils.execute(
+                    ['sysctl', '-b', key], run_as_root=True,
+                    log_fail_as_error=False, check_exit_code=False)
+                if enabled == '1':
+                    status = 'enabled'
+                    log_method = LOG.debug
+                else:
+                    status = 'disabled'
+                    log_method = LOG.warning
+                    log_warning = True
+                log_method('Key %(key)s is %(status)s',
+                           {'key': key, 'status': status})
+
+        if log_warning:
+            LOG.warning('Please ensure that netfilter options for bridge are '
+                        'enabled to provide working security groups.')
 
     @property
     def ports(self):
@@ -108,6 +146,29 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             else:
                 self._update_remote_security_group_members(sec_group_ids)
 
+    def process_trusted_ports(self, port_ids):
+        """Process ports that are trusted and shouldn't be filtered."""
+        for port in port_ids:
+            if port not in self.trusted_ports:
+                jump_rule = self._generate_trusted_port_rules(port)
+                self._add_rules_to_chain_v4v6(
+                    'FORWARD', jump_rule, jump_rule, comment=ic.TRUSTED_ACCEPT)
+                self.trusted_ports.append(port)
+
+    def remove_trusted_ports(self, port_ids):
+        for port in port_ids:
+            if port in self.trusted_ports:
+                jump_rule = self._generate_trusted_port_rules(port)
+                self._remove_rule_from_chain_v4v6(
+                    'FORWARD', jump_rule, jump_rule)
+                self.trusted_ports.remove(port)
+
+    def _generate_trusted_port_rules(self, port):
+        rt = '-m physdev --%%s %s --physdev-is-bridged -j ACCEPT' % (
+            self._get_device_name(port))
+        return [rt % (self.IPTABLES_DIRECTION[constants.INGRESS_DIRECTION]),
+                rt % (self.IPTABLES_DIRECTION[constants.EGRESS_DIRECTION])]
+
     def update_security_group_rules(self, sg_id, sg_rules):
         LOG.debug("Update rules of security group (%s)", sg_id)
         self.sg_rules[sg_id] = sg_rules
@@ -205,14 +266,14 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         for pname in sorted(ports):
             port = ports[pname]
             self._add_conntrack_jump(port)
-            self._setup_chain(port, firewall.INGRESS_DIRECTION)
-            self._setup_chain(port, firewall.EGRESS_DIRECTION)
+            self._setup_chain(port, constants.INGRESS_DIRECTION)
+            self._setup_chain(port, constants.EGRESS_DIRECTION)
         self.iptables.ipv4['filter'].add_rule(SG_CHAIN, '-j ACCEPT')
         self.iptables.ipv6['filter'].add_rule(SG_CHAIN, '-j ACCEPT')
 
         for port in unfiltered_ports.values():
-            self._add_accept_rule_port_sec(port, firewall.INGRESS_DIRECTION)
-            self._add_accept_rule_port_sec(port, firewall.EGRESS_DIRECTION)
+            self._add_accept_rule_port_sec(port, constants.INGRESS_DIRECTION)
+            self._add_accept_rule_port_sec(port, constants.EGRESS_DIRECTION)
 
     def _remove_chains(self):
         """Remove ingress and egress chain for a port."""
@@ -222,13 +283,13 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
 
     def _remove_chains_apply(self, ports, unfiltered_ports):
         for port in ports.values():
-            self._remove_chain(port, firewall.INGRESS_DIRECTION)
-            self._remove_chain(port, firewall.EGRESS_DIRECTION)
+            self._remove_chain(port, constants.INGRESS_DIRECTION)
+            self._remove_chain(port, constants.EGRESS_DIRECTION)
             self._remove_chain(port, SPOOF_FILTER)
             self._remove_conntrack_jump(port)
         for port in unfiltered_ports.values():
-            self._remove_rule_port_sec(port, firewall.INGRESS_DIRECTION)
-            self._remove_rule_port_sec(port, firewall.EGRESS_DIRECTION)
+            self._remove_rule_port_sec(port, constants.INGRESS_DIRECTION)
+            self._remove_rule_port_sec(port, constants.EGRESS_DIRECTION)
         self._remove_chain_by_name_v4v6(SG_CHAIN)
 
     def _setup_chain(self, port, DIRECTION):
@@ -256,16 +317,18 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         self.iptables.ipv6['filter'].remove_chain(chain_name)
 
     def _add_rules_to_chain_v4v6(self, chain_name, ipv4_rules, ipv6_rules,
-                                 comment=None):
+                                 top=False, comment=None):
         for rule in ipv4_rules:
             self.iptables.ipv4['filter'].add_rule(chain_name, rule,
-                                                  comment=comment)
+                                                  top=top, comment=comment)
 
         for rule in ipv6_rules:
             self.iptables.ipv6['filter'].add_rule(chain_name, rule,
-                                                  comment=comment)
+                                                  top=top, comment=comment)
 
     def _get_device_name(self, port):
+        if not isinstance(port, dict):
+            return port
         return port['device']
 
     def _update_port_sec_rules(self, port, direction, add=False):
@@ -281,7 +344,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         else:
             self._remove_rule_from_chain_v4v6('FORWARD', jump_rule, jump_rule)
 
-        if direction == firewall.EGRESS_DIRECTION:
+        if direction == constants.EGRESS_DIRECTION:
             if add:
                 self._add_rules_to_chain_v4v6('INPUT', jump_rule, jump_rule,
                                               comment=ic.PORT_SEC_ACCEPT)
@@ -304,8 +367,10 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
                      '-j $%s' % (self.IPTABLES_DIRECTION[direction],
                                  device,
                                  SG_CHAIN)]
+        # Security group chain has to be applied before unfiltered
+        # or trusted ports
         self._add_rules_to_chain_v4v6('FORWARD', jump_rule, jump_rule,
-                                      comment=ic.VM_INT_SG)
+                                      top=True, comment=ic.VM_INT_SG)
 
         # jump to the chain based on the device
         jump_rule = ['-m physdev --%s %s --physdev-is-bridged '
@@ -315,37 +380,79 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         self._add_rules_to_chain_v4v6(SG_CHAIN, jump_rule, jump_rule,
                                       comment=ic.SG_TO_VM_SG)
 
-        if direction == firewall.EGRESS_DIRECTION:
+        if direction == constants.EGRESS_DIRECTION:
             self._add_rules_to_chain_v4v6('INPUT', jump_rule, jump_rule,
                                           comment=ic.INPUT_TO_SG)
 
     def _get_br_device_name(self, port):
-        return ('brq' + port['network_id'])[:n_const.LINUX_DEV_LEN]
+        return ('brq' + port['network_id'])[:constants.LINUX_DEV_LEN]
 
-    def _get_jump_rules(self, port):
-        zone = self.ipconntrack.get_device_zone(port)
+    def _get_port_device_name(self, port):
+        if port['device'].startswith(constants.TAP_DEVICE_PREFIX):
+            return port['device'][4:]
+        return port['device']
+
+    def _get_jump_rules(self, port, create=True):
+        zone = self.ipconntrack.get_device_zone(port, create=create)
+        if not zone:
+            return []
         br_dev = self._get_br_device_name(port)
         port_dev = self._get_device_name(port)
         # match by interface for bridge input
         match_interface = '-i %s'
         match_physdev = '-m physdev --physdev-in %s'
-        # comment to prevent duplicate warnings for different devices using
-        # same bridge. truncate start to remove prefixes
-        comment = '-m comment --comment "Set zone for %s"' % port['device'][4:]
+        port_sg_rules = self._get_port_sg_rules(port)
+        if self._are_sg_rules_stateful(port_sg_rules):
+            # comment to prevent duplicate warnings for different devices using
+            # same bridge. truncate start to remove prefixes
+            comment = 'Set zone for %s' % self._get_port_device_name(port)
+            conntrack = '--zone %s' % self.ipconntrack.get_device_zone(port)
+        else:
+            comment = 'Make %s stateless' % self._get_port_device_name(port)
+            conntrack = '--notrack'
         rules = []
         for dev, match in ((br_dev, match_physdev), (br_dev, match_interface),
                            (port_dev, match_physdev)):
             match = match % dev
-            rule = '%s %s -j CT --zone %s' % (match, comment, zone)
+            rule = '%s -m comment --comment "%s" -j CT %s' % (match, comment,
+                                                              conntrack)
             rules.append(rule)
         return rules
 
+    def _get_port_sg_rules(self, port):
+        port_sg_rules = []
+        if not any(port.get('device_owner', '').startswith(prefix)
+                   for prefix in constants.DEVICE_OWNER_PREFIXES):
+            port_sg_ids = port.get('security_groups', [])
+            if port_sg_ids:
+                for rule in self.sg_rules.get(port_sg_ids[0], []):
+                    if self.enable_ipset:
+                        port_sg_rules.append(rule)
+                        break
+                    port_sg_rules.extend(
+                        self._expand_sg_rule_with_remote_ips(
+                            rule, port, constants.INGRESS_DIRECTION))
+                    if port_sg_rules:
+                        break
+                    port_sg_rules.extend(
+                        self._expand_sg_rule_with_remote_ips(
+                            rule, port, constants.EGRESS_DIRECTION))
+                    if port_sg_rules:
+                        break
+        return port_sg_rules
+
+    @staticmethod
+    def _are_sg_rules_stateful(security_group_rules):
+        for rule in security_group_rules:
+            return rule.get('stateful', True)
+        return True
+
     def _add_conntrack_jump(self, port):
         for jump_rule in self._get_jump_rules(port):
             self._add_raw_rule('PREROUTING', jump_rule)
 
     def _remove_conntrack_jump(self, port):
-        for jump_rule in self._get_jump_rules(port):
+        for jump_rule in self._get_jump_rules(port, create=False):
             self._remove_raw_rule('PREROUTING', jump_rule)
 
     def _add_raw_rule(self, chain, rule, comment=None):
@@ -363,8 +470,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             if rule.get('ethertype') == constants.IPv4:
                 ipv4_sg_rules.append(rule)
             elif rule.get('ethertype') == constants.IPv6:
-                if rule.get('protocol') == 'icmp':
-                    rule['protocol'] = 'ipv6-icmp'
+                if rule.get('protocol') in const.IPV6_ICMP_LEGACY_PROTO_LIST:
+                    rule['protocol'] = constants.PROTO_NAME_IPV6_ICMP
                 ipv6_sg_rules.append(rule)
         return ipv4_sg_rules, ipv6_sg_rules
 
@@ -457,7 +564,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
                                     '-j RETURN', comment=ic.DHCP_CLIENT)]
 
     def _drop_dhcp_rule(self, ipv4_rules, ipv6_rules):
-        #Note(nati) Drop dhcp packet from VM
+        # Note(nati) Drop dhcp packet from VM
         ipv4_rules += [comment_rule('-p udp -m udp --sport 67 '
                                     '--dport 68 '
                                     '-j DROP', comment=ic.DHCP_SPOOF)]
@@ -497,7 +604,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             ethertype = rule['ethertype']
             port_ips = port.get('fixed_ips', [])
 
-            for ip in self.sg_members[remote_group_id][ethertype]:
+            for ip, _mac in self.sg_members[remote_group_id][ethertype]:
                 if ip not in port_ips:
                     ip_rule = rule.copy()
                     direction_ip_prefix = firewall.DIRECTION_IP_PREFIX[
@@ -532,11 +639,11 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         ipv4_iptables_rules = []
         ipv6_iptables_rules = []
         # include fixed egress/ingress rules
-        if direction == firewall.EGRESS_DIRECTION:
+        if direction == constants.EGRESS_DIRECTION:
             self._add_fixed_egress_rules(port,
                                          ipv4_iptables_rules,
                                          ipv6_iptables_rules)
-        elif direction == firewall.INGRESS_DIRECTION:
+        elif direction == constants.INGRESS_DIRECTION:
             ipv6_iptables_rules += self._accept_inbound_icmpv6()
         # include IPv4 and IPv6 iptable rules from security group
         ipv4_iptables_rules += self._convert_sgr_to_iptables_rules(
@@ -559,8 +666,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         ethertype = sg_rule.get('ethertype')
         ipset_name = self.ipset.get_name(remote_gid, ethertype)
         if not self.ipset.set_name_exists(ipset_name):
-            #NOTE(mangelajo): ipsets for empty groups are not created
-            #                 thus we can't reference them.
+            # NOTE(mangelajo): ipsets for empty groups are not created
+            #                  thus we can't reference them.
             return None
         ipset_direction = IPSET_DIRECTION[sg_rule.get('direction')]
         args = self._generate_protocol_and_port_args(sg_rule)
@@ -633,18 +740,53 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             comment=ic.ALLOW_ASSOC)]
         return iptables_rules
 
+    def _local_protocol_name_map(self):
+        local_protocol_name_map = {}
+        try:
+            class protoent(ctypes.Structure):
+                _fields_ = [("p_name", ctypes.c_char_p),
+                            ("p_aliases", ctypes.POINTER(ctypes.c_char_p)),
+                            ("p_proto", ctypes.c_int)]
+            libc.getprotoent.restype = ctypes.POINTER(protoent)
+            libc.setprotoent(0)
+            while True:
+                pr = libc.getprotoent()
+                if not pr:
+                    break
+                r = pr[0]
+                p_name = helpers.safe_decode_utf8(r.p_name)
+                local_protocol_name_map[str(r.p_proto)] = p_name
+        except Exception:
+            LOG.exception("Unable to create local protocol name map: %s",
+                          sys.exc_info()[0])
+        finally:
+            libc.endprotoent()
+        return local_protocol_name_map
+
+    def _protocol_name_map(self):
+        if not self._iptables_protocol_name_map:
+            tmp_map = constants.IPTABLES_PROTOCOL_NAME_MAP.copy()
+            tmp_map.update(self._local_protocol_name_map())
+            self._iptables_protocol_name_map = tmp_map
+        return self._iptables_protocol_name_map
+
+    def _iptables_protocol_name(self, protocol):
+        # protocol zero is a special case and requires no '-p'
+        if protocol and protocol != '0':
+            return self._protocol_name_map().get(protocol, protocol)
+
     def _protocol_arg(self, protocol, is_port):
         iptables_rule = []
-        rule_protocol = n_const.IPTABLES_PROTOCOL_NAME_MAP.get(protocol,
-                                                               protocol)
+        rule_protocol = self._iptables_protocol_name(protocol)
         # protocol zero is a special case and requires no '-p'
         if rule_protocol:
             iptables_rule = ['-p', rule_protocol]
 
             if (is_port and rule_protocol in constants.IPTABLES_PROTOCOL_MAP):
                 # iptables adds '-m protocol' when the port number is specified
-                iptables_rule += ['-m',
-                    constants.IPTABLES_PROTOCOL_MAP[rule_protocol]]
+                iptables_rule += [
+                    '-m', constants.IPTABLES_PROTOCOL_MAP[rule_protocol]
+                ]
         return iptables_rule
 
     def _port_arg(self, direction, protocol, port_range_min, port_range_max):
@@ -652,7 +794,7 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
         if port_range_min is None:
             return args
 
-        protocol = n_const.IPTABLES_PROTOCOL_NAME_MAP.get(protocol, protocol)
+        protocol = self._iptables_protocol_name(protocol)
         if protocol in ['icmp', 'ipv6-icmp']:
             protocol_type = 'icmpv6' if protocol == 'ipv6-icmp' else 'icmp'
             # Note(xuhanp): port_range_min/port_range_max represent
@@ -661,15 +803,22 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             # icmp code can be 0 so we cannot use "if port_range_max" here
             if port_range_max is not None:
                 args[-1] += '/%s' % port_range_max
-        elif port_range_min == port_range_max:
-            args += ['--%s' % direction, '%s' % (port_range_min,)]
-        else:
-            args += ['-m', 'multiport', '--%ss' % direction,
-                     '%s:%s' % (port_range_min, port_range_max)]
+        elif protocol in const.SG_PORT_PROTO_NAMES:
+            # iptables protocols that support --dport, --sport and -m multiport
+            if port_range_min == port_range_max:
+                if protocol in const.IPTABLES_MULTIPORT_ONLY_PROTOCOLS:
+                    # use -m multiport, but without a port range
+                    args += ['-m', 'multiport', '--%ss' % direction,
+                             '%s' % port_range_min]
+                else:
+                    args += ['--%s' % direction, '%s' % port_range_min]
+            else:
+                args += ['-m', 'multiport', '--%ss' % direction,
+                         '%s:%s' % (port_range_min, port_range_max)]
         return args
 
     def _ip_prefix_arg(self, direction, ip_prefix):
-        #NOTE (nati) : source_group_id is converted to list of source_
+        # NOTE (nati) : source_group_id is converted to list of source_
         # ip_prefix in server side
         if ip_prefix:
             if '/' not in ip_prefix:
@@ -843,7 +992,8 @@ class IptablesFirewallDriver(firewall.FirewallDriver):
             self._clean_deleted_remote_sg_members_conntrack_entries()
 
     def _get_sg_members(self, sg_info, sg_id, ethertype):
-        return set(sg_info.get(sg_id, {}).get(ethertype, []))
+        ip_mac_addresses = sg_info.get(sg_id, {}).get(ethertype, [])
+        return set([ip_mac[0] for ip_mac in ip_mac_addresses])
 
     def filter_defer_apply_off(self):
         if self._defer_apply:
@@ -868,7 +1018,9 @@ class OVSHybridIptablesFirewallDriver(IptablesFirewallDriver):
             '%s%s' % (CHAIN_NAME_PREFIX[direction], port['device']))
 
     def _get_br_device_name(self, port):
-        return ('qvb' + port['device'])[:n_const.LINUX_DEV_LEN]
+        return ('qvb' + port['device'])[:constants.LINUX_DEV_LEN]
 
     def _get_device_name(self, port):
-        return get_hybrid_port_name(port['device'])
+        device_name = super(
+            OVSHybridIptablesFirewallDriver, self)._get_device_name(port)
+        return get_hybrid_port_name(device_name)
