@@ -25,6 +25,7 @@ import os
 import re
 import sys
 
+from neutron_lib import constants
 from neutron_lib import exceptions
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.utils import runtime
@@ -37,7 +38,7 @@ from neutron._i18n import _
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import iptables_comments as ic
 from neutron.agent.linux import utils as linux_utils
-from neutron.common import constants
+from neutron.common import _constants as n_const
 from neutron.common import utils
 from neutron.conf.agent import common as config
 
@@ -307,7 +308,8 @@ class IptablesManager(object):
     _random_fully = None
 
     def __init__(self, _execute=None, state_less=False, use_ipv6=False,
-                 namespace=None, binary_name=binary_name):
+                 nat=True, namespace=None, binary_name=binary_name,
+                 external_lock=True):
         if _execute:
             self.execute = _execute
         else:
@@ -317,6 +319,7 @@ class IptablesManager(object):
         self.namespace = namespace
         self.iptables_apply_deferred = False
         self.wrap_name = binary_name[:16]
+        self.external_lock = external_lock
 
         self.ipv4 = {'filter': IptablesTable(binary_name=self.wrap_name)}
         self.ipv6 = {'filter': IptablesTable(binary_name=self.wrap_name)}
@@ -347,7 +350,8 @@ class IptablesManager(object):
 
         if not state_less:
             self.initialize_mangle_table()
-            self.initialize_nat_table()
+            if nat:
+                self.initialize_nat_table()
 
     def initialize_mangle_table(self):
         self.ipv4.update(
@@ -370,9 +374,12 @@ class IptablesManager(object):
     def initialize_nat_table(self):
         self.ipv4.update(
             {'nat': IptablesTable(binary_name=self.wrap_name)})
+        self.ipv6.update(
+            {'nat': IptablesTable(binary_name=self.wrap_name)})
 
         builtin_chains = {
-            4: {'nat': ['PREROUTING', 'OUTPUT', 'POSTROUTING']}}
+            4: {'nat': ['PREROUTING', 'OUTPUT', 'POSTROUTING']},
+            6: {'nat': ['PREROUTING']}}
         self._configure_builtin_chains(builtin_chains)
 
         # Add a neutron-postrouting-bottom chain. It's intended to be
@@ -458,10 +465,12 @@ class IptablesManager(object):
         # NOTE(ihrachys) we may get rid of the lock once all supported
         # platforms get iptables with 999eaa241212d3952ddff39a99d0d55a74e3639e
         # ("iptables-restore: support acquiring the lock.")
-        with lockutils.lock(lock_name, runtime.SYNCHRONIZED_PREFIX, True):
+        with lockutils.lock(lock_name, runtime.SYNCHRONIZED_PREFIX,
+                            external=self.external_lock):
             first = self._apply_synchronized()
             if not cfg.CONF.AGENT.debug_iptables_rules:
                 return first
+            LOG.debug('List of IPTables Rules applied: %s', '\n'.join(first))
             second = self._apply_synchronized()
             if second:
                 msg = (_("IPTables Rules did not converge. Diff: %s") %
@@ -491,7 +500,7 @@ class IptablesManager(object):
 
         version = self._get_version()
         self.__class__._random_fully = utils.is_version_greater_equal(
-            version, constants.IPTABLES_RANDOM_FULLY_VERSION)
+            version, n_const.IPTABLES_RANDOM_FULLY_VERSION)
         return self._random_fully
 
     @property
@@ -845,7 +854,7 @@ def _generate_chain_diff_iptables_commands(chain, old_chain_rules,
         if line.startswith('?'):
             # skip ? because that's a guide string for intraline differences
             continue
-        elif line.startswith('-'):  # line deleted
+        if line.startswith('-'):  # line deleted
             statements.append('-D %s %d' % (chain, old_index))
             # since we are removing a line from the old rules, we
             # backup the index by 1
