@@ -32,6 +32,7 @@ from oslo_utils import netutils
 from neutron._i18n import _
 from neutron.agent.common import ovs_lib
 from neutron.agent import firewall
+from neutron.agent.linux import ip_conntrack
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import iptables
@@ -476,13 +477,12 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         """
         self.permitted_ethertypes = cfg.CONF.SECURITYGROUP.permitted_ethertypes
         self.int_br = self.initialize_bridge(integration_bridge)
-        self.sg_port_map = SGPortMap()
-        self.conj_ip_manager = ConjIPFlowManager(self)
-        self.sg_to_delete = set()
+        self._initialize_sg()
         self._update_cookie = None
         self._deferred = False
         self.iptables_helper = iptables.Helper(self.int_br.br)
         self.iptables_helper.load_driver_if_needed()
+        self.ipconntrack = ip_conntrack.OvsIpConntrackManager()
         self._initialize_firewall()
 
         callbacks_registry.subscribe(
@@ -492,8 +492,14 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def _init_firewall_callback(self, resource, event, trigger, payload=None):
         LOG.info("Reinitialize Openvswitch firewall after OVS restart.")
+        self._initialize_sg()
         self._initialize_firewall()
 
+    def _initialize_sg(self):
+        self.sg_port_map = SGPortMap()
+        self.conj_ip_manager = ConjIPFlowManager(self)
+        self.sg_to_delete = set()
+
     def _initialize_firewall(self):
         self._drop_all_unmatched_flows()
         self._initialize_common_flows()
@@ -608,6 +614,12 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         return get_physical_network_from_other_config(
             self.int_br.br, port_name)
 
+    def _delete_invalid_conntrack_entries_for_port(self, port, of_port):
+        port['of_port'] = of_port
+        for ethertype in [lib_const.IPv4, lib_const.IPv6]:
+            self.ipconntrack.delete_conntrack_state_by_remote_ips(
+                [port], ethertype, set(), mark=ovsfw_consts.CT_MARK_INVALID)
+
     def get_ofport(self, port):
         port_id = port['device']
         return self.sg_port_map.ports.get(port_id)
@@ -662,6 +674,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 self._update_flows_for_port(of_port, old_of_port)
             else:
                 self._set_port_filters(of_port)
+            self._delete_invalid_conntrack_entries_for_port(port, of_port)
         except exceptions.OVSFWPortNotFound as not_found_error:
             LOG.info("port %(port_id)s does not exist in ovsdb: %(err)s.",
                      {'port_id': port['device'],
@@ -701,6 +714,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             else:
                 self._set_port_filters(of_port)
 
+            self._delete_invalid_conntrack_entries_for_port(port, of_port)
+
         except exceptions.OVSFWPortNotFound as not_found_error:
             LOG.info("port %(port_id)s does not exist in ovsdb: %(err)s.",
                      {'port_id': port['device'],
@@ -894,19 +909,36 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         self._initialize_egress(port)
         self._initialize_ingress(port)
 
-    def _initialize_egress_ipv6_icmp(self, port):
-        for icmp_type in firewall.ICMPV6_ALLOWED_EGRESS_TYPES:
-            self._add_flow(
-                table=ovs_consts.BASE_EGRESS_TABLE,
-                priority=95,
-                in_port=port.ofport,
-                reg_port=port.ofport,
-                dl_type=lib_const.ETHERTYPE_IPV6,
-                nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
-                icmp_type=icmp_type,
-                actions='resubmit(,%d)' % (
-                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
-            )
+    def _initialize_egress_ipv6_icmp(self, port, allowed_pairs):
+        allowed_pairs = allowed_pairs.union({(port.mac, port.lla_address)})
+        for mac_addr, ip_addr in allowed_pairs:
+            for icmp_type in firewall.ICMPV6_ALLOWED_EGRESS_TYPES:
+                self._add_flow(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=95,
+                    in_port=port.ofport,
+                    reg_port=port.ofport,
+                    dl_type=lib_const.ETHERTYPE_IPV6,
+                    nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
+                    icmp_type=icmp_type,
+                    dl_src=mac_addr,
+                    ipv6_src=ip_addr,
+                    actions='resubmit(,%d)' % (
+                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+                )
+            for icmp_type in firewall.ICMPV6_RESTRICTED_EGRESS_TYPES:
+                self._add_flow(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=95,
+                    in_port=port.ofport,
+                    reg_port=port.ofport,
+                    dl_type=lib_const.ETHERTYPE_IPV6,
+                    nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
+                    icmp_type=icmp_type,
+                    nd_target=ip_addr,
+                    actions='resubmit(,%d)' % (
+                        ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE)
+                )
 
     def _initialize_egress_no_port_security(self, port_id, ovs_ports=None):
         try:
@@ -979,12 +1011,11 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def _initialize_egress(self, port):
         """Identify egress traffic and send it to egress base"""
-        self._initialize_egress_ipv6_icmp(port)
 
         # Apply mac/ip pairs for IPv4
-        allowed_pairs = port.allowed_pairs_v4.union(
+        allowed_mac_ipv4_pairs = port.allowed_pairs_v4.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv4_addresses})
-        for mac_addr, ip_addr in allowed_pairs:
+        for mac_addr, ip_addr in allowed_mac_ipv4_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=95,
@@ -1010,9 +1041,10 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # Apply mac/ip pairs for IPv6
-        allowed_pairs = port.allowed_pairs_v6.union(
+        allowed_mac_ipv6_pairs = port.allowed_pairs_v6.union(
             {(port.mac, ip_addr) for ip_addr in port.ipv6_addresses})
-        for mac_addr, ip_addr in allowed_pairs:
+        self._initialize_egress_ipv6_icmp(port, allowed_mac_ipv6_pairs)
+        for mac_addr, ip_addr in allowed_mac_ipv6_pairs:
             self._add_flow(
                 table=ovs_consts.BASE_EGRESS_TABLE,
                 priority=65,
@@ -1027,21 +1059,30 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
         # DHCP discovery
-        for dl_type, src_port, dst_port in (
-                (lib_const.ETHERTYPE_IP, 68, 67),
-                (lib_const.ETHERTYPE_IPV6, 546, 547)):
-            self._add_flow(
-                table=ovs_consts.BASE_EGRESS_TABLE,
-                priority=80,
-                reg_port=port.ofport,
-                in_port=port.ofport,
-                dl_type=dl_type,
-                nw_proto=lib_const.PROTO_NUM_UDP,
-                tp_src=src_port,
-                tp_dst=dst_port,
-                actions='resubmit(,{:d})'.format(
-                    ovs_consts.ACCEPT_OR_INGRESS_TABLE)
-            )
+        additional_ipv4_filters = [
+            {"dl_src": mac, "nw_src": ip}
+            for mac, ip in (*allowed_mac_ipv4_pairs,
+                            (port.mac, '0.0.0.0'),)]
+        additional_ipv6_filters = [
+            {"dl_src": mac, "ipv6_src": ip}
+            for mac, ip in allowed_mac_ipv6_pairs]
+        for dl_type, src_port, dst_port, filters_list in (
+                (lib_const.ETHERTYPE_IP, 68, 67, additional_ipv4_filters),
+                (lib_const.ETHERTYPE_IPV6, 546, 547, additional_ipv6_filters)):
+            for additional_filters in filters_list:
+                self._add_flow(
+                    table=ovs_consts.BASE_EGRESS_TABLE,
+                    priority=80,
+                    reg_port=port.ofport,
+                    in_port=port.ofport,
+                    dl_type=dl_type,
+                    **additional_filters,
+                    nw_proto=lib_const.PROTO_NUM_UDP,
+                    tp_src=src_port,
+                    tp_dst=dst_port,
+                    actions='resubmit(,{:d})'.format(
+                        ovs_consts.ACCEPT_OR_INGRESS_TABLE)
+                )
         # Ban dhcp service running on an instance
         for dl_type, src_port, dst_port in (
                 (lib_const.ETHERTYPE_IP, 67, 68),
