@@ -13,43 +13,38 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import fcntl
 import glob
 import grp
+from http import client as httplib
 import os
 import pwd
 import shlex
 import socket
-import struct
 import threading
+import time
 
-import debtcollector
 import eventlet
 from eventlet.green import subprocess
-from eventlet import greenthread
-from neutron_lib import constants
+from neutron_lib import exceptions
+from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_rootwrap import client
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-from six.moves import http_client as httplib
+from oslo_utils import fileutils
 
-from neutron._i18n import _, _LE
-from neutron.agent.common import config
+from neutron._i18n import _
+from neutron.agent.linux import xenapi_root_helper
 from neutron.common import utils
+from neutron.conf.agent import common as config
+from neutron.privileged.agent.linux import utils as priv_utils
 from neutron import wsgi
 
 
 LOG = logging.getLogger(__name__)
 
 
-class ProcessExecutionError(RuntimeError):
-    def __init__(self, message, returncode):
-        super(ProcessExecutionError, self).__init__(message)
-        self.returncode = returncode
-
-
 class RootwrapDaemonHelper(object):
     __client = None
     __lock = threading.Lock()
@@ -62,8 +57,12 @@ class RootwrapDaemonHelper(object):
     def get_client(cls):
         with cls.__lock:
             if cls.__client is None:
-                cls.__client = client.Client(
-                    shlex.split(cfg.CONF.AGENT.root_helper_daemon))
+                if (xenapi_root_helper.ROOT_HELPER_DAEMON_TOKEN ==
+                        cfg.CONF.AGENT.root_helper_daemon):
+                    cls.__client = xenapi_root_helper.XenAPIClient()
+                else:
+                    cls.__client = client.Client(
+                        shlex.split(cfg.CONF.AGENT.root_helper_daemon))
             return cls.__client
 
 
@@ -104,7 +103,11 @@ def execute_rootwrap_daemon(cmd, process_input, addl_env):
     # just logging the execution error.
     LOG.debug("Running command (rootwrap daemon): %s", cmd)
     client = RootwrapDaemonHelper.get_client()
-    return client.execute(cmd, process_input)
+    try:
+        return client.execute(cmd, process_input)
+    except Exception:
+        with excutils.save_and_reraise_exception():
+            LOG.error("Rootwrap error running command: %s", cmd)
 
 
 def execute(cmd, process_input=None, addl_env=None,
@@ -124,16 +127,18 @@ def execute(cmd, process_input=None, addl_env=None,
             _stdout, _stderr = obj.communicate(_process_input)
             returncode = obj.returncode
             obj.stdin.close()
-        _stdout = utils.safe_decode_utf8(_stdout)
-        _stderr = utils.safe_decode_utf8(_stderr)
+        _stdout = helpers.safe_decode_utf8(_stdout)
+        _stderr = helpers.safe_decode_utf8(_stderr)
 
         extra_ok_codes = extra_ok_codes or []
         if returncode and returncode not in extra_ok_codes:
             msg = _("Exit code: %(returncode)d; "
+                    "Cmd: %(cmd)s; "
                     "Stdin: %(stdin)s; "
                     "Stdout: %(stdout)s; "
                     "Stderr: %(stderr)s") % {
                         'returncode': returncode,
+                        'cmd': cmd,
                         'stdin': process_input or '',
                         'stdout': _stdout,
                         'stderr': _stderr}
@@ -141,37 +146,27 @@ def execute(cmd, process_input=None, addl_env=None,
             if log_fail_as_error:
                 LOG.error(msg)
             if check_exit_code:
-                raise ProcessExecutionError(msg, returncode=returncode)
-        else:
-            LOG.debug("Exit code: %d", returncode)
+                raise exceptions.ProcessExecutionError(msg,
+                                                       returncode=returncode)
 
     finally:
         # NOTE(termie): this appears to be necessary to let the subprocess
         #               call clean something up in between calls, without
         #               it two execute calls in a row hangs the second one
-        greenthread.sleep(0)
+        time.sleep(0)
 
     return (_stdout, _stderr) if return_stderr else _stdout
 
 
-def get_interface_mac(interface):
-    MAC_START = 18
-    MAC_END = 24
-    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-    dev = interface[:constants.DEVICE_NAME_MAX_LEN]
-    dev = encodeutils.to_utf8(dev)
-    info = fcntl.ioctl(s.fileno(), 0x8927, struct.pack('256s', dev))
-    return ''.join(['%02x:' % ord(char)
-                    for char in info[MAC_START:MAC_END]])[:-1]
-
-
-def find_child_pids(pid):
-    """Retrieve a list of the pids of child processes of the given pid."""
+def find_child_pids(pid, recursive=False):
+    """Retrieve a list of the pids of child processes of the given pid.
 
+    It can also find all children through the hierarchy if recursive=True
+    """
     try:
         raw_pids = execute(['ps', '--ppid', pid, '-o', 'pid='],
                            log_fail_as_error=False)
-    except ProcessExecutionError as e:
+    except exceptions.ProcessExecutionError as e:
         # Unexpected errors are the responsibility of the caller
         with excutils.save_and_reraise_exception() as ctxt:
             # Exception has already been logged by execute
@@ -179,16 +174,75 @@ def find_child_pids(pid):
             if no_children_found:
                 ctxt.reraise = False
                 return []
-    return [x.strip() for x in raw_pids.split('\n') if x.strip()]
+    child_pids = [x.strip() for x in raw_pids.split('\n') if x.strip()]
+    if recursive:
+        for child in child_pids:
+            child_pids = child_pids + find_child_pids(child, True)
+    return child_pids
+
+
+def find_parent_pid(pid):
+    """Retrieve the pid of the parent process of the given pid.
+
+    If the pid doesn't exist in the system, this function will return
+    None
+    """
+    try:
+        ppid = execute(['ps', '-o', 'ppid=', pid],
+                       log_fail_as_error=False)
+    except exceptions.ProcessExecutionError as e:
+        # Unexpected errors are the responsibility of the caller
+        with excutils.save_and_reraise_exception() as ctxt:
+            # Exception has already been logged by execute
+            no_such_pid = e.returncode == 1
+            if no_such_pid:
+                ctxt.reraise = False
+                return
+    return ppid.strip()
+
+
+def get_process_count_by_name(name):
+    """Find the process count by name."""
+    try:
+        out = execute(['ps', '-C', name, '-o', 'comm='],
+                      log_fail_as_error=False)
+    except exceptions.ProcessExecutionError:
+        with excutils.save_and_reraise_exception(reraise=False):
+            return 0
+    return len(out.strip('\n').split('\n'))
+
+
+def find_fork_top_parent(pid):
+    """Retrieve the pid of the top parent of the given pid through a fork.
+
+    This function will search the top parent with its same cmdline. If the
+    given pid has no parent, its own pid will be returned
+    """
+    while True:
+        ppid = find_parent_pid(pid)
+        if (ppid and ppid != pid and
+                pid_invoked_with_cmdline(ppid, get_cmdline_from_pid(pid))):
+            pid = ppid
+        else:
+            return pid
+
+
+def kill_process(pid, signal, run_as_root=False):
+    """Kill the process with the given pid using the given signal."""
+    try:
+        execute(['kill', '-%d' % signal, pid], run_as_root=run_as_root)
+    except exceptions.ProcessExecutionError:
+        if process_is_running(pid):
+            raise
 
 
 def _get_conf_base(cfg_root, uuid, ensure_conf_dir):
-    #TODO(mangelajo): separate responsibilities here, ensure_conf_dir
-    #                 should be a separate function
+    # TODO(mangelajo): separate responsibilities here, ensure_conf_dir
+    #                  should be a separate function
     conf_dir = os.path.abspath(os.path.normpath(cfg_root))
     conf_base = os.path.join(conf_dir, uuid)
     if ensure_conf_dir:
-        utils.ensure_dir(conf_dir)
+        fileutils.ensure_tree(conf_dir, mode=0o755)
     return conf_base
 
 
@@ -205,9 +259,10 @@ def get_value_from_file(filename, converter=None):
             try:
                 return converter(f.read()) if converter else f.read()
             except ValueError:
-                LOG.error(_LE('Unable to convert value in %s'), filename)
-    except IOError:
-        LOG.debug('Unable to access %s', filename)
+                LOG.error('Unable to convert value in %s', filename)
+    except IOError as error:
+        LOG.debug('Unable to access %(filename)s; Error: %(error)s',
+                  {'filename': filename, 'error': error})
 
 
 def remove_conf_files(cfg_root, uuid):
@@ -217,8 +272,7 @@ def remove_conf_files(cfg_root, uuid):
 
 
 def get_root_helper_child_pid(pid, expected_cmd, run_as_root=False):
-    """
-    Get the first non root_helper child pid in the process hierarchy.
+    """Get the first non root_helper child pid in the process hierarchy.
 
     If root helper was used, two or more processes would be created:
 
@@ -263,14 +317,44 @@ def remove_abs_path(cmd):
     return cmd
 
 
+def process_is_running(pid):
+    """Find if the given PID is running in the system.
+
+    """
+    return pid and os.path.exists('/proc/%s' % pid)
+
+
 def get_cmdline_from_pid(pid):
-    if pid is None or not os.path.exists('/proc/%s' % pid):
+    if not process_is_running(pid):
         return []
-    with open('/proc/%s/cmdline' % pid, 'r') as f:
-        return f.readline().split('\0')[:-1]
+    # NOTE(jh): Even after the above check, the process may terminate
+    # before the open below happens
+    try:
+        with open('/proc/%s/cmdline' % pid, 'r') as f:
+            cmdline = f.readline().split('\0')[:-1]
+    except IOError:
+        return []
+
+    # NOTE(slaweq): sometimes it may happen that values in
+    # /proc/{pid}/cmdline are separated by space instead of NUL char,
+    # in such case we would have everything in one element of cmdline_args
+    # list and it would not match to expected cmd so we need to try to
+    # split it by spaces
+    if len(cmdline) == 1:
+        cmdline = cmdline[0].split(' ')
+
+    LOG.debug("Found cmdline %s for process with PID %s.", cmdline, pid)
+    return cmdline
 
 
-def cmd_matches_expected(cmd, expected_cmd):
+def cmd_matches_expected(cmd, expected_cmd, process_name):
+    if process_name and cmd and cmd[0] == process_name:
+        # If Neutron has defined the title (setproctitle) of the running
+        # process, the "ps" output will be "<process_name> (cmd)"
+        cmd = cmd[1:]
+        cmd[0] = cmd[0].strip('(')
+        cmd[-1] = cmd[-1].strip(')')
+
     abs_cmd = remove_abs_path(cmd)
     abs_expected_cmd = remove_abs_path(expected_cmd)
     if abs_cmd != abs_expected_cmd:
@@ -281,17 +365,12 @@ def cmd_matches_expected(cmd, expected_cmd):
     return abs_cmd == abs_expected_cmd
 
 
-def pid_invoked_with_cmdline(pid, expected_cmd):
+def pid_invoked_with_cmdline(pid, expected_cmd, process_name=None):
     """Validate process with given pid is running with provided parameters
 
     """
     cmd = get_cmdline_from_pid(pid)
-    return cmd_matches_expected(cmd, expected_cmd)
-
-
-wait_until_true = debtcollector.moves.moved_function(
-    utils.wait_until_true, 'wait_until_true', __name__,
-    version='Newton', removal_version='Ocata')
+    return cmd_matches_expected(cmd, expected_cmd, process_name)
 
 
 def ensure_directory_exists_without_file(path):
@@ -304,7 +383,7 @@ def ensure_directory_exists_without_file(path):
                 if not os.path.exists(path):
                     ctxt.reraise = False
     else:
-        utils.ensure_dir(dirname)
+        fileutils.ensure_tree(dirname, mode=0o755)
 
 
 def is_effective_user(user_id_or_name):
@@ -325,6 +404,14 @@ def is_effective_group(group_id_or_name):
     return group_id_or_name == effective_group_name
 
 
+def delete_if_exists(path, run_as_root=False):
+    """Delete a path, executed as normal user or with elevated privileges"""
+    if run_as_root:
+        priv_utils.delete_if_exists(path)
+    else:
+        fileutils.delete_if_exists(path)
+
+
 class UnixDomainHTTPConnection(httplib.HTTPConnection):
     """Connection class for HTTP over UNIX domain socket."""
     def __init__(self, host, port=None, strict=None, timeout=None,
@@ -341,18 +428,36 @@ class UnixDomainHTTPConnection(httplib.HTTPConnection):
 
 
 class UnixDomainHttpProtocol(eventlet.wsgi.HttpProtocol):
-    # TODO(jlibosva): This is just a workaround not to set TCP_NODELAY on
-    # socket due to 40714b1ffadd47b315ca07f9b85009448f0fe63d evenlet change
-    # This should be removed once
-    # https://github.com/eventlet/eventlet/issues/301 is fixed
-    disable_nagle_algorithm = False
+    def __init__(self, *args):
+        # NOTE(yamahata): from eventlet v0.22 HttpProtocol.__init__
+        # signature was changed by changeset of
+        # 7f53465578543156e7251e243c0636e087a8445f
+        # Both have server as last arg, but first arg(s) differ
+        server = args[-1]
 
-    def __init__(self, request, client_address, server):
-        if client_address == '':
-            client_address = ('<local>', 0)
-        # base class is old-style, so super does not work properly
-        eventlet.wsgi.HttpProtocol.__init__(self, request, client_address,
-                                            server)
+        # Because the caller is eventlet.wsgi.Server.process_request,
+        # the number of arguments will dictate if it is new or old style.
+        if len(args) == 2:
+            conn_state = args[0]
+            client_address = conn_state[0]
+            if not client_address:
+                conn_state[0] = ('<local>', 0)
+            # base class is old-style, so super does not work properly
+            eventlet.wsgi.HttpProtocol.__init__(self, conn_state, server)
+        elif len(args) == 3:
+            request = args[0]
+            client_address = args[1]
+            if not client_address:
+                client_address = ('<local>', 0)
+            # base class is old-style, so super does not work properly
+            # NOTE: eventlet 0.22 or later changes the number of args to 2.
+            # If we install eventlet 0.22 or later into a venv for pylint,
+            # pylint complains this. Let's skip it. (bug 1791178)
+            # pylint: disable=too-many-function-args
+            eventlet.wsgi.HttpProtocol.__init__(
+                self, request, client_address, server)
+        else:
+            eventlet.wsgi.HttpProtocol.__init__(self, *args)
 
 
 class UnixDomainWSGIServer(wsgi.Server):
@@ -379,4 +484,5 @@ class UnixDomainWSGIServer(wsgi.Server):
                              application,
                              max_size=self.num_threads,
                              protocol=UnixDomainHttpProtocol,
-                             log=logger)
+                             log=logger,
+                             log_format=cfg.CONF.wsgi_log_format)
