@@ -13,10 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import errno
 import grp
 import os
 import pwd
+import signal
 
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -32,10 +32,15 @@ from neutron.agent.l3 import ha_router
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
+from neutron.agent.linux import utils as linux_utils
+from neutron.common import coordination
+from neutron.common import utils as common_utils
 
 
 LOG = logging.getLogger(__name__)
 
+SIGTERM_TIMEOUT = 5
+
 METADATA_SERVICE_NAME = 'metadata-proxy'
 HAPROXY_SERVICE = 'haproxy'
 
@@ -175,13 +180,7 @@ class HaproxyConfigurator(object):
         cfg_path = os.path.join(
             HaproxyConfigurator.get_config_path(state_path),
             "%s.conf" % uuid)
-        try:
-            os.unlink(cfg_path)
-        except OSError as ex:
-            # It can happen that this function is called but metadata proxy
-            # was never spawned so its config file won't exist
-            if ex.errno != errno.ENOENT:
-                raise
+        linux_utils.delete_if_exists(cfg_path, run_as_root=True)
 
 
 class MetadataDriver(object):
@@ -279,10 +278,19 @@ class MetadataDriver(object):
         monitor.unregister(uuid, METADATA_SERVICE_NAME)
         pm = cls._get_metadata_proxy_process_manager(uuid, conf,
                                                      ns_name=ns_name)
-        pm.disable()
+        pm.disable(sig=str(int(signal.SIGTERM)))
+        try:
+            common_utils.wait_until_true(lambda: not pm.active,
+                                         timeout=SIGTERM_TIMEOUT)
+        except common_utils.WaitTimeout:
+            LOG.warning('Metadata process %s did not finish after SIGTERM '
+                        'signal in %s seconds, sending SIGKILL signal',
+                        pm.pid, SIGTERM_TIMEOUT)
+            pm.disable(sig=str(int(signal.SIGKILL)))
 
-        # Delete metadata proxy config file
+        # Delete metadata proxy config and PID files.
         HaproxyConfigurator.cleanup_config_file(uuid, cfg.CONF.state_path)
+        linux_utils.delete_if_exists(pm.get_pid_file_name(), run_as_root=True)
 
         cls.monitors.pop(uuid, None)
 
@@ -300,26 +308,10 @@ class MetadataDriver(object):
 def after_router_added(resource, event, l3_agent, **kwargs):
     router = kwargs['router']
     proxy = l3_agent.metadata_driver
-    ipv6_enabled = netutils.is_ipv6_enabled()
-    for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
-                                            proxy.metadata_access_mark):
-        router.iptables_manager.ipv4['filter'].add_rule(c, r)
-    if ipv6_enabled:
-        for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
-                                                proxy.metadata_access_mark):
-            router.iptables_manager.ipv6['filter'].add_rule(c, r)
-    for c, r in proxy.metadata_nat_rules(proxy.metadata_port):
-        router.iptables_manager.ipv4['nat'].add_rule(c, r)
-    if ipv6_enabled:
-        for c, r in proxy.metadata_nat_rules(
-                proxy.metadata_port,
-                metadata_address=(constants.METADATA_V6_IP + '/128')):
-            router.iptables_manager.ipv6['nat'].add_rule(c, r)
-    router.iptables_manager.apply()
-
+    apply_metadata_nat_rules(router, proxy)
     if not isinstance(router, ha_router.HaRouter):
         spawn_kwargs = {}
-        if ipv6_enabled:
+        if netutils.is_ipv6_enabled():
             spawn_kwargs['bind_address'] = '::'
         proxy.spawn_monitored_metadata_proxy(
             l3_agent.process_monitor,
@@ -355,3 +347,22 @@ def before_router_removed(resource, event, l3_agent, payload=None):
                                            router.router['id'],
                                            l3_agent.conf,
                                            router.ns_name)
+
+
+@coordination.synchronized('router-lock-ns-{router.ns_name}')
+def apply_metadata_nat_rules(router, proxy):
+    for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
+                                            proxy.metadata_access_mark):
+        router.iptables_manager.ipv4['filter'].add_rule(c, r)
+    if netutils.is_ipv6_enabled():
+        for c, r in proxy.metadata_filter_rules(proxy.metadata_port,
+                                                proxy.metadata_access_mark):
+            router.iptables_manager.ipv6['filter'].add_rule(c, r)
+    for c, r in proxy.metadata_nat_rules(proxy.metadata_port):
+        router.iptables_manager.ipv4['nat'].add_rule(c, r)
+    if netutils.is_ipv6_enabled():
+        for c, r in proxy.metadata_nat_rules(
+                proxy.metadata_port,
+                metadata_address=(constants.METADATA_V6_IP + '/128')):
+            router.iptables_manager.ipv6['nat'].add_rule(c, r)
+    router.iptables_manager.apply()
