@@ -25,13 +25,15 @@ from neutron_lib.callbacks import resources as callback_resources
 from neutron_lib import constants
 from neutron_lib.plugins import utils
 from neutron_lib import rpc as lib_rpc
+from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
+from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 
 from neutron.agent import resource_cache
 from neutron.api.rpc.callbacks import resources
-from neutron.common import constants as n_const
+from neutron.common import _constants as n_const
 from neutron import objects
 
 LOG = logging.getLogger(__name__)
@@ -78,7 +80,7 @@ class PluginReportStateAPI(object):
     """
     def __init__(self, topic):
         target = oslo_messaging.Target(topic=topic, version='1.2',
-                                       namespace=n_const.RPC_NAMESPACE_STATE)
+                                       namespace=constants.RPC_NAMESPACE_STATE)
         self.client = lib_rpc.get_client(target)
 
     def has_alive_neutron_server(self, context, **kwargs):
@@ -112,6 +114,10 @@ class PluginApi(object):
         1.4 - tunnel_sync rpc signature upgrade to obtain 'host'
         1.5 - Support update_device_list and
               get_devices_details_list_and_failed_devices
+        1.6 - Support get_network_details
+        1.7 - Support get_ports_by_vnic_type_and_host
+        1.8 - Rename agent_restarted to refresh_tunnels in
+              update_device_list to reflect its expanded purpose
     '''
 
     def __init__(self, topic):
@@ -129,7 +135,8 @@ class PluginApi(object):
                           devices=devices, agent_id=agent_id, host=host)
 
     def get_devices_details_list_and_failed_devices(self, context, devices,
-                                                    agent_id, host=None):
+                                                    agent_id, host=None,
+                                                    **kwargs):
         """Get devices details and the list of devices that failed.
 
         This method returns the devices details. If an error is thrown when
@@ -142,6 +149,11 @@ class PluginApi(object):
             'get_devices_details_list_and_failed_devices',
             devices=devices, agent_id=agent_id, host=host)
 
+    def get_network_details(self, context, network, agent_id, host=None):
+        cctxt = self.client.prepare(version='1.6')
+        return cctxt.call(context, 'get_network_details', network=network,
+                          agent_id=agent_id, host=host)
+
     def update_device_down(self, context, device, agent_id, host=None):
         cctxt = self.client.prepare()
         return cctxt.call(context, 'update_device_down', device=device,
@@ -153,8 +165,8 @@ class PluginApi(object):
                           agent_id=agent_id, host=host)
 
     def update_device_list(self, context, devices_up, devices_down,
-                           agent_id, host, agent_restarted=False):
-        cctxt = self.client.prepare(version='1.5')
+                           agent_id, host, refresh_tunnels=False):
+        cctxt = self.client.prepare(version='1.8')
 
         ret_devices_up = []
         failed_devices_up = []
@@ -170,7 +182,7 @@ class PluginApi(object):
                              devices_up=devices_up[i:i + step],
                              devices_down=devices_down[i:i + step],
                              agent_id=agent_id, host=host,
-                             agent_restarted=agent_restarted)
+                             refresh_tunnels=refresh_tunnels)
             ret_devices_up.extend(ret.get("devices_up", []))
             failed_devices_up.extend(ret.get("failed_devices_up", []))
             ret_devices_down.extend(ret.get("devices_down", []))
@@ -186,28 +198,24 @@ class PluginApi(object):
         return cctxt.call(context, 'tunnel_sync', tunnel_ip=tunnel_ip,
                           tunnel_type=tunnel_type, host=host)
 
-
-def create_cache_for_l2_agent():
-    """Create a push-notifications cache for L2 agent related resources."""
-
-    objects.register_objects()
-    resource_types = [
-        resources.PORT,
-        resources.SECURITYGROUP,
-        resources.SECURITYGROUPRULE,
-        resources.NETWORK,
-        resources.SUBNET
-    ]
-    rcache = resource_cache.RemoteResourceCache(resource_types)
-    rcache.start_watcher()
-    return rcache
+    def get_ports_by_vnic_type_and_host(self, context, vnic_type, host):
+        cctxt = self.client.prepare(version='1.7')
+        return cctxt.call(context, 'get_ports_by_vnic_type_and_host',
+                          vnic_type=vnic_type, host=host)
 
 
 class CacheBackedPluginApi(PluginApi):
 
+    RESOURCE_TYPES = [resources.PORT,
+                      resources.SECURITYGROUP,
+                      resources.SECURITYGROUPRULE,
+                      resources.NETWORK,
+                      resources.SUBNET]
+
     def __init__(self, *args, **kwargs):
         super(CacheBackedPluginApi, self).__init__(*args, **kwargs)
-        self.remote_resource_cache = create_cache_for_l2_agent()
+        self.remote_resource_cache = None
+        self._create_cache_for_l2_agent()
 
     def register_legacy_notification_callbacks(self, legacy_interface):
         """Emulates the server-side notifications from ml2 AgentNotifierApi.
@@ -228,6 +236,7 @@ class CacheBackedPluginApi(PluginApi):
         the payloads the handlers are expecting (an ID).
         """
         rtype = rtype.lower()  # all legacy handlers don't camelcase
+        agent_restarted = kwargs.pop("agent_restarted", None)
         method, host_with_activation, host_with_deactivation = (
             self._get_method_host(rtype, event, **kwargs))
         if not hasattr(self._legacy_interface, method):
@@ -244,6 +253,9 @@ class CacheBackedPluginApi(PluginApi):
         else:
             payload = {rtype: {'id': resource_id},
                        '%s_id' % rtype: resource_id}
+            if method == "port_update" and agent_restarted is not None:
+                # Mark ovs-agent restart for local port_update
+                payload["agent_restarted"] = agent_restarted
             getattr(self._legacy_interface, method)(context, **payload)
 
     def _get_method_host(self, rtype, event, **kwargs):
@@ -288,20 +300,23 @@ class CacheBackedPluginApi(PluginApi):
         return method, host_with_activation, host_with_deactivation
 
     def get_devices_details_list_and_failed_devices(self, context, devices,
-                                                    agent_id, host=None):
+                                                    agent_id, host=None,
+                                                    agent_restarted=False):
         result = {'devices': [], 'failed_devices': []}
         for device in devices:
             try:
                 result['devices'].append(
-                    self.get_device_details(context, device, agent_id, host))
+                    self.get_device_details(context, device, agent_id, host,
+                                            agent_restarted))
             except Exception:
                 LOG.exception("Failed to get details for device %s", device)
                 result['failed_devices'].append(device)
         return result
 
-    def get_device_details(self, context, device, agent_id, host=None):
+    def get_device_details(self, context, device, agent_id, host=None,
+                           agent_restarted=False):
         port_obj = self.remote_resource_cache.get_resource_by_id(
-            resources.PORT, device)
+            resources.PORT, device, agent_restarted)
         if not port_obj:
             LOG.debug("Device %s does not exist in cache.", device)
             return {'device': device}
@@ -315,13 +330,15 @@ class CacheBackedPluginApi(PluginApi):
         binding = utils.get_port_binding_by_status_and_host(
             port_obj.bindings, constants.ACTIVE, raise_if_not_found=True,
             port_id=port_obj.id)
-        if (port_obj.device_owner.startswith(
-                constants.DEVICE_OWNER_COMPUTE_PREFIX) and
+        migrating_to = migrating_to_host(port_obj.bindings)
+        if (not (migrating_to and cfg.CONF.nova.live_migration_events) and
+                port_obj.device_owner.startswith(
+                    constants.DEVICE_OWNER_COMPUTE_PREFIX) and
                 binding[pb_ext.HOST] != host):
             LOG.debug("Device %s has no active binding in this host",
                       port_obj)
             return {'device': device,
-                    n_const.NO_ACTIVE_BINDING: True}
+                    constants.NO_ACTIVE_BINDING: True}
         net = self.remote_resource_cache.get_resource_by_id(
             resources.NETWORK, port_obj.network_id)
         net_qos_policy_id = net.qos_policy_id
@@ -330,6 +347,7 @@ class CacheBackedPluginApi(PluginApi):
                                    dialect=netaddr.mac_unix_expanded))
         entry = {
             'device': device,
+            'device_id': port_obj.device_id,
             'network_id': port_obj.network_id,
             'port_id': port_obj.id,
             'mac_address': mac_addr,
@@ -349,7 +367,10 @@ class CacheBackedPluginApi(PluginApi):
             'qos_policy_id': port_obj.qos_policy_id,
             'network_qos_policy_id': net_qos_policy_id,
             'profile': binding.profile,
-            'security_groups': list(port_obj.security_group_ids)
+            'vif_type': binding.vif_type,
+            'vnic_type': binding.vnic_type,
+            'security_groups': list(port_obj.security_group_ids),
+            'migrating_to': migrating_to,
         }
         LOG.debug("Returning: %s", entry)
         return entry
@@ -357,3 +378,37 @@ class CacheBackedPluginApi(PluginApi):
     def get_devices_details_list(self, context, devices, agent_id, host=None):
         return [self.get_device_details(context, device, agent_id, host)
                 for device in devices]
+
+    def _create_cache_for_l2_agent(self):
+        """Create a push-notifications cache for L2 agent related resources."""
+        objects.register_objects()
+        rcache = resource_cache.RemoteResourceCache(self.RESOURCE_TYPES)
+        rcache.start_watcher()
+        self.remote_resource_cache = rcache
+
+
+# TODO(ralonsoh): move this method to neutron_lib.plugins.utils
+def migrating_to_host(bindings, host=None):
+    """Return the host the port is being migrated.
+
+    If the host is passed, the port binding profile with the "migrating_to",
+    that contains the host the port is being migrated, is compared to this
+    value. If no value is passed, this method will return if the port is
+    being migrated ("migrating_to" is present in any port binding profile).
+
+    The function returns None or the matching host.
+    """
+    for binding in (binding for binding in bindings if
+                    binding[pb_ext.STATUS] == constants.ACTIVE):
+        profile = binding.get('profile')
+        if not profile:
+            continue
+        profile = (jsonutils.loads(profile) if isinstance(profile, str) else
+                   profile)
+        migrating_to = profile.get('migrating_to')
+        if migrating_to:
+            if not host:  # Just know if the port is being migrated.
+                return migrating_to
+            if migrating_to == host:
+                return migrating_to
+    return None
