@@ -16,13 +16,15 @@
 
 import functools
 
+from neutron_lib.api.definitions import rbac_security_groups as rbac_sg_apidef
+from neutron_lib.api.definitions import stateful_security_group as stateful_sg
+from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron._i18n import _LI, _LW
 from neutron.agent import firewall
-from neutron.api.rpc.handlers import securitygroups_rpc
+from neutron.common import _constants as common_constants
 from neutron.conf.agent import securitygroups_rpc as sc_cfg
 
 
@@ -32,24 +34,7 @@ LOG = logging.getLogger(__name__)
 sc_cfg.register_securitygroups_opts()
 
 
-#This is backward compatibility check for Havana
-def _is_valid_driver_combination():
-    return ((cfg.CONF.SECURITYGROUP.enable_security_group and
-             (cfg.CONF.SECURITYGROUP.firewall_driver and
-              cfg.CONF.SECURITYGROUP.firewall_driver !=
-             'neutron.agent.firewall.NoopFirewallDriver')) or
-            (not cfg.CONF.SECURITYGROUP.enable_security_group and
-             (cfg.CONF.SECURITYGROUP.firewall_driver ==
-             'neutron.agent.firewall.NoopFirewallDriver' or
-              cfg.CONF.SECURITYGROUP.firewall_driver is None)
-             ))
-
-
 def is_firewall_enabled():
-    if not _is_valid_driver_combination():
-        LOG.warning(_LW("Driver configuration doesn't match with "
-                        "enable_security_group"))
-
     return cfg.CONF.SECURITYGROUP.enable_security_group
 
 
@@ -60,10 +45,14 @@ def _disable_extension(extension, aliases):
 
 def disable_security_group_extension_by_config(aliases):
     if not is_firewall_enabled():
-        LOG.info(_LI('Disabled security-group extension.'))
+        LOG.info('Disabled security-group extension.')
         _disable_extension('security-group', aliases)
-        LOG.info(_LI('Disabled allowed-address-pairs extension.'))
+        _disable_extension(rbac_sg_apidef.ALIAS, aliases)
+        _disable_extension(stateful_sg.ALIAS, aliases)
+        LOG.info('Disabled allowed-address-pairs extension.')
         _disable_extension('allowed-address-pairs', aliases)
+        LOG.info('Disabled address-group extension.')
+        _disable_extension('address-group', aliases)
 
 
 class SecurityGroupAgentRpc(object):
@@ -74,14 +63,49 @@ class SecurityGroupAgentRpc(object):
         self.context = context
         self.plugin_rpc = plugin_rpc
         self.init_firewall(defer_refresh_firewall, integration_bridge)
+        # _latest_port_filter_lock will point to the lock created for the
+        # most recent thread to enter _apply_port_filters().
+        self._latest_port_filter_lock = lockutils.ReaderWriterLock()
+
+    def _get_trusted_devices(self, device_ids, devices):
+        trusted_devices = []
+        # Devices which are already added in firewall ports should
+        # not be treated as trusted devices but as regular ports
+        all_devices = devices.copy()
+        all_devices.update(self.firewall.ports)
+        device_names = [
+            dev['device'] for dev in all_devices.values()]
+        for device_id in device_ids:
+            if (device_id not in all_devices.keys() and
+                    device_id not in device_names):
+                trusted_devices.append(device_id)
+        return trusted_devices
+
+    def _port_filter_lock(func):
+        """Decorator to acquire a new lock while applying port filters"""
+        @functools.wraps(func)
+        def decorated_function(self, *args, **kwargs):
+            lock = lockutils.ReaderWriterLock()
+            # Tracking the most recent lock at the instance level allows
+            # waiters to only wait for the most recent lock to be released
+            # instead of waiting until all locks have been released.
+            self._latest_port_filter_lock = lock
+            with lock.write_lock():
+                return func(self, *args, **kwargs)
+        return decorated_function
+
+    def _port_filter_wait(func):
+        """Decorator to wait for the latest port filter lock to be released"""
+        @functools.wraps(func)
+        def decorated_function(self, *args, **kwargs):
+            with self._latest_port_filter_lock.read_lock():
+                return func(self, *args, **kwargs)
+        return decorated_function
 
     def init_firewall(self, defer_refresh_firewall=False,
                       integration_bridge=None):
         firewall_driver = cfg.CONF.SECURITYGROUP.firewall_driver or 'noop'
         LOG.debug("Init firewall settings (driver=%s)", firewall_driver)
-        if not _is_valid_driver_combination():
-            LOG.warning(_LW("Driver configuration doesn't match "
-                            "with enable_security_group"))
         firewall_class = firewall.load_firewall_driver_class(firewall_driver)
         try:
             self.firewall = firewall_class(
@@ -94,8 +118,6 @@ class SecurityGroupAgentRpc(object):
         # Stores devices for which firewall should be refreshed when
         # deferred refresh is enabled.
         self.devices_to_refilter = set()
-        # Flag raised when a global refresh is needed
-        self.global_refresh_firewall = False
         self._use_enhanced_rpc = None
 
     @property
@@ -110,10 +132,10 @@ class SecurityGroupAgentRpc(object):
             self.plugin_rpc.security_group_info_for_devices(
                 self.context, devices=[])
         except oslo_messaging.UnsupportedVersion:
-            LOG.warning(_LW('security_group_info_for_devices rpc call not '
-                            'supported by the server, falling back to old '
-                            'security_group_rules_for_devices which scales '
-                            'worse.'))
+            LOG.warning('security_group_info_for_devices rpc call not '
+                        'supported by the server, falling back to old '
+                        'security_group_rules_for_devices which scales '
+                        'worse.')
             return False
         return True
 
@@ -121,32 +143,43 @@ class SecurityGroupAgentRpc(object):
         @functools.wraps(func)
         def decorated_function(self, *args, **kwargs):
             if (isinstance(self.firewall, firewall.NoopFirewallDriver) or
-                not is_firewall_enabled()):
-                LOG.info(_LI("Skipping method %s as firewall is disabled "
-                         "or configured as NoopFirewallDriver."),
+                    not is_firewall_enabled()):
+                LOG.info("Skipping method %s as firewall is disabled "
+                         "or configured as NoopFirewallDriver.",
                          func.__name__)
             else:
                 return func(self,  # pylint: disable=not-callable
                             *args, **kwargs)
         return decorated_function
 
+    @skip_if_noopfirewall_or_firewall_disabled
+    def init_ovs_dvr_firewall(self, dvr_agent):
+        dvr_agent.set_firewall(self.firewall)
+
     @skip_if_noopfirewall_or_firewall_disabled
     def prepare_devices_filter(self, device_ids):
         if not device_ids:
             return
-        LOG.info(_LI("Preparing filters for devices %s"), device_ids)
+        LOG.info("Preparing filters for devices %s", device_ids)
         self._apply_port_filter(device_ids)
 
+    @_port_filter_lock
     def _apply_port_filter(self, device_ids, update_filter=False):
+        step = common_constants.AGENT_RES_PROCESSING_STEP
         if self.use_enhanced_rpc:
-            devices_info = self.plugin_rpc.security_group_info_for_devices(
-                self.context, list(device_ids))
-            devices = devices_info['devices']
-            security_groups = devices_info['security_groups']
-            security_group_member_ips = devices_info['sg_member_ips']
+            devices = {}
+            security_groups = {}
+            security_group_member_ips = {}
+            for i in range(0, len(device_ids), step):
+                devices_info = self.plugin_rpc.security_group_info_for_devices(
+                    self.context, list(device_ids)[i:i + step])
+                devices.update(devices_info['devices'])
+                security_groups.update(devices_info['security_groups'])
+                security_group_member_ips.update(devices_info['sg_member_ips'])
         else:
             devices = self.plugin_rpc.security_group_rules_for_devices(
                 self.context, list(device_ids))
+        trusted_devices = self._get_trusted_devices(device_ids, devices)
 
         with self.firewall.defer_apply():
             if self.use_enhanced_rpc:
@@ -161,6 +194,7 @@ class SecurityGroupAgentRpc(object):
                 else:
                     LOG.debug("Prepare port filter for %s", device['device'])
                     self.firewall.prepare_port_filter(device)
+            self.firewall.process_trusted_ports(trusted_devices)
 
     def _update_security_group_info(self, security_groups,
                                     security_group_member_ips):
@@ -172,21 +206,22 @@ class SecurityGroupAgentRpc(object):
                 remote_sg_id, member_ips)
 
     def security_groups_rule_updated(self, security_groups):
-        LOG.info(_LI("Security group "
-                 "rule updated %r"), security_groups)
+        LOG.info("Security group "
+                 "rule updated %r", security_groups)
         self._security_group_updated(
             security_groups,
             'security_groups',
             'sg_rule')
 
     def security_groups_member_updated(self, security_groups):
-        LOG.info(_LI("Security group "
-                 "member updated %r"), security_groups)
+        LOG.info("Security group "
+                 "member updated %r", security_groups)
         self._security_group_updated(
             security_groups,
             'security_group_source_groups',
             'sg_member')
 
+    @_port_filter_wait
     def _security_group_updated(self, security_groups, attribute, action_type):
         devices = []
         sec_grp_set = set(security_groups)
@@ -204,39 +239,30 @@ class SecurityGroupAgentRpc(object):
             else:
                 self.refresh_firewall(devices)
 
-    def security_groups_provider_updated(self, devices_to_update):
-        LOG.info(_LI("Provider rule updated"))
-        if self.defer_refresh_firewall:
-            if devices_to_update is None:
-                self.global_refresh_firewall = True
-            else:
-                self.devices_to_refilter |= set(devices_to_update)
-        else:
-            self.refresh_firewall(devices_to_update)
-
     def remove_devices_filter(self, device_ids):
         if not device_ids:
             return
-        LOG.info(_LI("Remove device filter for %r"), device_ids)
+        LOG.info("Remove device filter for %r", device_ids)
         with self.firewall.defer_apply():
             for device_id in device_ids:
                 device = self.firewall.ports.get(device_id)
-                if not device:
-                    continue
-                self.firewall.remove_port_filter(device)
+                if device:
+                    self.firewall.remove_port_filter(device)
+                else:
+                    self.firewall.remove_trusted_ports([device_id])
 
     @skip_if_noopfirewall_or_firewall_disabled
     def refresh_firewall(self, device_ids=None):
-        LOG.info(_LI("Refresh firewall rules"))
+        LOG.info("Refresh firewall rules")
         if not device_ids:
             device_ids = self.firewall.ports.keys()
             if not device_ids:
-                LOG.info(_LI("No ports here to refresh firewall"))
+                LOG.info("No ports here to refresh firewall")
                 return
         self._apply_port_filter(device_ids, update_filter=True)
 
     def firewall_refresh_needed(self):
-        return self.global_refresh_firewall or self.devices_to_refilter
+        return bool(self.devices_to_refilter)
 
     def setup_port_filters(self, new_devices, updated_devices):
         """Configure port filters for devices.
@@ -252,9 +278,7 @@ class SecurityGroupAgentRpc(object):
         # These data structures are cleared here in order to avoid
         # losing updates occurring during firewall refresh
         devices_to_refilter = self.devices_to_refilter
-        global_refresh_firewall = self.global_refresh_firewall
         self.devices_to_refilter = set()
-        self.global_refresh_firewall = False
         # We must call prepare_devices_filter() after we've grabbed
         # self.devices_to_refilter since an update for a new port
         # could arrive while we're processing, and we need to make
@@ -263,36 +287,14 @@ class SecurityGroupAgentRpc(object):
             LOG.debug("Preparing device filters for %d new devices",
                       len(new_devices))
             self.prepare_devices_filter(new_devices)
-        # TODO(salv-orlando): Avoid if possible ever performing the global
-        # refresh providing a precise list of devices for which firewall
-        # should be refreshed
-        if global_refresh_firewall:
-            LOG.debug("Refreshing firewall for all filtered devices")
-            self.refresh_firewall()
-        else:
-            if self.use_enhanced_rpc:
-                self.firewall.security_group_updated('sg_member', [],
-                                                     updated_devices)
-            # If a device is both in new and updated devices
-            # avoid reprocessing it
-            updated_devices = ((updated_devices | devices_to_refilter) -
-                               new_devices)
-            if updated_devices:
-                LOG.debug("Refreshing firewall for %d devices",
-                          len(updated_devices))
-                self.refresh_firewall(updated_devices)
-
-
-# TODO(armax): For bw compat with external dependencies; to be dropped in M.
-SG_RPC_VERSION = (
-    securitygroups_rpc.SecurityGroupAgentRpcApiMixin.SG_RPC_VERSION
-)
-SecurityGroupServerRpcApi = (
-    securitygroups_rpc.SecurityGroupServerRpcApi
-)
-SecurityGroupAgentRpcApiMixin = (
-    securitygroups_rpc.SecurityGroupAgentRpcApiMixin
-)
-SecurityGroupAgentRpcCallbackMixin = (
-    securitygroups_rpc.SecurityGroupAgentRpcCallbackMixin
-)
+        if self.use_enhanced_rpc and updated_devices:
+            self.firewall.security_group_updated('sg_member', [],
+                                                 updated_devices)
+        # If a device is both in new and updated devices
+        # avoid reprocessing it
+        updated_devices = ((updated_devices | devices_to_refilter) -
+                           new_devices)
+        if updated_devices:
+            LOG.debug("Refreshing firewall for %d devices",
+                      len(updated_devices))
+            self.refresh_firewall(updated_devices)
