@@ -17,25 +17,24 @@ import copy
 import itertools
 import operator
 
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.api import extensions
+from neutron_lib.callbacks import resources
 from neutron_lib import constants
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions
+from neutron_lib.plugins import directory
+from neutron_lib.plugins import utils as p_utils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_utils import excutils
 
-from neutron._i18n import _, _LW
-from neutron.callbacks import resources
-from neutron.common import constants as n_const
-from neutron.common import exceptions as n_exc
+from neutron._i18n import _
 from neutron.common import utils
-from neutron.db import api as db_api
 from neutron.db import provisioning_blocks
-from neutron.extensions import portbindings
 from neutron.extensions import segment as segment_ext
-from neutron import manager
-from neutron.plugins.common import utils as p_utils
 from neutron.quota import resource_registry
 
 
@@ -48,7 +47,7 @@ class DhcpRpcCallback(object):
     This class implements the server side of an rpc interface.  The client
     side of this interface can be found in
     neutron.agent.dhcp.agent.DhcpPluginApi.  For more information about
-    changing rpc interfaces, see doc/source/devref/rpc_api.rst.
+    changing rpc interfaces, see doc/source/contributor/internals/rpc_api.rst.
     """
 
     # API version history:
@@ -71,17 +70,19 @@ class DhcpRpcCallback(object):
     #     1.6 - Removed get_active_networks. It's not used by reference
     #           DHCP agent since Havana, so similar rationale for not bumping
     #           the major version as above applies here too.
+    #     1.7 - Add get_networks
+    #     1.8 - Add get_dhcp_port
 
     target = oslo_messaging.Target(
-        namespace=n_const.RPC_NAMESPACE_DHCP_PLUGIN,
-        version='1.6')
+        namespace=constants.RPC_NAMESPACE_DHCP_PLUGIN,
+        version='1.8')
 
     def _get_active_networks(self, context, **kwargs):
         """Retrieve and return a list of the active networks."""
         host = kwargs.get('host')
-        plugin = manager.NeutronManager.get_plugin()
-        if utils.is_extension_supported(
-            plugin, constants.DHCP_AGENT_SCHEDULER_EXT_ALIAS):
+        plugin = directory.get_plugin()
+        if extensions.is_extension_supported(
+                plugin, constants.DHCP_AGENT_SCHEDULER_EXT_ALIAS):
             if cfg.CONF.network_auto_schedule:
                 plugin.auto_schedule_networks(context, host)
             nets = plugin.list_active_networks_on_active_dhcp_agent(
@@ -120,9 +121,9 @@ class DhcpRpcCallback(object):
                         ctxt.reraise = True
                 if ctxt.reraise:
                     net_id = port['port']['network_id']
-                    LOG.warning(_LW("Action %(action)s for network %(net_id)s "
-                                    "could not complete successfully: "
-                                    "%(reason)s"),
+                    LOG.warning("Action %(action)s for network %(net_id)s "
+                                "could not complete successfully: "
+                                "%(reason)s",
                                 {"action": action,
                                  "net_id": net_id,
                                  'reason': e})
@@ -140,10 +141,12 @@ class DhcpRpcCallback(object):
         host = kwargs.get('host')
         LOG.debug('get_active_networks_info from %s', host)
         networks = self._get_active_networks(context, **kwargs)
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         filters = {'network_id': [network['id'] for network in networks]}
         ports = plugin.get_ports(context, filters=filters)
-        filters['enable_dhcp'] = [True]
+        # default is to filter subnets based on 'enable_dhcp' flag
+        if kwargs.get('enable_dhcp_filter', True):
+            filters['enable_dhcp'] = [True]
         # NOTE(kevinbenton): we sort these because the agent builds tags
         # based on position in the list and has to restart the process if
         # the order changes.
@@ -153,10 +156,11 @@ class DhcpRpcCallback(object):
         # inside a segment.  If the segment service plugin is loaded and
         # there are active dhcp enabled subnets, then filter out the subnets
         # that are not on the host's segment.
-        seg_plug = manager.NeutronManager.get_service_plugins().get(
+        seg_plug = directory.get_plugin(
             segment_ext.SegmentPluginBase.get_plugin_type())
         seg_subnets = [subnet for subnet in subnets
                        if subnet.get('segment_id')]
+        nonlocal_subnets = []
         if seg_plug and seg_subnets:
             host_segment_ids = seg_plug.get_segments_by_hosts(context, [host])
             # Gather the ids of all the subnets that are on a segment that
@@ -170,14 +174,19 @@ class DhcpRpcCallback(object):
             # segments as the host.  Do this only for the networks that are
             # routed because we want non-routed networks to work as
             # before.
+            nonlocal_subnets = [subnet for subnet in seg_subnets
+                                if subnet['id'] not in seg_subnet_ids]
             subnets = [subnet for subnet in subnets
                        if subnet['network_id'] not in routed_net_ids or
                        subnet['id'] in seg_subnet_ids]
 
         grouped_subnets = self._group_by_network_id(subnets)
+        grouped_nonlocal_subnets = self._group_by_network_id(nonlocal_subnets)
         grouped_ports = self._group_by_network_id(ports)
         for network in networks:
             network['subnets'] = grouped_subnets.get(network['id'], [])
+            network['non_local_subnets'] = (
+                grouped_nonlocal_subnets.get(network['id'], []))
             network['ports'] = grouped_ports.get(network['id'], [])
 
         return networks
@@ -189,7 +198,7 @@ class DhcpRpcCallback(object):
         LOG.debug('Network %(network_id)s requested from '
                   '%(host)s', {'network_id': network_id,
                                'host': host})
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         try:
             network = plugin.get_network(context, network_id)
         except exceptions.NetworkNotFound:
@@ -198,8 +207,9 @@ class DhcpRpcCallback(object):
             return
         filters = dict(network_id=[network_id])
         subnets = plugin.get_subnets(context, filters=filters)
-        seg_plug = manager.NeutronManager.get_service_plugins().get(
+        seg_plug = directory.get_plugin(
             segment_ext.SegmentPluginBase.get_plugin_type())
+        nonlocal_subnets = []
         if seg_plug and subnets:
             seg_subnets = [subnet for subnet in subnets
                            if subnet.get('segment_id')]
@@ -212,12 +222,16 @@ class DhcpRpcCallback(object):
                 # host is not mapped to any segments and this is a routed
                 # network, then this host shouldn't have even been scheduled
                 # to.
+                nonlocal_subnets = [subnet for subnet in seg_subnets
+                                    if subnet['segment_id'] not in segment_ids]
                 subnets = [subnet for subnet in seg_subnets
                            if subnet['segment_id'] in segment_ids]
         # NOTE(kevinbenton): we sort these because the agent builds tags
         # based on position in the list and has to restart the process if
         # the order changes.
         network['subnets'] = sorted(subnets, key=operator.itemgetter('id'))
+        network['non_local_subnets'] = sorted(nonlocal_subnets,
+                                              key=operator.itemgetter('id'))
         network['ports'] = plugin.get_ports(context, filters=filters)
         return network
 
@@ -231,9 +245,10 @@ class DhcpRpcCallback(object):
         LOG.debug('DHCP port deletion for %(network_id)s request from '
                   '%(host)s',
                   {'network_id': network_id, 'host': host})
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         plugin.delete_ports_by_device_id(context, device_id, network_id)
 
+    @oslo_messaging.expected_exceptions(exceptions.IpAddressGenerationFailure)
     @db_api.retry_db_errors
     @resource_registry.mark_resources_dirty
     def create_dhcp_port(self, context, **kwargs):
@@ -255,9 +270,18 @@ class DhcpRpcCallback(object):
         port['port'][portbindings.HOST_ID] = host
         if 'mac_address' not in port['port']:
             port['port']['mac_address'] = constants.ATTR_NOT_SPECIFIED
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         return self._port_action(plugin, context, port, 'create_port')
 
+    def _is_dhcp_agent_hosting_network(self, plugin, context, host,
+                                       network_id):
+        """Check whether a DHCP agent (host) is hosting a network."""
+        agents = plugin.get_dhcp_agents_hosting_networks(context, [network_id],
+                                                         hosts=[host])
+        return len(agents) != 0
+
+    @oslo_messaging.expected_exceptions(exceptions.NetworkNotFound)
+    @oslo_messaging.expected_exceptions(exceptions.IpAddressGenerationFailure)
     @db_api.retry_db_errors
     def update_dhcp_port(self, context, **kwargs):
         """Update the dhcp port."""
@@ -265,14 +289,21 @@ class DhcpRpcCallback(object):
         port = kwargs.get('port')
         port['id'] = kwargs.get('port_id')
         port['port'][portbindings.HOST_ID] = host
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         try:
+            network_id = port['port']['network_id']
             old_port = plugin.get_port(context, port['id'])
-            if (old_port['device_id'] != n_const.DEVICE_ID_RESERVED_DHCP_PORT
-                and old_port['device_id'] !=
-                utils.get_dhcp_agent_device_id(port['port']['network_id'],
-                                               host)):
-                raise n_exc.DhcpPortInUse(port_id=port['id'])
+            if (old_port['device_id'] !=
+                    constants.DEVICE_ID_RESERVED_DHCP_PORT and
+                old_port['device_id'] !=
+                    utils.get_dhcp_agent_device_id(network_id, host)):
+                return
+            if not self._is_dhcp_agent_hosting_network(plugin, context, host,
+                                                       network_id):
+                LOG.warning("The DHCP agent on %(host)s does not host the "
+                            "network %(net_id)s.", {"host": host,
+                                                    "net_id": network_id})
+                raise exceptions.NetworkNotFound(net_id=network_id)
             LOG.debug('Update dhcp port %(port)s '
                       'from %(host)s.',
                       {'port': port,
@@ -282,7 +313,13 @@ class DhcpRpcCallback(object):
             LOG.debug('Host %(host)s tried to update port '
                       '%(port_id)s which no longer exists.',
                       {'host': host, 'port_id': port['id']})
-            return None
+
+    @db_api.retry_db_errors
+    def get_dhcp_port(self, context, **kwargs):
+        """Retrieve the DHCP port"""
+        port_id = kwargs.get('port_id')
+        plugin = directory.get_plugin()
+        return plugin.get_port(context, port_id)
 
     @db_api.retry_db_errors
     def dhcp_ready_on_ports(self, context, port_ids):
@@ -290,3 +327,12 @@ class DhcpRpcCallback(object):
             provisioning_blocks.provisioning_complete(
                 context, port_id, resources.PORT,
                 provisioning_blocks.DHCP_ENTITY)
+
+    def get_networks(self, context, filters=None, fields=None):
+        """Retrieve and return a list of networks."""
+        # NOTE(adrianc): This RPC is being used by out of tree interface
+        # drivers, MultiInterfaceDriver and IPoIBInterfaceDriver, located in
+        # networking-mlnx.
+        plugin = directory.get_plugin()
+        return plugin.get_networks(
+            context, filters=filters, fields=fields)
