@@ -21,14 +21,14 @@ import sys
 
 from keystoneauth1 import loading as ks_loading
 from neutron_lib.api import validators
+from neutron_lib import rpc as n_rpc
 from oslo_config import cfg
-from oslo_db import options as db_options
 from oslo_log import log as logging
 import oslo_messaging
 from oslo_middleware import cors
 from oslo_service import wsgi
 
-from neutron._i18n import _, _LI
+from neutron._i18n import _
 from neutron.conf import common as common_config
 from neutron import policy
 from neutron import version
@@ -41,8 +41,8 @@ LOG = logging.getLogger(__name__)
 EXTRA_LOG_LEVEL_DEFAULTS = [
     'OFPHandler=INFO',
     'OfctlService=INFO',
-    'ryu.base.app_manager=INFO',
-    'ryu.controller.controller=INFO',
+    'os_ken.base.app_manager=INFO',
+    'os_ken.controller.controller=INFO',
 ]
 
 # Register the configuration options
@@ -52,17 +52,6 @@ common_config.register_core_common_config_opts()
 oslo_messaging.set_transport_defaults(control_exchange='neutron')
 
 
-def set_db_defaults():
-    # Update the default QueuePool parameters. These can be tweaked by the
-    # conf variables - max_pool_size, max_overflow and pool_timeout
-    db_options.set_defaults(
-        cfg.CONF,
-        connection='sqlite://',
-        max_pool_size=10,
-        max_overflow=20, pool_timeout=10)
-
-set_db_defaults()
-
 NOVA_CONF_SECTION = 'nova'
 
 ks_loading.register_auth_conf_options(cfg.CONF, NOVA_CONF_SECTION)
@@ -72,17 +61,32 @@ ks_loading.register_session_conf_options(cfg.CONF, NOVA_CONF_SECTION)
 # Register the nova configuration options
 common_config.register_nova_opts()
 
+ks_loading.register_auth_conf_options(cfg.CONF,
+                                      common_config.PLACEMENT_CONF_SECTION)
+ks_loading.register_session_conf_options(cfg.CONF,
+                                         common_config.PLACEMENT_CONF_SECTION)
+
+# Register the placement configuration options
+common_config.register_placement_opts()
+
 logging.register_options(cfg.CONF)
 
+# Register the ironic configuration options
+ks_loading.register_auth_conf_options(cfg.CONF,
+                                      common_config.IRONIC_CONF_SECTION)
+ks_loading.register_session_conf_options(cfg.CONF,
+                                         common_config.IRONIC_CONF_SECTION)
+ks_loading.register_adapter_conf_options(cfg.CONF,
+                                         common_config.IRONIC_CONF_SECTION)
+common_config.register_ironic_opts()
 
-def init(args, **kwargs):
+
+def init(args, default_config_files=None, **kwargs):
     cfg.CONF(args=args, project='neutron',
              version='%%(prog)s %s' % version.version_info.release_string(),
+             default_config_files=default_config_files,
              **kwargs)
 
-    # FIXME(ihrachys): if import is put in global, circular import
-    # failure occurs
-    from neutron.common import rpc as n_rpc
     n_rpc.init(cfg.CONF)
 
     # Validate that the base_mac is of the correct format
@@ -100,8 +104,8 @@ def setup_logging():
     logging.set_defaults(default_log_levels=logging.get_default_log_levels() +
                          EXTRA_LOG_LEVEL_DEFAULTS)
     logging.setup(cfg.CONF, product_name)
-    LOG.info(_LI("Logging enabled!"))
-    LOG.info(_LI("%(prog)s version %(version)s"),
+    LOG.info("Logging enabled!")
+    LOG.info("%(prog)s version %(version)s",
              {'prog': sys.argv[0],
               'version': version.version_info.release_string()})
     LOG.debug("command line: %s", " ".join(sys.argv))
@@ -122,6 +126,10 @@ def load_paste_app(app_name):
     :param app_name: Name of the application to load
     """
     loader = wsgi.Loader(cfg.CONF)
+
+    # Log the values of registered opts
+    if cfg.CONF.debug:
+        cfg.CONF.log_opt_values(LOG, logging.DEBUG)
     app = loader.load_app(app_name)
     return app
 
@@ -133,24 +141,22 @@ def set_config_defaults():
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
-    # CORS Defaults
-    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
-    cfg.set_defaults(cors.CORS_OPTS,
-                     allow_headers=['X-Auth-Token',
-                                    'X-Identity-Status',
-                                    'X-Roles',
-                                    'X-Service-Catalog',
-                                    'X-User-Id',
-                                    'X-Tenant-Id',
-                                    'X-OpenStack-Request-ID'],
-                     expose_headers=['X-Auth-Token',
-                                     'X-Subject-Token',
-                                     'X-Service-Token',
-                                     'X-OpenStack-Request-ID',
-                                     'OpenStack-Volume-microversion'],
-                     allow_methods=['GET',
-                                    'PUT',
-                                    'POST',
-                                    'DELETE',
-                                    'PATCH']
-                     )
+    cors.set_defaults(
+        allow_headers=['X-Auth-Token',
+                       'X-Identity-Status',
+                       'X-Roles',
+                       'X-Service-Catalog',
+                       'X-User-Id',
+                       'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        expose_headers=['X-Auth-Token',
+                        'X-Subject-Token',
+                        'X-Service-Token',
+                        'X-OpenStack-Request-ID',
+                        'OpenStack-Volume-microversion'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH']
+    )
