@@ -17,9 +17,16 @@ import datetime
 import random
 import time
 
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants
 from neutron_lib import context as ncontext
+from neutron_lib.db import api as db_api
+from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import agent as agent_exc
+from neutron_lib.exceptions import dhcpagentscheduler as das_exc
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
@@ -33,6 +40,7 @@ from neutron.db import agents_db
 from neutron.db.availability_zone import network as network_az
 from neutron.extensions import dhcpagentscheduler
 from neutron.objects import network
+from neutron.objects import subnet as subnet_obj
 from neutron import worker as neutron_worker
 
 
@@ -50,7 +58,6 @@ class AgentSchedulerDbMixin(agents_db.AgentDbMixin):
     agent_notifiers = {
         constants.AGENT_TYPE_DHCP: None,
         constants.AGENT_TYPE_L3: None,
-        constants.AGENT_TYPE_LOADBALANCER: None,
     }
 
     @staticmethod
@@ -63,8 +70,14 @@ class AgentSchedulerDbMixin(agents_db.AgentDbMixin):
             #                   filter is set, only agents which are 'up'
             #                   (i.e. have a recent heartbeat timestamp)
             #                   are eligible, even if active is False
-            return not agent_utils.is_agent_down(
-                agent['heartbeat_timestamp'])
+            if agent_utils.is_agent_down(agent['heartbeat_timestamp']):
+                LOG.warning('Agent %(agent)s is down. Type: %(type)s, host: '
+                            '%(host)s, heartbeat: %(heartbeat)s',
+                            {'agent': agent['id'], 'type': agent['agent_type'],
+                             'host': agent['host'],
+                             'heartbeat': agent['heartbeat_timestamp']})
+                return False
+            return True
 
     def update_agent(self, context, id, agent):
         original_agent = self.get_agent(context, id)
@@ -73,8 +86,9 @@ class AgentSchedulerDbMixin(agents_db.AgentDbMixin):
         agent_data = agent['agent']
         agent_notifier = self.agent_notifiers.get(original_agent['agent_type'])
         if (agent_notifier and
-            'admin_state_up' in agent_data and
-            original_agent['admin_state_up'] != agent_data['admin_state_up']):
+                'admin_state_up' in agent_data and
+                original_agent['admin_state_up'] !=
+                agent_data['admin_state_up']):
             agent_notifier.agent_updated(context,
                                          agent_data['admin_state_up'],
                                          original_agent['host'])
@@ -137,15 +151,14 @@ class AgentSchedulerDbMixin(agents_db.AgentDbMixin):
                 binding_resource_id = getattr(binding, resource_id_attr)
                 if binding_agent_id in agents_back_online:
                     continue
-                else:
-                    # we need new context to make sure we use different DB
-                    # transaction - otherwise we may fetch same agent record
-                    # each time due to REPEATABLE_READ isolation level
-                    context = ncontext.get_admin_context()
-                    agent = self._get_agent(context, binding_agent_id)
-                    if agent.is_active:
-                        agents_back_online.add(binding_agent_id)
-                        continue
+                # we need new context to make sure we use different DB
+                # transaction - otherwise we may fetch same agent record
+                # each time due to REPEATABLE_READ isolation level
+                context = ncontext.get_admin_context()
+                agent = self._get_agent(context, binding_agent_id)
+                if agent.is_active:
+                    agents_back_online.add(binding_agent_id)
+                    continue
 
                 LOG.warning(
                     "Rescheduling %(resource_name)s %(resource)s from agent "
@@ -218,12 +231,15 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
                              additional_time)
         return agent_expected_up > timeutils.utcnow()
 
-    def _schedule_network(self, context, network_id, dhcp_notifier):
+    def _schedule_network(self, context, network_id, dhcp_notifier,
+                          candidate_hosts=None):
         LOG.info("Scheduling unhosted network %s", network_id)
         try:
             # TODO(enikanorov): have to issue redundant db query
             # to satisfy scheduling interface
             network = self.get_network(context, network_id)
+            if candidate_hosts:
+                network['candidate_hosts'] = candidate_hosts
             agents = self.schedule_network(context, network)
             if not agents:
                 LOG.info("Failed to schedule network %s, "
@@ -290,8 +306,7 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
             down_bindings = network.NetworkDhcpAgentBinding.get_down_bindings(
                 context, cutoff)
             dhcp_notifier = self.agent_notifiers.get(constants.AGENT_TYPE_DHCP)
-            dead_bindings = [b for b in
-                             self._filter_bindings(context, down_bindings)]
+            dead_bindings = list(self._filter_bindings(context, down_bindings))
             agents = self.get_agent_objects(
                 context, {'agent_type': [constants.AGENT_TYPE_DHCP]})
             if not agents:
@@ -323,7 +338,7 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
                                                         binding.dhcp_agent_id,
                                                         binding.network_id,
                                                         notify=False)
-                except dhcpagentscheduler.NetworkNotHostedByDhcpAgent:
+                except das_exc.NetworkNotHostedByDhcpAgent:
                     # measures against concurrent operation
                     LOG.debug("Network %(net)s already removed from DHCP "
                               "agent %(agent)s",
@@ -369,19 +384,19 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
 
     def add_network_to_dhcp_agent(self, context, id, network_id):
         self._get_network(context, network_id)
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(context):
             agent_db = self._get_agent(context, id)
             if (agent_db['agent_type'] != constants.AGENT_TYPE_DHCP or
                     not services_available(agent_db['admin_state_up'])):
-                raise dhcpagentscheduler.InvalidDHCPAgent(id=id)
+                raise das_exc.InvalidDHCPAgent(id=id)
             dhcp_agents = self.get_dhcp_agents_hosting_networks(
                 context, [network_id])
             for dhcp_agent in dhcp_agents:
                 if id == dhcp_agent.id:
-                    raise dhcpagentscheduler.NetworkHostedByDHCPAgent(
+                    raise das_exc.NetworkHostedByDHCPAgent(
                         network_id=network_id, agent_id=id)
-            network.NetworkDhcpAgentBinding(context, dhcp_agent_id=id,
-                 network_id=network_id).create()
+            self.network_scheduler.resource_filter.bind(
+                context, [agent_db], network_id, force_scheduling=True)
         dhcp_notifier = self.agent_notifiers.get(constants.AGENT_TYPE_DHCP)
         if dhcp_notifier:
             dhcp_notifier.network_added_to_agent(
@@ -393,7 +408,7 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
         binding_obj = network.NetworkDhcpAgentBinding.get_object(
             context, network_id=network_id, dhcp_agent_id=id)
         if not binding_obj:
-            raise dhcpagentscheduler.NetworkNotHostedByDhcpAgent(
+            raise das_exc.NetworkNotHostedByDhcpAgent(
                 network_id=network_id, agent_id=id)
 
         # reserve the port, so the ip is reused on a subsequent add
@@ -406,7 +421,11 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
         # update_port passing and another failing
         for port in ports:
             port['device_id'] = constants.DEVICE_ID_RESERVED_DHCP_PORT
-            self.update_port(context, port['id'], dict(port=port))
+            try:
+                self.update_port(context, port['id'], dict(port=port))
+            except n_exc.PortNotFound:
+                LOG.debug("DHCP port %s has been deleted concurrently",
+                          port['id'])
         binding_obj.delete()
 
         if not notify:
@@ -462,7 +481,7 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
             return {'agents': []}
 
     def schedule_network(self, context, created_network):
-        if self.network_scheduler:
+        if self.network_scheduler and cfg.CONF.network_auto_schedule:
             return self.network_scheduler.schedule(
                 self, context, created_network)
 
@@ -470,6 +489,25 @@ class DhcpAgentSchedulerDbMixin(dhcpagentscheduler
         if self.network_scheduler:
             self.network_scheduler.auto_schedule_networks(self, context, host)
 
+    @registry.receives(resources.SEGMENT_HOST_MAPPING, [events.AFTER_CREATE])
+    def auto_schedule_new_network_segments(self, resource, event, trigger,
+                                           payload=None):
+        if not cfg.CONF.network_auto_schedule:
+            return
+        segment_plugin = directory.get_plugin('segments')
+        dhcp_notifier = self.agent_notifiers.get(constants.AGENT_TYPE_DHCP)
+        segment_ids = payload.metadata.get('current_segment_ids')
+        segments = segment_plugin.get_segments(
+            payload.context, filters={'id': segment_ids})
+        subnets = subnet_obj.Subnet.get_objects(
+            payload.context, segment_id=segment_ids)
+        network_ids = {s.network_id for s in subnets}
+        for network_id in network_ids:
+            for segment in segments:
+                self._schedule_network(
+                    payload.context, network_id, dhcp_notifier,
+                    candidate_hosts=segment['hosts'])
+
 
 class AZDhcpAgentSchedulerDbMixin(DhcpAgentSchedulerDbMixin,
                                   network_az.NetworkAvailabilityZoneMixin):
