@@ -16,6 +16,7 @@
 import functools
 
 import netaddr
+from neutron_lib.api.definitions import external_net as extnet_def
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
 from neutron_lib.api.definitions import portbindings as portbindings_def
 from neutron_lib.api.definitions import subnetpool as subnetpool_def
@@ -418,10 +419,12 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         return network
 
     @db_api.retry_if_session_inactive()
-    def update_network(self, context, id, network):
+    def update_network(self, context, id, network, db_network=None):
         n = network['network']
+        # we dont't use DB objects not belonging to the current active session
+        db_network = db_network if context.session.is_active else None
         with db_api.CONTEXT_WRITER.using(context):
-            network = self._get_network(context, id)
+            network = db_network or self._get_network(context, id)
             # validate 'shared' parameter
             if 'shared' in n:
                 entry = None
@@ -504,7 +507,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 network_db = self._get_network(context, id)
                 network = self._make_network_dict(network_db, context=context)
                 registry.notify(resources.NETWORK, events.PRECOMMIT_DELETE,
-                                self, context=context, network_id=id)
+                                self, context=context, network_id=id,
+                                network=network)
                 # We expire network_db here because precommit deletion
                 # might have left the relationship stale, for example,
                 # if we deleted a segment.
@@ -736,11 +740,12 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     @db_api.retry_if_session_inactive()
     def _create_subnet_postcommit(self, context, result,
                                   network=None, ipam_subnet=None):
-        if not network:
+        # need to get db net obj with full subnet info either if no network
+        # is passed or if passed network is an external net dict from
+        # ml2 plugin (with only IDs in 'subnets')
+        if not network or network.get(extnet_def.EXTERNAL):
             network = self._get_network(context,
                                         result['network_id'])
-        if not ipam_subnet:
-            ipam_subnet = self.ipam.get_subnet(context, result['id'])
 
         if hasattr(network, 'external') and network.external:
             self._update_router_gw_ports(context,
@@ -749,6 +754,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         # If this subnet supports auto-addressing, then update any
         # internal ports on the network with addresses for this subnet.
         if ipv6_utils.is_auto_address_subnet(result):
+            if not ipam_subnet:
+                ipam_subnet = self.ipam.get_subnet(context, result['id'])
             updated_ports = self.ipam.add_auto_addrs_on_network_ports(
                 context, result, ipam_subnet)
             for port_id in updated_ports:
@@ -941,8 +948,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                         self, **kwargs)
 
         with db_api.CONTEXT_WRITER.using(context):
-            subnet, changes = self.ipam.update_db_subnet(context, id, s,
-                                                         db_pools)
+            subnet, changes = self.ipam.update_db_subnet(
+                context, id, s, db_pools, subnet_obj=subnet_obj)
         return self._make_subnet_dict(subnet, context=context), orig
 
     @property
@@ -1053,7 +1060,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         with db_api.exc_to_retry(sql_exc.IntegrityError), \
                 db_api.CONTEXT_WRITER.using(context):
             registry.notify(resources.SUBNET, events.PRECOMMIT_DELETE,
-                            self, context=context, subnet_id=subnet.id)
+                            self, context=context, subnet_id=subnet.id,
+                            subnet_obj=subnet)
             subnet.delete()
             # Delete related ipam subnet manually,
             # since there is no FK relationship
@@ -1311,7 +1319,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         if not validators.is_attr_set(network_id):
             msg = _("network_id must be specified.")
             raise exc.InvalidInput(error_message=msg)
-        if not network_obj.Network.objects_exist(context, id=network_id):
+        if not self._network_exists(context, network_id):
             raise exc.NetworkNotFound(net_id=network_id)
 
         subnetpool = subnetpool_obj.SubnetPool.get_object(context,
@@ -1421,7 +1429,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             port_data['mac_address'] = p.get('mac_address')
         with db_api.CONTEXT_WRITER.using(context):
             # Ensure that the network exists.
-            self._get_network(context, network_id)
+            if not self._network_exists(context, network_id):
+                raise exc.NetworkNotFound(net_id=network_id)
 
             # Create the port
             db_port = self._create_db_port_obj(context, port_data)
@@ -1459,11 +1468,13 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                                         new_mac, current_owner)
 
     @db_api.retry_if_session_inactive()
-    def update_port(self, context, id, port):
+    def update_port(self, context, id, port, db_port=None):
+        # we dont't use DB objects not belonging to the current active session
+        db_port = db_port if context.session.is_active else None
         new_port = port['port']
 
         with db_api.CONTEXT_WRITER.using(context):
-            db_port = self._get_port(context, id)
+            db_port = db_port or self._get_port(context, id)
             new_mac = new_port.get('mac_address')
             self._validate_port_for_update(context, db_port, new_port, new_mac)
             # Note: _make_port_dict is called here to load extension data
