@@ -18,31 +18,29 @@ import copy
 import itertools
 
 import netaddr
+from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
+from neutron_lib.api.definitions import portbindings
 from neutron_lib.api import validators
 from neutron_lib import constants as const
+from neutron_lib.db import api as db_api
+from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as exc
+from neutron_lib.exceptions import address_scope as addr_scope_exc
+from neutron_lib.utils import net as net_utils
 from oslo_config import cfg
-from oslo_db import exception as db_exc
 from oslo_log import log as logging
-from sqlalchemy import and_, or_
 from sqlalchemy.orm import exc as orm_exc
 
-from neutron._i18n import _, _LI
-from neutron.common import constants
-from neutron.common import exceptions as n_exc
+from neutron._i18n import _
 from neutron.common import ipv6_utils
-from neutron.common import utils as common_utils
 from neutron.db import db_base_plugin_common
-from neutron.db.models import subnet_service_type as sst_model
 from neutron.db import models_v2
-from neutron.db import segments_db
-from neutron.extensions import ip_allocation as ipa
-from neutron.extensions import portbindings
 from neutron.extensions import segment
 from neutron.ipam import exceptions as ipam_exceptions
 from neutron.ipam import utils as ipam_utils
+from neutron.objects import address_scope as addr_scope_obj
+from neutron.objects import network as network_obj
 from neutron.objects import subnet as subnet_obj
-from neutron.services.segments import db as segment_svc_db
 from neutron.services.segments import exceptions as segment_exc
 
 LOG = logging.getLogger(__name__)
@@ -58,7 +56,14 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
     @staticmethod
     def _gateway_ip_str(subnet, cidr_net):
         if subnet.get('gateway_ip') is const.ATTR_NOT_SPECIFIED:
-            return str(netaddr.IPNetwork(cidr_net).network + 1)
+            if subnet.get('ip_version') == const.IP_VERSION_6:
+                gateway_ip = netaddr.IPNetwork(cidr_net).network
+                pd_net = netaddr.IPNetwork(const.PROVISIONAL_IPV6_PD_PREFIX)
+                if gateway_ip == pd_net.network:
+                    return
+            else:
+                gateway_ip = netaddr.IPNetwork(cidr_net).network + 1
+            return str(gateway_ip)
         return subnet.get('gateway_ip')
 
     @staticmethod
@@ -69,13 +74,17 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                 ip_range_pools.append(netaddr.IPRange(ip_pool['start'],
                                                       ip_pool['end']))
             except netaddr.AddrFormatError:
-                LOG.info(_LI("Found invalid IP address in pool: "
-                             "%(start)s - %(end)s:"),
+                LOG.info("Found invalid IP address in pool: "
+                         "%(start)s - %(end)s:",
                          {'start': ip_pool['start'],
                           'end': ip_pool['end']})
-                raise n_exc.InvalidAllocationPool(pool=ip_pool)
+                raise exc.InvalidAllocationPool(pool=ip_pool)
         return ip_range_pools
 
+    @staticmethod
+    def _is_distributed_service(port):
+        return port.get('device_owner') == const.DEVICE_OWNER_DISTRIBUTED
+
     def delete_subnet(self, context, subnet_id):
         pass
 
@@ -108,13 +117,13 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         if (new_mac and new_mac != db_port.mac_address and
                 self._is_mac_in_use(context, network_id, new_mac)):
             raise exc.MacAddressInUse(net_id=network_id, mac=new_mac)
-        db_port.update(self._filter_non_model_columns(new_port,
-                                                      models_v2.Port))
+        db_port.update(db_utils.filter_non_model_columns(new_port,
+                                                         models_v2.Port))
 
     def _update_subnet_host_routes(self, context, id, s):
 
         def _combine(ht):
-            return ht['destination'] + "_" + ht['nexthop']
+            return "{}_{}".format(ht['destination'], ht['nexthop'])
 
         old_route_list = self._get_route_by_subnet(context, id)
 
@@ -127,13 +136,15 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         for route_str in old_route_set - new_route_set:
             for route in old_route_list:
                 if _combine(route) == route_str:
-                    context.session.delete(route)
+                    route.delete()
         for route_str in new_route_set - old_route_set:
-            route = models_v2.SubnetRoute(
-                destination=route_str.partition("_")[0],
-                nexthop=route_str.partition("_")[2],
+            route = subnet_obj.Route(
+                context,
+                destination=net_utils.AuthenticIPNetwork(
+                    route_str.partition("_")[0]),
+                nexthop=netaddr.IPAddress(route_str.partition("_")[2]),
                 subnet_id=id)
-            context.session.add(route)
+            route.create()
 
         # Gather host routes for result
         new_routes = []
@@ -145,14 +156,12 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         return new_routes
 
     def _update_subnet_dns_nameservers(self, context, id, s):
-        old_dns_list = self._get_dns_by_subnet(context, id)
         new_dns_addr_list = s["dns_nameservers"]
 
         # NOTE(changzhi) delete all dns nameservers from db
         # when update subnet's DNS nameservers. And store new
         # nameservers with order one by one.
-        for dns in old_dns_list:
-            dns.delete()
+        subnet_obj.DNSNameServer.delete_objects(context, subnet_id=id)
 
         for order, server in enumerate(new_dns_addr_list):
             dns = subnet_obj.DNSNameServer(context,
@@ -163,17 +172,16 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         del s["dns_nameservers"]
         return new_dns_addr_list
 
+    @db_api.CONTEXT_WRITER
     def _update_subnet_allocation_pools(self, context, subnet_id, s):
-        context.session.query(models_v2.IPAllocationPool).filter_by(
-            subnet_id=subnet_id).delete()
+        subnet_obj.IPAllocationPool.delete_objects(context,
+                                                   subnet_id=subnet_id)
         pools = [(netaddr.IPAddress(p.first, p.version).format(),
                   netaddr.IPAddress(p.last, p.version).format())
                  for p in s['allocation_pools']]
-        new_pools = [models_v2.IPAllocationPool(first_ip=p[0],
-                                                last_ip=p[1],
-                                                subnet_id=subnet_id)
-                     for p in pools]
-        context.session.add_all(new_pools)
+        for p in pools:
+            subnet_obj.IPAllocationPool(context, start=p[0], end=p[1],
+                                        subnet_id=subnet_id).create()
 
         # Gather new pools for result
         result_pools = [{'start': p[0], 'end': p[1]} for p in pools]
@@ -181,19 +189,18 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         return result_pools
 
     def _update_subnet_service_types(self, context, subnet_id, s):
-        old_types = context.session.query(
-            sst_model.SubnetServiceType).filter_by(subnet_id=subnet_id)
-        for service_type in old_types:
-            context.session.delete(service_type)
+        subnet_obj.SubnetServiceType.delete_objects(context,
+                                                    subnet_id=subnet_id)
         updated_types = s.pop('service_types')
         for service_type in updated_types:
-            new_type = sst_model.SubnetServiceType(
-                           subnet_id=subnet_id,
-                           service_type=service_type)
-            context.session.add(new_type)
+            new_type = subnet_obj.SubnetServiceType(context,
+                                                    subnet_id=subnet_id,
+                                                    service_type=service_type)
+            new_type.create()
         return updated_types
 
-    def update_db_subnet(self, context, subnet_id, s, oldpools):
+    def update_db_subnet(self, context, subnet_id, s, oldpools,
+            subnet_obj=None):
         changes = {}
         if "dns_nameservers" in s:
             changes['dns_nameservers'] = (
@@ -211,9 +218,10 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
             changes['service_types'] = (
                 self._update_subnet_service_types(context, subnet_id, s))
 
-        subnet = self._get_subnet(context, subnet_id)
-        subnet.update(s)
-        return subnet, changes
+        subnet_obj = subnet_obj or self._get_subnet_object(context, subnet_id)
+        subnet_obj.update_fields(s)
+        subnet_obj.update()
+        return subnet_obj, changes
 
     def _validate_subnet_cidr(self, context, network, new_subnet_cidr):
         """Validate the CIDR for a subnet.
@@ -233,37 +241,65 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
             if cidr.prefixlen == 0:
                 err_msg = _("0 is not allowed as CIDR prefix length")
                 raise exc.InvalidInput(error_message=err_msg)
-
         if cfg.CONF.allow_overlapping_ips:
-            subnet_list = network.subnets
+            subnet_list = [{'id': s.id, 'cidr': s.cidr}
+                           for s in network.subnets]
         else:
-            subnet_list = self._get_all_subnets(context)
+            subnet_list = subnet_obj.Subnet.get_subnet_cidrs(context)
         for subnet in subnet_list:
-            if ((netaddr.IPSet([subnet.cidr]) & new_subnet_ipset) and
-                subnet.cidr != constants.PROVISIONAL_IPV6_PD_PREFIX):
+            if ((netaddr.IPSet([subnet['cidr']]) & new_subnet_ipset) and
+                    str(subnet['cidr']) != const.PROVISIONAL_IPV6_PD_PREFIX):
                 # don't give out details of the overlapping subnet
-                err_msg = (_("Requested subnet with cidr: %(cidr)s for "
-                             "network: %(network_id)s overlaps with another "
-                             "subnet") %
+                err_msg = ("Requested subnet with cidr: %(cidr)s for "
+                           "network: %(network_id)s overlaps with another "
+                           "subnet" %
                            {'cidr': new_subnet_cidr,
                             'network_id': network.id})
-                LOG.info(_LI("Validation for CIDR: %(new_cidr)s failed - "
-                             "overlaps with subnet %(subnet_id)s "
-                             "(CIDR: %(cidr)s)"),
+                LOG.info("Validation for CIDR: %(new_cidr)s failed - "
+                         "overlaps with subnet %(subnet_id)s "
+                         "(CIDR: %(cidr)s)",
                          {'new_cidr': new_subnet_cidr,
-                          'subnet_id': subnet.id,
-                          'cidr': subnet.cidr})
+                          'subnet_id': subnet['id'],
+                          'cidr': subnet['cidr']})
                 raise exc.InvalidInput(error_message=err_msg)
 
-    def _validate_network_subnetpools(self, network,
-                                      new_subnetpool_id, ip_version):
+    def _validate_network_subnetpools(self, network, subnet_ip_version,
+                                      new_subnetpool, network_scope):
         """Validate all subnets on the given network have been allocated from
-           the same subnet pool as new_subnetpool_id
+           the same subnet pool as new_subnetpool if no address scope is
+           used. If address scopes are used, validate that all subnets on the
+           given network participate in the same address scope.
         """
+        # 'new_subnetpool' might just be the Prefix Delegation ID
+        ipv6_pd_subnetpool = new_subnetpool == const.IPV6_PD_POOL_ID
+
+        # Check address scope affinities
+        if network_scope:
+            if (ipv6_pd_subnetpool or
+                    new_subnetpool and
+                    new_subnetpool.address_scope_id != network_scope.id):
+                raise addr_scope_exc.NetworkAddressScopeAffinityError()
+
+        # Checks for situations where address scopes aren't involved
         for subnet in network.subnets:
-            if (subnet.ip_version == ip_version and
-                    new_subnetpool_id != subnet.subnetpool_id):
-                raise n_exc.NetworkSubnetPoolAffinityError()
+            if ipv6_pd_subnetpool:
+                # Check the prefix delegation case.  Since there is no
+                # subnetpool object, we just check against the PD ID.
+                if (subnet.ip_version == const.IP_VERSION_6 and
+                        subnet.subnetpool_id != const.IPV6_PD_POOL_ID):
+                    raise exc.NetworkSubnetPoolAffinityError()
+            else:
+                if new_subnetpool:
+                    # In this case we have the new subnetpool object, so
+                    # we can check the ID and IP version.
+                    if (subnet.subnetpool_id != new_subnetpool.id and
+                            subnet.ip_version == new_subnetpool.ip_version and
+                            not network_scope):
+                        raise exc.NetworkSubnetPoolAffinityError()
+                else:
+                    if (subnet.subnetpool_id and
+                            subnet.ip_version == subnet_ip_version):
+                        raise exc.NetworkSubnetPoolAffinityError()
 
     def validate_allocation_pools(self, ip_pools, subnet_cidr):
         """Validate IP allocation pools.
@@ -285,14 +321,14 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
             end_ip = netaddr.IPAddress(ip_pool.last, ip_pool.version)
             if (start_ip.version != subnet.version or
                     end_ip.version != subnet.version):
-                LOG.info(_LI("Specified IP addresses do not match "
-                             "the subnet IP version"))
-                raise n_exc.InvalidAllocationPool(pool=ip_pool)
+                LOG.info("Specified IP addresses do not match "
+                         "the subnet IP version")
+                raise exc.InvalidAllocationPool(pool=ip_pool)
             if start_ip < subnet_first_ip or end_ip > subnet_last_ip:
-                LOG.info(_LI("Found pool larger than subnet "
-                             "CIDR:%(start)s - %(end)s"),
+                LOG.info("Found pool larger than subnet "
+                         "CIDR:%(start)s - %(end)s",
                          {'start': start_ip, 'end': end_ip})
-                raise n_exc.OutOfBoundsAllocationPool(
+                raise exc.OutOfBoundsAllocationPool(
                     pool=ip_pool,
                     subnet_cidr=subnet_cidr)
             # Valid allocation pool
@@ -310,42 +346,49 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                 if ip_sets[l_cursor] & ip_sets[r_cursor]:
                     l_range = ip_ranges[l_cursor]
                     r_range = ip_ranges[r_cursor]
-                    LOG.info(_LI("Found overlapping ranges: %(l_range)s and "
-                                 "%(r_range)s"),
+                    LOG.info("Found overlapping ranges: %(l_range)s and "
+                             "%(r_range)s",
                              {'l_range': l_range, 'r_range': r_range})
-                    raise n_exc.OverlappingAllocationPools(
+                    raise exc.OverlappingAllocationPools(
                         pool_1=l_range,
                         pool_2=r_range,
                         subnet_cidr=subnet_cidr)
 
-    def _validate_max_ips_per_port(self, fixed_ip_list, device_owner):
-        if common_utils.is_port_trusted({'device_owner': device_owner}):
-            return
-
-        if len(fixed_ip_list) > cfg.CONF.max_fixed_ips_per_port:
-            msg = _('Exceeded maximum amount of fixed ips per port.')
-            raise exc.InvalidInput(error_message=msg)
-
-    def _validate_segment(self, context, network_id, segment_id):
-        query = context.session.query(models_v2.Subnet.segment_id)
-        query = query.filter(models_v2.Subnet.network_id == network_id)
-        associated_segments = set(row.segment_id for row in query)
+    def _validate_segment(self, context, network_id, segment_id, action=None,
+                          old_segment_id=None):
+        segments = subnet_obj.Subnet.get_values(
+            context, 'segment_id', network_id=network_id)
+        associated_segments = set(segments)
         if None in associated_segments and len(associated_segments) > 1:
             raise segment_exc.SubnetsNotAllAssociatedWithSegments(
                 network_id=network_id)
 
+        if action == 'update' and old_segment_id != segment_id:
+            # Check the current state of segments and subnets on the network
+            # before allowing migration from non-routed to routed network.
+            if len(segments) > 1:
+                raise segment_exc.SubnetsNotAllAssociatedWithSegments(
+                    network_id=network_id)
+            if (None not in associated_segments and
+                    segment_id not in associated_segments):
+                raise segment_exc.SubnetSegmentAssociationChangeNotAllowed()
+
+            if network_obj.NetworkSegment.count(
+                    context, network_id=network_id) > 1:
+                raise segment_exc.NoUpdateSubnetWhenMultipleSegmentsOnNetwork(
+                    network_id=network_id)
+
         if segment_id:
-            query = context.session.query(segments_db.NetworkSegment)
-            query = query.filter(segments_db.NetworkSegment.id == segment_id)
-            segment_model = query.one()
-            if segment_model.network_id != network_id:
+            segment = network_obj.NetworkSegment.get_object(context,
+                                                            id=segment_id)
+            if segment.network_id != network_id:
                 raise segment_exc.NetworkIdsDontMatch(
                     subnet_network=network_id,
                     segment_id=segment_id)
-            if segment_model.is_dynamic:
+            if segment.is_dynamic:
                 raise segment_exc.SubnetCantAssociateToDynamicSegment()
 
-    def _get_subnet_for_fixed_ip(self, context, fixed, subnets):
+    def _get_subnet_for_fixed_ip(self, context, fixed, subnets, network_id):
         # Subnets are all the subnets belonging to the same network.
         if not subnets:
             msg = _('IP allocation requires subnets for network')
@@ -358,17 +401,20 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                         return subnet
             subnet = get_matching_subnet()
             if not subnet:
-                subnet = self._get_subnet(context, fixed['subnet_id'])
+                # Call this method to keep original status code
+                # in case subnet is not found
+                self._get_subnet_object(context, fixed['subnet_id'])
                 msg = (_("Failed to create port on network %(network_id)s"
                          ", because fixed_ips included invalid subnet "
                          "%(subnet_id)s") %
-                       {'network_id': subnet['network_id'],
+                       {'network_id': network_id,
                         'subnet_id': fixed['subnet_id']})
                 raise exc.InvalidInput(error_message=msg)
             # Ensure that the IP is valid on the subnet
             if ('ip_address' in fixed and
                 not ipam_utils.check_subnet_ip(subnet['cidr'],
-                                               fixed['ip_address'])):
+                                               fixed['ip_address'],
+                                               fixed['device_owner'])):
                 raise exc.InvalidIpForSubnet(ip_address=fixed['ip_address'])
             return subnet
 
@@ -378,7 +424,8 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
 
         for subnet in subnets:
             if ipam_utils.check_subnet_ip(subnet['cidr'],
-                                          fixed['ip_address']):
+                                          fixed['ip_address'],
+                                          fixed['device_owner']):
                 return subnet
         raise exc.InvalidIpForNetwork(ip_address=fixed['ip_address'])
 
@@ -399,7 +446,7 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
     def validate_gw_out_of_pools(self, gateway_ip, pools):
         for pool_range in pools:
             if netaddr.IPAddress(gateway_ip) in pool_range:
-                raise n_exc.GatewayConflictWithAllocationPools(
+                raise exc.GatewayConflictWithAllocationPools(
                     pool=pool_range,
                     ip_address=gateway_ip)
 
@@ -411,9 +458,9 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         if device_owner in const.ROUTER_INTERFACE_OWNERS_SNAT:
             return True
 
-        subnet = self._get_subnet(context, subnet_id)
-        return not (ipv6_utils.is_auto_address_subnet(subnet) and
-                    not ipv6_utils.is_ipv6_pd_enabled(subnet))
+        subnet_obj = self._get_subnet_object(context, subnet_id)
+        return not (ipv6_utils.is_auto_address_subnet(subnet_obj) and
+                    not ipv6_utils.is_ipv6_pd_enabled(subnet_obj))
 
     def _get_changed_ips_for_port(self, context, original_ips,
                                   new_ips, device_owner):
@@ -423,8 +470,6 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                                 if ip.get('delete_subnet'))
         ips = [ip for ip in new_ips
                if ip.get('subnet_id') not in delete_subnet_ids]
-        # the new_ips contain all of the fixed_ips that are to be updated
-        self._validate_max_ips_per_port(ips, device_owner)
 
         add_ips, prev_ips, remove_candidates = [], [], []
 
@@ -512,20 +557,35 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                      dns_nameservers,
                      host_routes,
                      subnet_request):
+        network_scope = addr_scope_obj.AddressScope.get_network_address_scope(
+            context, network.id, subnet_args['ip_version'])
+        # 'subnetpool' is not necessarily an object
+        subnetpool = subnet_args.get('subnetpool_id')
+        if subnetpool and subnetpool != const.IPV6_PD_POOL_ID:
+            subnetpool = self._get_subnetpool(context, subnetpool)
+
         self._validate_subnet_cidr(context, network, subnet_args['cidr'])
         self._validate_network_subnetpools(network,
-                                           subnet_args['subnetpool_id'],
-                                           subnet_args['ip_version'])
+                                           subnet_args['ip_version'],
+                                           subnetpool,
+                                           network_scope)
 
         service_types = subnet_args.pop('service_types', [])
 
-        subnet = models_v2.Subnet(**subnet_args)
         segment_id = subnet_args.get('segment_id')
-        try:
-            context.session.add(subnet)
-            context.session.flush()
-        except db_exc.DBReferenceError:
-            raise segment_exc.SegmentNotFound(segment_id=segment_id)
+        if segment_id:
+            # TODO(slaweq): integrate check if segment exists in
+            # self._validate_segment() method
+            segment = network_obj.NetworkSegment.get_object(context,
+                                                            id=segment_id)
+            if not segment:
+                raise segment_exc.SegmentNotFound(segment_id=segment_id)
+
+        subnet = subnet_obj.Subnet(context, **subnet_args)
+        subnet.create()
+        # TODO(slaweq): when check is segment exists will be integrated in
+        # self._validate_segment() method, it should be moved to be done before
+        # subnet object is created
         self._validate_segment(context, network['id'], segment_id)
 
         # NOTE(changzhi) Store DNS nameservers with order into DB one
@@ -540,23 +600,24 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
 
         if validators.is_attr_set(host_routes):
             for rt in host_routes:
-                route = models_v2.SubnetRoute(
+                route = subnet_obj.Route(
+                    context,
                     subnet_id=subnet.id,
-                    destination=rt['destination'],
-                    nexthop=rt['nexthop'])
-                context.session.add(route)
+                    destination=net_utils.AuthenticIPNetwork(
+                        rt['destination']),
+                    nexthop=netaddr.IPAddress(rt['nexthop']))
+                route.create()
 
         if validators.is_attr_set(service_types):
             for service_type in service_types:
-                service_type_entry = sst_model.SubnetServiceType(
-                    subnet_id=subnet.id,
-                    service_type=service_type)
-                context.session.add(service_type_entry)
+                service_type_obj = subnet_obj.SubnetServiceType(
+                    context, subnet_id=subnet.id, service_type=service_type)
+                service_type_obj.create()
 
         self.save_allocation_pools(context, subnet,
                                    subnet_request.allocation_pools)
 
-        return subnet
+        return subnet_obj.Subnet.get_object(context, id=subnet.id)
 
     def _classify_subnets(self, context, subnets):
         """Split into v4, v6 stateless and v6 stateful subnets"""
@@ -590,75 +651,24 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                 fixed_ip_list.append({'subnet_id': subnet['id']})
         return fixed_ip_list
 
-    def _query_subnets_on_network(self, context, network_id):
-        query = self._get_collection_query(context, models_v2.Subnet)
-        return query.filter(models_v2.Subnet.network_id == network_id)
-
-    def _query_filter_service_subnets(self, query, service_type):
-        ServiceType = sst_model.SubnetServiceType
-        query = query.add_entity(ServiceType)
-        query = query.outerjoin(ServiceType)
-        query = query.filter(or_(ServiceType.service_type.is_(None),
-                                 ServiceType.service_type == service_type))
-        return query.from_self(models_v2.Subnet)
-
-    @staticmethod
-    def _query_filter_by_segment_host_mapping(query, host):
-        """Excludes subnets on segments not reachable by the host
-
-        The query gets two kinds of subnets: those that are on segments that
-        the host can reach and those that are not on segments at all (assumed
-        reachable by all hosts). Hence, subnets on segments that the host
-        *cannot* reach are excluded.
-        """
-        Subnet = models_v2.Subnet
-        SegmentHostMapping = segment_svc_db.SegmentHostMapping
-
-        # A host has been provided.  Consider these two scenarios
-        # 1. Not a routed network:  subnets are not on segments
-        # 2. Is a routed network:  only subnets on segments mapped to host
-        # The following join query returns results for either.  The two are
-        # guaranteed to be mutually exclusive when subnets are created.
-        query = query.add_entity(SegmentHostMapping)
-        query = query.outerjoin(
-            SegmentHostMapping,
-            and_(Subnet.segment_id == SegmentHostMapping.segment_id,
-                 SegmentHostMapping.host == host))
-
-        # Essentially "segment_id IS NULL XNOR host IS NULL"
-        query = query.filter(or_(and_(Subnet.segment_id.isnot(None),
-                                      SegmentHostMapping.host.isnot(None)),
-                                 and_(Subnet.segment_id.is_(None),
-                                      SegmentHostMapping.host.is_(None))))
-        return query
-
-    @staticmethod
-    def _query_exclude_subnets_on_segments(query):
-        """Excludes all subnets associated with segments
-
-        For the case where the host is not known, we don't consider any subnets
-        that are on segments. But, we still consider subnets that are not
-        associated with any segment (i.e. for non-routed networks).
-        """
-        return query.filter(models_v2.Subnet.segment_id.is_(None))
-
-    @staticmethod
-    def is_host_set(host):
-        """Utility to tell if the host is set in the port binding"""
-        # This seems redundant, but its not. Host is unset if its None, '',
-        # or ATTR_NOT_SPECIFIED due to differences in host binding
-        # implementations.
-        return host and validators.is_attr_set(host)
-
-    def _ipam_get_subnets(self, context, network_id, host, service_type=None):
+    def _ipam_get_subnets(self, context, network_id, host, service_type=None,
+                          fixed_configured=False, fixed_ips=None,
+                          distributed_service=False):
         """Return eligible subnets
 
         If no eligible subnets are found, determine why and potentially raise
         an appropriate error.
         """
-        subnets = self._find_candidate_subnets(
-            context, network_id, host, service_type)
+        subnets = subnet_obj.Subnet.find_candidate_subnets(
+            context, network_id, host, service_type, fixed_configured,
+            fixed_ips, distributed_service=distributed_service)
         if subnets:
+            msg = ('This subnet is being modified by another concurrent '
+                   'operation')
+            for subnet in subnets:
+                subnet.lock_register(
+                    context, exc.SubnetInUse(subnet_id=subnet.id, reason=msg),
+                    id=subnet.id)
             subnet_dicts = [self._make_subnet_dict(subnet, context=context)
                             for subnet in subnets]
             # Give priority to subnets with service_types
@@ -666,56 +676,12 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                 subnet_dicts,
                 key=lambda subnet: not subnet.get('service_types'))
 
-        # Determine why we found no subnets to raise the right error
-        query = self._query_subnets_on_network(context, network_id)
-
-        if self.is_host_set(host):
-            # Empty because host isn't mapped to a segment with a subnet?
-            s_query = query.filter(models_v2.Subnet.segment_id.isnot(None))
-            if s_query.limit(1).count() != 0:
-                # It is a routed network but no subnets found for host
-                raise segment_exc.HostNotConnectedToAnySegment(
-                    host=host, network_id=network_id)
-
-        if not query.limit(1).count():
-            # Network has *no* subnets of any kind. This isn't an error.
+        if subnet_obj.Subnet.network_has_no_subnet(
+                context, network_id, host, service_type):
             return []
 
-        # Does filtering ineligible service subnets makes the list empty?
-        query = self._query_filter_service_subnets(query, service_type)
-        if query.limit(1).count():
-            # No, must be a deferred IP port because there are matching
-            # subnets. Happens on routed networks when host isn't known.
-            raise ipam_exceptions.DeferIpam()
-
-        raise ipam_exceptions.IpAddressGenerationFailureNoMatchingSubnet()
-
-    def _find_candidate_subnets(self, context, network_id, host, service_type):
-        """Find canditate subnets for the network, host, and service_type"""
-        query = self._query_subnets_on_network(context, network_id)
-        query = self._query_filter_service_subnets(query, service_type)
-
-        # Select candidate subnets and return them
-        if not self.is_host_set(host):
-            # If the host isn't known, we can't allocate on a routed network.
-            # So, exclude any subnets attached to segments.
-            return self._query_exclude_subnets_on_segments(query).all()
-
-        # The host is known. Consider both routed and non-routed networks
-        results = self._query_filter_by_segment_host_mapping(query, host).all()
-
-        # For now, we're using a simplifying assumption that a host will only
-        # touch one segment in a given routed network.  Raise exception
-        # otherwise.  This restriction may be relaxed as use cases for multiple
-        # mappings are understood.
-        segment_ids = {subnet.segment_id
-                       for subnet, mapping in results
-                       if mapping}
-        if 1 < len(segment_ids):
-            raise segment_exc.HostConnectedToMultipleSegments(
-                host=host, network_id=network_id)
-
-        return [subnet for subnet, _mapping in results]
+        raise ipam_exceptions.IpAddressGenerationFailureNoMatchingSubnet(
+            network_id=network_id, service_type=service_type)
 
     def _make_subnet_args(self, detail, subnet, subnetpool_id):
         args = super(IpamBackendMixin, self)._make_subnet_args(
@@ -752,16 +718,19 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         fixed_ips_requested = validators.is_attr_set(new_port.get('fixed_ips'))
         old_ips = old_port.get('fixed_ips')
         deferred_ip_allocation = (
-            old_port.get('ip_allocation') == ipa.IP_ALLOCATION_DEFERRED
-            and host and not old_host
-            and not old_ips
-            and not fixed_ips_requested)
+            old_port.get('ip_allocation') ==
+            ipalloc_apidef.IP_ALLOCATION_DEFERRED and
+            host and not old_host and
+            not old_ips and
+            not fixed_ips_requested)
         if not deferred_ip_allocation:
             # Check that any existing IPs are valid on the new segment
             new_host_requested = host and host != old_host
             if old_ips and new_host_requested and not fixed_ips_requested:
                 valid_subnets = self._ipam_get_subnets(
-                    context, old_port['network_id'], host)
+                    context, old_port['network_id'], host,
+                    service_type=old_port.get('device_owner'),
+                    distributed_service=self._is_distributed_service(old_port))
                 valid_subnet_ids = {s['id'] for s in valid_subnets}
                 for fixed_ip in old_ips:
                     if fixed_ip['subnet_id'] not in valid_subnet_ids:
@@ -776,4 +745,5 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         context.session.expire(old_port_db, ['fixed_ips'])
         ips = self.allocate_ips_for_port_and_store(
             context, {'port': port_copy}, port_copy['id'])
+        getattr(old_port_db, 'fixed_ips')  # refresh relationship before return
         return self.Changes(add=ips, original=[], remove=[])
