@@ -118,6 +118,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         allocated ip addresses.
         """
         allocated = []
+        factory = ipam_driver.get_address_request_factory()
 
         # we need to start with entries that asked for a specific IP in case
         # those IPs happen to be next in the line for allocation for ones that
@@ -132,7 +133,6 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                 ip_list = [ip] if isinstance(ip, dict) else ip
                 subnets = [ip_dict['subnet_id'] for ip_dict in ip_list]
                 try:
-                    factory = ipam_driver.get_address_request_factory()
                     ip_request = factory.get_request(context, port, ip_list[0])
                     ipam_allocator = ipam_driver.get_allocator(subnets)
                     ip_address, subnet_id = ipam_allocator.allocate(ip_request)
@@ -230,11 +230,15 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         """
         p = port['port']
         fixed_configured = p['fixed_ips'] is not constants.ATTR_NOT_SPECIFIED
-        subnets = self._ipam_get_subnets(context,
-                                         network_id=p['network_id'],
-                                         host=p.get(portbindings.HOST_ID),
-                                         service_type=p.get('device_owner'),
-                                         fixed_configured=fixed_configured)
+        fixed_ips = p['fixed_ips'] if fixed_configured else []
+        subnets = self._ipam_get_subnets(
+            context,
+            network_id=p['network_id'],
+            host=p.get(portbindings.HOST_ID),
+            service_type=p.get('device_owner'),
+            fixed_configured=fixed_configured,
+            fixed_ips=fixed_ips,
+            distributed_service=self._is_distributed_service(p))
 
         v4, v6_stateful, v6_stateless = self._classify_subnets(
             context, subnets)
@@ -289,7 +293,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         fixed_ip_list = []
         for fixed in fixed_ips:
             fixed['device_owner'] = device_owner
-            subnet = self._get_subnet_for_fixed_ip(context, fixed, subnets)
+            subnet = self._get_subnet_for_fixed_ip(context, fixed,
+                                                   subnets, network_id)
 
             is_auto_addr_subnet = ipv6_utils.is_auto_address_subnet(subnet)
             if ('ip_address' in fixed and
@@ -345,7 +350,9 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         try:
             subnets = self._ipam_get_subnets(
                 context, network_id=port['network_id'], host=host,
-                service_type=port.get('device_owner'))
+                service_type=port.get('device_owner'), fixed_configured=True,
+                fixed_ips=changes.add + changes.original,
+                distributed_service=self._is_distributed_service(port))
         except ipam_exc.DeferIpam:
             subnets = []
 
@@ -488,8 +495,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         self._ipam_deallocate_ips(context, ipam_driver, port,
                                   port['fixed_ips'])
 
-    def update_db_subnet(self, context, id, s, old_pools):
-        subnet = obj_subnet.Subnet.get_object(context, id=id)
+    def update_db_subnet(self, context, id, s, old_pools, subnet_obj=None):
+        subnet = subnet_obj or obj_subnet.Subnet.get_object(context, id=id)
         old_segment_id = subnet.segment_id if subnet else None
         if 'segment_id' in s:
             self._validate_segment(
@@ -500,7 +507,7 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
         # so create unchanged copy for ipam driver
         subnet_copy = copy.deepcopy(s)
         subnet, changes = super(IpamPluggableBackend, self).update_db_subnet(
-            context, id, s, old_pools)
+            context, id, s, old_pools, subnet_obj=subnet_obj)
         ipam_driver = driver.Pool.get_instance(None, context)
 
         # Set old allocation pools if no new pools are provided by user.
@@ -527,6 +534,22 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
             ipam_driver = driver.Pool.get_instance(None, context)
             factory = ipam_driver.get_address_request_factory()
             for port in ports:
+                # Find candidate subnets based on host_id and existing
+                # fixed_ips. This will filter subnets on other segments. Only
+                # allocate if this subnet is a valid candidate.
+                p = self._make_port_dict(port)
+                fixed_configured = (p['fixed_ips'] is not
+                                    constants.ATTR_NOT_SPECIFIED)
+                subnet_candidates = obj_subnet.Subnet.find_candidate_subnets(
+                    context,
+                    network_id,
+                    p.get(portbindings.HOST_ID),
+                    p.get('device_owner'),
+                    fixed_configured,
+                    p.get('fixed_ips'))
+                if subnet['id'] not in [s['id'] for s in subnet_candidates]:
+                    continue
+
                 ip = {'subnet_id': subnet['id'],
                       'subnet_cidr': subnet['cidr'],
                       'eui64_address': True,
@@ -596,6 +619,8 @@ class IpamPluggableBackend(ipam_backend_mixin.IpamBackendMixin):
                                        subnet['dns_nameservers'],
                                        subnet['host_routes'],
                                        subnet_request)
+            obj_subnet.NetworkSubnetLock.lock_subnet(context, network.id,
+                                                     subnet.id)
         except Exception:
             # Note(pbondar): Third-party ipam servers can't rely
             # on transaction rollback, so explicit rollback call needed.
