@@ -52,6 +52,7 @@ from neutron.ipam import utils as ipam_utils
 from neutron.objects import agent as ag_obj
 from neutron.objects import base as base_obj
 from neutron.objects import l3agent as rb_obj
+from neutron.objects import ports as port_obj
 from neutron.objects import router as l3_obj
 
 
@@ -768,6 +769,61 @@ class DVRResourceOperationHandler(object):
                                                            p['id'],
                                                            l3_port_check=False)
 
+    def _get_ext_nets_by_host(self, context, host):
+        ext_nets = set()
+        ports_ids = port_obj.Port.get_ports_by_host(context, host)
+        for port_id in ports_ids:
+            fips = self._get_floatingips_by_port_id(context, port_id)
+            for fip in fips:
+                ext_nets.add(fip.floating_network_id)
+        return ext_nets
+
+    @registry.receives(resources.AGENT, [events.AFTER_CREATE])
+    def create_fip_agent_gw_ports(self, resource, event,
+                                  trigger, payload=None):
+        """Create floating agent gw ports for DVR L3 agent.
+
+        Create floating IP Agent gateway ports when an L3 agent is created.
+        """
+        if not payload:
+            return
+        agent = payload.latest_state
+        if agent.get('agent_type') != const.AGENT_TYPE_L3:
+            return
+        # NOTE(slaweq) agent is passed in payload as dict so to avoid getting
+        # again from db, lets just get configuration from this dict directly
+        l3_agent_mode = agent.get('configurations', {}).get('agent_mode')
+        if l3_agent_mode not in [const.L3_AGENT_MODE_DVR,
+                                 const.L3_AGENT_MODE_DVR_SNAT]:
+            return
+
+        host = agent['host']
+        context = payload.context.elevated()
+        for ext_net in self._get_ext_nets_by_host(context, host):
+            self.create_fip_agent_gw_port_if_not_exists(
+                context, ext_net, host)
+
+    @registry.receives(resources.AGENT, [events.AFTER_DELETE])
+    def delete_fip_agent_gw_ports(self, resource, event,
+                                  trigger, payload=None):
+        """Delete floating agent gw ports for DVR.
+
+        Delete floating IP Agent gateway ports from host when an L3 agent is
+        deleted.
+        """
+        if not payload:
+            return
+        agent = payload.latest_state
+        if agent.get('agent_type') != const.AGENT_TYPE_L3:
+            return
+        if self._get_agent_mode(agent) not in [const.L3_AGENT_MODE_DVR,
+                                               const.L3_AGENT_MODE_DVR_SNAT]:
+            return
+
+        agent_gw_ports = self._get_agent_gw_ports(payload.context, agent['id'])
+        for gw_port in agent_gw_ports:
+            self._core_plugin.delete_port(payload.context, gw_port['id'])
+
 
 class _DVRAgentInterfaceMixin(object):
     """Contains calls made by the DVR scheduler and RPC interface.
@@ -1044,6 +1100,14 @@ class _DVRAgentInterfaceMixin(object):
         if ports:
             return ports[0]
 
+    def _get_agent_gw_ports(self, context, agent_id):
+        """Return agent gw ports."""
+        filters = {
+            'device_id': [agent_id],
+            'device_owner': [const.DEVICE_OWNER_AGENT_GW]
+        }
+        return self._core_plugin.get_ports(context, filters)
+
     def check_for_fip_and_create_agent_gw_port_on_host_if_not_exists(
             self, context, port, host):
         """Create fip agent_gw_port on host if not exists"""
@@ -1331,11 +1395,23 @@ class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
     def get_ports_under_dvr_connected_subnet(self, context, subnet_id):
         query = dvr_mac_db.get_ports_query_by_subnet_and_ip(context, subnet_id)
         ports = [p for p in query.all() if is_port_bound(p)]
-        return [
+        # TODO(slaweq): if there would be way to pass to neutron-lib only
+        # list of extensions which actually should be processed, than setting
+        # process_extensions=True below could avoid that second loop and
+        # getting allowed_address_pairs for each of the ports
+        ports_list = [
             self.l3plugin._core_plugin._make_port_dict(
                 port, process_extensions=False)
             for port in ports
         ]
+        ports_ids = [p['id'] for p in ports_list]
+        allowed_address_pairs = (
+            self._core_plugin.get_allowed_address_pairs_for_ports(
+                context, ports_ids))
+        for port in ports_list:
+            port['allowed_address_pairs'] = allowed_address_pairs.get(
+                port['id'], [])
+        return ports_list
 
 
 def is_distributed_router(router):
