@@ -25,11 +25,9 @@ from alembic import util as alembic_util
 from oslo_config import cfg
 from oslo_utils import fileutils
 from oslo_utils import importutils
-import pkg_resources
-import six
 
 from neutron._i18n import _
-from neutron.common import utils
+from neutron.conf.db import migration_cli
 from neutron.db import migration
 from neutron.db.migration.connection import DBConnection
 
@@ -39,57 +37,34 @@ HEADS_FILENAME = 'HEADS'
 CONTRACT_HEAD_FILENAME = 'CONTRACT_HEAD'
 EXPAND_HEAD_FILENAME = 'EXPAND_HEAD'
 
-CURRENT_RELEASE = migration.NEWTON
+CURRENT_RELEASE = migration.VICTORIA
 RELEASES = (
     migration.LIBERTY,
     migration.MITAKA,
     migration.NEWTON,
+    migration.OCATA,
+    migration.PIKE,
+    migration.QUEENS,
+    migration.ROCKY,
+    migration.STEIN,
+    migration.TRAIN,
+    migration.USSURI,
+    migration.VICTORIA,
 )
 
 EXPAND_BRANCH = 'expand'
 CONTRACT_BRANCH = 'contract'
 MIGRATION_BRANCHES = (EXPAND_BRANCH, CONTRACT_BRANCH)
 
-MIGRATION_ENTRYPOINTS = 'neutron.db.alembic_migrations'
-migration_entrypoints = {
-    entrypoint.name: entrypoint
-    for entrypoint in pkg_resources.iter_entry_points(MIGRATION_ENTRYPOINTS)
-}
-
-
 neutron_alembic_ini = os.path.join(os.path.dirname(__file__), 'alembic.ini')
 
-
-INSTALLED_SUBPROJECTS = [project_ for project_ in migration_entrypoints]
-
-_core_opts = [
-    cfg.StrOpt('subproject',
-               choices=INSTALLED_SUBPROJECTS,
-               help=(_("The subproject to execute the command against. "
-                       "Can be one of: '%s'.")
-                     % "', '".join(INSTALLED_SUBPROJECTS))),
-    cfg.BoolOpt('split_branches',
-                default=True,
-                deprecated_for_removal=True,
-                help=_("DEPRECATED. Alembic environments integrating with "
-                       "Neutron must implement split (contract and expand) "
-                       "branches file structure."))
-]
-
-_db_opts = [
-    cfg.StrOpt('connection',
-               default='',
-               secret=True,
-               help=_('URL to database')),
-    cfg.StrOpt('engine',
-               default='',
-               help=_('Database engine for which script will be generated '
-                      'when using offline migration.')),
-]
-
 CONF = cfg.ConfigOpts()
-CONF.register_cli_opts(_core_opts)
-CONF.register_cli_opts(_db_opts, 'database')
+migration_cli.register_db_cli_opts(CONF)
+
+
+log_error = alembic_util.err
+log_warning = alembic_util.warn
+log_info = alembic_util.msg
 
 
 def do_alembic_command(config, cmd, revision=None, desc=None, **kwargs):
@@ -99,22 +74,22 @@ def do_alembic_command(config, cmd, revision=None, desc=None, **kwargs):
 
     project = config.get_main_option('neutron_project')
     if desc:
-        alembic_util.msg(_('Running %(cmd)s (%(desc)s) for %(project)s ...') %
-                         {'cmd': cmd, 'desc': desc, 'project': project})
+        log_info(_('Running %(cmd)s (%(desc)s) for %(project)s ...') %
+                 {'cmd': cmd, 'desc': desc, 'project': project})
     else:
-        alembic_util.msg(_('Running %(cmd)s for %(project)s ...') %
-                         {'cmd': cmd, 'project': project})
+        log_info(_('Running %(cmd)s for %(project)s ...') %
+                 {'cmd': cmd, 'project': project})
     try:
         getattr(alembic_command, cmd)(config, *args, **kwargs)
     except alembic_util.CommandError as e:
-        alembic_util.err(six.text_type(e))
-    alembic_util.msg(_('OK'))
+        log_error(str(e))
+    log_info(_('OK'))
 
 
 def _get_alembic_entrypoint(project):
-    if project not in migration_entrypoints:
-        alembic_util.err(_('Sub-project %s not installed.') % project)
-    return migration_entrypoints[project]
+    if project not in migration_cli.migration_entrypoints:
+        log_error(_('Sub-project %s not installed.') % project)
+    return migration_cli.migration_entrypoints[project]
 
 
 def do_generic_show(config, cmd):
@@ -155,7 +130,7 @@ def do_upgrade(config, cmd):
     branch = None
 
     if ((CONF.command.revision or CONF.command.delta) and
-        (CONF.command.expand or CONF.command.contract)):
+            (CONF.command.expand or CONF.command.contract)):
         raise SystemExit(_(
             'Phase upgrade options do not accept revision specification'))
 
@@ -227,7 +202,7 @@ def _check_bootstrap_new_branch(branch, version_path, addn_kwargs):
     addn_kwargs['head'] = _get_branch_head(branch)
     if not os.path.exists(version_path):
         # Bootstrap initial directory structure
-        utils.ensure_dir(version_path)
+        fileutils.ensure_tree(version_path, mode=0o755)
 
 
 def do_revision(config, cmd):
@@ -280,14 +255,14 @@ def _compare_labels(revision, expected_labels):
         bad_labels_with_release = (revision.branch_labels -
                                    _get_release_labels(expected_labels))
         if not bad_labels_with_release:
-            alembic_util.warn(
+            log_warning(
                 _('Release aware branch labels (%s) are deprecated. '
                   'Please switch to expand@ and contract@ '
                   'labels.') % bad_labels)
             return
 
         script_name = os.path.basename(revision.path)
-        alembic_util.err(
+        log_error(
             _('Unexpected label for script %(script_name)s: %(labels)s') %
             {'script_name': script_name,
              'labels': bad_labels}
@@ -329,7 +304,7 @@ def validate_revisions(config):
     branchpoints = _get_branch_points(script_dir)
     if len(branchpoints) > 1:
         branchpoints = ', '.join(p.revision for p in branchpoints)
-        alembic_util.err(
+        log_error(
             _('Unexpected number of alembic branch points: %(branchpoints)s') %
             {'branchpoints': branchpoints}
         )
@@ -367,7 +342,7 @@ def _check_head(branch_name, head_file, head):
         pass
     else:
         if observed_head != head:
-            alembic_util.err(
+            log_error(
                 _('%(branch)s HEAD file does not match migration timeline '
                   'head, expected: %(head)s') % {'branch': branch_name.title(),
                                                  'head': head})
@@ -378,8 +353,8 @@ def validate_head_files(config):
     contract_head = _get_contract_head_file_path(config)
     expand_head = _get_expand_head_file_path(config)
     if not os.path.exists(contract_head) or not os.path.exists(expand_head):
-        alembic_util.warn(_("Repository does not contain HEAD files for "
-                            "contract and expand branches."))
+        log_warning(_("Repository does not contain HEAD files for "
+                      "contract and expand branches."))
         return
     head_map = _get_heads_map(config)
     _check_head(CONTRACT_BRANCH, contract_head, head_map[CONTRACT_BRANCH])
@@ -419,11 +394,11 @@ def has_offline_migrations(config, cmd):
         # it means we should shut down all neutron-server instances before
         # proceeding with upgrade.
         project = config.get_main_option('neutron_project')
-        alembic_util.msg(_('Need to apply migrations from %(project)s '
-                           'contract branch. This will require all Neutron '
-                           'server instances to be shutdown before '
-                           'proceeding with the upgrade.') %
-            {"project": project})
+        log_info(_('Need to apply migrations from %(project)s '
+                   'contract branch. This will require all Neutron '
+                   'server instances to be shutdown before '
+                   'proceeding with the upgrade.') %
+                 {"project": project})
         return True
     return False
 
@@ -496,7 +471,7 @@ def _get_package_root_dir(config):
     root_module = importutils.try_import(_get_project_base(config))
     if not root_module:
         project = config.get_main_option('neutron_project')
-        alembic_util.err(_("Failed to locate source for %s.") % project)
+        log_error(_("Failed to locate source for %s.") % project)
     # The root_module.__file__ property is a path like
     #    '/opt/stack/networking-foo/networking_foo/__init__.py'
     # We return just
@@ -526,7 +501,8 @@ def _get_head_file_path(config):
 
 
 def _get_heads_file_path(config):
-    '''
+    '''Get heads file path
+
     Return the path of the file that was once used to maintain the list of
     latest heads.
     '''
@@ -536,18 +512,14 @@ def _get_heads_file_path(config):
 
 
 def _get_contract_head_file_path(config):
-    '''
-    Return the path of the file that is used to maintain contract head
-    '''
+    '''Return the path of the file that is used to maintain contract head'''
     return os.path.join(
         _get_root_versions_dir(config),
         CONTRACT_HEAD_FILENAME)
 
 
 def _get_expand_head_file_path(config):
-    '''
-    Return the path of the file that is used to maintain expand head
-    '''
+    '''Return the path of the file that is used to maintain expand head'''
     return os.path.join(
         _get_root_versions_dir(config),
         EXPAND_HEAD_FILENAME)
@@ -576,9 +548,9 @@ def _set_version_locations(config):
 
 def _get_installed_entrypoint(subproject):
     '''Get the entrypoint for the subproject, which must be installed.'''
-    if subproject not in migration_entrypoints:
-        alembic_util.err(_('Package %s not installed') % subproject)
-    return migration_entrypoints[subproject]
+    if subproject not in migration_cli.migration_entrypoints:
+        log_error(_('Package %s not installed') % subproject)
+    return migration_cli.migration_entrypoints[subproject]
 
 
 def _get_subproject_script_location(subproject):
@@ -602,7 +574,7 @@ def get_alembic_version_table(config):
         return []
 
     with environment.EnvironmentContext(config, script_dir,
-                                       fn=alembic_version_table_from_env):
+                                        fn=alembic_version_table_from_env):
         script_dir.run_env()
 
     return alembic_version_table[0]
@@ -622,7 +594,7 @@ def get_alembic_configs():
         script_location = _get_subproject_script_location(CONF.subproject)
         script_locations[CONF.subproject] = script_location
     else:
-        for subproject, ep in migration_entrypoints.items():
+        for subproject in migration_cli.migration_entrypoints:
             script_locations[subproject] = _get_subproject_script_location(
                 subproject)
 
@@ -671,7 +643,7 @@ def run_sanity_checks(config, revision):
 
 
 def get_engine_config():
-    return [obj for obj in _db_opts if obj.name == 'engine']
+    return [obj for obj in migration_cli.DB_OPTS if obj.name == 'engine']
 
 
 def main():
@@ -682,10 +654,10 @@ def main():
     CONF(project='neutron')
     return_val = False
     for config in get_alembic_configs():
-        #TODO(gongysh) enable logging
+        # TODO(gongysh) enable logging
         return_val |= bool(CONF.command.func(config, CONF.command.name))
 
     if CONF.command.name == 'has_offline_migrations' and not return_val:
-        alembic_util.msg(_('No offline migrations pending.'))
+        log_info(_('No offline migrations pending.'))
 
     return return_val
