@@ -12,12 +12,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.db import constants as db_const
 from neutron_lib.db import model_base
 import sqlalchemy as sa
 from sqlalchemy import orm
+from sqlalchemy import sql
 
-from neutron.api.v2 import attributes
 from neutron.db import models_v2
+from neutron.db import rbac_db_models
 from neutron.db import standard_attr
 from neutron.extensions import securitygroup as sg
 
@@ -26,8 +28,17 @@ class SecurityGroup(standard_attr.HasStandardAttributes, model_base.BASEV2,
                     model_base.HasId, model_base.HasProject):
     """Represents a v2 neutron security group."""
 
-    name = sa.Column(sa.String(attributes.NAME_MAX_LEN))
+    name = sa.Column(sa.String(db_const.NAME_FIELD_SIZE))
+    stateful = sa.Column(sa.Boolean,
+                         default=True, server_default=sql.true(),
+                         nullable=False)
+    rbac_entries = sa.orm.relationship(rbac_db_models.SecurityGroupRBAC,
+                                       backref='security_group',
+                                       lazy='subquery',
+                                       cascade='all, delete, delete-orphan')
     api_collections = [sg.SECURITYGROUPS]
+    collection_resource_map = {sg.SECURITYGROUPS: 'security_group'}
+    tag_support = True
 
 
 class DefaultSecurityGroup(model_base.BASEV2, model_base.HasProjectPrimaryKey):
@@ -58,7 +69,7 @@ class SecurityGroupPortBinding(model_base.BASEV2):
     # Add a relationship to the Port model in order to instruct SQLAlchemy to
     # eagerly load security group bindings
     ports = orm.relationship(
-        models_v2.Port,
+        models_v2.Port, load_on_pending=True,
         backref=orm.backref("security_groups",
                             lazy='joined', cascade='delete'))
 
@@ -85,8 +96,8 @@ class SecurityGroupRule(standard_attr.HasStandardAttributes, model_base.BASEV2,
     port_range_max = sa.Column(sa.Integer)
     remote_ip_prefix = sa.Column(sa.String(255))
     security_group = orm.relationship(
-        SecurityGroup,
-        backref=orm.backref('rules', cascade='all,delete', lazy='joined'),
+        SecurityGroup, load_on_pending=True,
+        backref=orm.backref('rules', cascade='all,delete', lazy='dynamic'),
         primaryjoin="SecurityGroup.id==SecurityGroupRule.security_group_id")
     source_group = orm.relationship(
         SecurityGroup,
