@@ -14,21 +14,20 @@
 #    under the License.
 
 import netaddr
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as const
-from oslo_log import log as logging
-from oslo_utils import netutils
-from sqlalchemy.orm import exc
+from neutron_lib.db import api as db_api
+from neutron_lib.utils import helpers
 
-from neutron._i18n import _, _LW
-from neutron.common import utils
-from neutron.db import api as db_api
+from neutron._i18n import _
 from neutron.db.models import allowed_address_pair as aap_models
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import models_v2
 from neutron.db import securitygroups_db as sg_db
 from neutron.extensions import securitygroup as ext_sg
-
-LOG = logging.getLogger(__name__)
+from neutron.objects import securitygroup as sg_obj
 
 
 DIRECTION_IP_PREFIX = {'ingress': 'source_ip_prefix',
@@ -37,9 +36,102 @@ DIRECTION_IP_PREFIX = {'ingress': 'source_ip_prefix',
 DHCP_RULE_PORT = {4: (67, 68, const.IPv4), 6: (547, 546, const.IPv6)}
 
 
-class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
+@registry.has_registry_receivers
+class SecurityGroupServerNotifierRpcMixin(sg_db.SecurityGroupDbMixin):
     """Mixin class to add agent-based security group implementation."""
 
+    @registry.receives(resources.PORT, [events.AFTER_CREATE,
+                                        events.AFTER_UPDATE,
+                                        events.AFTER_DELETE])
+    def notify_sg_on_port_change(self, resource, event, trigger, context,
+                                 port, *args, **kwargs):
+        """Trigger notification to other SG members on port changes."""
+        if event == events.AFTER_UPDATE:
+            original_port = kwargs.get('original_port')
+            self.check_and_notify_security_group_member_changed(
+                context, original_port, port)
+        else:
+            self.notify_security_groups_member_updated(context, port)
+
+    def create_security_group_rule(self, context, security_group_rule):
+        rule = super(SecurityGroupServerNotifierRpcMixin,
+                     self).create_security_group_rule(context,
+                                                      security_group_rule)
+        sgids = [rule['security_group_id']]
+        self.notifier.security_groups_rule_updated(context, sgids)
+        return rule
+
+    def create_security_group_rule_bulk(self, context, security_group_rules):
+        rules = super(SecurityGroupServerNotifierRpcMixin,
+                      self).create_security_group_rule_bulk_native(
+                          context, security_group_rules)
+        sgids = set([r['security_group_id'] for r in rules])
+        self.notifier.security_groups_rule_updated(context, list(sgids))
+        return rules
+
+    def delete_security_group_rule(self, context, sgrid):
+        rule = self.get_security_group_rule(context, sgrid)
+        super(SecurityGroupServerNotifierRpcMixin,
+              self).delete_security_group_rule(context, sgrid)
+        self.notifier.security_groups_rule_updated(context,
+                                                   [rule['security_group_id']])
+
+    def check_and_notify_security_group_member_changed(
+            self, context, original_port, updated_port):
+        sg_change = not helpers.compare_elements(
+            original_port.get(ext_sg.SECURITYGROUPS),
+            updated_port.get(ext_sg.SECURITYGROUPS))
+        if sg_change:
+            self.notify_security_groups_member_updated_bulk(
+                context, [original_port, updated_port])
+        elif original_port['fixed_ips'] != updated_port['fixed_ips']:
+            self.notify_security_groups_member_updated(context, updated_port)
+
+    def is_security_group_member_updated(self, context,
+                                         original_port, updated_port):
+        """Check security group member updated or not.
+
+        This method returns a flag which indicates request notification
+        is required and does not perform notification itself.
+        It is because another changes for the port may require notification.
+        """
+        need_notify = False
+        if (original_port['fixed_ips'] != updated_port['fixed_ips'] or
+            original_port['mac_address'] != updated_port['mac_address'] or
+            not helpers.compare_elements(
+                original_port.get(ext_sg.SECURITYGROUPS),
+                updated_port.get(ext_sg.SECURITYGROUPS))):
+            need_notify = True
+        return need_notify
+
+    def notify_security_groups_member_updated_bulk(self, context, ports):
+        """Notify update event of security group members for ports.
+
+        The agent setups the iptables rule to allow
+        ingress packet from the dhcp server (as a part of provider rules),
+        so we need to notify an update of dhcp server ip
+        address to the plugin agent.
+        """
+        sec_groups = set()
+        for port in ports:
+            # NOTE (Swami): ROUTER_INTERFACE_OWNERS check is required
+            # since it includes the legacy router interface device owners
+            # and DVR router interface device owners.
+            if (port['device_owner'] not in
+                    [const.DEVICE_OWNER_DHCP, const.ROUTER_INTERFACE_OWNERS]):
+                sec_groups |= set(port.get(ext_sg.SECURITYGROUPS))
+
+        if sec_groups:
+            self.notifier.security_groups_member_updated(
+                context, list(sec_groups))
+
+    def notify_security_groups_member_updated(self, context, port):
+        self.notify_security_groups_member_updated_bulk(context, [port])
+
+
+class SecurityGroupInfoAPIMixin(object):
+    """API for retrieving security group info for SG agent code."""
+
     def get_port_from_device(self, context, device):
         """Get port dict from device name on an agent.
 
@@ -70,102 +162,6 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
         return [self.get_port_from_device(context, device)
                 for device in devices]
 
-    def create_security_group_rule(self, context, security_group_rule):
-        rule = super(SecurityGroupServerRpcMixin,
-                     self).create_security_group_rule(context,
-                                                      security_group_rule)
-        sgids = [rule['security_group_id']]
-        self.notifier.security_groups_rule_updated(context, sgids)
-        return rule
-
-    def create_security_group_rule_bulk(self, context, security_group_rules):
-        rules = super(SecurityGroupServerRpcMixin,
-                      self).create_security_group_rule_bulk_native(
-                          context, security_group_rules)
-        sgids = set([r['security_group_id'] for r in rules])
-        self.notifier.security_groups_rule_updated(context, list(sgids))
-        return rules
-
-    def delete_security_group_rule(self, context, sgrid):
-        rule = self.get_security_group_rule(context, sgrid)
-        super(SecurityGroupServerRpcMixin,
-              self).delete_security_group_rule(context, sgrid)
-        self.notifier.security_groups_rule_updated(context,
-                                                   [rule['security_group_id']])
-
-    def check_and_notify_security_group_member_changed(
-            self, context, original_port, updated_port):
-        sg_change = not utils.compare_elements(
-            original_port.get(ext_sg.SECURITYGROUPS),
-            updated_port.get(ext_sg.SECURITYGROUPS))
-        if sg_change:
-            self.notify_security_groups_member_updated_bulk(
-                context, [original_port, updated_port])
-        elif original_port['fixed_ips'] != updated_port['fixed_ips']:
-            self.notify_security_groups_member_updated(context, updated_port)
-
-    def is_security_group_member_updated(self, context,
-                                         original_port, updated_port):
-        """Check security group member updated or not.
-
-        This method returns a flag which indicates request notification
-        is required and does not perform notification itself.
-        It is because another changes for the port may require notification.
-        """
-        need_notify = False
-        if (original_port['fixed_ips'] != updated_port['fixed_ips'] or
-            original_port['mac_address'] != updated_port['mac_address'] or
-            not utils.compare_elements(
-                original_port.get(ext_sg.SECURITYGROUPS),
-                updated_port.get(ext_sg.SECURITYGROUPS))):
-            need_notify = True
-        return need_notify
-
-    def notify_security_groups_member_updated_bulk(self, context, ports):
-        """Notify update event of security group members for ports.
-
-        The agent setups the iptables rule to allow
-        ingress packet from the dhcp server (as a part of provider rules),
-        so we need to notify an update of dhcp server ip
-        address to the plugin agent.
-        security_groups_provider_updated() just notifies that an event
-        occurs and the plugin agent fetches the update provider
-        rule in the other RPC call (security_group_rules_for_devices).
-        """
-        sg_provider_updated_networks = set()
-        sec_groups = set()
-        for port in ports:
-            if port['device_owner'] == const.DEVICE_OWNER_DHCP:
-                sg_provider_updated_networks.add(
-                    port['network_id'])
-            # For IPv6, provider rule need to be updated in case router
-            # interface is created or updated after VM port is created.
-            # NOTE (Swami): ROUTER_INTERFACE_OWNERS check is required
-            # since it includes the legacy router interface device owners
-            # and DVR router interface device owners.
-            elif port['device_owner'] in const.ROUTER_INTERFACE_OWNERS:
-                if any(netaddr.IPAddress(fixed_ip['ip_address']).version == 6
-                       for fixed_ip in port['fixed_ips']):
-                    sg_provider_updated_networks.add(
-                        port['network_id'])
-            else:
-                sec_groups |= set(port.get(ext_sg.SECURITYGROUPS))
-
-        if sg_provider_updated_networks:
-            ports_query = context.session.query(models_v2.Port.id).filter(
-                models_v2.Port.network_id.in_(
-                    sg_provider_updated_networks)).all()
-            ports_to_update = [p.id for p in ports_query]
-            self.notifier.security_groups_provider_updated(
-                context, ports_to_update)
-        if sec_groups:
-            self.notifier.security_groups_member_updated(
-                context, list(sec_groups))
-
-    def notify_security_groups_member_updated(self, context, port):
-        self.notify_security_groups_member_updated_bulk(context, [port])
-
-    @db_api.retry_if_session_inactive()
     def security_group_info_for_ports(self, context, ports):
         sg_info = {'devices': ports,
                    'security_groups': {},
@@ -177,7 +173,7 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
             security_group_id = rule_in_db.get('security_group_id')
             ethertype = rule_in_db['ethertype']
             if ('security_group_source_groups'
-                not in sg_info['devices'][port_id]):
+                    not in sg_info['devices'][port_id]):
                 sg_info['devices'][port_id][
                     'security_group_source_groups'] = []
 
@@ -194,9 +190,12 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
                     remote_security_group_info[remote_gid][ethertype] = set()
 
             direction = rule_in_db['direction']
+            stateful = self._is_security_group_stateful(context,
+                                                        security_group_id)
             rule_dict = {
                 'direction': direction,
-                'ethertype': ethertype}
+                'ethertype': ethertype,
+                'stateful': stateful}
 
             for key in ('protocol', 'port_range_min', 'port_range_max',
                         'remote_ip_prefix', 'remote_group_id'):
@@ -229,68 +228,11 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
             context, sg_info['sg_member_ips'].keys())
         for sg_id, member_ips in ips.items():
             for ip in member_ips:
-                ethertype = 'IPv%d' % netaddr.IPNetwork(ip).version
+                ethertype = 'IPv%d' % netaddr.IPNetwork(ip[0]).version
                 if ethertype in sg_info['sg_member_ips'][sg_id]:
                     sg_info['sg_member_ips'][sg_id][ethertype].add(ip)
         return sg_info
 
-    def _select_sg_ids_for_ports(self, context, ports):
-        if not ports:
-            return []
-        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
-        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
-        query = context.session.query(sg_binding_sgid)
-        query = query.filter(sg_binding_port.in_(ports.keys()))
-        return query.all()
-
-    def _select_rules_for_ports(self, context, ports):
-        if not ports:
-            return []
-        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
-        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
-
-        sgr_sgid = sg_models.SecurityGroupRule.security_group_id
-
-        query = context.session.query(sg_binding_port,
-                                      sg_models.SecurityGroupRule)
-        query = query.join(sg_models.SecurityGroupRule,
-                           sgr_sgid == sg_binding_sgid)
-        query = query.filter(sg_binding_port.in_(ports.keys()))
-        return query.all()
-
-    def _select_ips_for_remote_group(self, context, remote_group_ids):
-        ips_by_group = {}
-        if not remote_group_ids:
-            return ips_by_group
-        for remote_group_id in remote_group_ids:
-            ips_by_group[remote_group_id] = set()
-
-        ip_port = models_v2.IPAllocation.port_id
-        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
-        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
-
-        # Join the security group binding table directly to the IP allocation
-        # table instead of via the Port table skip an unnecessary intermediary
-        query = context.session.query(sg_binding_sgid,
-                                      models_v2.IPAllocation.ip_address,
-                                      aap_models.AllowedAddressPair.ip_address)
-        query = query.join(models_v2.IPAllocation,
-                           ip_port == sg_binding_port)
-        # Outerjoin because address pairs may be null and we still want the
-        # IP for the port.
-        query = query.outerjoin(
-            aap_models.AllowedAddressPair,
-            sg_binding_port == aap_models.AllowedAddressPair.port_id)
-        query = query.filter(sg_binding_sgid.in_(remote_group_ids))
-        # Each allowed address pair IP record for a port beyond the 1st
-        # will have a duplicate regular IP in the query response since
-        # the relationship is 1-to-many. Dedup with a set
-        for security_group_id, ip_address, allowed_addr_ip in query:
-            ips_by_group[security_group_id].add(ip_address)
-            if allowed_addr_ip:
-                ips_by_group[security_group_id].add(allowed_addr_ip)
-        return ips_by_group
-
     def _select_remote_group_ids(self, ports):
         remote_group_ids = []
         for port in ports.values():
@@ -300,92 +242,6 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
                     remote_group_ids.append(remote_group_id)
         return remote_group_ids
 
-    def _select_network_ids(self, ports):
-        return set((port['network_id'] for port in ports.values()))
-
-    def _select_dhcp_ips_for_network_ids(self, context, network_ids):
-        if not network_ids:
-            return {}
-        query = context.session.query(models_v2.Port.mac_address,
-                                      models_v2.Port.network_id,
-                                      models_v2.IPAllocation.ip_address)
-        query = query.join(models_v2.IPAllocation)
-        query = query.filter(models_v2.Port.network_id.in_(network_ids))
-        owner = const.DEVICE_OWNER_DHCP
-        query = query.filter(models_v2.Port.device_owner == owner)
-        ips = {}
-
-        for network_id in network_ids:
-            ips[network_id] = []
-
-        for mac_address, network_id, ip in query:
-            if (netaddr.IPAddress(ip).version == 6
-                and not netaddr.IPAddress(ip).is_link_local()):
-                ip = str(netutils.get_ipv6_addr_by_EUI64(const.IPv6_LLA_PREFIX,
-                    mac_address))
-            if ip not in ips[network_id]:
-                ips[network_id].append(ip)
-
-        return ips
-
-    def _select_ra_ips_for_network_ids(self, context, network_ids):
-        """Select IP addresses to allow sending router advertisement from.
-
-        If the OpenStack managed radvd process sends an RA, get link local
-        address of gateway and allow RA from this Link Local address.
-        The gateway port link local address will only be obtained
-        when router is created before VM instance is booted and
-        subnet is attached to router.
-
-        If OpenStack doesn't send RA, allow RA from gateway IP.
-        Currently, the gateway IP needs to be link local to be able
-        to send RA to VM.
-        """
-        if not network_ids:
-            return {}
-        ips = {}
-        for network_id in network_ids:
-            ips[network_id] = set([])
-        query = context.session.query(models_v2.Subnet)
-        subnets = query.filter(models_v2.Subnet.network_id.in_(network_ids))
-        for subnet in subnets:
-            gateway_ip = subnet['gateway_ip']
-            if subnet['ip_version'] != 6 or not gateway_ip:
-                continue
-            if not netaddr.IPAddress(gateway_ip).is_link_local():
-                if subnet['ipv6_ra_mode']:
-                    gateway_ip = self._get_lla_gateway_ip_for_subnet(context,
-                                                                     subnet)
-                else:
-                    # TODO(xuhanp):Figure out how to allow gateway IP from
-                    # existing device to be global address and figure out the
-                    # link local address by other method.
-                    continue
-            if gateway_ip:
-                ips[subnet['network_id']].add(gateway_ip)
-
-        return ips
-
-    def _get_lla_gateway_ip_for_subnet(self, context, subnet):
-        query = context.session.query(models_v2.Port.mac_address)
-        query = query.join(models_v2.IPAllocation)
-        query = query.filter(
-            models_v2.IPAllocation.subnet_id == subnet['id'])
-        query = query.filter(
-            models_v2.IPAllocation.ip_address == subnet['gateway_ip'])
-        query = query.filter(
-            models_v2.Port.device_owner.in_(const.ROUTER_INTERFACE_OWNERS))
-        try:
-            mac_address = query.one()[0]
-        except (exc.NoResultFound, exc.MultipleResultsFound):
-            LOG.warning(_LW('No valid gateway port on subnet %s is '
-                            'found for IPv6 RA'), subnet['id'])
-            return
-        lla_ip = str(netutils.get_ipv6_addr_by_EUI64(
-            const.IPv6_LLA_PREFIX,
-            mac_address))
-        return lla_ip
-
     def _convert_remote_group_id_to_ip_prefix(self, context, ports):
         remote_group_ids = self._select_remote_group_ids(ports)
         ips = self._select_ips_for_remote_group(context, remote_group_ids)
@@ -401,7 +257,8 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
 
                 port['security_group_source_groups'].append(remote_group_id)
                 base_rule = rule
-                for ip in ips[remote_group_id]:
+                ip_list = [ip[0] for ip in ips[remote_group_id]]
+                for ip in ip_list:
                     if ip in port.get('fixed_ips', []):
                         continue
                     ip_rule = base_rule.copy()
@@ -415,40 +272,47 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
             port['security_group_rules'] = updated_rule
         return ports
 
-    def _add_ingress_dhcp_rule(self, port, ips):
-        dhcp_ips = ips.get(port['network_id'])
-        for dhcp_ip in dhcp_ips:
-            source_port, dest_port, ethertype = DHCP_RULE_PORT[
-                netaddr.IPAddress(dhcp_ip).version]
-            dhcp_rule = {'direction': 'ingress',
-                         'ethertype': ethertype,
-                         'protocol': 'udp',
-                         'port_range_min': dest_port,
-                         'port_range_max': dest_port,
-                         'source_port_range_min': source_port,
-                         'source_port_range_max': source_port,
-                         'source_ip_prefix': dhcp_ip}
-            port['security_group_rules'].append(dhcp_rule)
+    def _add_ingress_dhcp_rule(self, port):
+        for ip_version in (4, 6):
+            # only allow DHCP servers to talk to the appropriate IP address
+            # to avoid getting leases that don't match the Neutron IPs
+            prefix = '32' if ip_version == 4 else '128'
+            dests = ['%s/%s' % (ip, prefix) for ip in port['fixed_ips']
+                     if netaddr.IPNetwork(ip).version == ip_version]
+            if ip_version == 4:
+                # v4 dhcp servers can also talk to broadcast
+                dests.append('255.255.255.255/32')
+            elif ip_version == 6:
+                # v6 dhcp responses can target link-local addresses
+                dests.append('fe80::/64')
+            source_port, dest_port, ethertype = DHCP_RULE_PORT[ip_version]
+            for dest in dests:
+                dhcp_rule = {'direction': 'ingress',
+                             'ethertype': ethertype,
+                             'protocol': 'udp',
+                             'port_range_min': dest_port,
+                             'port_range_max': dest_port,
+                             'source_port_range_min': source_port,
+                             'source_port_range_max': source_port,
+                             'dest_ip_prefix': dest}
+                port['security_group_rules'].append(dhcp_rule)
 
-    def _add_ingress_ra_rule(self, port, ips):
-        ra_ips = ips.get(port['network_id'])
-        for ra_ip in ra_ips:
-            ra_rule = {'direction': 'ingress',
-                       'ethertype': const.IPv6,
-                       'protocol': const.PROTO_NAME_IPV6_ICMP,
-                       'source_ip_prefix': ra_ip,
-                       'source_port_range_min': const.ICMPV6_TYPE_RA}
-            port['security_group_rules'].append(ra_rule)
+    def _add_ingress_ra_rule(self, port):
+        has_v6 = [ip for ip in port['fixed_ips']
+                  if netaddr.IPNetwork(ip).version == 6]
+        if not has_v6:
+            return
+        ra_rule = {'direction': 'ingress',
+                   'ethertype': const.IPv6,
+                   'protocol': const.PROTO_NAME_IPV6_ICMP,
+                   'source_port_range_min': const.ICMPV6_TYPE_RA}
+        port['security_group_rules'].append(ra_rule)
 
     def _apply_provider_rule(self, context, ports):
-        network_ids = self._select_network_ids(ports)
-        ips_dhcp = self._select_dhcp_ips_for_network_ids(context, network_ids)
-        ips_ra = self._select_ra_ips_for_network_ids(context, network_ids)
         for port in ports.values():
-            self._add_ingress_ra_rule(port, ips_ra)
-            self._add_ingress_dhcp_rule(port, ips_dhcp)
+            self._add_ingress_ra_rule(port)
+            self._add_ingress_dhcp_rule(port)
 
-    @db_api.retry_if_session_inactive()
     def security_group_rules_for_ports(self, context, ports):
         rules_in_db = self._select_rules_for_ports(context, ports)
         for (port_id, rule_in_db) in rules_in_db:
@@ -470,3 +334,107 @@ class SecurityGroupServerRpcMixin(sg_db.SecurityGroupDbMixin):
             port['security_group_rules'].append(rule_dict)
         self._apply_provider_rule(context, ports)
         return self._convert_remote_group_id_to_ip_prefix(context, ports)
+
+    def _select_ips_for_remote_group(self, context, remote_group_ids):
+        """Get all IP addresses (including allowed addr pairs) for each sg.
+
+        Return dict of lists of IPs keyed by group_id.
+        """
+        raise NotImplementedError()
+
+    def _select_rules_for_ports(self, context, ports):
+        """Get all security group rules associated with a list of ports.
+
+        Return list of tuples of (port_id, sg_rule)
+        """
+        raise NotImplementedError()
+
+    def _select_sg_ids_for_ports(self, context, ports):
+        """Return security group IDs for a list of ports.
+
+        Return list of tuples with a single element of sg_id.
+        """
+        raise NotImplementedError()
+
+    def _is_security_group_stateful(self, context, sg_id):
+        """Return whether the security group is stateful or not.
+
+        Return True if the security group associated with the given ID
+        is stateful, else False.
+        """
+        return True
+
+
+class SecurityGroupServerRpcMixin(SecurityGroupInfoAPIMixin,
+                                  SecurityGroupServerNotifierRpcMixin):
+    """Server-side RPC mixin using DB for SG notifications and responses."""
+
+    @db_api.retry_if_session_inactive()
+    def _select_sg_ids_for_ports(self, context, ports):
+        if not ports:
+            return []
+        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
+        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
+        query = context.session.query(sg_binding_sgid)
+        query = query.filter(sg_binding_port.in_(ports.keys()))
+        return query.all()
+
+    @db_api.retry_if_session_inactive()
+    def _select_rules_for_ports(self, context, ports):
+        if not ports:
+            return []
+        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
+        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
+
+        sgr_sgid = sg_models.SecurityGroupRule.security_group_id
+
+        query = context.session.query(sg_binding_port,
+                                      sg_models.SecurityGroupRule)
+        query = query.join(sg_models.SecurityGroupRule,
+                           sgr_sgid == sg_binding_sgid)
+        query = query.filter(sg_binding_port.in_(ports.keys()))
+        return query.all()
+
+    @db_api.retry_if_session_inactive()
+    def _select_ips_for_remote_group(self, context, remote_group_ids):
+        ips_by_group = {}
+        if not remote_group_ids:
+            return ips_by_group
+        for remote_group_id in remote_group_ids:
+            ips_by_group[remote_group_id] = set()
+
+        ip_port = models_v2.IPAllocation.port_id
+        sg_binding_port = sg_models.SecurityGroupPortBinding.port_id
+        sg_binding_sgid = sg_models.SecurityGroupPortBinding.security_group_id
+
+        # Join the security group binding table directly to the IP allocation
+        # table instead of via the Port table skip an unnecessary intermediary
+        query = context.session.query(
+            sg_binding_sgid,
+            models_v2.IPAllocation.ip_address,
+            aap_models.AllowedAddressPair.ip_address,
+            aap_models.AllowedAddressPair.mac_address)
+        query = query.join(models_v2.IPAllocation,
+                           ip_port == sg_binding_port)
+        # Outerjoin because address pairs may be null and we still want the
+        # IP for the port.
+        query = query.outerjoin(
+            aap_models.AllowedAddressPair,
+            sg_binding_port == aap_models.AllowedAddressPair.port_id)
+        query = query.filter(sg_binding_sgid.in_(remote_group_ids))
+        # Each allowed address pair IP record for a port beyond the 1st
+        # will have a duplicate regular IP in the query response since
+        # the relationship is 1-to-many. Dedup with a set
+        for security_group_id, ip_address, allowed_addr_ip, mac in query:
+            # Since port mac will not be used further, but in order to align
+            # the data structure we directly set None to it to avoid bother
+            # the ports table.
+            ips_by_group[security_group_id].add((ip_address, None))
+            if allowed_addr_ip:
+                ips_by_group[security_group_id].add(
+                    (allowed_addr_ip, mac))
+        return ips_by_group
+
+    @db_api.retry_if_session_inactive()
+    def _is_security_group_stateful(self, context, sg_id):
+        return sg_obj.SecurityGroup.get_sg_by_id(context, sg_id).stateful
