@@ -21,7 +21,6 @@ from neutron_lib.api import extensions as api_extensions
 from neutron_lib import constants
 from neutron_lib.db import constants as db_const
 from neutron_lib.plugins import directory
-import six
 
 from neutron.api import extensions
 from neutron.api.v2 import base
@@ -45,17 +44,20 @@ RESOURCE_ATTRIBUTE_MAP = {
                'allow_put': False,
                'validate': {'type:uuid': None},
                'is_filter': True,
+               'is_sort_key': True,
                'is_visible': True,
                'primary_key': True},
         'tenant_id': {'allow_post': True,
                       'allow_put': False,
                       'validate': {'type:string':
                                    db_const.PROJECT_ID_FIELD_SIZE},
+                      'is_sort_key': True,
                       'is_visible': False},
         'network_id': {'allow_post': True,
                        'allow_put': False,
                        'validate': {'type:uuid': None},
                        'is_filter': True,
+                       'is_sort_key': True,
                        'is_visible': True},
         PHYSICAL_NETWORK: {'allow_post': True,
                            'allow_put': False,
@@ -63,23 +65,27 @@ RESOURCE_ATTRIBUTE_MAP = {
                            'validate': {'type:string':
                                         providernet.PHYSICAL_NETWORK_MAX_LEN},
                            'is_filter': True,
+                           'is_sort_key': True,
                            'is_visible': True},
         NETWORK_TYPE: {'allow_post': True,
                        'allow_put': False,
                        'validate': {'type:string':
                                     providernet.NETWORK_TYPE_MAX_LEN},
                        'is_filter': True,
+                       'is_sort_key': True,
                        'is_visible': True},
         SEGMENTATION_ID: {'allow_post': True,
                           'allow_put': False,
                           'default': constants.ATTR_NOT_SPECIFIED,
                           'convert_to': converters.convert_to_int,
+                          'is_sort_key': True,
                           'is_visible': True},
         'name': {'allow_post': True,
                  'allow_put': True,
                  'default': constants.ATTR_NOT_SPECIFIED,
                  'validate': {'type:string_or_none': NAME_LEN},
                  'is_filter': True,
+                 'is_sort_key': True,
                  'is_visible': True}
     },
     subnet_def.COLLECTION_NAME: {
@@ -142,8 +148,7 @@ class Segment(api_extensions.ExtensionDescriptor):
                 stdattrseg_apidef.ALIAS]
 
 
-@six.add_metaclass(abc.ABCMeta)
-class SegmentPluginBase(object):
+class SegmentPluginBase(object, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def create_segment(self, context, segment):
