@@ -18,21 +18,24 @@ import random
 
 import netaddr
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import directory
 from oslo_db import exception as db_exc
 from oslo_log import log
 from oslo_utils import uuidutils
 
-from neutron._i18n import _, _LE
+from neutron._i18n import _
 from neutron.ipam import driver as ipam_base
 from neutron.ipam.drivers.neutrondb_ipam import db_api as ipam_db_api
 from neutron.ipam import exceptions as ipam_exc
 from neutron.ipam import requests as ipam_req
 from neutron.ipam import subnet_alloc
 from neutron.ipam import utils as ipam_utils
-from neutron import manager
 
 
 LOG = log.getLogger(__name__)
+MAX_WIN = 1000
+MULTIPLIER = 100
+MAX_WIN_MULTI = MAX_WIN * MULTIPLIER
 
 
 class NeutronDbSubnet(ipam_base.Subnet):
@@ -43,14 +46,14 @@ class NeutronDbSubnet(ipam_base.Subnet):
     """
 
     @classmethod
-    def create_allocation_pools(cls, subnet_manager, session, pools, cidr):
+    def create_allocation_pools(cls, subnet_manager, context, pools, cidr):
         for pool in pools:
             # IPv6 addresses that start '::1', '::2', etc cause IP version
             # ambiguity when converted to integers by pool.first and pool.last.
             # Infer the IP version from the subnet cidr.
             ip_version = cidr.version
             subnet_manager.create_pool(
-                session,
+                context,
                 netaddr.IPAddress(pool.first, ip_version).format(),
                 netaddr.IPAddress(pool.last, ip_version).format())
 
@@ -61,8 +64,7 @@ class NeutronDbSubnet(ipam_base.Subnet):
             ipam_subnet_id,
             subnet_request.subnet_id)
         # Create subnet resource
-        session = ctx.session
-        subnet_manager.create(session)
+        subnet_manager.create(ctx)
         # If allocation pools are not specified, define them around
         # the subnet's gateway IP
         if not subnet_request.allocation_pools:
@@ -71,7 +73,7 @@ class NeutronDbSubnet(ipam_base.Subnet):
         else:
             pools = subnet_request.allocation_pools
         # Create IPAM allocation pools
-        cls.create_allocation_pools(subnet_manager, session, pools,
+        cls.create_allocation_pools(subnet_manager, ctx, pools,
                                     subnet_request.subnet_cidr)
 
         return cls(ipam_subnet_id,
@@ -89,30 +91,29 @@ class NeutronDbSubnet(ipam_base.Subnet):
         :param neutron_subnet_id: neutron subnet identifier.
         """
         ipam_subnet = ipam_db_api.IpamSubnetManager.load_by_neutron_subnet_id(
-            ctx.session, neutron_subnet_id)
+            ctx, neutron_subnet_id)
         if not ipam_subnet:
-            LOG.error(_LE("IPAM subnet referenced to "
-                          "Neutron subnet %s does not exist"),
-                      neutron_subnet_id)
+            LOG.error("IPAM subnet referenced to "
+                      "Neutron subnet %s does not exist", neutron_subnet_id)
             raise n_exc.SubnetNotFound(subnet_id=neutron_subnet_id)
         pools = []
         for pool in ipam_subnet.allocation_pools:
             pools.append(netaddr.IPRange(pool['first_ip'], pool['last_ip']))
 
-        neutron_subnet = cls._fetch_subnet(ctx, neutron_subnet_id)
+        neutron_subnet_obj = cls._fetch_subnet(ctx, neutron_subnet_id)
 
         return cls(ipam_subnet['id'],
                    ctx,
-                   cidr=neutron_subnet['cidr'],
+                   cidr=neutron_subnet_obj.cidr,
                    allocation_pools=pools,
-                   gateway_ip=neutron_subnet['gateway_ip'],
-                   tenant_id=neutron_subnet['tenant_id'],
+                   gateway_ip=neutron_subnet_obj.gateway_ip,
+                   tenant_id=neutron_subnet_obj.tenant_id,
                    subnet_id=neutron_subnet_id)
 
     @classmethod
     def _fetch_subnet(cls, context, id):
-        plugin = manager.NeutronManager.get_plugin()
-        return plugin._get_subnet(context, id)
+        plugin = directory.get_plugin()
+        return plugin._get_subnet_object(context, id)
 
     def __init__(self, internal_id, ctx, cidr=None,
                  allocation_pools=None, gateway_ip=None, tenant_id=None,
@@ -132,15 +133,15 @@ class NeutronDbSubnet(ipam_base.Subnet):
                                                             self._subnet_id)
         self._context = ctx
 
-    def _verify_ip(self, session, ip_address):
+    def _verify_ip(self, context, ip_address):
         """Verify whether IP address can be allocated on subnet.
 
-        :param session: database session
+        :param context: neutron api request context
         :param ip_address: String representing the IP address to verify
         :raises: InvalidInput, IpAddressAlreadyAllocated
         """
         # Ensure that the IP's are unique
-        if not self.subnet_manager.check_unique_allocation(session,
+        if not self.subnet_manager.check_unique_allocation(context,
                                                            ip_address):
             raise ipam_exc.IpAddressAlreadyAllocated(
                 subnet_id=self.subnet_manager.neutron_id,
@@ -152,28 +153,76 @@ class NeutronDbSubnet(ipam_base.Subnet):
                 subnet_id=self.subnet_manager.neutron_id,
                 ip=ip_address)
 
-    def _generate_ip(self, session, prefer_next=False):
+    def _generate_ip(self, context, prefer_next=False):
         """Generate an IP address from the set of available addresses."""
-        ip_allocations = netaddr.IPSet()
-        for ipallocation in self.subnet_manager.list_allocations(session):
-            ip_allocations.add(netaddr.IPAddress(ipallocation.ip_address))
+        return self._generate_ips(context, prefer_next)[0]
 
-        for ip_pool in self.subnet_manager.list_pools(session):
+    def _generate_ips(self, context, prefer_next=False, num_addresses=1):
+        """Generate a set of IPs from the set of available addresses."""
+        allocated_ips = []
+        requested_num_addresses = num_addresses
+
+        allocations = self.subnet_manager.list_allocations(context)
+        # It is better not to use 'netaddr.IPSet.add',
+        # because _compact_single_network in 'IPSet.add'
+        # is quite time consuming.
+        ip_allocations = netaddr.IPSet(
+            [netaddr.IPAddress(allocation.ip_address)
+             for allocation in allocations])
+
+        for ip_pool in self.subnet_manager.list_pools(context):
             ip_set = netaddr.IPSet()
             ip_set.add(netaddr.IPRange(ip_pool.first_ip, ip_pool.last_ip))
             av_set = ip_set.difference(ip_allocations)
             if av_set.size == 0:
                 continue
 
-            if prefer_next:
-                window = 1
+            if av_set.size < requested_num_addresses:
+                # All addresses of the address pool are allocated
+                # for the first time and the remaining addresses
+                # will be allocated in the next address pools.
+                allocated_num_addresses = av_set.size
             else:
-                # Compute a value for the selection window
-                window = min(av_set.size, 10)
-            ip_index = random.randint(1, window)
-            candidate_ips = list(itertools.islice(av_set, ip_index))
-            allocated_ip = candidate_ips[-1]
-            return str(allocated_ip), ip_pool.id
+                # All expected addresses can be assigned in this loop.
+                allocated_num_addresses = requested_num_addresses
+
+            if prefer_next:
+                allocated_ip_pool = list(itertools.islice(
+                    av_set, allocated_num_addresses))
+                allocated_ips.extend([str(allocated_ip)
+                                      for allocated_ip in allocated_ip_pool])
+
+                requested_num_addresses -= allocated_num_addresses
+                if requested_num_addresses:
+                    # More addresses need to be allocated in the next loop.
+                    continue
+                return allocated_ips
+
+            window = min(av_set.size, MAX_WIN)
+
+            # NOTE(gryf): If there is more than one address, make the window
+            # bigger, so that are chances to fulfill demanded amount of IPs.
+            if allocated_num_addresses > 1:
+                window = min(av_set.size,
+                             allocated_num_addresses * MULTIPLIER,
+                             MAX_WIN_MULTI)
+
+            if window < allocated_num_addresses:
+                continue
+            # Maximize randomness by using the random module's built in
+            # sampling function
+            av_ips = list(itertools.islice(av_set, 0, window))
+            allocated_ip_pool = random.sample(av_ips,
+                                              allocated_num_addresses)
+            allocated_ips.extend([str(allocated_ip)
+                                  for allocated_ip in allocated_ip_pool])
+
+            requested_num_addresses -= allocated_num_addresses
+            if requested_num_addresses:
+                # More addresses need to be allocated in the next loop.
+                continue
+
+            return allocated_ips
 
         raise ipam_exc.IpAddressGenerationFailure(
                   subnet_id=self.subnet_manager.neutron_id)
@@ -183,8 +232,6 @@ class NeutronDbSubnet(ipam_base.Subnet):
         # running transaction, which is started on create_port or upper level.
         # To be able to do rollback/retry actions correctly ipam driver
         # should not create new nested transaction blocks.
-        session = self._context.session
-        all_pool_id = None
         # NOTE(salv-orlando): It would probably better to have a simpler
         # model for address requests and just check whether there is a
         # specific IP address specified in address_request
@@ -192,44 +239,65 @@ class NeutronDbSubnet(ipam_base.Subnet):
             # This handles both specific and automatic address requests
             # Check availability of requested IP
             ip_address = str(address_request.address)
-            self._verify_ip(session, ip_address)
+            self._verify_ip(self._context, ip_address)
         else:
             prefer_next = isinstance(address_request,
                                      ipam_req.PreferNextAddressRequest)
-            ip_address, all_pool_id = self._generate_ip(session, prefer_next)
+            ip_address = self._generate_ip(self._context, prefer_next)
 
         # Create IP allocation request object
         # The only defined status at this stage is 'ALLOCATED'.
         # More states will be available in the future - e.g.: RECYCLABLE
         try:
-            with session.begin(subtransactions=True):
+            # TODO(ataraday): revisit this after objects switched to
+            # new enginefacade
+            with self._context.session.begin(subtransactions=True):
                 # NOTE(kevinbenton): we use a subtransaction to force
                 # a flush here so we can capture DBReferenceErrors due
                 # to concurrent subnet deletions. (galera would deadlock
                 # later on final commit)
-                self.subnet_manager.create_allocation(session, ip_address)
+                self.subnet_manager.create_allocation(self._context,
+                                                      ip_address)
         except db_exc.DBReferenceError:
             raise n_exc.SubnetNotFound(
                 subnet_id=self.subnet_manager.neutron_id)
         return ip_address
 
+    def bulk_allocate(self, address_request):
+        # The signature of this function differs from allocate only in that it
+        # returns a list of addresses, as opposed to a single address.
+        if not isinstance(address_request, ipam_req.BulkAddressRequest):
+            return [self.allocate(address_request)]
+        num_addrs = address_request.num_addresses
+        allocated_ip_pool = self._generate_ips(self._context,
+                                               False,
+                                               num_addrs)
+        # Create IP allocation request objects
+        try:
+            with self._context.session.begin(subtransactions=True):
+                for ip_address in allocated_ip_pool:
+                    self.subnet_manager.create_allocation(self._context,
+                                                          ip_address)
+        except db_exc.DBReferenceError:
+            raise n_exc.SubnetNotFound(
+                subnet_id=self.subnet_manager.neutron_id)
+        return allocated_ip_pool
+
     def deallocate(self, address):
         # This is almost a no-op because the Neutron DB IPAM driver does not
-        # delete IPAllocation objects at every deallocation. The only operation
-        # it performs is to delete an IPRequest entry.
-        session = self._context.session
-
+        # delete IPAllocation objects at every deallocation. The only
+        # operation it performs is to delete an IPRequest entry.
         count = self.subnet_manager.delete_allocation(
-            session, address)
+            self._context, address)
         # count can hardly be greater than 1, but it can be 0...
         if not count:
             raise ipam_exc.IpAddressAllocationNotFound(
                 subnet_id=self.subnet_manager.neutron_id,
                 ip_address=address)
 
-    def _no_pool_changes(self, session, pools):
+    def _no_pool_changes(self, context, pools):
         """Check if pool updates in db are required."""
-        db_pools = self.subnet_manager.list_pools(session)
+        db_pools = self.subnet_manager.list_pools(context)
         iprange_pools = [netaddr.IPRange(pool.first_ip, pool.last_ip)
                          for pool in db_pools]
         return pools == iprange_pools
@@ -238,11 +306,11 @@ class NeutronDbSubnet(ipam_base.Subnet):
         # Pools have already been validated in the subnet request object which
         # was sent to the subnet pool driver. Further validation should not be
         # required.
-        session = self._context.session
-        if self._no_pool_changes(session, pools):
+        if self._no_pool_changes(self._context, pools):
             return
-        self.subnet_manager.delete_allocation_pools(session)
-        self.create_allocation_pools(self.subnet_manager, session, pools, cidr)
+        self.subnet_manager.delete_allocation_pools(self._context)
+        self.create_allocation_pools(self.subnet_manager, self._context, pools,
+                                     cidr)
         self._pools = pools
 
     def get_details(self):
@@ -313,12 +381,11 @@ class NeutronDbPool(subnet_alloc.SubnetAllocator):
         IPAM-related data has no foreign key relationships to neutron subnet,
         so removing ipam subnet manually
         """
-        count = ipam_db_api.IpamSubnetManager.delete(self._context.session,
+        count = ipam_db_api.IpamSubnetManager.delete(self._context,
                                                      subnet_id)
         if count < 1:
-            LOG.error(_LE("IPAM subnet referenced to "
-                          "Neutron subnet %s does not exist"),
-                      subnet_id)
+            LOG.error("IPAM subnet referenced to "
+                      "Neutron subnet %s does not exist", subnet_id)
             raise n_exc.SubnetNotFound(subnet_id=subnet_id)
 
     def needs_rollback(self):
