@@ -14,20 +14,24 @@
 
 import netaddr
 from neutron_lib import constants
+from neutron_lib.objects import common_types
 from neutron_lib.utils import net as net_utils
+from oslo_log import log as logging
 from oslo_utils import versionutils
 from oslo_versionedobjects import fields as obj_fields
 
+from neutron.common import _constants
 from neutron.db.models import dns as dns_models
 from neutron.db.models import l3
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import models_v2
 from neutron.objects import base
-from neutron.objects import common_types
 from neutron.objects.db import api as obj_db_api
 from neutron.objects.qos import binding
 from neutron.plugins.ml2 import models as ml2_models
 
+LOG = logging.getLogger(__name__)
+
 
 class PortBindingBase(base.NeutronDbObject):
 
@@ -211,6 +215,14 @@ class IPAllocation(base.NeutronDbObject):
         if alloc_db:
             return True
 
+    @classmethod
+    def delete_alloc_by_subnet_id(cls, context, subnet_id):
+        allocs = context.session.query(models_v2.IPAllocation).filter_by(
+            subnet_id=subnet_id).all()
+        for alloc in allocs:
+            alloc_obj = super(IPAllocation, cls)._load_object(context, alloc)
+            alloc_obj.delete()
+
 
 @base.NeutronObjectRegistry.register
 class PortDNS(base.NeutronDbObject):
@@ -236,11 +248,6 @@ class PortDNS(base.NeutronDbObject):
         'dns_domain': common_types.DomainNameField(),
     }
 
-    def obj_make_compatible(self, primitive, target_version):
-        _target_version = versionutils.convert_version_to_tuple(target_version)
-        if _target_version < (1, 1):
-            primitive.pop('dns_domain', None)
-
 
 @base.NeutronObjectRegistry.register
 class SecurityGroupPortBinding(base.NeutronDbObject):
@@ -408,18 +415,36 @@ class Port(base.NeutronDbObject):
                 kwargs['id'] = list(set(port_ids) & set(ports_with_sg))
             else:
                 kwargs['id'] = ports_with_sg
-        return super(Port, cls).get_objects(context, _pager, validate_filters,
-                                            **kwargs)
+        port_array = super(Port, cls).get_objects(context, _pager,
+                                                  validate_filters,
+                                                  **kwargs)
+        sg_count = len(security_group_ids) if security_group_ids else 0
+        LOG.debug("Time-cost: Fetching %(port_count)s ports in %(sg_count)s "
+                  "security groups",
+                  {'port_count': len(port_array),
+                   'sg_count': sg_count})
+        return port_array
 
     @classmethod
-    def get_port_ids_filter_by_segment_id(cls, context, segment_id):
+    def get_auto_deletable_port_ids_and_proper_port_count_by_segment(
+            cls, context, segment_id):
+
         query = context.session.query(models_v2.Port.id)
         query = query.join(
             ml2_models.PortBindingLevel,
             ml2_models.PortBindingLevel.port_id == models_v2.Port.id)
         query = query.filter(
             ml2_models.PortBindingLevel.segment_id == segment_id)
-        return [p.id for p in query]
+
+        q_delete = query.filter(
+            models_v2.Port.device_owner.in_(
+                _constants.AUTO_DELETE_PORT_OWNERS))
+
+        q_proper = query.filter(
+            ~models_v2.Port.device_owner.in_(
+                _constants.AUTO_DELETE_PORT_OWNERS))
+
+        return ([r.id for r in q_delete.all()], q_proper.count())
 
     @classmethod
     def modify_fields_to_db(cls, fields):
@@ -480,8 +505,6 @@ class Port(base.NeutronDbObject):
 
     def obj_make_compatible(self, primitive, target_version):
         _target_version = versionutils.convert_version_to_tuple(target_version)
-        if _target_version < (1, 1):
-            primitive.pop('data_plane_status', None)
         if _target_version < (1, 2):
             binding_levels = primitive.get('binding_levels', [])
             for lvl in binding_levels:
@@ -508,15 +531,65 @@ class Port(base.NeutronDbObject):
             primitive.pop('qos_network_policy_id', None)
 
     @classmethod
-    def get_ports_by_router(cls, context, router_id, owner, subnet):
-        rport_qry = context.session.query(models_v2.Port).join(
-            l3.RouterPort)
-        ports = rport_qry.filter(
-            l3.RouterPort.router_id == router_id,
-            l3.RouterPort.port_type == owner,
-            models_v2.Port.network_id == subnet['network_id']
-        )
-        return [cls._load_object(context, db_obj) for db_obj in ports.all()]
+    def get_ports_by_router_and_network(cls, context, router_id, owner,
+                                        network_id):
+        """Returns port objects filtering by router ID, owner and network ID"""
+        rports_filter = (models_v2.Port.network_id == network_id, )
+        router_filter = (models_v2.Port.network_id == network_id, )
+        return cls._get_ports_by_router(context, router_id, owner,
+                                        rports_filter, router_filter)
+
+    @classmethod
+    def get_ports_by_router_and_port(cls, context, router_id, owner, port_id):
+        """Returns port objects filtering by router ID, owner and port ID"""
+        rports_filter = (l3.RouterPort.port_id == port_id, )
+        router_filter = (models_v2.Port.id == port_id, )
+        return cls._get_ports_by_router(context, router_id, owner,
+                                        rports_filter, router_filter)
+
+    @classmethod
+    def _get_ports_by_router(cls, context, router_id, owner, rports_filter,
+                             router_filter):
+        """Returns port objects filtering by router id and owner
+
+        The method will receive extra filters depending of the caller (filter
+        by network or filter by port).
+
+        The ports are retrieved using:
+        - The RouterPort registers. Each time a port is assigned to a router,
+          a new RouterPort register is added to the DB.
+        - The port owner and device_id information.
+
+        Both searches should return the same result. If not, a warning message
+        is logged and the port list to be returned is completed with the
+        missing ones.
+        """
+        rports_filter += (l3.RouterPort.router_id == router_id,
+                          l3.RouterPort.port_type == owner)
+        router_filter += (models_v2.Port.device_id == router_id,
+                          models_v2.Port.device_owner == owner)
+
+        ports = context.session.query(models_v2.Port).join(
+            l3.RouterPort).filter(*rports_filter)
+        ports_rports = [cls._load_object(context, db_obj)
+                        for db_obj in ports.all()]
+
+        ports = context.session.query(models_v2.Port).filter(*router_filter)
+        ports_router = [cls._load_object(context, db_obj)
+                        for db_obj in ports.all()]
+
+        ports_rports_ids = {p.id for p in ports_rports}
+        ports_router_ids = {p.id for p in ports_router}
+        missing_port_ids = ports_router_ids - ports_rports_ids
+        if missing_port_ids:
+            LOG.warning('The following ports, assigned to router '
+                        '%(router_id)s, do not have a "routerport" register: '
+                        '%(port_ids)s', {'router_id': router_id,
+                                         'port_ids': missing_port_ids})
+            port_objs = [p for p in ports_router if p.id in missing_port_ids]
+            ports_rports += port_objs
+
+        return ports_rports
 
     @classmethod
     def get_ports_ids_by_security_groups(cls, context, security_group_ids,
@@ -531,6 +604,14 @@ class Port(base.NeutronDbObject):
                 ~models_v2.Port.device_owner.in_(excluded_device_owners))
         return [port_binding['port_id'] for port_binding in query.all()]
 
+    @classmethod
+    def get_ports_by_host(cls, context, host):
+        query = context.session.query(models_v2.Port.id).join(
+            ml2_models.PortBinding)
+        query = query.filter(
+            ml2_models.PortBinding.host == host)
+        return [port_id[0] for port_id in query.all()]
+
     @classmethod
     def get_ports_by_binding_type_and_host(cls, context,
                                            binding_type, host):
@@ -574,3 +655,10 @@ class Port(base.NeutronDbObject):
             query = query.filter(
                 ml2_models.PortBinding.vif_type.in_(binding_types))
         return bool(query.count())
+
+    @classmethod
+    def get_ports_allocated_by_subnet_id(cls, context, subnet_id):
+        """Return ports with fixed IPs in a subnet"""
+        return context.session.query(models_v2.Port).filter(
+            models_v2.IPAllocation.port_id == models_v2.Port.id).filter(
+            models_v2.IPAllocation.subnet_id == subnet_id).all()
