@@ -17,16 +17,15 @@ import abc
 import sys
 
 from neutron_lib import constants
+from neutron_lib.objects import common_types
 from neutron_lib.services.qos import constants as qos_consts
 from neutron_lib.utils import helpers
 from oslo_utils import versionutils
 from oslo_versionedobjects import exception
 from oslo_versionedobjects import fields as obj_fields
-import six
 
 from neutron.db.qos import models as qos_db_model
 from neutron.objects import base
-from neutron.objects import common_types
 
 DSCP_MARK = 'dscp_mark'
 
@@ -43,16 +42,15 @@ def get_rules(obj_cls, context, qos_policy_id):
     return all_rules
 
 
-@six.add_metaclass(abc.ABCMeta)
-class QosRule(base.NeutronDbObject):
+class QosRule(base.NeutronDbObject, metaclass=abc.ABCMeta):
     # Version 1.0: Initial version, only BandwidthLimitRule
     #         1.1: Added DscpMarkingRule
     #         1.2: Added QosMinimumBandwidthRule
     #         1.3: Added direction for BandwidthLimitRule
     #
-    #NOTE(mangelajo): versions need to be handled from the top QosRule object
-    #                 because it's the only reference QosPolicy can make
-    #                 to them via obj_relationships version map
+    # NOTE(mangelajo): versions need to be handled from the top QosRule object
+    #                  because it's the only reference QosPolicy can make
+    #                  to them via obj_relationships version map
     VERSION = '1.3'
 
     fields = {
@@ -65,6 +63,25 @@ class QosRule(base.NeutronDbObject):
     # should be redefined in subclasses
     rule_type = None
 
+    duplicates_compare_fields = ()
+
+    def duplicates(self, other_rule):
+        """Returns True if rules have got same values in fields defined in
+        'duplicates_compare_fields' list.
+
+        In case when subclass don't have defined any field in
+        duplicates_compare_fields, only rule types are compared.
+        """
+
+        if self.rule_type != other_rule.rule_type:
+            return False
+
+        if self.duplicates_compare_fields:
+            for field in self.duplicates_compare_fields:
+                if getattr(self, field) != getattr(other_rule, field):
+                    return False
+        return True
+
     def to_dict(self):
         dict_ = super(QosRule, self).to_dict()
         dict_['type'] = self.rule_type
@@ -94,8 +111,15 @@ class QosRule(base.NeutronDbObject):
         #    - Is a port QoS policy (not a network QoS policy)
         #    - Is not an internal network device (e.g. router) and is a network
         #      QoS policy and there is no port QoS policy
-        return (is_port_policy or ((is_router_gw or not is_network_device_port)
-                                   and is_network_policy_only))
+        return (is_port_policy or
+                ((is_router_gw or not is_network_device_port) and
+                 is_network_policy_only))
+
+    def obj_make_compatible(self, primitive, target_version):
+        _target_version = versionutils.convert_version_to_tuple(target_version)
+        if _target_version < (1, 3):
+            raise exception.IncompatibleObjectVersion(
+                objver=target_version, objtype=self.__class__.__name__)
 
 
 @base.NeutronObjectRegistry.register
@@ -110,16 +134,9 @@ class QosBandwidthLimitRule(QosRule):
             default=constants.EGRESS_DIRECTION)
     }
 
-    rule_type = qos_consts.RULE_TYPE_BANDWIDTH_LIMIT
+    duplicates_compare_fields = ['direction']
 
-    def obj_make_compatible(self, primitive, target_version):
-        _target_version = versionutils.convert_version_to_tuple(target_version)
-        if _target_version < (1, 3) and 'direction' in primitive:
-            direction = primitive.pop('direction')
-            if direction == constants.INGRESS_DIRECTION:
-                raise exception.IncompatibleObjectVersion(
-                    objver=target_version,
-                    objtype="QosBandwidthLimitRule")
+    rule_type = qos_consts.RULE_TYPE_BANDWIDTH_LIMIT
 
 
 @base.NeutronObjectRegistry.register
@@ -133,13 +150,6 @@ class QosDscpMarkingRule(QosRule):
 
     rule_type = qos_consts.RULE_TYPE_DSCP_MARKING
 
-    def obj_make_compatible(self, primitive, target_version):
-        _target_version = versionutils.convert_version_to_tuple(target_version)
-        if _target_version < (1, 1):
-            raise exception.IncompatibleObjectVersion(
-                                 objver=target_version,
-                                 objname="QosDscpMarkingRule")
-
 
 @base.NeutronObjectRegistry.register
 class QosMinimumBandwidthRule(QosRule):
@@ -151,11 +161,6 @@ class QosMinimumBandwidthRule(QosRule):
         'direction': common_types.FlowDirectionEnumField(),
     }
 
-    rule_type = qos_consts.RULE_TYPE_MINIMUM_BANDWIDTH
+    duplicates_compare_fields = ['direction']
 
-    def obj_make_compatible(self, primitive, target_version):
-        _target_version = versionutils.convert_version_to_tuple(target_version)
-        if _target_version < (1, 2):
-            raise exception.IncompatibleObjectVersion(
-                                 objver=target_version,
-                                 objname="QosMinimumBandwidthRule")
+    rule_type = qos_consts.RULE_TYPE_MINIMUM_BANDWIDTH
