@@ -13,18 +13,49 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import api
 from oslo_log import log
 from oslo_serialization import jsonutils
+import sqlalchemy
 
-from neutron._i18n import _LW
 from neutron.db import segments_db
-from neutron.extensions import portbindings
-from neutron.plugins.ml2 import driver_api as api
 
 LOG = log.getLogger(__name__)
 
 
+class InstanceSnapshot(object):
+    """Used to avoid holding references to DB objects in PortContext."""
+    def __init__(self, obj):
+        self._model_class = obj.__class__
+        self._identity_key = sqlalchemy.orm.util.identity_key(instance=obj)[1]
+        self._cols = [col.key
+                      for col in sqlalchemy.inspect(self._model_class).columns]
+        for col in self._cols:
+            setattr(self, col, getattr(obj, col))
+
+    def persist_state_to_session(self, session):
+        """Updates the state of the snapshot in the session.
+
+        Finds the SQLA object in the session if it exists or creates a new
+        object and updates the object with the column values stored in this
+        snapshot.
+        """
+        db_obj = session.query(self._model_class).get(self._identity_key)
+        if db_obj:
+            for col in self._cols:
+                setattr(db_obj, col, getattr(self, col))
+        else:
+            session.add(self._model_class(**{col: getattr(self, col)
+                                             for col in self._cols}))
+
+    def __getitem__(self, item):
+        if item not in self._cols:
+            raise KeyError(item)
+        return getattr(self, item)
+
+
 class MechanismDriverContext(object):
     """MechanismDriver context base class."""
     def __init__(self, plugin, plugin_context):
@@ -38,12 +69,12 @@ class MechanismDriverContext(object):
 class NetworkContext(MechanismDriverContext, api.NetworkContext):
 
     def __init__(self, plugin, plugin_context, network,
-                 original_network=None):
+                 original_network=None, segments=None):
         super(NetworkContext, self).__init__(plugin, plugin_context)
         self._network = network
         self._original_network = original_network
         self._segments = segments_db.get_network_segments(
-            plugin_context.session, network['id'])
+            plugin_context, network['id']) if segments is None else segments
 
     @property
     def current(self):
@@ -66,7 +97,7 @@ class SubnetContext(MechanismDriverContext, api.SubnetContext):
         self._subnet = subnet
         self._original_subnet = original_subnet
         self._network_context = NetworkContext(plugin, plugin_context,
-                                               network)
+                                               network) if network else None
 
     @property
     def current(self):
@@ -78,6 +109,11 @@ class SubnetContext(MechanismDriverContext, api.SubnetContext):
 
     @property
     def network(self):
+        if self._network_context is None:
+            network = self._plugin.get_network(
+                self._plugin_context, self.current['network_id'])
+            self._network_context = NetworkContext(
+                self._plugin, self._plugin_context, network)
         return self._network_context
 
 
@@ -88,10 +124,15 @@ class PortContext(MechanismDriverContext, api.PortContext):
         super(PortContext, self).__init__(plugin, plugin_context)
         self._port = port
         self._original_port = original_port
-        self._network_context = NetworkContext(plugin, plugin_context,
-                                               network)
-        self._binding = binding
-        self._binding_levels = binding_levels
+        if isinstance(network, NetworkContext):
+            self._network_context = network
+        else:
+            self._network_context = NetworkContext(
+                plugin, plugin_context, network) if network else None
+        # NOTE(kevinbenton): InstanceSnapshot can go away once we are working
+        # with OVO objects instead of native SQLA objects.
+        self._binding = InstanceSnapshot(binding)
+        self._binding_levels = binding_levels or []
         self._segments_to_bind = None
         self._new_bound_segment = None
         self._next_segments_to_bind = None
@@ -117,6 +158,8 @@ class PortContext(MechanismDriverContext, api.PortContext):
         self._binding_levels = []
 
     def _push_binding_level(self, binding_level):
+        # NOTE(slaweq): binding_level should be always OVO with no reference
+        # to DB object
         self._binding_levels.append(binding_level)
 
     def _pop_binding_level(self):
@@ -151,6 +194,11 @@ class PortContext(MechanismDriverContext, api.PortContext):
 
     @property
     def network(self):
+        if not self._network_context:
+            network = self._plugin.get_network(
+                self._plugin_context, self.current['network_id'])
+            self._network_context = NetworkContext(
+                self._plugin, self._plugin_context, network)
         return self._network_context
 
     @property
@@ -192,10 +240,16 @@ class PortContext(MechanismDriverContext, api.PortContext):
                 self._original_binding_levels[-1].segment_id)
 
     def _expand_segment(self, segment_id):
-        segment = segments_db.get_segment_by_id(self._plugin_context.session,
+        for s in self.network.network_segments:
+            if s['id'] == segment_id:
+                return s
+        # TODO(kevinbenton): eliminate the query below. The above should
+        # always return since the port is bound to a network segment. Leaving
+        # in for now for minimally invasive change for back-port.
+        segment = segments_db.get_segment_by_id(self._plugin_context,
                                                 segment_id)
         if not segment:
-            LOG.warning(_LW("Could not expand segment %s"), segment_id)
+            LOG.warning("Could not expand segment %s", segment_id)
         return segment
 
     @property
@@ -250,6 +304,16 @@ class PortContext(MechanismDriverContext, api.PortContext):
         self._binding.vif_details = jsonutils.dumps(vif_details)
         self._new_port_status = status
 
+    def _unset_binding(self):
+        '''Undo a previous call to set_binding() before it gets committed.
+
+        This method is for MechanismManager and is not part of the driver API.
+        '''
+        self._new_bound_segment = None
+        self._binding.vif_type = portbindings.VIF_TYPE_UNBOUND
+        self._binding.vif_details = ''
+        self._new_port_status = None
+
     def continue_binding(self, segment_id, next_segments_to_bind):
         # TODO(rkukura) Verify binding allowed, segment in network
         self._new_bound_segment = segment_id
@@ -263,4 +327,4 @@ class PortContext(MechanismDriverContext, api.PortContext):
 
     def release_dynamic_segment(self, segment_id):
         return self._plugin.type_manager.release_dynamic_segment(
-                self._plugin_context.session, segment_id)
+                self._plugin_context, segment_id)
