@@ -15,8 +15,8 @@
 #    under the License.
 
 from neutron_lib import constants as n_const
+from neutron_lib.plugins import utils as plugin_utils
 
-from neutron.plugins.common import utils as p_utils
 
 MAX_VLAN_POSTFIX_LEN = 5
 
@@ -25,7 +25,6 @@ def get_vlan_device_name(src_dev, vlan):
     """Generating the vlan device name."""
 
     # Ensure that independent of the vlan len the same name prefix is used.
-    src_dev = p_utils.get_interface_name(src_dev,
-                                         max_len=n_const.DEVICE_NAME_MAX_LEN -
-                                         MAX_VLAN_POSTFIX_LEN)
+    src_dev = plugin_utils.get_interface_name(
+        src_dev, max_len=n_const.DEVICE_NAME_MAX_LEN - MAX_VLAN_POSTFIX_LEN)
     return "%s.%s" % (src_dev, vlan)
