@@ -13,15 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import glob
 import os
 import re
 
+from neutron_lib.utils import helpers
 from oslo_log import log as logging
-import six
 
-from neutron._i18n import _, _LE, _LW
-from neutron.agent.linux import ip_link_support
-from neutron.common import utils
+from neutron._i18n import _
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common \
     import exceptions as exc
 from neutron.plugins.ml2.drivers.mech_sriov.agent import pci_lib
@@ -29,13 +28,23 @@ from neutron.plugins.ml2.drivers.mech_sriov.agent import pci_lib
 LOG = logging.getLogger(__name__)
 
 
+IP_LINK_CAPABILITY_STATE = 'state'
+IP_LINK_CAPABILITY_VLAN = 'vlan'
+IP_LINK_CAPABILITY_RATE = 'max_tx_rate'
+IP_LINK_CAPABILITY_MIN_TX_RATE = 'min_tx_rate'
+IP_LINK_CAPABILITY_SPOOFCHK = 'spoofchk'
+IP_LINK_SUB_CAPABILITY_QOS = 'qos'
+
+
 class PciOsWrapper(object):
     """OS wrapper for checking virtual functions"""
 
     DEVICE_PATH = "/sys/class/net/%s/device"
     PCI_PATH = "/sys/class/net/%s/device/virtfn%s/net"
+    NUMVFS_PATH = "/sys/class/net/%s/device/sriov_numvfs"
     VIRTFN_FORMAT = r"^virtfn(?P<vf_index>\d+)"
     VIRTFN_REG_EX = re.compile(VIRTFN_FORMAT)
+    MAC_VTAP_PREFIX = "upper_macvtap*"
 
     @classmethod
     def scan_vf_devices(cls, dev_name):
@@ -47,7 +56,7 @@ class PciOsWrapper(object):
         vf_list = []
         dev_path = cls.DEVICE_PATH % dev_name
         if not os.path.isdir(dev_path):
-            LOG.error(_LE("Failed to get devices for %s"), dev_name)
+            LOG.error("Failed to get devices for %s", dev_name)
             raise exc.InvalidDeviceError(dev_name=dev_name,
                                          reason=_("Device not found"))
         file_list = os.listdir(dev_path)
@@ -63,32 +72,69 @@ class PciOsWrapper(object):
         return vf_list
 
     @classmethod
-    def is_assigned_vf(cls, dev_name, vf_index):
+    def pf_device_exists(cls, dev_name):
+        return os.path.isdir(cls.DEVICE_PATH % dev_name)
+
+    @classmethod
+    def is_assigned_vf_direct(cls, dev_name, vf_index):
         """Check if VF is assigned.
 
         Checks if a given vf index of a given device name is assigned
-        by checking the relevant path in the system:
+        as PCI passthrough by checking the relevant path in the system:
         VF is assigned if:
             Direct VF: PCI_PATH does not exist.
-            Macvtap VF: macvtap@<vf interface> interface exists in ip link show
         @param dev_name: pf network device name
         @param vf_index: vf index
+        @return: True if VF is assigned, False otherwise
         """
         path = cls.PCI_PATH % (dev_name, vf_index)
+        return not os.path.isdir(path)
 
+    @classmethod
+    def get_vf_macvtap_upper_devs(cls, dev_name, vf_index):
+        """Retrieve VF netdev upper (macvtap) devices.
+
+        @param dev_name: pf network device name
+        @param vf_index: vf index
+        @return: list of upper net devices associated with the VF
+        """
+        path = cls.PCI_PATH % (dev_name, vf_index)
+        upper_macvtap_path = os.path.join(path, "*", cls.MAC_VTAP_PREFIX)
+        devs = [os.path.basename(dev) for dev in glob.glob(upper_macvtap_path)]
+        # file name is in the format of upper_<netdev_name> extract netdev name
+        return [dev.split('_')[1] for dev in devs]
+
+    @classmethod
+    def is_assigned_vf_macvtap(cls, dev_name, vf_index):
+        """Check if VF is assigned.
+
+        Checks if a given vf index of a given device name is assigned
+        as macvtap by checking the relevant path in the system:
+            Macvtap VF: upper_macvtap path exists.
+        @param dev_name: pf network device name
+        @param vf_index: vf index
+        @return: True if VF is assigned, False otherwise
+        """
+        return bool(cls.get_vf_macvtap_upper_devs(dev_name, vf_index))
+
+    @classmethod
+    def get_numvfs(cls, dev_name):
+        """Get configured number of VFs on device
+
+        @param dev_name: pf network device name
+        @return: integer number of VFs or -1
+        if sriov_numvfs file not found (device doesn't support this config)
+        """
         try:
-            ifname_list = os.listdir(path)
-        except OSError:
-            # PCI_PATH does not exist means that the DIRECT VF assigend
-            return True
-
-        # Note(moshele) kernel < 3.13 doesn't create symbolic link
-        # for macvtap interface. Therefore we workaround it
-        # by parsing ip link show and checking if macvtap interface exists
-        for ifname in ifname_list:
-            if pci_lib.PciDeviceIPWrapper.is_macvtap_assigned(ifname):
-                return True
-        return False
+            with open(cls.NUMVFS_PATH % dev_name) as f:
+                numvfs = int(f.read())
+                LOG.debug("Number of VFs configured on device %s: %s",
+                    dev_name, numvfs)
+                return numvfs
+        except IOError:
+            LOG.warning("Error reading sriov_numvfs file for device %s, "
+                        "probably not supported by this device", dev_name)
+            return -1
 
 
 class EmbSwitch(object):
@@ -102,16 +148,15 @@ class EmbSwitch(object):
     @ivar pci_dev_wrapper: pci device wrapper
     """
 
-    def __init__(self, phys_net, dev_name, exclude_devices):
+    def __init__(self, dev_name, exclude_devices):
         """Constructor
 
-        @param phys_net: physical network
         @param dev_name: network device name
         @param exclude_devices: list of pci slots to exclude
         """
-        self.phys_net = phys_net
         self.dev_name = dev_name
         self.pci_slot_map = {}
+        self.scanned_pci_list = []
         self.pci_dev_wrapper = pci_lib.PciDeviceIPWrapper(dev_name)
 
         self._load_devices(exclude_devices)
@@ -121,8 +166,8 @@ class EmbSwitch(object):
 
         @param exclude_devices: excluded devices mapping device_name: pci slots
         """
-        scanned_pci_list = PciOsWrapper.scan_vf_devices(self.dev_name)
-        for pci_slot, vf_index in scanned_pci_list:
+        self.scanned_pci_list = PciOsWrapper.scan_vf_devices(self.dev_name)
+        for pci_slot, vf_index in self.scanned_pci_list:
             if pci_slot not in exclude_devices:
                 self.pci_slot_map[pci_slot] = vf_index
 
@@ -136,17 +181,10 @@ class EmbSwitch(object):
 
         @return: list of VF pair (mac address, pci slot)
         """
-        vf_to_pci_slot_mapping = {}
         assigned_devices_info = []
         for pci_slot, vf_index in self.pci_slot_map.items():
-            if not PciOsWrapper.is_assigned_vf(self.dev_name, vf_index):
-                continue
-            vf_to_pci_slot_mapping[vf_index] = pci_slot
-        if vf_to_pci_slot_mapping:
-            vf_to_mac_mapping = self.pci_dev_wrapper.get_assigned_macs(
-                list(vf_to_pci_slot_mapping.keys()))
-            for vf_index, mac in vf_to_mac_mapping.items():
-                pci_slot = vf_to_pci_slot_mapping[vf_index]
+            mac = self.get_pci_device(pci_slot)
+            if mac:
                 assigned_devices_info.append((mac, pci_slot))
         return assigned_devices_info
 
@@ -158,32 +196,33 @@ class EmbSwitch(object):
         vf_index = self._get_vf_index(pci_slot)
         return self.pci_dev_wrapper.get_vf_state(vf_index)
 
-    def set_device_state(self, pci_slot, state):
+    def set_device_state(self, pci_slot, state, propagate_uplink_state):
         """Set device state.
 
         @param pci_slot: Virtual Function address
         @param state: link state
         """
         vf_index = self._get_vf_index(pci_slot)
-        return self.pci_dev_wrapper.set_vf_state(vf_index, state)
+        return self.pci_dev_wrapper.set_vf_state(vf_index, state,
+                                                 auto=propagate_uplink_state)
 
     def set_device_rate(self, pci_slot, rate_type, rate_kbps):
-        """Set device rate: rate (max_tx_rate), min_tx_rate
+        """Set device rate: max_tx_rate, min_tx_rate
 
         @param pci_slot: Virtual Function address
-        @param rate_type: device rate name type. Could be 'rate' and
-                          'min_tx_rate'.
+        @param rate_type: device rate name type. Could be 'max_tx_rate' and
+                          'min_tx_rate' ('rate' is not supported anymore).
         @param rate_kbps: device rate in kbps
         """
         vf_index = self._get_vf_index(pci_slot)
-        #NOTE(ralonsoh): ip link sets rate in Mbps therefore we need to convert
-        #the rate_kbps value from kbps to Mbps.
-        #Zero means to disable the rate so the lowest rate available is 1Mbps.
-        #Floating numbers are not allowed
-        if rate_kbps > 0 and rate_kbps < 1000:
+        # NOTE(ralonsoh): ip link sets rate in Mbps therefore we need to
+        # convert the rate_kbps value from kbps to Mbps.
+        # Zero means to disable the rate so the lowest rate available is 1Mbps.
+        # Floating numbers are not allowed
+        if 0 < rate_kbps < 1000:
             rate_mbps = 1
         else:
-            rate_mbps = utils.round_val(rate_kbps / 1000.0)
+            rate_mbps = helpers.round_val(rate_kbps / 1000.0)
 
         log_dict = {
             'rate_mbps': rate_mbps,
@@ -205,7 +244,7 @@ class EmbSwitch(object):
     def _get_vf_index(self, pci_slot):
         vf_index = self.pci_slot_map.get(pci_slot)
         if vf_index is None:
-            LOG.warning(_LW("Cannot find vf index for pci slot %s"),
+            LOG.warning("Cannot find vf index for pci slot %s",
                         pci_slot)
             raise exc.InvalidPciSlotError(pci_slot=pci_slot)
         return vf_index
@@ -221,18 +260,45 @@ class EmbSwitch(object):
             raise exc.InvalidPciSlotError(pci_slot=pci_slot)
         return self.pci_dev_wrapper.set_vf_spoofcheck(vf_index, enabled)
 
+    def _get_macvtap_mac(self, vf_index):
+        upperdevs = PciOsWrapper.get_vf_macvtap_upper_devs(
+            self.dev_name, vf_index)
+        # NOTE(adrianc) although there can be many macvtap upper
+        # devices, we expect to have excatly one.
+        if len(upperdevs) > 1:
+            LOG.warning("Found more than one macvtap upper device for PF "
+                        "%(pf)s with VF index %(vf_index)s.",
+                        {"pf": self.dev_name, "vf_index": vf_index})
+        upperdev = upperdevs[0]
+        return pci_lib.PciDeviceIPWrapper(
+            upperdev).device(upperdev).link.address
+
     def get_pci_device(self, pci_slot):
         """Get mac address for given Virtual Function address
 
         @param pci_slot: pci slot
         @return: MAC address of virtual function
         """
+        if not PciOsWrapper.pf_device_exists(self.dev_name):
+            # If the root PCI path does not exist, then the VF cannot
+            # actually have been allocated and there is no way we can
+            # manage it.
+            return None
+
         vf_index = self.pci_slot_map.get(pci_slot)
         mac = None
+
         if vf_index is not None:
-            if PciOsWrapper.is_assigned_vf(self.dev_name, vf_index):
+            # NOTE(adrianc) for VF passthrough take administrative mac from PF
+            # netdevice, for macvtap take mac directly from macvtap interface.
+            # This is done to avoid relying on hypervisor [lack of] logic to
+            # keep effective and administrative mac in sync.
+            if PciOsWrapper.is_assigned_vf_direct(self.dev_name, vf_index):
                 macs = self.pci_dev_wrapper.get_assigned_macs([vf_index])
                 mac = macs.get(vf_index)
+            elif PciOsWrapper.is_assigned_vf_macvtap(
+                    self.dev_name, vf_index):
+                mac = self._get_macvtap_mac(vf_index)
         return mac
 
 
@@ -245,6 +311,7 @@ class ESwitchManager(object):
             cls._instance = super(ESwitchManager, cls).__new__(cls)
             cls.emb_switches_map = {}
             cls.pci_slot_map = {}
+            cls.skipped_devices = set()
         return cls._instance
 
     def device_exists(self, device_mac, pci_slot):
@@ -281,15 +348,15 @@ class ESwitchManager(object):
     def get_device_state(self, device_mac, pci_slot):
         """Get device state.
 
-        Get the device state (up/True or down/False)
+        Get the device state (up/enable, down/disable, or auto)
         @param device_mac: device mac
         @param pci_slot: VF PCI slot
-        @return: device state (True/False) None if failed
+        @return: device state (enable/disable/auto) None if failed
         """
         embedded_switch = self._get_emb_eswitch(device_mac, pci_slot)
         if embedded_switch:
             return embedded_switch.get_device_state(pci_slot)
-        return False
+        return pci_lib.LinkState.disable.name
 
     def set_device_max_rate(self, device_mac, pci_slot, max_kbps):
         """Set device max rate
@@ -302,9 +369,7 @@ class ESwitchManager(object):
         embedded_switch = self._get_emb_eswitch(device_mac, pci_slot)
         if embedded_switch:
             embedded_switch.set_device_rate(
-                pci_slot,
-                ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_RATE,
-                max_kbps)
+                pci_slot, IP_LINK_CAPABILITY_RATE, max_kbps)
 
     def set_device_min_tx_rate(self, device_mac, pci_slot, min_kbps):
         """Set device min_tx_rate
@@ -317,22 +382,23 @@ class ESwitchManager(object):
         embedded_switch = self._get_emb_eswitch(device_mac, pci_slot)
         if embedded_switch:
             embedded_switch.set_device_rate(
-                pci_slot,
-                ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_MIN_TX_RATE,
-                min_kbps)
+                pci_slot, IP_LINK_CAPABILITY_MIN_TX_RATE, min_kbps)
 
-    def set_device_state(self, device_mac, pci_slot, admin_state_up):
+    def set_device_state(self, device_mac, pci_slot, admin_state_up,
+                         propagate_uplink_state):
         """Set device state
 
         Sets the device state (up or down)
         @param device_mac: device mac
         @param pci_slot: pci slot
         @param admin_state_up: device admin state True/False
+        @param propagate_uplink_state: follow uplink state True/False
         """
         embedded_switch = self._get_emb_eswitch(device_mac, pci_slot)
         if embedded_switch:
             embedded_switch.set_device_state(pci_slot,
-                                             admin_state_up)
+                                             admin_state_up,
+                                             propagate_uplink_state)
 
     def set_device_spoofcheck(self, device_mac, pci_slot, enabled):
         """Set device spoofcheck
@@ -347,6 +413,27 @@ class ESwitchManager(object):
             embedded_switch.set_device_spoofcheck(pci_slot,
                                                   enabled)
 
+    def _process_emb_switch_map(self, phys_net, dev_name, exclude_devices):
+        """Process emb_switch_map
+        @param phys_net: physical network
+        @param dev_name: device name
+        @param exclude_devices: PCI devices to ignore.
+        """
+        emb_switches = self.emb_switches_map.get(phys_net, [])
+        for switch in emb_switches:
+            if switch.dev_name == dev_name:
+                if not PciOsWrapper.pf_device_exists(dev_name):
+                    # If the device is given to the VM as PCI-PT
+                    # then delete the respective emb_switch from map
+                    self.emb_switches_map.get(phys_net).remove(switch)
+                return
+
+        # We don't know about this device at the moment, so add to the map.
+        if PciOsWrapper.pf_device_exists(dev_name):
+            self._create_emb_switch(
+                phys_net, dev_name,
+                exclude_devices.get(dev_name, set()))
+
     def discover_devices(self, device_mappings, exclude_devices):
         """Discover which Virtual functions to manage.
 
@@ -356,16 +443,40 @@ class ESwitchManager(object):
         """
         if exclude_devices is None:
             exclude_devices = {}
-        for phys_net, dev_names in six.iteritems(device_mappings):
+        for phys_net, dev_names in device_mappings.items():
             for dev_name in dev_names:
-                self._create_emb_switch(phys_net, dev_name,
-                                        exclude_devices.get(dev_name, set()))
+                self._process_emb_switch_map(phys_net, dev_name,
+                                             exclude_devices)
 
     def _create_emb_switch(self, phys_net, dev_name, exclude_devices):
-        embedded_switch = EmbSwitch(phys_net, dev_name, exclude_devices)
+        embedded_switch = EmbSwitch(dev_name, exclude_devices)
+        numvfs = PciOsWrapper.get_numvfs(dev_name)
+        if numvfs == 0:
+            # numvfs might be 0 on pre-up state of a device
+            # giving such devices one more chance to initialize
+            if dev_name not in self.skipped_devices:
+                self.skipped_devices.add(dev_name)
+                LOG.info("Device %s has 0 VFs configured. Skipping "
+                         "for now to let the device initialize", dev_name)
+                return
+            else:
+                # looks like device indeed has 0 VFs configured
+                # it is probably used just as direct-physical
+                LOG.info("Device %s has 0 VFs configured", dev_name)
+
+        numvfs_cur = len(embedded_switch.scanned_pci_list)
+        if numvfs >= 0 and numvfs > numvfs_cur:
+            LOG.info("Not all VFs were initialized on device %(device)s: "
+                     "expected - %(expected)s, actual - %(actual)s. Skipping.",
+                     {'device': dev_name, 'expected': numvfs,
+                      'actual': numvfs_cur})
+            self.skipped_devices.add(dev_name)
+            return
+
         self.emb_switches_map.setdefault(phys_net, []).append(embedded_switch)
         for pci_slot in embedded_switch.get_pci_slot_list():
             self.pci_slot_map[pci_slot] = embedded_switch
+        self.skipped_devices.discard(dev_name)
 
     def _get_emb_eswitch(self, device_mac, pci_slot):
         """Get embedded switch.
@@ -378,35 +489,51 @@ class ESwitchManager(object):
         if embedded_switch:
             used_device_mac = embedded_switch.get_pci_device(pci_slot)
             if used_device_mac != device_mac:
-                LOG.warning(_LW("device pci mismatch: %(device_mac)s "
-                                "- %(pci_slot)s"),
+                LOG.warning("device pci mismatch: %(device_mac)s "
+                            "- %(pci_slot)s",
                             {"device_mac": device_mac, "pci_slot": pci_slot})
                 embedded_switch = None
         return embedded_switch
 
-    def clear_rate(self, pci_slot, rate_type):
-        """Clear the VF rate
+    def clear_max_rate(self, pci_slot):
+        """Clear the VF "rate" parameter
+
+        Clear the "rate" configuration from VF by setting it to 0.
+        @param pci_slot: VF PCI slot
+        """
+        self._clear_rate(pci_slot, IP_LINK_CAPABILITY_RATE)
+
+    def clear_min_tx_rate(self, pci_slot):
+        """Clear the VF "min_tx_rate" parameter
+
+        Clear the "min_tx_rate" configuration from VF by setting it to 0.
+        @param pci_slot: VF PCI slot
+        """
+        self._clear_rate(pci_slot, IP_LINK_CAPABILITY_MIN_TX_RATE)
+
+    def _clear_rate(self, pci_slot, rate_type):
+        """Clear the VF rate parameter specified in rate_type
 
         Clear the rate configuration from VF by setting it to 0.
         @param pci_slot: VF PCI slot
-        @param rate_type: rate to clear ('rate', 'min_tx_rate')
+        @param rate_type: rate to clear ('max_tx_rate', 'min_tx_rate')
         """
-        #NOTE(Moshe Levi): we don't use the self._get_emb_eswitch here, because
-        #when clearing the VF it may be not assigned. This happens when
-        #libvirt releases the VF back to the hypervisor on delete VM. Therefore
-        #we should just clear the VF rate according to pci_slot no matter
-        #if VF is assigned or not.
+        # NOTE(Moshe Levi): we don't use the self._get_emb_eswitch here,
+        # because when clearing the VF it may be not assigned. This happens
+        # when libvirt releases the VF back to the hypervisor on delete VM.
+        # Therefore we should just clear the VF rate according to pci_slot no
+        # matter if VF is assigned or not.
         embedded_switch = self.pci_slot_map.get(pci_slot)
         if embedded_switch:
-            #NOTE(Moshe Levi): check the pci_slot is not assigned to some
-            #other port before resetting the rate.
+            # NOTE(Moshe Levi): check the pci_slot is not assigned to some
+            # other port before resetting the rate.
             if embedded_switch.get_pci_device(pci_slot) is None:
                 embedded_switch.set_device_rate(pci_slot, rate_type, 0)
             else:
-                LOG.warning(_LW("VF with PCI slot %(pci_slot)s is already "
-                                "assigned; skipping reset for '%(rate_type)s' "
-                                "device configuration parameter"),
+                LOG.warning("VF with PCI slot %(pci_slot)s is already "
+                            "assigned; skipping reset for '%(rate_type)s' "
+                            "device configuration parameter",
                             {'pci_slot': pci_slot, 'rate_type': rate_type})
         else:
-            LOG.error(_LE("PCI slot %(pci_slot)s has no mapping to Embedded "
-                          "Switch; skipping"), {'pci_slot': pci_slot})
+            LOG.error("PCI slot %(pci_slot)s has no mapping to Embedded "
+                      "Switch; skipping", {'pci_slot': pci_slot})
