@@ -31,22 +31,25 @@ from oslo_log import log as logging
 import oslo_messaging
 from oslo_service import loopingcall
 from osprofiler import profiler
-import six
+import pyroute2
 
 from neutron._i18n import _
+from neutron.agent.common import utils
 from neutron.agent.l2 import l2_agent_extensions_manager as ext_manager
 from neutron.agent import rpc as agent_rpc
 from neutron.agent import securitygroups_rpc as agent_sg_rpc
 from neutron.api.rpc.callbacks import resources
 from neutron.api.rpc.handlers import securitygroups_rpc as sg_rpc
 from neutron.common import config as common_config
-from neutron.common import constants as c_const
 from neutron.common import profiler as setup_profiler
 from neutron.common import utils as n_utils
+from neutron.conf.agent import common as agent_config
+from neutron.conf import service as service_conf
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common import config
 from neutron.plugins.ml2.drivers.mech_sriov.agent.common \
     import exceptions as exc
 from neutron.plugins.ml2.drivers.mech_sriov.agent import eswitch_manager as esm
+from neutron.privileged.agent.linux import ip_lib as priv_ip_lib
 
 
 LOG = logging.getLogger(__name__)
@@ -134,7 +137,8 @@ class SriovNicSwitchRpcCallbacks(sg_rpc.SecurityGroupAgentRpcCallbackMixin):
 @profiler.trace_cls("rpc")
 class SriovNicSwitchAgent(object):
     def __init__(self, physical_devices_mappings, exclude_devices,
-                 polling_interval, rp_bandwidths, rp_inventory_defaults):
+                 polling_interval, rp_bandwidths, rp_inventory_defaults,
+                 rp_hypervisors):
 
         self.polling_interval = polling_interval
         self.network_ports = collections.defaultdict(list)
@@ -160,13 +164,15 @@ class SriovNicSwitchAgent(object):
             self.connection)
 
         configurations = {'device_mappings': physical_devices_mappings,
-                          c_const.RP_BANDWIDTHS: rp_bandwidths,
-                          c_const.RP_INVENTORY_DEFAULTS: rp_inventory_defaults,
+                          n_constants.RP_BANDWIDTHS: rp_bandwidths,
+                          n_constants.RP_INVENTORY_DEFAULTS:
+                              rp_inventory_defaults,
+                          'resource_provider_hypervisors': rp_hypervisors,
                           'extensions': self.ext_manager.names()}
 
         # TODO(mangelajo): optimize resource_versions (see ovs agent)
         self.agent_state = {
-            'binary': 'neutron-sriov-nic-agent',
+            'binary': n_constants.AGENT_PROCESS_NIC_SWITCH,
             'host': self.conf.host,
             'topic': n_constants.L2_AGENT_TOPIC,
             'configurations': configurations,
@@ -288,10 +294,9 @@ class SriovNicSwitchAgent(object):
                 self.eswitch_mgr.set_device_state(device, pci_slot,
                                                   admin_state_up,
                                                   propagate_uplink_state)
-            except exc.IpCommandOperationNotSupportedError:
-                LOG.warning("Device %s does not support state change",
-                            device)
-            except exc.SriovNicError:
+            except priv_ip_lib.InterfaceOperationNotSupported:
+                LOG.warning("Device %s does not support state change", device)
+            except pyroute2.NetlinkError:
                 LOG.warning("Failed to set device %s state", device)
                 return False
         else:
@@ -355,7 +360,7 @@ class SriovNicSwitchAgent(object):
                                            port_id,
                                            (device, profile.get('pci_slot')))
                 self.ext_manager.handle_port(self.context, device_details)
-            elif c_const.NO_ACTIVE_BINDING in device_details:
+            elif n_constants.NO_ACTIVE_BINDING in device_details:
                 # Port was added but its binding in this agent
                 # hasn't been activated yet. It will be treated as
                 # added when binding is activated
@@ -514,6 +519,11 @@ class SriovNicAgentConfigParser(object):
             cfg.CONF.SRIOV_NIC.resource_provider_bandwidths)
         self.rp_inventory_defaults = place_utils.parse_rp_inventory_defaults(
             cfg.CONF.SRIOV_NIC.resource_provider_inventory_defaults)
+        self.rp_hypervisors = utils.default_rp_hypervisors(
+            cfg.CONF.SRIOV_NIC.resource_provider_hypervisors,
+            self.device_mappings,
+            cfg.CONF.SRIOV_NIC.resource_provider_default_hypervisor,
+        )
         self._validate()
 
     def _validate(self):
@@ -525,7 +535,7 @@ class SriovNicAgentConfigParser(object):
         exists in device mappings.
         """
         dev_net_set = set(itertools.chain.from_iterable(
-                          six.itervalues(self.device_mappings)))
+            self.device_mappings.values()))
         for dev_name in self.exclude_devices.keys():
             if dev_name not in dev_net_set:
                 raise ValueError(_(
@@ -539,6 +549,9 @@ def main():
     common_config.init(sys.argv[1:])
 
     common_config.setup_logging()
+    agent_config.setup_privsep()
+    service_conf.register_service_opts(service_conf.RPC_EXTRA_OPTS, cfg.CONF)
+
     try:
         config_parser = SriovNicAgentConfigParser()
         config_parser.parse()
@@ -546,6 +559,7 @@ def main():
         exclude_devices = config_parser.exclude_devices
         rp_bandwidths = config_parser.rp_bandwidths
         rp_inventory_defaults = config_parser.rp_inventory_defaults
+        rp_hypervisors = config_parser.rp_hypervisors
 
     except ValueError:
         LOG.exception("Failed on Agent configuration parse. "
@@ -553,6 +567,9 @@ def main():
         raise SystemExit(1)
     LOG.info("Physical Devices mappings: %s", device_mappings)
     LOG.info("Exclude Devices: %s", exclude_devices)
+    LOG.info("Resource provider bandwidths: %s", rp_bandwidths)
+    LOG.info("Resource provider inventory defaults: %s", rp_inventory_defaults)
+    LOG.info("Resource provider hypervisors: %s", rp_hypervisors)
 
     polling_interval = cfg.CONF.AGENT.polling_interval
     try:
@@ -560,11 +577,12 @@ def main():
                                     exclude_devices,
                                     polling_interval,
                                     rp_bandwidths,
-                                    rp_inventory_defaults)
+                                    rp_inventory_defaults,
+                                    rp_hypervisors)
     except exc.SriovNicError:
         LOG.exception("Agent Initialization Failed")
         raise SystemExit(1)
     # Start everything.
-    setup_profiler.setup("neutron-sriov-nic-agent", cfg.CONF.host)
+    setup_profiler.setup(n_constants.AGENT_PROCESS_NIC_SWITCH, cfg.CONF.host)
     LOG.info("Agent initialized successfully, now running... ")
     agent.daemon_loop()
