@@ -29,8 +29,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from ryu.lib.packet import arp
-from ryu.lib.packet import ether_types
+from os_ken.lib.packet import arp
+from os_ken.lib.packet import ether_types
 
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
@@ -46,6 +46,7 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
     # Used by OVSDVRProcessMixin
     dvr_process_table_id = constants.DVR_PROCESS
     dvr_process_next_table_id = constants.PATCH_LV_TO_TUN
+    of_tables = constants.TUN_BR_ALL_TABLES
 
     def setup_default_table(self, patch_int_ofport, arp_responder_enabled):
         (dp, ofp, ofpp) = self._get_dp()
@@ -164,7 +165,7 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
         (_dp, ofp, ofpp) = self._get_dp()
         match = self._local_vlan_match(ofp, ofpp, segmentation_id)
         table_id = constants.TUN_TABLE[network_type]
-        self.delete_flows(table_id=table_id, match=match)
+        self.uninstall_flows(table_id=table_id, match=match)
 
     @staticmethod
     def _flood_to_tun_match(ofp, ofpp, vlan):
@@ -185,7 +186,7 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
     def delete_flood_to_tun(self, vlan):
         (_dp, ofp, ofpp) = self._get_dp()
         match = self._flood_to_tun_match(ofp, ofpp, vlan)
-        self.delete_flows(table_id=constants.FLOOD_TO_TUN, match=match)
+        self.uninstall_flows(table_id=constants.FLOOD_TO_TUN, match=match)
 
     @staticmethod
     def _unicast_to_tun_match(ofp, ofpp, vlan, mac):
@@ -208,7 +209,7 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
             match = ofpp.OFPMatch(vlan_vid=vlan | ofp.OFPVID_PRESENT)
         else:
             match = self._unicast_to_tun_match(ofp, ofpp, vlan, mac)
-        self.delete_flows(table_id=constants.UCAST_TO_TUN, match=match)
+        self.uninstall_flows(table_id=constants.UCAST_TO_TUN, match=match)
 
     @staticmethod
     def _arp_responder_match(ofp, ofpp, vlan, ip):
@@ -247,7 +248,8 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
                                   eth_type=ether_types.ETH_TYPE_ARP)
         else:
             match = self._arp_responder_match(ofp, ofpp, vlan, ip)
-        self.delete_flows(table_id=constants.ARP_RESPONDER, match=match)
+        self.uninstall_flows(table_id=constants.ARP_RESPONDER,
+                             match=match)
 
     def setup_tunnel_port(self, network_type, port):
         self.install_goto(dest_table_id=constants.TUN_TABLE[network_type],
@@ -255,7 +257,7 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
                           in_port=port)
 
     def cleanup_tunnel_port(self, port):
-        self.delete_flows(in_port=port)
+        self.uninstall_flows(in_port=port)
 
     def add_dvr_mac_tun(self, mac, port):
         self.install_output(table_id=constants.DVR_NOT_LEARN,
@@ -265,28 +267,5 @@ class OVSTunnelBridge(ovs_bridge.OVSAgentBridge,
 
     def remove_dvr_mac_tun(self, mac):
         # REVISIT(yamamoto): match in_port as well?
-        self.delete_flows(table_id=constants.DVR_NOT_LEARN,
-                          eth_src=mac)
-
-    def deferred(self):
-        # REVISIT(yamamoto): This is for API compat with "ovs-ofctl"
-        # interface.  Consider removing this mechanism when obsoleting
-        # "ovs-ofctl" interface.
-        # For "ovs-ofctl" interface, "deferred" mechanism would improve
-        # performance by batching flow-mods with a single ovs-ofctl command
-        # invocation.
-        # On the other hand, for this "native" interface, the overheads of
-        # each flow-mods are already minimum and batching doesn't make much
-        # sense.  Thus this method is left as no-op.
-        # It might be possible to send multiple flow-mods with a single
-        # barrier.  But it's unclear that level of performance optimization
-        # is desirable while it would certainly complicate error handling.
-        return self
-
-    def __enter__(self):
-        # REVISIT(yamamoto): See the comment on deferred().
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        # REVISIT(yamamoto): See the comment on deferred().
-        pass
+        self.uninstall_flows(table_id=constants.DVR_NOT_LEARN,
+                             eth_src=mac)
