@@ -13,8 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import sys
 
+import netaddr
 from neutron_lib import constants as n_const
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -23,14 +25,12 @@ from oslo_utils import excutils
 from osprofiler import profiler
 
 from neutron.agent.common import ovs_lib
+from neutron.agent.linux.openvswitch_firewall import firewall as ovs_firewall
 from neutron.common import utils as n_utils
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 
 LOG = logging.getLogger(__name__)
 
-cfg.CONF.import_group('AGENT', 'neutron.plugins.ml2.drivers.openvswitch.'
-                      'agent.common.config')
-
 
 # A class to represent a DVR-hosted subnet including vif_ports resident on
 # that subnet
@@ -38,6 +38,8 @@ class LocalDVRSubnetMapping(object):
     def __init__(self, subnet, csnat_ofport=constants.OFPORT_INVALID):
         # set of compute ports on this dvr subnet
         self.compute_ports = {}
+        # set of dvr router interfaces on this subnet
+        self.dvr_ports = {}
         self.subnet = subnet
         self.csnat_ofport = csnat_ofport
         self.dvr_owned = False
@@ -75,6 +77,15 @@ class LocalDVRSubnetMapping(object):
     def get_csnat_ofport(self):
         return self.csnat_ofport
 
+    def add_dvr_ofport(self, vif_id, ofport):
+        self.dvr_ports[vif_id] = ofport
+
+    def remove_dvr_ofport(self, vif_id):
+        self.dvr_ports.pop(vif_id, 0)
+
+    def get_dvr_ofports(self):
+        return self.dvr_ports
+
 
 class OVSPort(object):
     def __init__(self, id, ofport, mac, device_owner):
@@ -83,21 +94,30 @@ class OVSPort(object):
         self.ofport = ofport
         self.subnets = set()
         self.device_owner = device_owner
+        # Currently, this is updated only for DVR router interfaces
+        self.ips = collections.defaultdict(list)
 
     def __str__(self):
         return ("OVSPort: id = %s, ofport = %s, mac = %s, "
-                "device_owner = %s, subnets = %s" %
+                "device_owner = %s, subnets = %s, ips = %s" %
                 (self.id, self.ofport, self.mac,
-                 self.device_owner, self.subnets))
+                 self.device_owner, self.subnets,
+                 self.ips))
 
-    def add_subnet(self, subnet_id):
+    def add_subnet(self, subnet_id, fixed_ip=None):
         self.subnets.add(subnet_id)
+        if fixed_ip is None:
+            return
+
+        self.ips[subnet_id].append(fixed_ip)
 
     def remove_subnet(self, subnet_id):
         self.subnets.remove(subnet_id)
+        self.ips.pop(subnet_id, None)
 
     def remove_all_subnets(self):
         self.subnets.clear()
+        self.ips.clear()
 
     def get_subnets(self):
         return self.subnets
@@ -111,12 +131,13 @@ class OVSPort(object):
     def get_ofport(self):
         return self.ofport
 
+    def get_ip(self, subnet_id):
+        return self.ips.get(subnet_id)
+
 
 @profiler.trace_cls("ovs_dvr_agent")
 class OVSDVRNeutronAgent(object):
-    '''
-    Implements OVS-based DVR(Distributed Virtual Router), for overlay networks.
-    '''
+    '''Implements OVS-based DVR (Distributed Virtual Router) agent'''
     # history
     #   1.0 Initial version
 
@@ -132,28 +153,33 @@ class OVSDVRNeutronAgent(object):
         self.enable_tunneling = enable_tunneling
         self.enable_distributed_routing = enable_distributed_routing
         self.bridge_mappings = bridge_mappings
-        self.phys_brs = phys_brs
         self.int_ofports = int_ofports
         self.phys_ofports = phys_ofports
-        self.reset_ovs_parameters(integ_br, tun_br,
+        self.reset_ovs_parameters(integ_br, tun_br, phys_brs,
                                   patch_int_ofport, patch_tun_ofport)
         self.reset_dvr_parameters()
         self.dvr_mac_address = None
         if self.enable_distributed_routing:
             self.get_dvr_mac_address()
         self.conf = cfg.CONF
+        self.firewall = None
 
-    def setup_dvr_flows(self):
+    def set_firewall(self, firewall=None):
+        self.firewall = firewall
+
+    def setup_dvr_flows(self, bridge_mappings=None):
+        bridge_mappings = bridge_mappings or self.bridge_mappings
         self.setup_dvr_flows_on_integ_br()
         self.setup_dvr_flows_on_tun_br()
-        self.setup_dvr_flows_on_phys_br()
+        self.setup_dvr_flows_on_phys_br(bridge_mappings)
         self.setup_dvr_mac_flows_on_all_brs()
 
-    def reset_ovs_parameters(self, integ_br, tun_br,
+    def reset_ovs_parameters(self, integ_br, tun_br, phys_brs,
                              patch_int_ofport, patch_tun_ofport):
         '''Reset the openvswitch parameters'''
         self.int_br = integ_br
         self.tun_br = tun_br
+        self.phys_brs = phys_brs
         self.patch_int_ofport = patch_int_ofport
         self.patch_tun_ofport = patch_tun_ofport
 
@@ -164,6 +190,15 @@ class OVSDVRNeutronAgent(object):
         self.local_ports = {}
         self.registered_dvr_macs = set()
 
+    def reset_dvr_flows(self, integ_br, tun_br, phys_brs,
+                        patch_int_ofport, patch_tun_ofport,
+                        bridge_mappings=None):
+        '''Reset the openvswitch and DVR parameters and DVR flows'''
+        self.reset_ovs_parameters(
+            integ_br, tun_br, phys_brs, patch_int_ofport, patch_tun_ofport)
+        self.reset_dvr_parameters()
+        self.setup_dvr_flows(bridge_mappings)
+
     def get_dvr_mac_address(self):
         try:
             self.get_dvr_mac_address_with_retry()
@@ -198,7 +233,9 @@ class OVSDVRNeutronAgent(object):
                 LOG.debug("L2 Agent DVR: Received response for "
                           "get_dvr_mac_address_by_host() from "
                           "plugin: %r", details)
-                self.dvr_mac_address = details['mac_address']
+                self.dvr_mac_address = (
+                    netaddr.EUI(details['mac_address'],
+                                dialect=netaddr.mac_unix_expanded))
                 return
 
     def setup_dvr_flows_on_integ_br(self):
@@ -206,17 +243,13 @@ class OVSDVRNeutronAgent(object):
 
         LOG.info("L2 Agent operating in DVR Mode with MAC %s",
                  self.dvr_mac_address)
-        # Remove existing flows in integration bridge
-        if self.conf.AGENT.drop_flows_on_start:
-            self.int_br.uninstall_flows(cookie=ovs_lib.COOKIE_ANY)
-
         # Add a canary flow to int_br to track OVS restarts
         self.int_br.setup_canary_table()
 
         # Insert 'drop' action as the default for Table DVR_TO_SRC_MAC
         self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC, priority=1)
 
-        self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC_VLAN,
+        self.int_br.install_drop(table_id=constants.DVR_TO_SRC_MAC_PHYSICAL,
                                  priority=1)
 
         for physical_network in self.bridge_mappings:
@@ -241,19 +274,19 @@ class OVSDVRNeutronAgent(object):
         self.tun_br.install_goto(table_id=constants.DVR_PROCESS,
                                  dest_table_id=constants.PATCH_LV_TO_TUN)
 
-    def setup_dvr_flows_on_phys_br(self):
+    def setup_dvr_flows_on_phys_br(self, bridge_mappings=None):
         '''Setup up initial dvr flows into br-phys'''
-
-        for physical_network in self.bridge_mappings:
+        bridge_mappings = bridge_mappings or self.bridge_mappings
+        for physical_network in bridge_mappings:
             self.phys_brs[physical_network].install_goto(
                 in_port=self.phys_ofports[physical_network],
                 priority=2,
-                dest_table_id=constants.DVR_PROCESS_VLAN)
+                dest_table_id=constants.DVR_PROCESS_PHYSICAL)
             self.phys_brs[physical_network].install_goto(
                 priority=1,
-                dest_table_id=constants.DVR_NOT_LEARN_VLAN)
+                dest_table_id=constants.DVR_NOT_LEARN_PHYSICAL)
             self.phys_brs[physical_network].install_goto(
-                table_id=constants.DVR_PROCESS_VLAN,
+                table_id=constants.DVR_PROCESS_PHYSICAL,
                 priority=0,
                 dest_table_id=constants.LOCAL_VLAN_TRANSLATION)
             self.phys_brs[physical_network].install_drop(
@@ -261,15 +294,19 @@ class OVSDVRNeutronAgent(object):
                 in_port=self.phys_ofports[physical_network],
                 priority=2)
             self.phys_brs[physical_network].install_normal(
-                table_id=constants.DVR_NOT_LEARN_VLAN,
+                table_id=constants.DVR_NOT_LEARN_PHYSICAL,
                 priority=1)
 
     def _add_dvr_mac_for_phys_br(self, physical_network, mac):
-        self.int_br.add_dvr_mac_vlan(mac=mac,
-                                     port=self.int_ofports[physical_network])
+        self.int_br.add_dvr_mac_physical(
+            mac=mac, port=self.int_ofports[physical_network])
         phys_br = self.phys_brs[physical_network]
-        phys_br.add_dvr_mac_vlan(mac=mac,
-                                 port=self.phys_ofports[physical_network])
+        phys_br.add_dvr_mac_physical(
+            mac=mac, port=self.phys_ofports[physical_network])
+
+    def _add_arp_dvr_mac_for_phys_br(self, physical_network, mac):
+        self.int_br.add_dvr_gateway_mac_arp_vlan(
+                mac=mac, port=self.int_ofports[physical_network])
 
     def _remove_dvr_mac_for_phys_br(self, physical_network, mac):
         # REVISIT(yamamoto): match in_port as well?
@@ -282,6 +319,10 @@ class OVSDVRNeutronAgent(object):
         self.int_br.add_dvr_mac_tun(mac=mac, port=self.patch_tun_ofport)
         self.tun_br.add_dvr_mac_tun(mac=mac, port=self.patch_int_ofport)
 
+    def _add_arp_dvr_mac_for_tun_br(self, mac):
+        self.int_br.add_dvr_gateway_mac_arp_tun(
+                mac=mac, port=self.patch_tun_ofport)
+
     def _remove_dvr_mac_for_tun_br(self, mac):
         self.int_br.remove_dvr_mac_tun(mac=mac, port=self.patch_tun_ofport)
         # REVISIT(yamamoto): match in_port as well?
@@ -295,6 +336,13 @@ class OVSDVRNeutronAgent(object):
         LOG.debug("Added DVR MAC flow for %s", mac)
         self.registered_dvr_macs.add(mac)
 
+    def _add_dvr_mac_for_arp(self, mac):
+        for physical_network in self.bridge_mappings:
+            self._add_arp_dvr_mac_for_phys_br(physical_network, mac)
+        if self.enable_tunneling:
+            self._add_arp_dvr_mac_for_tun_br(mac)
+        LOG.debug("Added ARP DVR MAC flow for %s", mac)
+
     def _remove_dvr_mac(self, mac):
         for physical_network in self.bridge_mappings:
             self._remove_dvr_mac_for_phys_br(physical_network, mac)
@@ -307,9 +355,13 @@ class OVSDVRNeutronAgent(object):
         dvr_macs = self.plugin_rpc.get_dvr_mac_address_list(self.context)
         LOG.debug("L2 Agent DVR: Received these MACs: %r", dvr_macs)
         for mac in dvr_macs:
-            if mac['mac_address'] == self.dvr_mac_address:
+            c_mac = netaddr.EUI(mac['mac_address'],
+                                dialect=netaddr.mac_unix_expanded)
+            if c_mac == self.dvr_mac_address:
+                self._add_dvr_mac_for_arp(c_mac)
+                LOG.debug("Added the DVR MAC rule for ARP %s", c_mac)
                 continue
-            self._add_dvr_mac(mac['mac_address'])
+            self._add_dvr_mac(c_mac)
 
     def dvr_mac_address_update(self, dvr_macs):
         if not self.dvr_mac_address:
@@ -319,9 +371,11 @@ class OVSDVRNeutronAgent(object):
 
         dvr_host_macs = set()
         for entry in dvr_macs:
-            if entry['mac_address'] == self.dvr_mac_address:
+            e_mac = netaddr.EUI(entry['mac_address'],
+                                dialect=netaddr.mac_unix_expanded)
+            if e_mac == self.dvr_mac_address:
                 continue
-            dvr_host_macs.add(entry['mac_address'])
+            dvr_host_macs.add(e_mac)
 
         if dvr_host_macs == self.registered_dvr_macs:
             LOG.debug("DVR Mac address already up to date")
@@ -373,11 +427,20 @@ class OVSDVRNeutronAgent(object):
         ldm.set_dvr_owned(True)
 
         vlan_to_use = lvm.vlan
-        if lvm.network_type == n_const.TYPE_VLAN:
+        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
 
         subnet_info = ldm.get_subnet_info()
         ip_version = subnet_info['ip_version']
+
+        if self.firewall and isinstance(self.firewall,
+                                        ovs_firewall.OVSFirewallDriver):
+            tunnel_direct_info = {"network_type": lvm.network_type,
+                                  "physical_network": lvm.physical_network}
+            self.firewall.install_accepted_egress_direct_flow(
+                subnet_info['gateway_mac'], lvm.vlan, port.ofport,
+                tunnel_direct_info=tunnel_direct_info)
+
         local_compute_ports = (
             self.plugin_rpc.get_ports_on_host_by_subnet(
                 self.context, self.host, subnet_uuid))
@@ -386,6 +449,13 @@ class OVSDVRNeutronAgent(object):
                   local_compute_ports)
         vif_by_id = self.int_br.get_vifs_by_ids(
             [local_port['id'] for local_port in local_compute_ports])
+
+        # A router port has an OVS interface with type internal. Once the
+        # interface is created, a valid ofport will be assigned.
+        vif_by_id = {k: v for k, v in vif_by_id.items()
+                     if not v or v.ofport not in
+                     (ovs_lib.INVALID_OFPORT, ovs_lib.UNASSIGNED_OFPORT)}
+
         for local_port in local_compute_ports:
             vif = vif_by_id.get(local_port['id'])
             if not vif:
@@ -401,7 +471,7 @@ class OVSDVRNeutronAgent(object):
                 # the compute port is discovered first here that its on
                 # a dvr routed subnet queue this subnet to that port
                 comp_ovsport = OVSPort(vif.vif_id, vif.ofport,
-                                  vif.vif_mac, local_port['device_owner'])
+                                       vif.vif_mac, local_port['device_owner'])
                 comp_ovsport.add_subnet(subnet_uuid)
                 self.local_ports[vif.vif_id] = comp_ovsport
             # create rule for just this vm port
@@ -411,8 +481,14 @@ class OVSDVRNeutronAgent(object):
                 gateway_mac=subnet_info['gateway_mac'],
                 dst_mac=comp_ovsport.get_mac(),
                 dst_port=comp_ovsport.get_ofport())
+        self.int_br.install_dvr_dst_mac_for_arp(
+            lvm.network_type,
+            vlan_tag=lvm.vlan,
+            gateway_mac=port.vif_mac,
+            dvr_mac=self.dvr_mac_address,
+            rtr_port=port.ofport)
 
-        if lvm.network_type == n_const.TYPE_VLAN:
+        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             # TODO(vivek) remove the IPv6 related flows once SNAT is not
             # used for IPv6 DVR.
             br = self.phys_brs[lvm.physical_network]
@@ -422,10 +498,10 @@ class OVSDVRNeutronAgent(object):
         # used for IPv6 DVR.
         if ip_version == 4:
             br.install_dvr_process_ipv4(
-                vlan_tag=lvm.vlan, gateway_ip=subnet_info['gateway_ip'])
+                vlan_tag=lvm.vlan, gateway_ip=fixed_ip['ip_address'])
         else:
             br.install_dvr_process_ipv6(
-                vlan_tag=lvm.vlan, gateway_mac=subnet_info['gateway_mac'])
+                vlan_tag=lvm.vlan, gateway_mac=port.vif_mac)
         br.install_dvr_process(
             vlan_tag=lvm.vlan, vif_mac=port.vif_mac,
             dvr_mac_address=self.dvr_mac_address)
@@ -435,8 +511,9 @@ class OVSDVRNeutronAgent(object):
         # a router interface on any given router
         ovsport = OVSPort(port.vif_id, port.ofport,
                           port.vif_mac, device_owner)
-        ovsport.add_subnet(subnet_uuid)
+        ovsport.add_subnet(subnet_uuid, fixed_ip['ip_address'])
         self.local_ports[port.vif_id] = ovsport
+        ldm.add_dvr_ofport(port.vif_id, port.ofport)
 
     def _bind_port_on_dvr_subnet(self, port, lvm, fixed_ips,
                                  device_owner):
@@ -473,7 +550,7 @@ class OVSDVRNeutronAgent(object):
                 ovsport.add_subnet(subnet_uuid)
                 self.local_ports[port.vif_id] = ovsport
             vlan_to_use = lvm.vlan
-            if lvm.network_type == n_const.TYPE_VLAN:
+            if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
                 vlan_to_use = lvm.segmentation_id
             # create a rule for this vm port
             self.int_br.install_dvr_to_src_mac(
@@ -485,32 +562,32 @@ class OVSDVRNeutronAgent(object):
 
     def _bind_centralized_snat_port_on_dvr_subnet(self, port, lvm,
                                                   fixed_ips, device_owner):
-        # since centralized-SNAT (CSNAT) port must have only one fixed
-        # IP, directly use fixed_ips[0]
-        fixed_ip = fixed_ips[0]
+        # We only pass the subnet uuid so the server code will correctly
+        # use the gateway_ip value from the subnet when looking up the
+        # centralized-SNAT (CSNAT) port, get it early from the first fixed_ip.
+        subnet_uuid = fixed_ips[0]['subnet_id']
         if port.vif_id in self.local_ports:
             # throw an error if CSNAT port is already on a different
             # dvr routed subnet
             ovsport = self.local_ports[port.vif_id]
             subs = list(ovsport.get_subnets())
-            if subs[0] == fixed_ip['subnet_id']:
+            if subs[0] == subnet_uuid:
                 return
             LOG.error("Centralized-SNAT port %(port)s on subnet "
                       "%(port_subnet)s already seen on a different "
                       "subnet %(orig_subnet)s", {
                           "port": port.vif_id,
-                          "port_subnet": fixed_ip['subnet_id'],
+                          "port_subnet": subnet_uuid,
                           "orig_subnet": subs[0],
                       })
             return
-        subnet_uuid = fixed_ip['subnet_id']
         ldm = None
         subnet_info = None
         if subnet_uuid not in self.local_dvr_map:
             # no csnat ports seen on this subnet - create csnat state
             # for this subnet
             subnet_info = self.plugin_rpc.get_subnet_for_dvr(
-                self.context, subnet_uuid, fixed_ips=fixed_ips)
+                self.context, subnet_uuid, fixed_ips=None)
             if not subnet_info:
                 LOG.warning("DVR: Unable to retrieve subnet information "
                             "for subnet_id %s. The subnet or the gateway "
@@ -533,7 +610,7 @@ class OVSDVRNeutronAgent(object):
         ovsport.add_subnet(subnet_uuid)
         self.local_ports[port.vif_id] = ovsport
         vlan_to_use = lvm.vlan
-        if lvm.network_type == n_const.TYPE_VLAN:
+        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         self.int_br.install_dvr_to_src_mac(
             network_type=lvm.network_type,
@@ -547,8 +624,9 @@ class OVSDVRNeutronAgent(object):
         if not self.in_distributed_mode():
             return
 
-        if local_vlan_map.network_type not in (constants.TUNNEL_NETWORK_TYPES
-                                               + [n_const.TYPE_VLAN]):
+        if (local_vlan_map.network_type not in
+                (constants.TUNNEL_NETWORK_TYPES +
+                 constants.DVR_PHYSICAL_NETWORK_TYPES)):
             LOG.debug("DVR: Port %s is with network_type %s not supported"
                       " for dvr plumbing", port.vif_id,
                       local_vlan_map.network_type)
@@ -560,6 +638,10 @@ class OVSDVRNeutronAgent(object):
                      "%(ofport)s, rebinding.",
                      {'vif': port.vif_id, 'ofport': port.ofport})
             self.unbind_port_from_dvr(port, local_vlan_map)
+            if port.ofport in (ovs_lib.INVALID_OFPORT,
+                               ovs_lib.UNASSIGNED_OFPORT):
+                return
+
         if device_owner == n_const.DEVICE_OWNER_DVR_INTERFACE:
             self._bind_distributed_router_interface_port(port,
                                                          local_vlan_map,
@@ -585,7 +667,7 @@ class OVSDVRNeutronAgent(object):
         network_type = lvm.network_type
         physical_network = lvm.physical_network
         vlan_to_use = lvm.vlan
-        if network_type == n_const.TYPE_VLAN:
+        if network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         # ensure we process for all the subnets laid on this removed port
         for sub_uuid in subnet_set:
@@ -594,36 +676,65 @@ class OVSDVRNeutronAgent(object):
             ldm = self.local_dvr_map[sub_uuid]
             subnet_info = ldm.get_subnet_info()
             ip_version = subnet_info['ip_version']
-            # DVR is no more owner
-            ldm.set_dvr_owned(False)
-            # remove all vm rules for this dvr subnet
-            # clear of compute_ports altogether
-            compute_ports = ldm.get_compute_ofports()
-            for vif_id in compute_ports:
-                comp_port = self.local_ports[vif_id]
-                self.int_br.delete_dvr_to_src_mac(
-                    network_type=network_type,
-                    vlan_tag=vlan_to_use, dst_mac=comp_port.get_mac())
-            ldm.remove_all_compute_ofports()
 
-            if ldm.get_csnat_ofport() == constants.OFPORT_INVALID:
-                # if there is no csnat port for this subnet, remove
-                # this subnet from local_dvr_map, as no dvr (or) csnat
-                # ports available on this agent anymore
+            fixed_ip = ovsport.get_ip(sub_uuid)
+            is_dvr_gateway_port = False
+            subnet_gateway = subnet_info.get('gateway_ip')
+            # since distributed router port must have only one fixed IP,
+            # directly use fixed_ip[0]
+            if fixed_ip and fixed_ip[0] == subnet_gateway:
+                is_dvr_gateway_port = True
+
+            # remove vm dvr src mac rules only if the ovsport
+            # is gateway for the subnet or if the gateway is
+            # not set on the subnet
+            if is_dvr_gateway_port or not subnet_gateway:
+                # DVR is no more owner
+                ldm.set_dvr_owned(False)
+                # remove all vm rules for this dvr subnet
+                # clear of compute_ports altogether
+                compute_ports = ldm.get_compute_ofports()
+                for vif_id in compute_ports:
+                    comp_port = self.local_ports[vif_id]
+                    self.int_br.delete_dvr_to_src_mac(
+                        network_type=network_type,
+                        vlan_tag=vlan_to_use, dst_mac=comp_port.get_mac())
+                ldm.remove_all_compute_ofports()
+
+            self.int_br.delete_dvr_dst_mac_for_arp(
+                network_type=network_type,
+                vlan_tag=vlan_to_use,
+                gateway_mac=port.vif_mac,
+                dvr_mac=self.dvr_mac_address,
+                rtr_port=port.ofport)
+            if (ldm.get_csnat_ofport() == constants.OFPORT_INVALID and
+                    len(ldm.get_dvr_ofports()) <= 1):
+                # if there is no csnat port for this subnet and if this is
+                # the last dvr port in the subnet, remove this subnet from
+                # local_dvr_map, as no dvr (or) csnat ports available on this
+                # agent anymore
                 self.local_dvr_map.pop(sub_uuid, None)
-            if network_type == n_const.TYPE_VLAN:
+            if network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
                 br = self.phys_brs[physical_network]
             if network_type in constants.TUNNEL_NETWORK_TYPES:
                 br = self.tun_br
             if ip_version == 4:
-                br.delete_dvr_process_ipv4(
-                    vlan_tag=lvm.vlan, gateway_ip=subnet_info['gateway_ip'])
+                if subnet_info['gateway_ip']:
+                    br.delete_dvr_process_ipv4(
+                        vlan_tag=lvm.vlan,
+                        gateway_ip=subnet_info['gateway_ip'])
             else:
                 br.delete_dvr_process_ipv6(
                     vlan_tag=lvm.vlan, gateway_mac=subnet_info['gateway_mac'])
             ovsport.remove_subnet(sub_uuid)
+            ldm.remove_dvr_ofport(port.vif_id)
 
-        if lvm.network_type == n_const.TYPE_VLAN:
+            if self.firewall and isinstance(self.firewall,
+                                            ovs_firewall.OVSFirewallDriver):
+                self.firewall.delete_accepted_egress_direct_flow(
+                    subnet_info['gateway_mac'], lvm.vlan)
+
+        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             br = self.phys_brs[physical_network]
         if lvm.network_type in constants.TUNNEL_NETWORK_TYPES:
             br = self.tun_br
@@ -645,7 +756,7 @@ class OVSDVRNeutronAgent(object):
             ldm = self.local_dvr_map[sub_uuid]
             ldm.remove_compute_ofport(port.vif_id)
             vlan_to_use = lvm.vlan
-            if lvm.network_type == n_const.TYPE_VLAN:
+            if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
                 vlan_to_use = lvm.segmentation_id
             # first remove this vm port rule
             self.int_br.delete_dvr_to_src_mac(
@@ -666,7 +777,7 @@ class OVSDVRNeutronAgent(object):
         ldm = self.local_dvr_map[sub_uuid]
         ldm.set_csnat_ofport(constants.OFPORT_INVALID)
         vlan_to_use = lvm.vlan
-        if lvm.network_type == n_const.TYPE_VLAN:
+        if lvm.network_type in constants.DVR_PHYSICAL_NETWORK_TYPES:
             vlan_to_use = lvm.segmentation_id
         # then remove csnat port rule
         self.int_br.delete_dvr_to_src_mac(
