@@ -666,6 +666,9 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # once per lock due to the use of periodics.NeverAgain().
     @periodics.periodic(spacing=600, run_immediately=True)
     def check_for_mcast_flood_reports(self):
+        if not self.has_lock:
+            return
+
         cmds = []
         for port in self._nb_idl.lsp_list().execute(check_error=True):
             port_type = port.type.strip()
@@ -678,7 +681,7 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
             options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true'})
             if port_type == ovn_const.LSP_TYPE_LOCALNET:
-                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'true'})
+                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'false'})
 
             cmds.append(self._nb_idl.lsp_set_options(port.name, **options))
 
