@@ -14,21 +14,26 @@
 #    under the License.
 
 import collections
+import itertools
 import re
+import sys
 
+from neutron_lib.api import attributes
+from neutron_lib.api.definitions import network as net_apidef
 from neutron_lib import constants
+from neutron_lib import context
 from neutron_lib import exceptions
+from neutron_lib.plugins import directory
+from neutron_lib.services import constants as service_const
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 from oslo_policy import policy
 from oslo_utils import excutils
-from oslo_utils import importutils
-import six
+import stevedore
 
-from neutron._i18n import _, _LE, _LW
-from neutron.api.v2 import attributes
-from neutron.common import constants as const
+from neutron._i18n import _
+from neutron.common import cache_utils as cache
 
 
 LOG = logging.getLogger(__name__)
@@ -37,6 +42,14 @@ _ENFORCER = None
 ADMIN_CTX_POLICY = 'context_is_admin'
 ADVSVC_CTX_POLICY = 'context_is_advsvc'
 
+# Identify the attribute used by a resource to reference another resource
+_RESOURCE_FOREIGN_KEYS = {
+    net_apidef.COLLECTION_NAME: 'network_id',
+    # TODO(slaweq): use SECURITYGROUPS constant from api def when
+    # securitygroups api def will be moved to neutron-lib
+    'security_groups': 'security_group_id'
+}
+
 
 def reset():
     global _ENFORCER
@@ -45,12 +58,23 @@ def reset():
         _ENFORCER = None
 
 
+def register_rules(enforcer):
+    extmgr = stevedore.extension.ExtensionManager('neutron.policies',
+                                                  invoke_on_load=True)
+    policies = [list(e.obj) for e in extmgr.extensions]
+    LOG.debug('Loaded default policies from %s '
+              'under neutron.policies entry points',
+              [e.name for e in extmgr.extensions])
+    enforcer.register_defaults(itertools.chain(*policies))
+
+
 def init(conf=cfg.CONF, policy_file=None):
     """Init an instance of the Enforcer class."""
 
     global _ENFORCER
     if not _ENFORCER:
         _ENFORCER = policy.Enforcer(conf, policy_file=policy_file)
+        register_rules(_ENFORCER)
         _ENFORCER.load_rules(True)
 
 
@@ -85,11 +109,11 @@ def set_rules(policies, overwrite=True):
 
 def _is_attribute_explicitly_set(attribute_name, resource, target, action):
     """Verify that an attribute is present and is explicitly set."""
-    if 'update' in action:
+    if target.get(constants.ATTRIBUTES_TO_UPDATE):
         # In the case of update, the function should not pay attention to a
         # default value of an attribute, but check whether it was explicitly
         # marked as being updated instead.
-        return (attribute_name in target[const.ATTRIBUTES_TO_UPDATE] and
+        return (attribute_name in target[constants.ATTRIBUTES_TO_UPDATE] and
                 target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
     result = (attribute_name in target and
               target[attribute_name] is not constants.ATTR_NOT_SPECIFIED)
@@ -103,7 +127,7 @@ def _should_validate_sub_attributes(attribute, sub_attr):
     validate = attribute.get('validate')
     return (validate and isinstance(sub_attr, collections.Iterable) and
             any([k.startswith('type:dict') and
-                 v for (k, v) in six.iteritems(validate)]))
+                 v for (k, v) in validate.items()]))
 
 
 def _build_subattr_match_rule(attr_name, attr, action, target):
@@ -114,8 +138,7 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
     validate = attr['validate']
     key = [k for k in validate.keys() if k.startswith('type:dict')]
     if not key:
-        LOG.warning(_LW("Unable to find data type descriptor "
-                        "for attribute %s"),
+        LOG.warning("Unable to find data type descriptor for attribute %s",
                     attr_name)
         return
     data = validate[key[0]]
@@ -132,6 +155,17 @@ def _build_subattr_match_rule(attr_name, attr, action, target):
     return policy.AndCheck(sub_attr_rules)
 
 
+def _build_list_of_subattrs_rule(attr_name, attribute_value, action):
+    rules = []
+    for sub_attr in attribute_value:
+        if isinstance(sub_attr, dict):
+            for k in sub_attr:
+                rules.append(policy.RuleCheck(
+                    'rule', '%s:%s:%s' % (action, attr_name, k)))
+    if rules:
+        return policy.AndCheck(rules)
+
+
 def _process_rules_list(rules, match_rule):
     """Recursively walk a policy rule to extract a list of match entries."""
     if isinstance(match_rule, policy.RuleCheck):
@@ -159,7 +193,7 @@ def _build_match_rule(action, target, pluralized):
         action, pluralized)
     if enforce_attr_based_check:
         # assigning to variable with short name for improving readability
-        res_map = attributes.RESOURCE_ATTRIBUTE_MAP
+        res_map = attributes.RESOURCES
         if resource in res_map:
             for attribute_name in res_map[resource]:
                 if _is_attribute_explicitly_set(attribute_name,
@@ -167,8 +201,8 @@ def _build_match_rule(action, target, pluralized):
                                                 target, action):
                     attribute = res_map[resource][attribute_name]
                     if 'enforce_policy' in attribute:
-                        attr_rule = policy.RuleCheck('rule', '%s:%s' %
-                                                     (action, attribute_name))
+                        attr_rule = policy.RuleCheck(
+                            'rule', '%s:%s' % (action, attribute_name))
                         # Build match entries for sub-attributes
                         if _should_validate_sub_attributes(
                                 attribute, target[attribute_name]):
@@ -176,6 +210,15 @@ def _build_match_rule(action, target, pluralized):
                                 [attr_rule, _build_subattr_match_rule(
                                     attribute_name, attribute,
                                     action, target)])
+
+                        attribute_value = target[attribute_name]
+                        if isinstance(attribute_value, list):
+                            subattr_rule = _build_list_of_subattrs_rule(
+                                attribute_name, attribute_value, action)
+                            if subattr_rule:
+                                attr_rule = policy.AndCheck(
+                                    [attr_rule, subattr_rule])
+
                         match_rule = policy.AndCheck([match_rule, attr_rule])
     return match_rule
 
@@ -208,8 +251,38 @@ class OwnerCheck(policy.Check):
             raise exceptions.PolicyInitError(
                 policy="%s:%s" % (kind, match),
                 reason=err_reason)
+        self._cache = cache._get_memory_cache_region(expiration_time=5)
         super(OwnerCheck, self).__init__(kind, match)
 
+    @cache.cache_method_results
+    def _extract(self, resource_type, resource_id, field):
+        # NOTE(salv-orlando): This check currently assumes the parent
+        # resource is handled by the core plugin. It might be worth
+        # having a way to map resources to plugins so to make this
+        # check more general
+        plugin = directory.get_plugin()
+        if resource_type in service_const.EXT_PARENT_RESOURCE_MAPPING:
+            plugin = directory.get_plugin(
+                service_const.EXT_PARENT_RESOURCE_MAPPING[resource_type])
+        f = getattr(plugin, 'get_%s' % resource_type)
+        # f *must* exist, if not found it is better to let neutron
+        # explode. Check will be performed with admin context
+        try:
+            data = f(context.get_admin_context(),
+                     resource_id,
+                     fields=[field])
+        except exceptions.NotFound as e:
+            # NOTE(kevinbenton): a NotFound exception can occur if a
+            # list operation is happening at the same time as one of
+            # the parents and its children being deleted. So we issue
+            # a RetryRequest so the API will redo the lookup and the
+            # problem items will be gone.
+            raise db_exc.RetryRequest(e)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                LOG.exception('Policy check error while calling %s!', f)
+        return data[field]
+
     def __call__(self, target, creds, enforcer):
         if self.target_field not in target:
             # policy needs a plugin check
@@ -237,8 +310,15 @@ class OwnerCheck(policy.Check):
                 raise exceptions.PolicyCheckError(
                     policy="%s:%s" % (self.kind, self.match),
                     reason=err_reason)
-            parent_foreign_key = attributes.RESOURCE_FOREIGN_KEYS.get(
+            parent_foreign_key = _RESOURCE_FOREIGN_KEYS.get(
                 "%ss" % parent_res, None)
+            if parent_res == constants.EXT_PARENT_PREFIX:
+                for resource in service_const.EXT_PARENT_RESOURCE_MAPPING:
+                    key = "%s_%s_id" % (constants.EXT_PARENT_PREFIX, resource)
+                    if key in target:
+                        parent_foreign_key = key
+                        parent_res = resource
+                        break
             if not parent_foreign_key:
                 err_reason = (_("Unable to verify match:%(match)s as the "
                                 "parent resource: %(res)s was not found") %
@@ -247,37 +327,13 @@ class OwnerCheck(policy.Check):
                 raise exceptions.PolicyCheckError(
                     policy="%s:%s" % (self.kind, self.match),
                     reason=err_reason)
-            # NOTE(salv-orlando): This check currently assumes the parent
-            # resource is handled by the core plugin. It might be worth
-            # having a way to map resources to plugins so to make this
-            # check more general
-            # NOTE(ihrachys): if import is put in global, circular
-            # import failure occurs
-            manager = importutils.import_module('neutron.manager')
-            f = getattr(manager.NeutronManager.get_instance().plugin,
-                        'get_%s' % parent_res)
-            # f *must* exist, if not found it is better to let neutron
-            # explode. Check will be performed with admin context
-            context = importutils.import_module('neutron.context')
-            try:
-                data = f(context.get_admin_context(),
-                         target[parent_foreign_key],
-                         fields=[parent_field])
-                target[self.target_field] = data[parent_field]
-            except exceptions.NotFound as e:
-                # NOTE(kevinbenton): a NotFound exception can occur if a
-                # list operation is happening at the same time as one of
-                # the parents and its children being deleted. So we issue
-                # a RetryRequest so the API will redo the lookup and the
-                # problem items will be gone.
-                raise db_exc.RetryRequest(e)
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Policy check error while calling %s!'),
-                                  f)
+
+            target[self.target_field] = self._extract(
+                parent_res, target[parent_foreign_key], parent_field)
+
         match = self.match % target
         if self.kind in creds:
-            return match == six.text_type(creds[self.kind])
+            return match == str(creds[self.kind])
         return False
 
 
@@ -293,27 +349,51 @@ class FieldCheck(policy.Check):
 
         # Value might need conversion - we need help from the attribute map
         try:
-            attr = attributes.RESOURCE_ATTRIBUTE_MAP[resource][field]
+            attr = attributes.RESOURCES[resource][field]
             conv_func = attr['convert_to']
         except KeyError:
             conv_func = lambda x: x
 
         self.field = field
+        self.resource = resource
         self.value = conv_func(value)
         self.regex = re.compile(value[1:]) if value.startswith('~') else None
 
     def __call__(self, target_dict, cred_dict, enforcer):
-        target_value = target_dict.get(self.field)
+        target_value = self._get_target_value(target_dict)
         # target_value might be a boolean, explicitly compare with None
         if target_value is None:
-            LOG.debug("Unable to find requested field: %(field)s in target: "
-                      "%(target_dict)s",
-                      {'field': self.field, 'target_dict': target_dict})
             return False
         if self.regex:
             return bool(self.regex.match(target_value))
         return target_value == self.value
 
+    def _get_target_value(self, target_dict):
+        if self.field in target_dict:
+            return target_dict[self.field]
+        # NOTE(slaweq): In case that target field is "networks:shared" we need
+        # to treat it in "special" way as it may be used for resources other
+        # than network, e.g. for port or subnet
+        target_value = None
+        if self.resource == "networks" and self.field == constants.SHARED:
+            target_network_id = target_dict.get("network_id")
+            if not target_network_id:
+                LOG.debug("Unable to find network_id field in target: "
+                          "%(target_dict)s",
+                          {'field': self.field, 'target_dict': target_dict})
+                return
+            project_id = target_dict.get('project_id')
+            ctx = (context.Context(tenant_id=project_id) if project_id
+                   else context.get_admin_context())
+            plugin = directory.get_plugin()
+            network = plugin.get_network(ctx, target_network_id)
+            target_value = network.get(self.field)
+        if target_value is None:
+            LOG.debug("Unable to find requested field: %(field)s in target: "
+                      "%(target_dict)s",
+                      {'field': self.field, 'target_dict': target_dict})
+        return target_value
+
 
 def _prepare_check(context, action, target, pluralized):
     """Prepare rule, target, and credentials for the policy engine."""
@@ -321,7 +401,7 @@ def _prepare_check(context, action, target, pluralized):
     if target is None:
         target = {}
     match_rule = _build_match_rule(action, target, pluralized)
-    credentials = context.to_dict()
+    credentials = context.to_policy_values()
     return match_rule, target, credentials
 
 
@@ -365,9 +445,6 @@ def check(context, action, target, plugin=None, might_not_exist=False,
                                target,
                                credentials,
                                pluralized=pluralized)
-    # logging applied rules in case of failure
-    if not result:
-        log_rule_list(match_rule)
     return result
 
 
@@ -406,21 +483,22 @@ def enforce(context, action, target, plugin=None, pluralized=None):
     return result
 
 
-def check_is_admin(context):
-    """Verify context has admin rights according to policy settings."""
-    init()
-    # the target is user-self
-    credentials = context.to_dict()
-    if ADMIN_CTX_POLICY not in _ENFORCER.rules:
-        return False
-    return _ENFORCER.enforce(ADMIN_CTX_POLICY, credentials, credentials)
+def get_enforcer():
+    # NOTE(amotoki): This was borrowed from nova/policy.py.
+    # This method is for use by oslo.policy CLI scripts. Those scripts need the
+    # 'output-file' and 'namespace' options, but having those in sys.argv means
+    # loading the neutron config options will fail as those are not expected to
+    # be present. So we pass in an arg list with those stripped out.
+    conf_args = []
+    # Start at 1 because cfg.CONF expects the equivalent of sys.argv[1:]
+    i = 1
+    while i < len(sys.argv):
+        if sys.argv[i].strip('-') in ['namespace', 'output-file']:
+            i += 2
+            continue
+        conf_args.append(sys.argv[i])
+        i += 1
 
-
-def check_is_advsvc(context):
-    """Verify context has advsvc rights according to policy settings."""
+    cfg.CONF(conf_args, project='neutron')
     init()
-    # the target is user-self
-    credentials = context.to_dict()
-    if ADVSVC_CTX_POLICY not in _ENFORCER.rules:
-        return False
-    return _ENFORCER.enforce(ADVSVC_CTX_POLICY, credentials, credentials)
+    return _ENFORCER
