@@ -15,22 +15,21 @@
 
 
 import collections
-
-from neutron_lib import constants
 from operator import itemgetter
-from oslo_config import cfg
-from oslo_db import exception as db_exc
-from oslo_log import log as logging
-from sqlalchemy import sql
 
-from neutron._i18n import _LI, _LW
-from neutron.db import agents_db
-from neutron.db import api as db_api
+from neutron_lib.api.definitions import availability_zone as az_def
+from neutron_lib import constants
+from neutron_lib.db import api as db_api
+from neutron_lib.objects import exceptions
+from oslo_config import cfg
+from oslo_log import log as logging
+
+from neutron.agent.common import utils as agent_utils
 from neutron.db.network_dhcp_agent_binding import models as ndab_model
-from neutron.extensions import availability_zone as az_ext
+from neutron.objects import agent as agent_obj
+from neutron.objects import network
 from neutron.scheduler import base_resource_filter
 from neutron.scheduler import base_scheduler
-from neutron.services.segments import db as segments_db
 
 LOG = logging.getLogger(__name__)
 
@@ -44,7 +43,9 @@ class AutoScheduler(object):
         agents_per_network = cfg.CONF.dhcp_agents_per_network
         # a list of (agent, net_ids) tuples
         bindings_to_add = []
-        with context.session.begin(subtransactions=True):
+        # NOTE(ralonsoh) use writer manager to call get_network. See
+        # https://review.opendev.org/#/c/483518/. Must be changed to READER.
+        with db_api.CONTEXT_WRITER.using(context):
             fields = ['network_id', 'enable_dhcp', 'segment_id']
             subnets = plugin.get_subnets(context, fields=fields)
             net_ids = {}
@@ -58,23 +59,18 @@ class AutoScheduler(object):
             if not net_ids:
                 LOG.debug('No non-hosted networks')
                 return False
-            query = context.session.query(agents_db.Agent)
-            query = query.filter(agents_db.Agent.agent_type ==
-                                 constants.AGENT_TYPE_DHCP,
-                                 agents_db.Agent.host == host,
-                                 agents_db.Agent.admin_state_up == sql.true())
-            dhcp_agents = query.all()
+            dhcp_agents = agent_obj.Agent.get_objects(
+                context, agent_type=constants.AGENT_TYPE_DHCP,
+                host=host, admin_state_up=True)
 
-            query = context.session.query(
-                segments_db.SegmentHostMapping.segment_id)
-            query = query.filter(segments_db.SegmentHostMapping.host == host)
-            segments_on_host = {s.segment_id for s in query}
+            segment_host_mapping = network.SegmentHostMapping.get_objects(
+                context, host=host)
+
+            segments_on_host = {s.segment_id for s in segment_host_mapping}
 
             for dhcp_agent in dhcp_agents:
-                if agents_db.AgentDbMixin.is_agent_down(
-                    dhcp_agent.heartbeat_timestamp):
-                    LOG.warning(_LW('DHCP agent %s is not active'),
-                                dhcp_agent.id)
+                if agent_utils.is_agent_down(dhcp_agent.heartbeat_timestamp):
+                    LOG.warning('DHCP agent %s is not active', dhcp_agent.id)
                     continue
                 for net_id, is_routed_network in net_ids.items():
                     agents = plugin.get_dhcp_agents_hosting_networks(
@@ -89,16 +85,24 @@ class AutoScheduler(object):
                     if any(dhcp_agent.id == agent.id for agent in agents):
                         continue
                     net = plugin.get_network(context, net_id)
-                    az_hints = (net.get(az_ext.AZ_HINTS) or
+                    az_hints = (net.get(az_def.AZ_HINTS) or
                                 cfg.CONF.default_availability_zones)
                     if (az_hints and
-                        dhcp_agent['availability_zone'] not in az_hints):
+                            dhcp_agent['availability_zone'] not in az_hints):
                         continue
-                    bindings_to_add.append((dhcp_agent, net_id))
+                    bindings_to_add.append(
+                        (dhcp_agent, net_id, is_routed_network))
         # do it outside transaction so particular scheduling results don't
         # make other to fail
-        for agent, net_id in bindings_to_add:
-            self.resource_filter.bind(context, [agent], net_id)
+        debug_data = []
+        for agent, net_id, is_routed_network in bindings_to_add:
+            self.resource_filter.bind(
+                context, [agent], net_id,
+                force_scheduling=is_routed_network)
+            debug_data.append('(%s, %s, %s)' % (agent['agent_type'],
+                                                agent['host'], net_id))
+        LOG.debug('Resources bound (agent type, host, resource id): %s',
+                  ', '.join(debug_data))
         return True
 
 
@@ -174,30 +178,58 @@ class AZAwareWeightScheduler(WeightScheduler):
 
 class DhcpFilter(base_resource_filter.BaseResourceFilter):
 
-    def bind(self, context, agents, network_id):
+    def get_vacant_network_dhcp_agent_binding_index(
+            self, context, network_id, force_scheduling):
+        """Return a vacant binding_index to use and whether or not it exists.
+
+        Each NetworkDhcpAgentBinding has a binding_index which is unique per
+        network_id, and when creating a single binding we require to find a
+        'vacant' binding_index which isn't yet used - for example if we have
+        bindings with indices 1 and 3, then clearly binding_index == 2 is free.
+
+        :returns: binding_index.
+        """
+        num_agents = agent_obj.Agent.count(
+            context, agent_type=constants.AGENT_TYPE_DHCP)
+        num_agents = min(num_agents, cfg.CONF.dhcp_agents_per_network)
+        bindings = network.NetworkDhcpAgentBinding.get_objects(
+            context, network_id=network_id)
+        return base_scheduler.get_vacant_binding_index(
+            num_agents, bindings, ndab_model.LOWEST_BINDING_INDEX,
+            force_scheduling=force_scheduling)
+
+    def bind(self, context, agents, network_id, force_scheduling=False):
         """Bind the network to the agents."""
         # customize the bind logic
         bound_agents = agents[:]
         for agent in agents:
+            binding_index = self.get_vacant_network_dhcp_agent_binding_index(
+                context, network_id, force_scheduling)
+            if binding_index < ndab_model.LOWEST_BINDING_INDEX:
+                LOG.debug('Unable to find a vacant binding_index for '
+                          'network %(network_id)s and agent %(agent_id)s',
+                          {'network_id': network_id,
+                           'agent_id': agent.id})
+                continue
+
             # saving agent_id to use it after rollback to avoid
             # DetachedInstanceError
             agent_id = agent.id
-            binding = ndab_model.NetworkDhcpAgentBinding()
-            binding.dhcp_agent_id = agent_id
-            binding.network_id = network_id
             try:
-                with db_api.autonested_transaction(context.session):
-                    context.session.add(binding)
-                    # try to actually write the changes and catch integrity
-                    # DBDuplicateEntry
-            except db_exc.DBDuplicateEntry:
+                network.NetworkDhcpAgentBinding(
+                     context, dhcp_agent_id=agent_id,
+                     network_id=network_id,
+                     binding_index=binding_index).create()
+            except exceptions.NeutronDbObjectDuplicateEntry:
                 # it's totally ok, someone just did our job!
                 bound_agents.remove(agent)
-                LOG.info(_LI('Agent %s already present'), agent_id)
+                LOG.info('Agent %s already present', agent_id)
             LOG.debug('Network %(network_id)s is scheduled to be '
-                      'hosted by DHCP agent %(agent_id)s',
+                      'hosted by DHCP agent %(agent_id)s with binding_index '
+                      '%(binding_index)d',
                       {'network_id': network_id,
-                       'agent_id': agent_id})
+                       'agent_id': agent_id,
+                       'binding_index': binding_index})
         super(DhcpFilter, self).bind(context, bound_agents, network_id)
 
     def filter_agents(self, plugin, context, network):
@@ -217,8 +249,8 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
                     'hosted_agents': agents_dict['hosted_agents']}
         return agents_dict
 
-    def _filter_agents_with_network_access(self, hostable_agents, plugin,
-                                           context, network):
+    def _filter_agents_with_network_access(self, plugin, context,
+                                           network, hostable_agents):
         if 'candidate_hosts' in network:
             hostable_dhcp_hosts = network['candidate_hosts']
         else:
@@ -234,9 +266,9 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
            network is already hosted by enough number of agents.
         """
         agents_per_network = cfg.CONF.dhcp_agents_per_network
-        #TODO(gongysh) don't schedule the networks with only
+        # TODO(gongysh) don't schedule the networks with only
         # subnets whose enable_dhcp is false
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(context):
             network_hosted_agents = plugin.get_dhcp_agents_hosting_networks(
                 context, [network['id']], hosts=network.get('candidate_hosts'))
             if len(network_hosted_agents) >= agents_per_network:
@@ -247,15 +279,15 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
 
     def _get_active_agents(self, plugin, context, az_hints):
         """Return a list of active dhcp agents."""
-        with context.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(context):
             filters = {'agent_type': [constants.AGENT_TYPE_DHCP],
                        'admin_state_up': [True]}
             if az_hints:
                 filters['availability_zone'] = az_hints
-            active_dhcp_agents = plugin.get_agents_db(
+            active_dhcp_agents = plugin.get_agent_objects(
                 context, filters=filters)
             if not active_dhcp_agents:
-                LOG.warning(_LW('No more DHCP agents'))
+                LOG.warning('No more DHCP agents')
                 return []
         return active_dhcp_agents
 
@@ -271,24 +303,32 @@ class DhcpFilter(base_resource_filter.BaseResourceFilter):
         if hosted_agents is None:
             return {'n_agents': 0, 'hostable_agents': [], 'hosted_agents': []}
         n_agents = cfg.CONF.dhcp_agents_per_network - len(hosted_agents)
-        az_hints = (network.get(az_ext.AZ_HINTS) or
+        az_hints = (network.get(az_def.AZ_HINTS) or
                     cfg.CONF.default_availability_zones)
         active_dhcp_agents = self._get_active_agents(plugin, context, az_hints)
+        hosted_agent_ids = [agent['id'] for agent in hosted_agents]
         if not active_dhcp_agents:
             return {'n_agents': 0, 'hostable_agents': [],
                     'hosted_agents': hosted_agents}
         hostable_dhcp_agents = [
-            agent for agent in set(active_dhcp_agents)
-            if agent not in hosted_agents and plugin.is_eligible_agent(
-                context, True, agent)
-        ]
-
+            agent for agent in active_dhcp_agents
+            if agent.id not in hosted_agent_ids and plugin.is_eligible_agent(
+                context, True, agent)]
         hostable_dhcp_agents = self._filter_agents_with_network_access(
-            hostable_dhcp_agents, plugin, context, network)
+            plugin, context, network, hostable_dhcp_agents)
 
         if not hostable_dhcp_agents:
-            return {'n_agents': 0, 'hostable_agents': [],
-                    'hosted_agents': hosted_agents}
-        n_agents = min(len(hostable_dhcp_agents), n_agents)
-        return {'n_agents': n_agents, 'hostable_agents': hostable_dhcp_agents,
-                'hosted_agents': hosted_agents}
+            result = {'n_agents': 0, 'hostable_agents': [],
+                      'hosted_agents': hosted_agents}
+        else:
+            result = {'n_agents': min(len(hostable_dhcp_agents), n_agents),
+                      'hostable_agents': hostable_dhcp_agents,
+                      'hosted_agents': hosted_agents}
+        hostable_agents_ids = [a['id'] for a in result['hostable_agents']]
+        hosted_agents_ids = [a['id'] for a in result['hosted_agents']]
+        LOG.debug('Network hostable DHCP agents. Network: %(network)s, '
+                  'hostable agents: %(hostable_agents)s, hosted agents: '
+                  '%(hosted_agents)s', {'network': network['id'],
+                                        'hostable_agents': hostable_agents_ids,
+                                        'hosted_agents': hosted_agents_ids})
+        return result
