@@ -26,11 +26,9 @@ from neutron_lib.exceptions import l3 as l3_exc
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
-import six
 
 from neutron.common import utils
 from neutron.conf.db import l3_hamode_db
-from neutron.db import api as db_api
 from neutron.db.models import l3agent as rb_model
 from neutron.objects import l3agent as rb_obj
 
@@ -39,8 +37,7 @@ LOG = logging.getLogger(__name__)
 cfg.CONF.register_opts(l3_hamode_db.L3_HA_OPTS)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class L3Scheduler(object):
+class L3Scheduler(object, metaclass=abc.ABCMeta):
 
     def __init__(self):
         self.max_ha_agents = cfg.CONF.max_l3_agents_per_router
@@ -56,7 +53,7 @@ class L3Scheduler(object):
     def _router_has_binding(self, context, router_id, l3_agent_id):
         router_binding_model = rb_model.RouterL3AgentBinding
 
-        query = context.session.query(router_binding_model)
+        query = context.session.query(router_binding_model.router_id)
         query = query.filter(router_binding_model.router_id == router_id,
                              router_binding_model.l3_agent_id == l3_agent_id)
 
@@ -118,7 +115,7 @@ class L3Scheduler(object):
     def _get_candidates(self, plugin, context, sync_router):
         """Return L3 agents where a router could be scheduled."""
         is_ha = sync_router.get('ha', False)
-        with context.session.begin(subtransactions=True):
+        with lib_db_api.CONTEXT_READER.using(context):
             # allow one router is hosted by just
             # one enabled l3 agent hosting since active is just a
             # timing problem. Non-active l3 agent can return to
@@ -169,7 +166,7 @@ class L3Scheduler(object):
         binding_index) fails because some other RouterL3AgentBinding was
         concurrently created using the same binding_index, then the function
         will retry to create an entry with a new binding_index. This creation
-        will be retried up to db_api.MAX_RETRIES times.
+        will be retried up to lib_db_api.MAX_RETRIES times.
         If, still in the HA router case, the creation failed because the
         router has already been bound to the l3 agent in question or has been
         removed (by a concurrent operation), then no further attempts will be
@@ -287,7 +284,9 @@ class L3Scheduler(object):
             port_binding = utils.create_object_with_dependency(
                 creator, dep_getter, dep_creator,
                 dep_id_attr, dep_deleter)[0]
-            with db_api.autonested_transaction(context.session):
+            # NOTE(ralonsoh): to be migrated to the new facade that can't be
+            # used with "create_object_with_dependency".
+            with lib_db_api.autonested_transaction(context.session):
                 port_binding.l3_agent_id = agent['id']
         except db_exc.DBDuplicateEntry:
             LOG.debug("Router %(router)s already scheduled for agent "
