@@ -14,65 +14,80 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib.api.definitions import constants as api_const
+from neutron_lib.api.definitions import l3 as l3_apidef
+from neutron_lib.api.definitions import network as net_def
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
+from neutron_lib.db import api as db_api
+from neutron_lib.db import resource_extend
+from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as n_exc
-from oslo_db import exception as db_exc
+from neutron_lib.objects import exceptions as obj_exc
+from neutron_lib.plugins import constants
+from neutron_lib.plugins import directory
+from neutron_lib.plugins import utils as p_utils
 from oslo_log import log as logging
-from sqlalchemy import sql
 
-from neutron._i18n import _, _LE
-from neutron.api.v2 import attributes
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.common import exceptions as c_exc
-from neutron.db import api as db_api
-from neutron.db import common_db_mixin
-from neutron.db import db_base_plugin_v2
-from neutron.db import external_net_db
-from neutron.db import models_v2
-from neutron.db import standard_attr
-from neutron.extensions import l3
-from neutron import manager
-from neutron.plugins.common import constants
-from neutron.plugins.common import utils as p_utils
+from neutron._i18n import _
+from neutron.objects import auto_allocate as auto_allocate_obj
+from neutron.objects import base as base_obj
+from neutron.objects import network as net_obj
 from neutron.services.auto_allocate import exceptions
-from neutron.services.auto_allocate import models
 
 LOG = logging.getLogger(__name__)
-IS_DEFAULT = 'is_default'
 CHECK_REQUIREMENTS = 'dry-run'
 
 
-def _extend_external_network_default(core_plugin, net_res, net_db):
-    """Add is_default field to 'show' response."""
-    if net_db.external is not None:
-        net_res[IS_DEFAULT] = net_db.external.is_default
-    return net_res
-
-
-@db_api.retry_if_session_inactive()
 def _ensure_external_network_default_value_callback(
-    resource, event, trigger, context, request, network):
+        resource, event, trigger, **kwargs):
     """Ensure the is_default db field matches the create/update request."""
-    is_default = request.get(IS_DEFAULT, False)
-    if event in (events.BEFORE_CREATE, events.BEFORE_UPDATE) and is_default:
-        # ensure there is only one default external network at any given time
-        obj = (context.session.query(external_net_db.ExternalNetwork).
-            filter_by(is_default=True)).first()
-        if obj and network['id'] != obj.network_id:
-            raise exceptions.DefaultExternalNetworkExists(
-                net_id=obj.network_id)
 
-    # Reflect the status of the is_default on the create/update request
-    obj = (context.session.query(external_net_db.ExternalNetwork).
-        filter_by(network_id=network['id']))
-    obj.update({IS_DEFAULT: is_default})
+    # TODO(boden): remove shim once all callbacks use payloads
+    if 'payload' in kwargs:
+        _request = kwargs['payload'].request_body
+        _context = kwargs['payload'].context
+        _network = kwargs['payload'].desired_state
+        _orig = kwargs['payload'].states[0]
+    else:
+        _request = kwargs['request']
+        _context = kwargs['context']
+        _network = kwargs['network']
+        _orig = kwargs.get('original_network')
+
+    @db_api.retry_if_session_inactive()
+    def _do_ensure_external_network_default_value_callback(
+            context, request, orig, network):
+        is_default = request.get(api_const.IS_DEFAULT)
+        if is_default is None:
+            return
+        if is_default:
+            # ensure only one default external network at any given time
+            pager = base_obj.Pager(limit=1)
+            objs = net_obj.ExternalNetwork.get_objects(context, _pager=pager,
+                                                       is_default=True)
+            if objs:
+                if objs[0] and network['id'] != objs[0].network_id:
+                    raise exceptions.DefaultExternalNetworkExists(
+                        net_id=objs[0].network_id)
+
+        if orig and orig.get(api_const.IS_DEFAULT) == is_default:
+            return
+        network[api_const.IS_DEFAULT] = is_default
+        # Reflect the status of the is_default on the create/update request
+        obj = net_obj.ExternalNetwork.get_object(context,
+                                                 network_id=network['id'])
+        if obj:
+            obj.is_default = is_default
+            obj.update()
+
+    _do_ensure_external_network_default_value_callback(
+        _context, _request, _orig, _network)
 
 
-class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
-
-    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
-        attributes.NETWORKS, [_extend_external_network_default])
+@resource_extend.has_resource_extenders
+class AutoAllocatedTopologyMixin(object):
 
     def __new__(cls, *args, **kwargs):
         # NOTE(kevinbenton): we subscribe on object construction because
@@ -80,11 +95,9 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
         new = super(AutoAllocatedTopologyMixin, cls).__new__(cls, *args,
                                                              **kwargs)
         registry.subscribe(_ensure_external_network_default_value_callback,
-            resources.EXTERNAL_NETWORK, events.BEFORE_CREATE)
+                           resources.NETWORK, events.PRECOMMIT_UPDATE)
         registry.subscribe(_ensure_external_network_default_value_callback,
-            resources.EXTERNAL_NETWORK, events.AFTER_CREATE)
-        registry.subscribe(_ensure_external_network_default_value_callback,
-            resources.EXTERNAL_NETWORK, events.BEFORE_UPDATE)
+                           resources.NETWORK, events.PRECOMMIT_CREATE)
         return new
 
     # TODO(armax): if a tenant modifies auto allocated resources under
@@ -100,16 +113,23 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
     @property
     def core_plugin(self):
         if not getattr(self, '_core_plugin', None):
-            self._core_plugin = manager.NeutronManager.get_plugin()
+            self._core_plugin = directory.get_plugin()
         return self._core_plugin
 
     @property
     def l3_plugin(self):
         if not getattr(self, '_l3_plugin', None):
-            self._l3_plugin = manager.NeutronManager.get_service_plugins().get(
-                constants.L3_ROUTER_NAT)
+            self._l3_plugin = directory.get_plugin(constants.L3)
         return self._l3_plugin
 
+    @staticmethod
+    @resource_extend.extends([net_def.COLLECTION_NAME])
+    def _extend_external_network_default(net_res, net_db):
+        """Add is_default field to 'show' response."""
+        if net_db.external is not None:
+            net_res[api_const.IS_DEFAULT] = net_db.external.is_default
+        return net_res
+
     def get_auto_allocated_topology(self, context, tenant_id, fields=None):
         """Return tenant's network associated to auto-allocated topology.
 
@@ -124,7 +144,7 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
             return self._check_requirements(context, tenant_id)
         elif fields:
             raise n_exc.BadRequest(resource='auto_allocate',
-                msg=_("Unrecognized field"))
+                                   msg=_("Unrecognized field"))
 
         # Check for an existent topology
         network_id = self._get_auto_allocated_network(context, tenant_id)
@@ -165,8 +185,8 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
         except exceptions.UnknownProvisioningError as e:
             # Clean partially provisioned topologies, and reraise the
             # error. If it can be retried, so be it.
-            LOG.error(_LE("Unknown error while provisioning topology for "
-                          "tenant %(tenant_id)s. Reason: %(reason)s"),
+            LOG.error("Unknown error while provisioning topology for "
+                      "tenant %(tenant_id)s. Reason: %(reason)s",
                       {'tenant_id': tenant_id, 'reason': e})
             self._cleanup(
                 context, network_id=e.network_id,
@@ -198,9 +218,8 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
 
     def _get_auto_allocated_topology(self, context, tenant_id):
         """Return the auto allocated topology record if present or None."""
-        with context.session.begin(subtransactions=True):
-            return (context.session.query(models.AutoAllocatedTopology).
-                filter_by(tenant_id=tenant_id).first())
+        return auto_allocate_obj.AutoAllocatedTopology.get_object(
+            context, project_id=tenant_id)
 
     def _get_auto_allocated_network(self, context, tenant_id):
         """Get the auto allocated network for the tenant."""
@@ -208,33 +227,30 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
         if network:
             return network['network_id']
 
-    def _response(self, network_id, tenant_id, fields=None):
+    @staticmethod
+    def _response(network_id, tenant_id, fields=None):
         """Build response for auto-allocated network."""
         res = {
             'id': network_id,
             'tenant_id': tenant_id
         }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     def _get_default_external_network(self, context):
         """Get the default external network for the deployment."""
-        with context.session.begin(subtransactions=True):
-            default_external_networks = (context.session.query(
-                external_net_db.ExternalNetwork).
-                filter_by(is_default=sql.true()).
-                join(models_v2.Network).
-                join(standard_attr.StandardAttribute).
-                order_by(standard_attr.StandardAttribute.id).all())
+
+        default_external_networks = net_obj.ExternalNetwork.get_objects(
+            context, is_default=True)
 
         if not default_external_networks:
-            LOG.error(_LE("Unable to find default external network "
-                          "for deployment, please create/assign one to "
-                          "allow auto-allocation to work correctly."))
+            LOG.error("Unable to find default external network "
+                      "for deployment, please create/assign one to "
+                      "allow auto-allocation to work correctly.")
             raise exceptions.AutoAllocationFailure(
                 reason=_("No default router:external network"))
         if len(default_external_networks) > 1:
-            LOG.error(_LE("Multiple external default networks detected. "
-                          "Network %s is true 'default'."),
+            LOG.error("Multiple external default networks detected. "
+                      "Network %s is true 'default'.",
                       default_external_networks[0]['network_id'])
         return default_external_networks[0].network_id
 
@@ -248,7 +264,7 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
             s for s in default_subnet_pools if s
         ]
         if not available_pools:
-            LOG.error(_LE("No default pools available"))
+            LOG.error("No default pools available")
             raise n_exc.NotFound()
 
         return available_pools
@@ -277,11 +293,11 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
                 subnets.append(p_utils.create_subnet(
                     self.core_plugin, context, {'subnet': subnet_args}))
             return subnets
-        except (c_exc.SubnetAllocationError, ValueError,
+        except (n_exc.SubnetAllocationError, ValueError,
                 n_exc.BadRequest, n_exc.NotFound) as e:
-            LOG.error(_LE("Unable to auto allocate topology for tenant "
-                          "%(tenant_id)s due to missing or unmet "
-                          "requirements. Reason: %(reason)s"),
+            LOG.error("Unable to auto allocate topology for tenant "
+                      "%(tenant_id)s due to missing or unmet "
+                      "requirements. Reason: %(reason)s",
                       {'tenant_id': tenant_id, 'reason': e})
             if network:
                 self._cleanup(context, network['id'])
@@ -291,12 +307,14 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
             network_id = network['id'] if network else None
             raise exceptions.UnknownProvisioningError(e, network_id=network_id)
 
-    def _provision_external_connectivity(
-        self, context, default_external_network, subnets, tenant_id):
+    def _provision_external_connectivity(self, context,
+                                         default_external_network, subnets,
+                                         tenant_id):
         """Uplink tenant subnet(s) to external network."""
         router_args = {
             'name': 'auto_allocated_router',
-            l3.EXTERNAL_GW_INFO: {'network_id': default_external_network},
+            l3_apidef.EXTERNAL_GW_INFO: {
+                'network_id': default_external_network},
             'tenant_id': tenant_id,
             'admin_state_up': True
         }
@@ -311,14 +329,14 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
                 attached_subnets.append(subnet)
             return router
         except n_exc.BadRequest as e:
-            LOG.error(_LE("Unable to auto allocate topology for tenant "
-                          "%(tenant_id)s because of router errors. "
-                          "Reason: %(reason)s"),
+            LOG.error("Unable to auto allocate topology for tenant "
+                      "%(tenant_id)s because of router errors. "
+                      "Reason: %(reason)s",
                       {'tenant_id': tenant_id, 'reason': e})
             router_id = router['id'] if router else None
             self._cleanup(context,
-                network_id=subnets[0]['network_id'],
-                router_id=router_id, subnets=attached_subnets)
+                          network_id=subnets[0]['network_id'],
+                          router_id=router_id, subnets=attached_subnets)
             raise exceptions.AutoAllocationFailure(
                 reason=_("Unable to provide external connectivity"))
         except Exception as e:
@@ -330,27 +348,17 @@ class AutoAllocatedTopologyMixin(common_db_mixin.CommonDbMixin):
     def _save(self, context, tenant_id, network_id, router_id, subnets):
         """Save auto-allocated topology, or revert in case of DB errors."""
         try:
-            # NOTE(armax): saving the auto allocated topology in a
-            # separate transaction will keep the Neutron DB and the
-            # Neutron plugin backend in sync, thus allowing for a
-            # more bullet proof cleanup. Any other error will have
-            # to bubble up.
-            with context.session.begin(subtransactions=True):
-                context.session.add(
-                    models.AutoAllocatedTopology(
-                        tenant_id=tenant_id,
-                        network_id=network_id,
-                        router_id=router_id))
+            auto_allocate_obj.AutoAllocatedTopology(
+                context, project_id=tenant_id, network_id=network_id,
+                router_id=router_id).create()
             self.core_plugin.update_network(
                 context, network_id,
                 {'network': {'admin_state_up': True}})
-        except db_exc.DBDuplicateEntry:
-            LOG.error(_LE("Multiple auto-allocated networks detected for "
-                          "tenant %(tenant)s. Attempting clean up for "
-                          "network %(network)s and router %(router)s"),
-                      {'tenant': tenant_id,
-                       'network': network_id,
-                       'router': router_id})
+        except obj_exc.NeutronDbObjectDuplicateEntry:
+            LOG.debug("Multiple auto-allocated networks detected for "
+                      "tenant %s. Attempting clean up for network %s "
+                      "and router %s.",
+                      tenant_id, network_id, router_id)
             self._cleanup(
                 context, network_id=network_id,
                 router_id=router_id, subnets=subnets)
