@@ -16,41 +16,48 @@
 """Base test cases for all neutron tests.
 """
 
+import abc
 import contextlib
-import gc
+import functools
+import inspect
+import logging
 import os
 import os.path
-import sys
-import weakref
+import queue
+import threading
+from unittest import mock
 
-from debtcollector import moves
 import eventlet.timeout
 import fixtures
-import mock
+from neutron_lib.callbacks import manager as registry_manager
+from neutron_lib.db import api as db_api
+from neutron_lib import fixture
+from neutron_lib.tests import tools as lib_test_tools
+from neutron_lib.tests.unit import fake_notifier
 from oslo_concurrency.fixture import lockutils
 from oslo_config import cfg
-from oslo_messaging import conffixture as messaging_conffixture
+from oslo_db import exception as db_exceptions
+from oslo_db import options as db_options
 from oslo_utils import excutils
+from oslo_utils import fileutils
 from oslo_utils import strutils
 from oslotest import base
-import six
+from osprofiler import profiler
+from sqlalchemy import exc as sqlalchemy_exc
 import testtools
+from testtools import content
 
 from neutron._i18n import _
 from neutron.agent.linux import external_process
 from neutron.api.rpc.callbacks.consumer import registry as rpc_consumer_reg
-from neutron.callbacks import manager as registry_manager
-from neutron.callbacks import registry
+from neutron.api.rpc.callbacks.producer import registry as rpc_producer_reg
 from neutron.common import config
-from neutron.common import rpc as n_rpc
-from neutron.common import utils
+from neutron.conf.agent import common as agent_config
 from neutron.db import agentschedulers_db
 from neutron import manager
 from neutron import policy
 from neutron.quota import resource_registry
-from neutron.tests import fake_notifier
 from neutron.tests import post_mortem_debug
-from neutron.tests import tools
 
 
 CONF = cfg.CONF
@@ -59,6 +66,10 @@ CONF.import_opt('state_path', 'neutron.conf.common')
 ROOTDIR = os.path.dirname(__file__)
 ETCDIR = os.path.join(ROOTDIR, 'etc')
 
+SUDO_CMD = 'sudo -n'
+
+TESTCASE_RETRIES = 3
+
 
 def etcdir(*p):
     return os.path.join(ETCDIR, *p)
@@ -68,16 +79,6 @@ def fake_use_fatal_exceptions(*args):
     return True
 
 
-for _name in ('get_related_rand_names',
-              'get_rand_name',
-              'get_rand_device_name',
-              'get_related_rand_device_names'):
-    setattr(sys.modules[__name__], _name, moves.moved_function(
-        getattr(utils, _name), _name, __name__,
-        message='use "neutron.common.utils.%s" instead' % _name,
-        version='Newton', removal_version='Ocata'))
-
-
 def bool_from_env(key, strict=False, default=False):
     value = os.environ.get(key)
     return strutils.bool_from_string(value, strict=strict, default=default)
@@ -86,27 +87,85 @@ def bool_from_env(key, strict=False, default=False):
 def setup_test_logging(config_opts, log_dir, log_file_path_template):
     # Have each test log into its own log file
     config_opts.set_override('debug', True)
-    utils.ensure_dir(log_dir)
+    fileutils.ensure_tree(log_dir, mode=0o755)
     log_file = sanitize_log_path(
         os.path.join(log_dir, log_file_path_template))
     config_opts.set_override('log_file', log_file)
-    config_opts.set_override('use_stderr', False)
     config.setup_logging()
 
 
 def sanitize_log_path(path):
     # Sanitize the string so that its log path is shell friendly
     replace_map = {' ': '-', '(': '_', ')': '_'}
-    for s, r in six.iteritems(replace_map):
+    for s, r in replace_map.items():
         path = path.replace(s, r)
     return path
 
 
+def unstable_test(reason):
+    def decor(f):
+        @functools.wraps(f)
+        def inner(self, *args, **kwargs):
+            try:
+                return f(self, *args, **kwargs)
+            except Exception as e:
+                msg = ("%s was marked as unstable because of %s, "
+                       "failure was: %s") % (self.id(), reason, e)
+                raise self.skipTest(msg)
+        return inner
+    return decor
+
+
+def skip_if_timeout(reason):
+    def decor(f):
+        @functools.wraps(f)
+        def inner(self, *args, **kwargs):
+            try:
+                return f(self, *args, **kwargs)
+            except fixtures.TimeoutException:
+                msg = ("Timeout raised for test %s, skipping it "
+                       "because of: %s") % (self.id(), reason)
+                raise self.skipTest(msg)
+            except (sqlalchemy_exc.InterfaceError,
+                    db_exceptions.DBConnectionError):
+                # In case of db tests very often TimeoutException is reason of
+                # some sqlalchemy InterfaceError exception and that is final
+                # raised exception which needs to be handled
+                msg = ("DB connection broken in test %s. It is very likely "
+                       "that this happend because of test timeout. "
+                       "Skipping test because of: %s") % (self.id(), reason)
+                raise self.skipTest(msg)
+        return inner
+    return decor
+
+
+def set_timeout(timeout):
+    """Timeout decorator for test methods.
+
+    Use this decorator for tests that are expected to pass in very specific
+    amount of time, not common for all other tests.
+    It can have either big or small value.
+    """
+    def decor(f):
+        @functools.wraps(f)
+        def inner(self, *args, **kwargs):
+            self.useFixture(fixtures.Timeout(timeout, gentle=True))
+            return f(self, *args, **kwargs)
+        return inner
+    return decor
+
+
+def get_rootwrap_cmd():
+    return os.environ.get('OS_ROOTWRAP_CMD', SUDO_CMD)
+
+
+def get_rootwrap_daemon_cmd():
+    return os.environ.get('OS_ROOTWRAP_DAEMON_CMD')
+
+
 class AttributeDict(dict):
 
-    """
-    Provide attribute access (dict.key) to dictionary values.
-    """
+    """Provide attribute access (dict.key) to dictionary values."""
 
     def __getattr__(self, name):
         """Allow attribute access for all keys in the dict."""
@@ -115,7 +174,48 @@ class AttributeDict(dict):
         raise AttributeError(_("Unknown attribute '%s'.") % name)
 
 
-class DietTestCase(base.BaseTestCase):
+def _catch_timeout(f):
+    @functools.wraps(f)
+    def func(self, *args, **kwargs):
+        for idx in range(1, TESTCASE_RETRIES + 1):
+            try:
+                return f(self, *args, **kwargs)
+            except eventlet.Timeout as e:
+                self.fail('Execution of this test timed out: %s' % e)
+            # NOTE(ralonsoh): exception catch added due to the constant
+            # occurrences of this exception during FT and UT execution.
+            # This is due to [1]. Once the sync decorators are removed or the
+            # privsep ones are decorated by those ones (swap decorator
+            # declarations) this catch can be remove.
+            # [1] https://review.opendev.org/#/c/631275/
+            except fixtures.TimeoutException:
+                if idx < TESTCASE_RETRIES:
+                    msg = ('"fixtures.TimeoutException" during test case '
+                           'execution no %s; test case re-executed' % idx)
+                    self.addDetail('DietTestCase',
+                                   content.text_content(msg))
+                    self._set_timeout()
+                else:
+                    self.fail('Execution of this test timed out')
+    return func
+
+
+class _CatchTimeoutMetaclass(abc.ABCMeta):
+    def __init__(cls, name, bases, dct):
+        super(_CatchTimeoutMetaclass, cls).__init__(name, bases, dct)
+        for name, method in inspect.getmembers(
+                # NOTE(ihrachys): we should use isroutine because it will catch
+                # both unbound methods (python2) and functions (python3)
+                cls, predicate=inspect.isroutine):
+            if name.startswith('test_'):
+                setattr(cls, name, _catch_timeout(method))
+
+
+# Test worker cannot survive eventlet's Timeout exception, which effectively
+# kills the whole worker, with all test cases scheduled to it. This metaclass
+# makes all test cases convert Timeout exceptions into unittest friendly
+# failure mode (self.fail).
+class DietTestCase(base.BaseTestCase, metaclass=_CatchTimeoutMetaclass):
     """Same great taste, less filling.
 
     BaseTestCase is responsible for doing lots of plugin-centric setup
@@ -126,10 +226,21 @@ class DietTestCase(base.BaseTestCase):
     def setUp(self):
         super(DietTestCase, self).setUp()
 
+        # Suppress some log messages during test runs, otherwise it may cause
+        # issues with subunit parser when running on Python 3. It happened for
+        # example for neutron-functional tests.
+        # With this suppress of log levels DEBUG logs will not be captured by
+        # stestr on pythonlogging stream and will not cause this parser issue.
+        supress_logs = ['neutron', 'neutron_lib', 'stevedore', 'oslo_policy',
+                        'oslo_concurrency', 'oslo_db', 'alembic', 'ovsdbapp']
+        for supress_log in supress_logs:
+            logger = logging.getLogger(supress_log)
+            logger.setLevel(logging.ERROR)
+
         # FIXME(amuller): this must be called in the Neutron unit tests base
-        # class to initialize the DB connection string. Moving this may cause
-        # non-deterministic failures. Bug #1489098 for more info.
-        config.set_db_defaults()
+        # class. Moving this may cause non-deterministic failures. Bug #1489098
+        # for more info.
+        db_options.set_defaults(cfg.CONF, connection='sqlite://')
 
         # Configure this first to ensure pm debugging support for setUp()
         debugger = os.environ.get('OS_POST_MORTEM_DEBUGGER')
@@ -138,7 +249,9 @@ class DietTestCase(base.BaseTestCase):
                 debugger))
 
         # Make sure we see all relevant deprecation warnings when running tests
-        self.useFixture(tools.WarningsFixture())
+        self.useFixture(fixture.WarningsFixture(module_re=['^neutron\\.']))
+
+        self.useFixture(fixture.DBQueryHooksFixture())
 
         # NOTE(ihrachys): oslotest already sets stopall for cleanup, but it
         # does it using six.moves.mock (the library was moved into
@@ -148,10 +261,12 @@ class DietTestCase(base.BaseTestCase):
         # six before removing the cleanup callback from here.
         self.addCleanup(mock.patch.stopall)
 
+        self.useFixture(fixture.DBResourceExtendFixture())
+
         self.addOnException(self.check_for_systemexit)
         self.orig_pid = os.getpid()
 
-        tools.reset_random_seed()
+        lib_test_tools.reset_random_seed()
 
     def addOnException(self, handler):
 
@@ -176,7 +291,7 @@ class DietTestCase(base.BaseTestCase):
 
     @contextlib.contextmanager
     def assert_max_execution_time(self, max_execution_time=5):
-        with eventlet.timeout.Timeout(max_execution_time, False):
+        with eventlet.Timeout(max_execution_time, False):
             yield
             return
         self.fail('Execution of this test timed out')
@@ -187,7 +302,7 @@ class DietTestCase(base.BaseTestCase):
         self.assertEqual(expect_val, actual_val)
 
     def sort_dict_lists(self, dic):
-        for key, value in six.iteritems(dic):
+        for key, value in dic.items():
             if isinstance(value, list):
                 dic[key] = sorted(value)
             elif isinstance(value, dict):
@@ -242,7 +357,6 @@ class BaseTestCase(DietTestCase):
     @staticmethod
     def config_parse(conf=None, args=None):
         """Create the default configurations."""
-        # neutron.conf includes rpc_backend which needs to be cleaned up
         if args is None:
             args = []
         args += ['--config-file', etcdir('neutron.conf')]
@@ -255,6 +369,7 @@ class BaseTestCase(DietTestCase):
         super(BaseTestCase, self).setUp()
 
         self.useFixture(lockutils.ExternalLockFixture())
+        self.useFixture(fixture.APIDefinitionFixture())
 
         cfg.CONF.set_override('state_path', self.get_default_temp_dir().path)
 
@@ -269,19 +384,28 @@ class BaseTestCase(DietTestCase):
             'oslo_config.cfg.find_config_files',
             lambda project=None, prog=None, extension=None: []))
 
-        self.setup_rpc_mocks()
+        self.useFixture(fixture.RPCFixture())
+
         self.setup_config()
-        self.setup_test_registry_instance()
+
+        self._callback_manager = registry_manager.CallbacksManager()
+        self.useFixture(fixture.CallbackRegistryFixture(
+            callback_manager=self._callback_manager))
+        # Give a private copy of the directory to each test.
+        self.useFixture(fixture.PluginDirectoryFixture())
 
         policy.init()
         self.addCleanup(policy.reset)
         self.addCleanup(resource_registry.unregister_all_resources)
+        self.addCleanup(db_api.sqla_remove_all)
         self.addCleanup(rpc_consumer_reg.clear)
+        self.addCleanup(rpc_producer_reg.clear)
+        self.addCleanup(profiler.clean)
 
     def get_new_temp_dir(self):
         """Create a new temporary directory.
 
-        :returns fixtures.TempDir
+        :returns: fixtures.TempDir
         """
         return self.useFixture(fixtures.TempDir())
 
@@ -290,7 +414,7 @@ class BaseTestCase(DietTestCase):
 
         Returns the same directory during the whole test case.
 
-        :returns fixtures.TempDir
+        :returns: fixtures.TempDir
         """
         if not hasattr(self, '_temp_dir'):
             self._temp_dir = self.get_new_temp_dir()
@@ -309,38 +433,11 @@ class BaseTestCase(DietTestCase):
         :type filename: string
         :param root: temporary directory to create a new file in
         :type root: fixtures.TempDir
-        :returns absolute file path string
+        :returns: absolute file path string
         """
         root = root or self.get_default_temp_dir()
         return root.join(filename)
 
-    def setup_rpc_mocks(self):
-        # don't actually start RPC listeners when testing
-        mock.patch(
-            'neutron.common.rpc.Connection.consume_in_threads',
-            return_value=[]).start()
-
-        self.useFixture(fixtures.MonkeyPatch(
-            'oslo_messaging.Notifier', fake_notifier.FakeNotifier))
-
-        self.messaging_conf = messaging_conffixture.ConfFixture(CONF)
-        self.messaging_conf.transport_driver = 'fake'
-        # NOTE(russellb) We want all calls to return immediately.
-        self.messaging_conf.response_timeout = 0
-        self.useFixture(self.messaging_conf)
-
-        self.addCleanup(n_rpc.clear_extra_exmods)
-        n_rpc.add_extra_exmods('neutron.test')
-
-        self.addCleanup(n_rpc.cleanup)
-        n_rpc.init(CONF)
-
-    def setup_test_registry_instance(self):
-        """Give a private copy of the registry to each test."""
-        self._callback_manager = registry_manager.CallbacksManager()
-        mock.patch.object(registry, '_get_callback_manager',
-                          return_value=self._callback_manager).start()
-
     def setup_config(self, args=None):
         """Tests that need a non-default config can override this method."""
         self.config_parse(args=args)
@@ -358,14 +455,16 @@ class BaseTestCase(DietTestCase):
         test by the fixtures cleanup process.
         """
         group = kw.pop('group', None)
-        for k, v in six.iteritems(kw):
+        for k, v in kw.items():
             CONF.set_override(k, v, group)
 
-    def setup_coreplugin(self, core_plugin=None):
+    def setup_coreplugin(self, core_plugin=None, load_plugins=True):
         cp = PluginFixture(core_plugin)
         self.useFixture(cp)
         self.patched_dhcp_periodic = cp.patched_dhcp_periodic
         self.patched_default_svc_plugins = cp.patched_default_svc_plugins
+        if load_plugins:
+            manager.init()
 
     def setup_notification_driver(self, notification_driver=None):
         self.addCleanup(fake_notifier.reset)
@@ -373,6 +472,56 @@ class BaseTestCase(DietTestCase):
             notification_driver = [fake_notifier.__name__]
         cfg.CONF.set_override("notification_driver", notification_driver)
 
+    def setup_rootwrap(self):
+        agent_config.register_root_helper(cfg.CONF)
+        self.config(group='AGENT',
+                    root_helper=get_rootwrap_cmd())
+        self.config(group='AGENT',
+                    root_helper_daemon=get_rootwrap_daemon_cmd())
+
+    def _simulate_concurrent_requests_process_and_raise(self, calls, args):
+
+        class SimpleThread(threading.Thread):
+            def __init__(self, q):
+                super(SimpleThread, self).__init__()
+                self.q = q
+                self.exception = None
+
+            def run(self):
+                try:
+                    while not self.q.empty():
+                        item = None
+                        try:
+                            item = self.q.get(False)
+                            func, func_args = item[0], item[1]
+                            func(*func_args)
+                        except queue.Empty:
+                            pass
+                        finally:
+                            if item:
+                                self.q.task_done()
+                except Exception as e:
+                    self.exception = e
+
+            def get_exception(self):
+                return self.exception
+
+        q = queue.Queue()
+        for func, func_args in zip(calls, args):
+            q.put_nowait((func, func_args))
+
+        threads = []
+        for z in range(len(calls)):
+            t = SimpleThread(q)
+            threads.append(t)
+            t.start()
+        q.join()
+
+        for t in threads:
+            e = t.get_exception()
+            if e:
+                raise e
+
 
 class PluginFixture(fixtures.Fixture):
 
@@ -409,24 +558,8 @@ class PluginFixture(fixtures.Fixture):
         # TODO(marun) Fix plugins that do not properly initialize notifiers
         agentschedulers_db.AgentSchedulerDbMixin.agent_notifiers = {}
 
-        # Perform a check for deallocation only if explicitly
-        # configured to do so since calling gc.collect() after every
-        # test increases test suite execution time by ~50%.
-        check_plugin_deallocation = (
-            bool_from_env('OS_CHECK_PLUGIN_DEALLOCATION'))
-        if check_plugin_deallocation:
-            plugin = weakref.ref(nm._instance.plugin)
-
         nm.clear_instance()
 
-        if check_plugin_deallocation:
-            gc.collect()
-
-            # TODO(marun) Ensure that mocks are deallocated?
-            if plugin() and not isinstance(plugin(), mock.Base):
-                raise AssertionError(
-                    'The plugin for this test was not deallocated.')
-
 
 class Timeout(fixtures.Fixture):
     """Setup per test timeouts.
