@@ -13,19 +13,19 @@
 #    under the License.
 
 import datetime
+from distutils import version
+import functools
 import os
 import random
 
+from neutron_lib.agent import topics
 from neutron_lib import constants
+from neutron_lib import context
 from oslo_utils import timeutils
-import six
-import testtools
 
 import neutron
-from neutron.common import topics
-from neutron import context
+from neutron.agent.common import ovs_lib
 from neutron.db import agents_db
-from neutron.db import common_db_mixin
 
 HOST = 'localhost'
 DEFAULT_AZ = 'nova'
@@ -49,46 +49,40 @@ def get_test_log_path():
     return os.environ.get('OS_LOG_PATH', '/tmp')
 
 
-class FakePlugin(common_db_mixin.CommonDbMixin,
-                 agents_db.AgentDbMixin):
+class FakePlugin(agents_db.AgentDbMixin):
     pass
 
 
 def _get_l3_agent_dict(host, agent_mode, internal_only=True,
-                       ext_net_id='', ext_bridge='',
                        az=DEFAULT_AZ):
     return {
         'agent_type': constants.AGENT_TYPE_L3,
-        'binary': 'neutron-l3-agent',
+        'binary': constants.AGENT_PROCESS_L3,
         'host': host,
         'topic': topics.L3_AGENT,
         'availability_zone': az,
         'configurations': {'agent_mode': agent_mode,
-                           'handle_internal_only_routers': internal_only,
-                           'external_network_bridge': ext_bridge,
-                           'gateway_external_network_id': ext_net_id}}
+                           'handle_internal_only_routers': internal_only}}
 
 
 def _register_agent(agent, plugin=None):
     if not plugin:
         plugin = FakePlugin()
     admin_context = context.get_admin_context()
-    plugin.create_or_update_agent(admin_context, agent)
+    plugin.create_or_update_agent(admin_context, agent, timeutils.utcnow())
     return plugin._get_agent_by_type_and_host(
         admin_context, agent['agent_type'], agent['host'])
 
 
 def register_l3_agent(host=HOST, agent_mode=constants.L3_AGENT_MODE_LEGACY,
-                      internal_only=True, ext_net_id='', ext_bridge='',
-                      az=DEFAULT_AZ):
-    agent = _get_l3_agent_dict(host, agent_mode, internal_only, ext_net_id,
-                               ext_bridge, az)
+                      internal_only=True, az=DEFAULT_AZ):
+    agent = _get_l3_agent_dict(host, agent_mode, internal_only, az)
     return _register_agent(agent)
 
 
 def _get_dhcp_agent_dict(host, networks=0, az=DEFAULT_AZ):
     agent = {
-        'binary': 'neutron-dhcp-agent',
+        'binary': constants.AGENT_PROCESS_DHCP,
         'host': host,
         'topic': topics.DHCP_AGENT,
         'agent_type': constants.AGENT_TYPE_DHCP,
@@ -139,7 +133,8 @@ def set_agent_admin_state(agent_id, admin_state_up=False):
 def _get_l2_agent_dict(host, agent_type, binary, tunnel_types=None,
                        tunneling_ip='20.0.0.1', interface_mappings=None,
                        bridge_mappings=None, l2pop_network_types=None,
-                       device_mappings=None, start_flag=True):
+                       device_mappings=None, start_flag=True,
+                       integration_bridge=None):
     agent = {
         'binary': binary,
         'host': host,
@@ -160,24 +155,28 @@ def _get_l2_agent_dict(host, agent_type, binary, tunnel_types=None,
         agent['configurations']['l2pop_network_types'] = l2pop_network_types
     if device_mappings is not None:
         agent['configurations']['device_mappings'] = device_mappings
+    if integration_bridge is not None:
+        agent['configurations']['integration_bridge'] = integration_bridge
     return agent
 
 
 def register_ovs_agent(host=HOST, agent_type=constants.AGENT_TYPE_OVS,
-                       binary='neutron-openvswitch-agent',
+                       binary=constants.AGENT_PROCESS_OVS,
                        tunnel_types=['vxlan'], tunneling_ip='20.0.0.1',
                        interface_mappings=None, bridge_mappings=None,
-                       l2pop_network_types=None, plugin=None, start_flag=True):
+                       l2pop_network_types=None, plugin=None, start_flag=True,
+                       integration_bridge=None):
     agent = _get_l2_agent_dict(host, agent_type, binary, tunnel_types,
                                tunneling_ip, interface_mappings,
                                bridge_mappings, l2pop_network_types,
-                               start_flag=start_flag)
+                               start_flag=start_flag,
+                               integration_bridge=integration_bridge)
     return _register_agent(agent, plugin)
 
 
 def register_linuxbridge_agent(host=HOST,
                                agent_type=constants.AGENT_TYPE_LINUXBRIDGE,
-                               binary='neutron-linuxbridge-agent',
+                               binary=constants.AGENT_PROCESS_LINUXBRIDGE,
                                tunnel_types=['vxlan'], tunneling_ip='20.0.0.1',
                                interface_mappings=None, bridge_mappings=None,
                                plugin=None):
@@ -190,7 +189,7 @@ def register_linuxbridge_agent(host=HOST,
 
 def register_macvtap_agent(host=HOST,
                            agent_type=constants.AGENT_TYPE_MACVTAP,
-                           binary='neutron-macvtap-agent',
+                           binary=constants.AGENT_PROCESS_MACVTAP,
                            interface_mappings=None, plugin=None):
     agent = _get_l2_agent_dict(host, agent_type, binary,
                                interface_mappings=interface_mappings)
@@ -199,24 +198,34 @@ def register_macvtap_agent(host=HOST,
 
 def register_sriovnicswitch_agent(host=HOST,
                                   agent_type=constants.AGENT_TYPE_NIC_SWITCH,
-                                  binary='neutron-sriov-nic-agent',
+                                  binary=constants.AGENT_PROCESS_NIC_SWITCH,
                                   device_mappings=None, plugin=None):
     agent = _get_l2_agent_dict(host, agent_type, binary,
                                device_mappings=device_mappings)
     return _register_agent(agent, plugin)
 
 
-def requires_py2(testcase):
-    return testtools.skipUnless(six.PY2, "requires python 2.x")(testcase)
-
-
-def requires_py3(testcase):
-    return testtools.skipUnless(six.PY3, "requires python 3.x")(testcase)
-
-
 def get_not_used_vlan(bridge, vlan_range):
     port_vlans = bridge.ovsdb.db_find(
         'Port', ('tag', '!=', []), columns=['tag']).execute()
     used_vlan_tags = {val['tag'] for val in port_vlans}
     available_vlans = vlan_range - used_vlan_tags
     return random.choice(list(available_vlans))
+
+
+def skip_if_ovs_older_than(ovs_version):
+    """Decorator for test method to skip if OVS version doesn't meet
+       minimal requirement.
+    """
+    def skip_if_bad_ovs(f):
+        @functools.wraps(f)
+        def check_ovs_and_skip(test):
+            ovs = ovs_lib.BaseOVS()
+            current_ovs_version = version.StrictVersion(
+                ovs.config['ovs_version'])
+            if current_ovs_version < version.StrictVersion(ovs_version):
+                test.skipTest("This test requires OVS version %s or higher." %
+                              ovs_version)
+            return f(test)
+        return check_ovs_and_skip
+    return skip_if_bad_ovs
