@@ -28,10 +28,10 @@ import time
 import fixtures
 import netaddr
 from neutron_lib import constants as n_const
+from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
-import six
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import bridge_lib
@@ -41,7 +41,7 @@ from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import utils
 from neutron.common import utils as common_utils
 from neutron.conf.agent import common as config
-from neutron.db import db_base_plugin_common
+from neutron.db import db_base_plugin_common as db_base
 from neutron.plugins.ml2.drivers.linuxbridge.agent import \
     linuxbridge_neutron_agent as linuxbridge_agent
 from neutron.tests.common import base as common_base
@@ -110,7 +110,7 @@ def assert_ping(src_namespace, dst_ip, timeout=1, count=3):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute([ping_command, '-c', count, '-W', timeout,
+    ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', count,
                                  dst_ip])
 
 
@@ -123,7 +123,7 @@ def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
     # cannot be used and it needs to be done using the following workaround.
     for _index in range(count):
         start_time = time.time()
-        ns_ip_wrapper.netns.execute([ping_command, '-c', '1', '-W', timeout,
+        ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', '1',
                                      dst_ip])
         end_time = time.time()
         diff = end_time - start_time
@@ -228,6 +228,18 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
     return get_unused_port(used_ports, start, end)
 
 
+def set_local_port_range(start, end):
+    utils.execute(
+        ['sysctl', '-w', 'net.ipv4.ip_local_port_range=%d %d' % (start, end)],
+        run_as_root=True)
+    utils.execute(['sysctl', '-p'], run_as_root=True)
+    # verify
+    port_range = utils.execute(
+        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+    assert int(port_range.split()[0]) == start
+    assert int(port_range.split()[1]) == end
+
+
 def create_patch_ports(source, destination):
     """Hook up two OVS bridges.
 
@@ -314,11 +326,25 @@ class RootHelperProcess(subprocess.Popen):
             if utils.pid_invoked_with_cmdline(child_pid, self.cmd):
                 return True
 
-        common_utils.wait_until_true(
-            child_is_running,
-            timeout,
-            exception=RuntimeError("Process %s hasn't been spawned "
-                                   "in %d seconds" % (self.cmd, timeout)))
+        try:
+            common_utils.wait_until_true(child_is_running, timeout)
+        except common_utils.WaitTimeout:
+            # If there is an error, the stderr and stdout pipes usually have
+            # information returned by the command executed. If not, timeout
+            # the pipe communication quickly.
+            stdout = stderr = ''
+            try:
+                stdout, stderr = self.communicate(timeout=0.5)
+            except subprocess.TimeoutExpired:
+                pass
+            msg = ("Process %(cmd)s hasn't been spawned in %(seconds)d "
+                   "seconds. Return code: %(ret_code)s, stdout: %(stdout)s, "
+                   "sdterr: %(stderr)s" %
+                   {'cmd': self.cmd, 'seconds': timeout,
+                    'ret_code': self.returncode, 'stdout': stdout,
+                    'stderr': stderr})
+            raise RuntimeError(msg)
+
         self.child_pid = utils.get_root_helper_child_pid(
             self.pid, self.cmd, run_as_root=True)
 
@@ -388,12 +414,13 @@ class Pinger(object):
         if self.proc and self.proc.is_running:
             raise RuntimeError("This pinger has already a running process")
         ip_version = common_utils.get_ip_version(self.address)
-        ping_exec = 'ping' if ip_version == 4 else 'ping6'
-        cmd = [ping_exec, self.address, '-W', str(self.timeout)]
+        ping_exec = 'ping' if ip_version == n_const.IP_VERSION_4 else 'ping6'
+        cmd = [ping_exec, '-W', str(self.timeout)]
         if self.count:
             cmd.extend(['-c', str(self.count)])
         if self.interval:
             cmd.extend(['-i', str(self.interval)])
+        cmd.append(self.address)
         self.proc = RootHelperProcess(cmd, namespace=self.namespace)
 
     def stop(self):
@@ -416,14 +443,15 @@ class NetcatTester(object):
     UDP = n_const.PROTO_NAME_UDP
     SCTP = n_const.PROTO_NAME_SCTP
     VERSION_TO_ALL_ADDRESS = {
-        4: '0.0.0.0',
-        6: '::',
+        n_const.IP_VERSION_4: '0.0.0.0',
+        n_const.IP_VERSION_6: '::',
     }
 
     def __init__(self, client_namespace, server_namespace, address,
                  dst_port, protocol, server_address=None, src_port=None):
 
-        """
+        """Initialize NetcatTester
+
         Tool for testing connectivity on transport layer using netcat
         executable.
 
@@ -510,10 +538,14 @@ class NetcatTester(object):
         if respawn:
             self.stop_processes()
 
-        self.client_process.writeline(testing_string)
-        message = self.server_process.read_stdout(READ_TIMEOUT).strip()
-        self.server_process.writeline(message)
-        message = self.client_process.read_stdout(READ_TIMEOUT).strip()
+        try:
+            self.client_process.writeline(testing_string)
+            message = self.server_process.read_stdout(READ_TIMEOUT).strip()
+            self.server_process.writeline(message)
+            message = self.client_process.read_stdout(READ_TIMEOUT).strip()
+        except ConnectionError as e:
+            LOG.debug("Error: %s occured during connectivity test.", e)
+            message = ""
 
         return message == testing_string
 
@@ -541,13 +573,22 @@ class NetcatTester(object):
         proc = RootHelperProcess(cmd, namespace=namespace)
         return proc
 
-    def stop_processes(self):
+    def stop_processes(self, skip_errors=None):
+        skip_errors = (['No such process'] if skip_errors is None
+                       else skip_errors)
         for proc_attr in ('_client_process', '_server_process'):
             proc = getattr(self, proc_attr)
             if proc:
-                if proc.poll() is None:
-                    proc.kill()
-                    proc.wait()
+                try:
+                    if proc.poll() is None:
+                        proc.kill()
+                        proc.wait()
+                except n_exc.ProcessExecutionError as exc:
+                    for skip_error in skip_errors:
+                        if skip_error in str(exc):
+                            break
+                    else:
+                        raise exc
                 setattr(self, proc_attr, None)
 
 
@@ -572,6 +613,8 @@ class NamespaceFixture(fixtures.Fixture):
 
     def destroy(self):
         if self.ip_wrapper.netns.exists(self.name):
+            for pid in ip_lib.list_namespace_pids(self.name):
+                utils.kill_process(pid, signal.SIGKILL, run_as_root=True)
             self.ip_wrapper.netns.delete(self.name)
 
 
@@ -595,7 +638,7 @@ class VethFixture(fixtures.Fixture):
         for port in self.ports:
             ip_wrapper = ip_lib.IPWrapper(port.namespace)
             if (ip_wrapper.netns.exists(port.namespace) or
-                port.namespace is None):
+                    port.namespace is None):
                 try:
                     ip_wrapper.del_veth(port.name)
                     break
@@ -666,15 +709,14 @@ class MacvtapFixture(fixtures.Fixture):
 
     def destroy(self):
         if (ip_lib.network_namespace_exists(self.ip_dev.namespace) or
-            self.ip_dev.namespace is None):
+                self.ip_dev.namespace is None):
             try:
                 self.ip_dev.link.delete()
             except RuntimeError:
                 pass
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PortFixture(fixtures.Fixture):
+class PortFixture(fixtures.Fixture, metaclass=abc.ABCMeta):
     """Create a port.
 
     :ivar port: created port
@@ -686,8 +728,8 @@ class PortFixture(fixtures.Fixture):
         super(PortFixture, self).__init__()
         self.bridge = bridge
         self.namespace = namespace
-        self.mac = (
-            mac or db_base_plugin_common.DbBasePluginCommon._generate_mac())
+        self.mac = (mac or
+                    db_base.DbBasePluginCommon._generate_macs()[0])
         self.port_id = port_id or uuidutils.generate_uuid()
 
     @abc.abstractmethod
@@ -782,6 +824,10 @@ class OVSPortFixture(PortFixture):
             self.mac,
             bridge=self.bridge.br_name,
             namespace=self.namespace)
+        # NOTE(mangelajo): for OVS implementations remove the DEAD VLAN tag
+        # on ports that we intend to use as fake vm interfaces, they
+        # need to be flat. This is related to lp#1767422
+        self.bridge.clear_db_attribute("Port", port_name, "tag")
         self.addCleanup(self.bridge.delete_port, port_name)
         self.port = ip_lib.IPDevice(port_name, self.namespace)
 
@@ -818,7 +864,7 @@ class OVSPortFixture(PortFixture):
         self.addCleanup(self.port.link.delete)
         ip_wrapper.add_device_to_namespace(self.port)
         bridge_port.link.set_up()
-        self.qbr.addif(bridge_port)
+        self.qbr.addif(bridge_port.name)
 
         self.port.link.set_address(self.mac)
         self.port.link.set_up()
@@ -917,7 +963,7 @@ class LinuxBridgePortFixture(PortFixture):
         # bridge side
         br_ip_wrapper = ip_lib.IPWrapper(self.bridge.namespace)
         br_ip_wrapper.add_device_to_namespace(self.br_port)
-        self.bridge.addif(self.br_port)
+        self.bridge.addif(self.br_port.name)
         self.br_port.link.set_up()
 
         # port side
