@@ -63,19 +63,25 @@ class ClientFixture(fixtures.Fixture):
         return delete(id)
 
     def create_router(self, tenant_id, name=None, ha=False,
-                      external_network=None):
+                      external_network=None, external_subnet=None):
         resource_type = 'router'
 
         name = name or utils.get_rand_name(prefix=resource_type)
         spec = {'tenant_id': tenant_id, 'name': name, 'ha': ha}
         if external_network:
             spec['external_gateway_info'] = {"network_id": external_network}
+            if external_subnet:
+                spec['external_gateway_info']['external_fixed_ips'] = (
+                    [{"subnet_id": external_subnet}])
 
         return self._create_resource(resource_type, spec)
 
+    def update_router(self, router_id, **kwargs):
+        return self._update_resource('router', router_id, kwargs)
+
     def create_network(self, tenant_id, name=None, external=False,
                        network_type=None, segmentation_id=None,
-                       physical_network=None, mtu=None):
+                       physical_network=None, mtu=None, qos_policy_id=None):
         resource_type = 'network'
 
         name = name or utils.get_rand_name(prefix=resource_type)
@@ -90,6 +96,8 @@ class ClientFixture(fixtures.Fixture):
             spec['provider:physical_network'] = physical_network
         if mtu is not None:
             spec['mtu'] = mtu
+        if qos_policy_id is not None:
+            spec['qos_policy_id'] = qos_policy_id
 
         return self._create_resource(resource_type, spec)
 
@@ -101,20 +109,25 @@ class ClientFixture(fixtures.Fixture):
 
     def create_subnet(self, tenant_id, network_id,
                       cidr, gateway_ip=None, name=None, enable_dhcp=True,
-                      ipv6_address_mode='slaac', ipv6_ra_mode='slaac'):
+                      ipv6_address_mode='slaac', ipv6_ra_mode='slaac',
+                      subnetpool_id=None, ip_version=None):
         resource_type = 'subnet'
 
         name = name or utils.get_rand_name(prefix=resource_type)
-        ip_version = netaddr.IPNetwork(cidr).version
+        if cidr and not ip_version:
+            ip_version = netaddr.IPNetwork(cidr).version
         spec = {'tenant_id': tenant_id, 'network_id': network_id, 'name': name,
-                'cidr': cidr, 'enable_dhcp': enable_dhcp,
-                'ip_version': ip_version}
+                'enable_dhcp': enable_dhcp, 'ip_version': ip_version}
         if ip_version == constants.IP_VERSION_6:
             spec['ipv6_address_mode'] = ipv6_address_mode
             spec['ipv6_ra_mode'] = ipv6_ra_mode
 
         if gateway_ip:
             spec['gateway_ip'] = gateway_ip
+        if subnetpool_id:
+            spec['subnetpool_id'] = subnetpool_id
+        if cidr:
+            spec['cidr'] = cidr
 
         return self._create_resource(resource_type, spec)
 
@@ -141,7 +154,7 @@ class ClientFixture(fixtures.Fixture):
         return self._update_resource('port', port_id, kwargs)
 
     def create_floatingip(self, tenant_id, floating_network_id,
-                          fixed_ip_address, port_id):
+                          fixed_ip_address, port_id, qos_policy_id=None):
         spec = {
             'floating_network_id': floating_network_id,
             'tenant_id': tenant_id,
@@ -149,6 +162,8 @@ class ClientFixture(fixtures.Fixture):
             'port_id': port_id
         }
 
+        if qos_policy_id:
+            spec['qos_policy_id'] = qos_policy_id
         return self._create_resource('floatingip', spec)
 
     def add_router_interface(self, router_id, subnet_id):
@@ -304,14 +319,18 @@ class ClientFixture(fixtures.Fixture):
         }
         return self.client.trunk_remove_subports(trunk_id, spec)
 
-    def create_security_group(self, tenant_id, name=None):
+    def create_security_group(self, tenant_id, name=None, stateful=True):
         resource_type = 'security_group'
 
         name = name or utils.get_rand_name(prefix=resource_type)
-        spec = {'tenant_id': tenant_id, 'name': name}
+        spec = {'tenant_id': tenant_id, 'name': name, 'stateful': stateful}
 
         return self._create_resource(resource_type, spec)
 
+    def update_security_group(self, security_group_id, **kwargs):
+        return self._update_resource('security_group', security_group_id,
+                                     kwargs)
+
     def create_security_group_rule(self, tenant_id, security_group_id,
                                    **kwargs):
         resource_type = 'security_group_rule'
@@ -334,3 +353,6 @@ class ClientFixture(fixtures.Fixture):
         self.addCleanup(
             _safe_method(self.client.delete_network_log), net_log['log']['id'])
         return net_log
+
+    def update_quota(self, project_id, tracked_resource, quota):
+        self._update_resource('quota', project_id, {tracked_resource: quota})
