@@ -40,7 +40,8 @@ class EnvironmentDescription(object):
                  global_mtu=constants.DEFAULT_NETWORK_MTU,
                  debug_iptables=False, log=False, report_bandwidths=False,
                  has_placement=False, placement_port=None,
-                 dhcp_scheduler_class=None, ml2_extension_drivers=None):
+                 dhcp_scheduler_class=None, ml2_extension_drivers=None,
+                 api_workers=1):
         self.network_type = network_type
         self.l2_pop = l2_pop
         self.qos = qos
@@ -62,6 +63,7 @@ class EnvironmentDescription(object):
         if self.log:
             self.service_plugins += ',log'
         self.ml2_extension_drivers = ml2_extension_drivers
+        self.api_workers = api_workers
 
     @property
     def tunneling_enabled(self):
