@@ -13,8 +13,11 @@
 #    under the License.
 
 from neutron_lib import constants
+from neutronclient.common import exceptions as nc_exc
 from oslo_utils import uuidutils
 
+from neutron.agent.linux import iptables_firewall
+from neutron.agent.linux import iptables_manager
 from neutron.cmd.sanity import checks
 from neutron.common import utils as common_utils
 from neutron.tests import base as tests_base
@@ -27,6 +30,10 @@ from neutron.tests.unit import testlib_api
 load_tests = testlib_api.module_load_tests
 
 
+class StatelessRulesNotConfiguredException(Exception):
+    pass
+
+
 class OVSVersionChecker(object):
     conntrack_supported = None
 
@@ -40,13 +47,10 @@ class OVSVersionChecker(object):
 
 class BaseSecurityGroupsSameNetworkTest(base.BaseFullStackTestCase):
 
-    of_interface = None
-
     def setUp(self):
         debug_iptables = self.firewall_driver.startswith("iptables")
         host_descriptions = [
             environment.HostDescription(
-                of_interface=self.of_interface,
                 l2_agent_type=self.l2_agent_type,
                 firewall_driver=self.firewall_driver,
                 dhcp_agent=True) for _ in range(self.num_hosts)]
@@ -88,26 +92,16 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
 
     network_type = 'vxlan'
     scenarios = [
+        # TODO(njohnston): Re-add the linuxbridge scenario once it is stable
         # The iptables_hybrid driver lacks isolation between agents and
         # because of that using only one host is enough
         ('ovs-hybrid', {
             'firewall_driver': 'iptables_hybrid',
-            'of_interface': 'native',
             'l2_agent_type': constants.AGENT_TYPE_OVS,
             'num_hosts': 1}),
-        ('ovs-openflow-cli', {
+        ('ovs-openflow', {
             'firewall_driver': 'openvswitch',
-            'of_interface': 'ovs-ofctl',
             'l2_agent_type': constants.AGENT_TYPE_OVS,
-            'num_hosts': 2}),
-        ('ovs-openflow-native', {
-            'firewall_driver': 'openvswitch',
-            'of_interface': 'native',
-            'l2_agent_type': constants.AGENT_TYPE_OVS,
-            'num_hosts': 2}),
-        ('linuxbridge-iptables', {
-            'firewall_driver': 'iptables',
-            'l2_agent_type': constants.AGENT_TYPE_LINUXBRIDGE,
             'num_hosts': 2})]
 
     index_to_sg = [0, 0, 1, 2]
@@ -115,7 +109,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
     # NOTE(toshii): As a firewall_driver can interfere with others,
     # the recommended way to add test is to expand this method, not
     # adding another.
-    @tests_base.unstable_test("bug 1779328")
+    @tests_base.unstable_test("bug 1742401")
     def test_securitygroup(self):
         """Tests if a security group rules are working, by confirming
         that 0. traffic is allowed when port security is disabled,
@@ -123,13 +117,16 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
              2. connection from allowed security group is permitted
              3. traffic not explicitly allowed (eg. ICMP) is blocked,
              4. a security group update takes effect,
-             5. a remote security group member addition works, and
-             6. an established connection stops by deleting a SG rule.
-             7. multiple overlapping remote rules work,
-             8. test other protocol functionality by using SCTP protocol
-             9. test two vms with same mac on the same host in different
-                networks
-             10. test using multiple security groups
+             5. a security group update for entire port range works
+             6. a remote security group member addition works, and
+             7. an established connection stops by deleting a SG rule.
+             8. multiple overlapping remote rules work,
+             9. test other protocol functionality by using SCTP protocol
+             10. test two vms with same mac on the same host in different
+                 networks
+             11. test using multiple security groups
+             12. test stateless security groups when firewall driver is
+                 iptables or iptables_hybrid.
         """
 
         tenant_uuid = uuidutils.generate_uuid()
@@ -138,15 +135,15 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             tenant_uuid, subnet_cidr)
 
         # 0. check that traffic is allowed when port security is disabled
+        vms[0].block_until_ping(vms[1].ip)
+        vms[0].block_until_ping(vms[2].ip)
+        vms[1].block_until_ping(vms[2].ip)
         self.assert_connection(
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3333,
             net_helpers.NetcatTester.TCP)
         self.assert_connection(
             vms[2].namespace, vms[0].namespace, vms[0].ip, 3333,
             net_helpers.NetcatTester.TCP)
-        vms[0].block_until_ping(vms[1].ip)
-        vms[0].block_until_ping(vms[2].ip)
-        vms[1].block_until_ping(vms[2].ip)
 
         # Apply security groups to the ports
         for port, sg in zip(ports, self.index_to_sg):
@@ -182,7 +179,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
             net_helpers.NetcatTester.TCP)
 
-        self.safe_client.create_security_group_rule(
+        rule1 = self.safe_client.create_security_group_rule(
             tenant_uuid, sgs[0]['id'],
             remote_group_id=sgs[0]['id'], direction='ingress',
             ethertype=constants.IPv4,
@@ -193,7 +190,35 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
             net_helpers.NetcatTester.TCP)
 
-        # 5. check if a remote security group member addition works
+        # 5. check if a security group update for entire port range works
+        self.client.delete_security_group_rule(rule1['id'])
+
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
+            net_helpers.NetcatTester.TCP)
+
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
+            net_helpers.NetcatTester.TCP)
+
+        rule1 = self.safe_client.create_security_group_rule(
+            tenant_uuid, sgs[0]['id'],
+            remote_group_id=sgs[0]['id'], direction='ingress',
+            ethertype=constants.IPv4,
+            protocol=constants.PROTO_NAME_TCP,
+            port_range_min=1, port_range_max=65535)
+
+        self.assert_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
+            net_helpers.NetcatTester.TCP)
+
+        self.assert_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
+            net_helpers.NetcatTester.TCP)
+
+        self.client.delete_security_group_rule(rule1['id'])
+
+        # 6. check if a remote security group member addition works
         rule2 = self.safe_client.create_security_group_rule(
             tenant_uuid, sgs[0]['id'],
             remote_group_id=sgs[1]['id'], direction='ingress',
@@ -205,7 +230,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             vms[2].namespace, vms[0].namespace, vms[0].ip, 3355,
             net_helpers.NetcatTester.TCP)
 
-        # 6. check if an established connection stops by deleting
+        # 7. check if an established connection stops by deleting
         #    the supporting SG rule.
         index_to_host.append(index_to_host[2])
         self.index_to_sg.append(1)
@@ -240,7 +265,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                                      sleep=8)
         netcat.stop_processes()
 
-        # 7. check if multiple overlapping remote rules work
+        # 8. check if multiple overlapping remote rules work
         self.safe_client.create_security_group_rule(
             tenant_uuid, sgs[0]['id'],
             remote_group_id=sgs[1]['id'], direction='ingress',
@@ -263,7 +288,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                 vms[3].namespace, vms[0].namespace, vms[0].ip, 8080,
                 net_helpers.NetcatTester.TCP)
 
-        # 8. check SCTP is supported by security group
+        # 9. check SCTP is supported by security group
         self.assert_no_connection(
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
             net_helpers.NetcatTester.SCTP)
@@ -279,12 +304,18 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
             vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
             net_helpers.NetcatTester.SCTP)
 
-        # 9. test two vms with same mac on the same host in different networks
+        # 10. test two vms with same mac on the same host in different networks
         self._test_overlapping_mac_addresses()
 
-        # 10. Check using multiple security groups
+        # 11. Check using multiple security groups
         self._test_using_multiple_security_groups()
 
+        # 12. test stateless security groups when firewall driver is iptables
+        # or iptables_hybrid.
+        # TODO(njohnston): Re-add the iptables here once it is stable
+        if self.firewall_driver == 'iptables_hybrid':
+            self._test_stateless_security_groups()
+
     def _test_using_multiple_security_groups(self):
         """Test using multiple security groups.
 
@@ -380,9 +411,127 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
         self.verify_no_connectivity_between_vms(
             vms[1], vms[0], net_helpers.NetcatTester.TCP, 22)
 
+    def _test_stateless_security_groups(self):
+        """Test stateless security groups.
+
+        This test will run basic tests for stateless security groups
+        1. Check the stateless rules are configured for vm interfaces when
+        firewall driver is iptables or iptables_hybrid.
+        2. Check connection is blocked for stateless security group.
+        3. Check if connection from allowed security group is permitted.
+        4. Check if traffic not explicitly allowed (eg. ICMP) is blocked.
+        5. Check if a security group update for entire port range works.
+        6. Check conflict when adding stateful and stateless security groups
+        to the same port
+        """
+        tenant_uuid = uuidutils.generate_uuid()
+        subnet_cidr = '40.0.0.0/24'
+        vms, ports, sgs, _, _ = self._create_resources(
+            tenant_uuid, subnet_cidr, stateful=False)
+
+        # Apply security groups to the ports
+        for port in ports[:2]:
+            self.safe_client.client.update_port(
+                port['id'],
+                body={'port': {'port_security_enabled': True,
+                               'security_groups': [sgs[0]['id']]}})
+
+        # Check the stateless rules are configured for vm interfaces when
+        # firewall driver is iptables or iptables_hybrid.
+        self._validate_stateless_rules(vms[:2])
+
+        # Connection is blocked for stateless security group
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3333,
+            net_helpers.NetcatTester.TCP)
+
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3333,
+            net_helpers.NetcatTester.TCP, src_port=3344)
+
+        rule1 = self.safe_client.create_security_group_rule(
+            tenant_uuid, sgs[0]['id'],
+            remote_group_id=sgs[0]['id'], direction='ingress',
+            ethertype=constants.IPv4,
+            protocol=constants.PROTO_NAME_TCP,
+            port_range_min=3344, port_range_max=3344)
+
+        # Check if connection from allowed security group is permitted
+        self.assert_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3333,
+            net_helpers.NetcatTester.TCP, src_port=3344)
+
+        # Check if traffic not explicitly allowed (eg. ICMP) is blocked
+        vms[0].block_until_no_ping(vms[1].ip)
+        vms[0].block_until_no_ping(vms[2].ip)
+        vms[1].block_until_no_ping(vms[2].ip)
+
+        # Check if a security group update for entire port range works
+        self.client.delete_security_group_rule(rule1['id'])
+
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
+            net_helpers.NetcatTester.TCP)
+
+        self.assert_no_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
+            net_helpers.NetcatTester.TCP)
+
+        self.safe_client.create_security_group_rule(
+            tenant_uuid, sgs[0]['id'],
+            remote_group_id=sgs[0]['id'], direction='ingress',
+            ethertype=constants.IPv4,
+            protocol=constants.PROTO_NAME_TCP,
+            port_range_min=1, port_range_max=65535)
+
+        self.assert_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3344,
+            net_helpers.NetcatTester.TCP)
+
+        self.assert_connection(
+            vms[1].namespace, vms[0].namespace, vms[0].ip, 3366,
+            net_helpers.NetcatTester.TCP)
+
+        # Update sg1 to be stateful
+        self.safe_client.update_security_group(sgs[1]['id'],
+                                               **{'stateful': True})
+
+        # Check conflict when adding stateful and stateless security groups
+        # to the same port
+        self.assertRaises(nc_exc.Conflict,
+                          self.safe_client.client.update_port,
+                          ports[0]['id'],
+                          body={'port': {
+                              'security_groups': [sgs[0]['id'],
+                                                  sgs[1]['id']]}})
+
+    def _validate_stateless_rules(self, vms):
+        """Check if stateless rules from iptables firewall are configured.
+
+        Raises StatelessRulesNotConfiguredException exception if no stateless
+        rules are found.
+        """
+        for vm in vms:
+            iptables = iptables_manager.IptablesManager(
+                namespace=vm.host.host_namespace)
+            vm_tap_device = iptables_firewall.get_hybrid_port_name(
+                vm.neutron_port['id'])
+            common_utils.wait_until_true(
+                lambda: self._is_stateless_configured(iptables,
+                                                      vm_tap_device),
+                exception=StatelessRulesNotConfiguredException(
+                    "There are no stateless rules configured for "
+                    "interface  %s" % vm_tap_device))
+
+    @staticmethod
+    def _is_stateless_configured(iptables, vm_tap_device):
+        filter_rules = iptables.get_rules_for_table('raw')
+        return any((vm_tap_device and '--notrack') in line for
+                   line in filter_rules)
+
     # NOTE: This can be used after refactor other tests to
     # one scenario one test.
-    def _create_resources(self, tenant_uuid, subnet_cidr):
+    def _create_resources(self, tenant_uuid, subnet_cidr, stateful=True):
         if self.firewall_driver == 'iptables_hybrid':
             # The iptables_hybrid driver lacks isolation between agents
             index_to_host = [0] * 4
@@ -393,7 +542,8 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
         self.safe_client.create_subnet(
             tenant_uuid, network['id'], subnet_cidr)
 
-        sgs = [self.safe_client.create_security_group(tenant_uuid)
+        sgs = [self.safe_client.create_security_group(tenant_uuid,
+                                                      stateful=stateful)
                for i in range(3)]
         ports = [
             self.safe_client.create_port(tenant_uuid, network['id'],
@@ -517,3 +667,27 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                 vm1, vm2, net_helpers.NetcatTester.TCP, port + 1)
             self.verify_no_connectivity_between_vms(
                 vm2, vm1, net_helpers.NetcatTester.TCP, port + 1)
+
+
+class SecurityGroupRulesTest(base.BaseFullStackTestCase):
+
+    def setUp(self):
+        host_descriptions = [environment.HostDescription()]
+        env = environment.Environment(environment.EnvironmentDescription(),
+                                      host_descriptions)
+        super(SecurityGroupRulesTest, self).setUp(env)
+
+    def test_security_group_rule_quota(self):
+        project_id = uuidutils.generate_uuid()
+        quota = self.client.show_quota_details(project_id)
+        sg_rules_used = quota['quota']['security_group_rule']['used']
+        self.assertEqual(0, sg_rules_used)
+
+        self.safe_client.create_security_group(project_id)
+        quota = self.client.show_quota_details(project_id)
+        sg_rules_used = quota['quota']['security_group_rule']['used']
+        self.safe_client.update_quota(project_id, 'security_group_rule',
+                                      sg_rules_used)
+
+        self.assertRaises(nc_exc.OverQuotaClient,
+                          self.safe_client.create_security_group, project_id)
