@@ -15,37 +15,99 @@
 #    under the License.
 
 import random
+from unittest import mock
 
 import eventlet
-import mock
+import fixtures
 from neutron_lib import constants as n_const
 from neutron_lib.utils import net
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
 from neutron.agent.common import ovs_lib
+from neutron.agent.common import polling
 from neutron.agent.l2 import l2_agent_extensions_manager as ext_manager
 from neutron.agent.linux import interface
-from neutron.agent.linux import polling
 from neutron.common import utils
 from neutron.conf.agent import common as agent_config
+from neutron.conf.agent import ovs_conf as ovs_agent_config
 from neutron.conf import common as common_config
 from neutron.conf.plugins.ml2.drivers import agent
 from neutron.conf.plugins.ml2.drivers import ovs_conf
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
-from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
-    import br_int
-from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
-    import br_phys
-from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
-    import br_tun
+from neutron.plugins.ml2.drivers.openvswitch.agent.extension_drivers \
+    import qos_driver as ovs_qos_driver
+from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
+    import main as main_mod
 from neutron.plugins.ml2.drivers.openvswitch.agent import ovs_neutron_agent \
     as ovs_agent
 from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import base
 
 
-class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
+class OVSOFControllerHelper(object):
+    """Helper class that runs os-ken openflow controller."""
+
+    def start_of_controller(self, conf):
+        self.br_int_cls = None
+        self.br_tun_cls = None
+        self.br_phys_cls = None
+        self.init_done = False
+        self.init_done_ev = eventlet.event.Event()
+        self.main_ev = eventlet.event.Event()
+        self.addCleanup(self._kill_main)
+        retry_count = 3
+        while True:
+            # Try a few different ports as a port conflict
+            # causes the test to fail.
+            conf.set_override('of_listen_port',
+                              net_helpers.get_free_namespace_port(
+                                  n_const.PROTO_NAME_TCP),
+                              group='OVS')
+            cfg.CONF.set_override('of_listen_port',
+                                  conf.OVS.of_listen_port,
+                                  group='OVS')
+            main_mod.init_config()
+            self._main_thread = eventlet.spawn(self._kick_main)
+
+            # Wait for _kick_main -> openflow main -> _agent_main
+            # NOTE(yamamoto): This complexity came from how we run openflow
+            # controller. Main routine blocks while running the embedded
+            # openflow controller.  In that case, the agent rpc_loop runs in
+            # another thread.  However, for FT we need to run setUp() and
+            # test_xxx() in the same thread.  So I made this run openflow main
+            # in a separate thread instead.
+            try:
+                while not self.init_done:
+                    self.init_done_ev.wait()
+                break
+            except fixtures.TimeoutException:
+                self._kill_main()
+            retry_count -= 1
+            if retry_count < 0:
+                raise Exception('port allocation failed')
+
+    def _kick_main(self):
+        with mock.patch.object(ovs_agent, 'main', self._agent_main):
+            main_mod.main()
+
+    def _kill_main(self):
+        self.main_ev.send()
+        self._main_thread.wait()
+
+    def _agent_main(self, bridge_classes):
+        self.br_int_cls = bridge_classes['br_int']
+        self.br_phys_cls = bridge_classes['br_phys']
+        self.br_tun_cls = bridge_classes['br_tun']
+
+        # signal to setUp()
+        self.init_done = True
+        self.init_done_ev.send()
+
+        self.main_ev.wait()
+
+
+class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
 
     def setUp(self):
         super(OVSAgentTestFramework, self).setUp()
@@ -66,6 +128,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         self.config = self._configure_agent()
         self.driver = interface.OVSInterfaceDriver(self.config)
         self.namespace = self.useFixture(net_helpers.NamespaceFixture()).name
+        self.start_of_controller(self.config)
 
     def _get_config_opts(self):
         config = cfg.ConfigOpts()
@@ -75,6 +138,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         agent_config.register_interface_opts(config)
         agent_config.register_interface_driver_opts_helper(config)
         agent_config.register_agent_state_opts_helper(config)
+        ovs_agent_config.register_ovs_agent_opts(config)
         ext_manager.register_opts(config)
         return config
 
@@ -84,7 +148,6 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
             'interface_driver',
             'neutron.agent.linux.interface.OVSInterfaceDriver')
         config.set_override('integration_bridge', self.br_int, "OVS")
-        config.set_override('ovs_integration_bridge', self.br_int)
         config.set_override('tunnel_bridge', self.br_tun, "OVS")
         config.set_override('int_peer_patch_port', self.patch_tun, "OVS")
         config.set_override('tun_peer_patch_port', self.patch_int, "OVS")
@@ -93,9 +156,9 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
 
     def _bridge_classes(self):
         return {
-            'br_int': br_int.OVSIntegrationBridge,
-            'br_phys': br_phys.OVSPhysicalBridge,
-            'br_tun': br_tun.OVSTunnelBridge
+            'br_int': self.br_int_cls,
+            'br_phys': self.br_phys_cls,
+            'br_tun': self.br_tun_cls
         }
 
     def create_agent(self, create_tunnels=True, ancillary_bridge=None,
@@ -112,8 +175,10 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         # Physical bridges should be created prior to running
         self._bridge_classes()['br_phys'](self.br_phys).create()
         ext_mgr = ext_manager.L2AgentExtensionsManager(self.config)
-        agent = ovs_agent.OVSNeutronAgent(self._bridge_classes(),
-                                          ext_mgr, self.config)
+        with mock.patch.object(ovs_qos_driver.QosOVSAgentDriver,
+                               '_minimum_bandwidth_initialize'):
+            agent = ovs_agent.OVSNeutronAgent(self._bridge_classes(),
+                                              ext_mgr, self.config)
         self.addCleanup(self.ovs.delete_bridge, self.br_int)
         if tunnel_types:
             self.addCleanup(self.ovs.delete_bridge, self.br_tun)
@@ -142,7 +207,8 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                     # because it keeps finding not ready ports that are created
                     # by other tests)
                     filtered_ports.append(dev)
-            return {'added': filtered_ports, 'removed': events['removed']}
+            return {'added': filtered_ports, 'removed': events['removed'],
+                    'modified': []}
         polling_manager.get_events = mock.Mock(side_effect=filter_events)
 
     def stop_agent(self, agent, rpc_loop_thread):
@@ -269,7 +335,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         return ports
 
     def _mock_update_device(self, context, devices_up, devices_down, agent_id,
-                            host=None):
+                            host=None, refresh_tunnels=False):
         dev_up = []
         dev_down = []
         for port in self.ports:
@@ -283,7 +349,8 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
                 'failed_devices_down': []}
 
     def setup_agent_rpc_mocks(self, agent, unplug_ports):
-        def mock_device_details(context, devices, agent_id, host=None):
+        def mock_device_details(context, devices, agent_id, host=None,
+                                agent_restarted=False):
             details = []
             for port in self.ports:
                 if port['id'] in devices:
@@ -313,7 +380,8 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
     def _prepare_failed_dev_up_trigger(self, agent):
 
         def mock_failed_devices_up(context, devices_up, devices_down,
-                                   agent_id, host=None):
+                                   agent_id, host=None,
+                                   refresh_tunnels=False):
             failed_devices = []
             devices = list(devices_up)
             # first port fails
@@ -334,7 +402,8 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
     def _prepare_failed_dev_down_trigger(self, agent):
 
         def mock_failed_devices_down(context, devices_up, devices_down,
-                                     agent_id, host=None):
+                                     agent_id, host=None,
+                                     refresh_tunnels=False):
             # first port fails
             failed_port_id = self.ports[0]['id']
             failed_devices_down = []
@@ -373,6 +442,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         self.ports = port_dicts
         self.agent = self.create_agent(create_tunnels=create_tunnels,
                                        ancillary_bridge=ancillary_bridge)
+        self.agent.iter_num += 1
         self.polling_manager = self.start_agent(self.agent, ports=self.ports)
         self.network = network or self._create_test_network_dict()
         if trigger_resync:
@@ -395,7 +465,13 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase):
         self._plug_ports(network, ports, self.agent, bridge=phys_br,
                          namespace=namespace)
 
-        if phys_segmentation_id and network_type == 'vlan':
+        if network_type == 'flat':
+            # NOTE(slaweq): for OVS implementations remove the DEAD VLAN tag
+            # on ports that belongs to flat network. DEAD VLAN tag is added
+            # to each newly created port. This is related to lp#1767422
+            for port in ports:
+                phys_br.clear_db_attribute("Port", port['vif_name'], "tag")
+        elif phys_segmentation_id and network_type == 'vlan':
             for port in ports:
                 phys_br.set_db_attribute(
                     "Port", port['vif_name'], "tag", phys_segmentation_id)
