@@ -14,15 +14,15 @@
 #    under the License.
 
 import copy
+from unittest import mock
 
-import mock
 from neutron_lib import constants
+from oslo_utils import netutils
 import testtools
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.l3 import agent as neutron_l3_agent
 from neutron.agent.linux import ip_lib
-from neutron.common import ipv6_utils
 from neutron.common import utils as common_utils
 from neutron.tests.common import l3_test_common
 from neutron.tests.common import net_helpers
@@ -37,19 +37,21 @@ class L3HATestCase(framework.L3AgentTestFramework):
 
     def test_keepalived_state_change_notification(self):
         enqueue_mock = mock.patch.object(
-            self.agent, 'enqueue_state_change').start()
+            self.agent, 'enqueue_state_change',
+            side_effect=self.change_router_state).start()
         router_info = self.generate_router_info(enable_ha=True)
         router = self.manage_router(self.agent, router_info)
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
 
         self.fail_ha_router(router)
         common_utils.wait_until_true(lambda: router.ha_state == 'backup')
 
-        common_utils.wait_until_true(lambda: enqueue_mock.call_count == 3)
+        common_utils.wait_until_true(lambda:
+            (enqueue_mock.call_count == 3 or enqueue_mock.call_count == 4))
         calls = [args[0] for args in enqueue_mock.call_args_list]
         self.assertEqual((router.router_id, 'backup'), calls[0])
-        self.assertEqual((router.router_id, 'master'), calls[1])
-        self.assertEqual((router.router_id, 'backup'), calls[2])
+        self.assertEqual((router.router_id, 'primary'), calls[1])
+        self.assertEqual((router.router_id, 'backup'), calls[-1])
 
     def _expected_rpc_report(self, expected):
         calls = (args[0][1] for args in
@@ -71,7 +73,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router2 = self.manage_router(self.agent, router_info)
 
         common_utils.wait_until_true(lambda: router1.ha_state == 'backup')
-        common_utils.wait_until_true(lambda: router2.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router2.ha_state == 'primary')
         common_utils.wait_until_true(
             lambda: self._expected_rpc_report(
                 {router1.router_id: 'standby', router2.router_id: 'active'}))
@@ -110,7 +112,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router_info = l3_test_common.prepare_router_data(
             enable_snat=True, enable_ha=True, dual_stack=True, enable_gw=False)
         router = self.manage_router(self.agent, router_info)
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
         if state == 'backup':
             self.fail_ha_router(router)
             common_utils.wait_until_true(lambda: router.ha_state == 'backup')
@@ -119,30 +121,28 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router_info['gw_port'] = ex_port
         router.process()
         self._assert_ipv6_accept_ra(router, expected_ra)
-        # As router is going first to master and than to backup mode,
+        # As router is going first to primary and than to backup mode,
         # ipv6_forwarding should be enabled on "all" interface always after
         # that transition
         self._assert_ipv6_forwarding(router, expected_forwarding,
                                      True)
 
-    @testtools.skipUnless(ipv6_utils.is_enabled_and_bind_by_default(),
-                          "IPv6 is not enabled")
-    def test_ipv6_router_advts_and_fwd_after_router_state_change_master(self):
+    @testtools.skipUnless(netutils.is_ipv6_enabled(), "IPv6 is not enabled")
+    def test_ipv6_router_advts_and_fwd_after_router_state_change_primary(self):
         # Check that RA and forwarding are enabled when there's no IPv6
         # gateway.
-        self._test_ipv6_router_advts_and_fwd_helper('master',
+        self._test_ipv6_router_advts_and_fwd_helper('primary',
                                                     enable_v6_gw=False,
                                                     expected_ra=True,
                                                     expected_forwarding=True)
         # Check that RA is disabled and forwarding is enabled when an IPv6
         # gateway is configured.
-        self._test_ipv6_router_advts_and_fwd_helper('master',
+        self._test_ipv6_router_advts_and_fwd_helper('primary',
                                                     enable_v6_gw=True,
                                                     expected_ra=False,
                                                     expected_forwarding=True)
 
-    @testtools.skipUnless(ipv6_utils.is_enabled_and_bind_by_default(),
-                          "IPv6 is not enabled")
+    @testtools.skipUnless(netutils.is_ipv6_enabled(), "IPv6 is not enabled")
     def test_ipv6_router_advts_and_fwd_after_router_state_change_backup(self):
         # Check that both RA and forwarding are disabled on backup instances
         self._test_ipv6_router_advts_and_fwd_helper('backup',
@@ -219,7 +219,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router_info = self.generate_router_info(
             ip_version=constants.IP_VERSION_6, enable_ha=True)
         router1 = self.manage_router(self.agent, router_info)
-        common_utils.wait_until_true(lambda: router1.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router1.ha_state == 'primary')
         common_utils.wait_until_true(lambda: router1.radvd.enabled)
 
         def _check_lla_status(router, expected):
@@ -263,7 +263,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
                 ipv6_subnet_modes=[slaac_mode],
                 interface_id=interface_id)
         router.process()
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
 
         # Verify that router internal interface is present and is configured
         # with IP address from both the subnets.
@@ -309,7 +309,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router = self.manage_router(self.agent, router_info)
         ex_gw_port = router.get_ex_gw_port()
         interface_name = router.get_external_device_interface_name(ex_gw_port)
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
         self._add_fip(router, '172.168.1.20', fixed_address='10.0.0.3')
         router.process()
         router.router[constants.FLOATINGIP_KEY] = []
@@ -328,7 +328,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router1.router[constants.HA_INTERFACE_KEY]['status'] = (
             constants.PORT_STATUS_ACTIVE)
         self.agent._process_updated_router(router1.router)
-        common_utils.wait_until_true(lambda: router1.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router1.ha_state == 'primary')
 
     def test_ha_router_namespace_has_ip_nonlocal_bind_disabled(self):
         router_info = self.generate_router_info(enable_ha=True)
@@ -345,8 +345,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
             raise
         self.assertEqual(0, ip_nonlocal_bind_value)
 
-    @testtools.skipUnless(ipv6_utils.is_enabled_and_bind_by_default(),
-                          "IPv6 is not enabled")
+    @testtools.skipUnless(netutils.is_ipv6_enabled(), "IPv6 is not enabled")
     def test_ha_router_namespace_has_ipv6_forwarding_disabled(self):
         router_info = self.generate_router_info(enable_ha=True)
         router_info[constants.HA_INTERFACE_KEY]['status'] = (
@@ -363,12 +362,11 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router.router[constants.HA_INTERFACE_KEY]['status'] = (
             constants.PORT_STATUS_ACTIVE)
         self.agent._process_updated_router(router.router)
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
         self._wait_until_ipv6_forwarding_has_state(
             router.ns_name, external_device_name, 1)
 
-    @testtools.skipUnless(ipv6_utils.is_enabled_and_bind_by_default(),
-                          "IPv6 is not enabled")
+    @testtools.skipUnless(netutils.is_ipv6_enabled(), "IPv6 is not enabled")
     def test_ha_router_without_gw_ipv6_forwarding_state(self):
         router_info = self.generate_router_info(
             enable_ha=True, enable_gw=False)
@@ -382,7 +380,7 @@ class L3HATestCase(framework.L3AgentTestFramework):
         router.router[constants.HA_INTERFACE_KEY]['status'] = (
             constants.PORT_STATUS_ACTIVE)
         self.agent._process_updated_router(router.router)
-        common_utils.wait_until_true(lambda: router.ha_state == 'master')
+        common_utils.wait_until_true(lambda: router.ha_state == 'primary')
         self._wait_until_ipv6_forwarding_has_state(router.ns_name, 'all', 1)
 
 
@@ -412,7 +410,7 @@ class L3HATestFailover(framework.L3AgentTestFramework):
         # enough because same IP address is also configured on
         # ovs_integration_bridge device from second router and it will still
         # respond to ping
-        r_br = ovs_lib.OVSBridge(router.driver.conf.ovs_integration_bridge)
+        r_br = ovs_lib.OVSBridge(router.driver.conf.OVS.integration_bridge)
         external_port = router.get_ex_gw_port()
         for subnet in external_port['subnets']:
             r_br.add_flow(
@@ -420,7 +418,7 @@ class L3HATestFailover(framework.L3AgentTestFramework):
 
     @staticmethod
     def restore_gw_router_port(router):
-        r_br = ovs_lib.OVSBridge(router.driver.conf.ovs_integration_bridge)
+        r_br = ovs_lib.OVSBridge(router.driver.conf.OVS.integration_bridge)
         external_port = router.get_ex_gw_port()
         for subnet in external_port['subnets']:
             r_br.delete_flows(proto='ip', nw_dst=subnet['gateway_ip'])
@@ -428,29 +426,29 @@ class L3HATestFailover(framework.L3AgentTestFramework):
     def test_ha_router_failover(self):
         router1, router2 = self.create_ha_routers()
 
-        master_router, slave_router = self._get_master_and_slave_routers(
+        primary_router, backup_router = self._get_primary_and_backup_routers(
             router1, router2)
 
-        self._assert_ipv6_accept_ra(master_router, True)
-        self._assert_ipv6_forwarding(master_router, True, True)
-        self._assert_ipv6_accept_ra(slave_router, False)
-        self._assert_ipv6_forwarding(slave_router, False, False)
+        self._assert_ipv6_accept_ra(primary_router, True)
+        self._assert_ipv6_forwarding(primary_router, True, True)
+        self._assert_ipv6_accept_ra(backup_router, False)
+        self._assert_ipv6_forwarding(backup_router, False, False)
 
         self.fail_ha_router(router1)
 
-        # NOTE: passing slave_router as first argument, because we expect
-        # that this router should be the master
-        new_master, new_slave = self._get_master_and_slave_routers(
-            slave_router, master_router)
+        # NOTE: passing backup_router as first argument, because we expect
+        # that this router should be the primary
+        new_primary, new_backup = self._get_primary_and_backup_routers(
+            backup_router, primary_router)
 
-        self.assertEqual(master_router, new_slave)
-        self.assertEqual(slave_router, new_master)
-        self._assert_ipv6_accept_ra(new_master, True)
-        self._assert_ipv6_forwarding(new_master, True, True)
-        self._assert_ipv6_accept_ra(new_slave, False)
-        # after transition from master -> slave, 'all' IPv6 forwarding should
+        self.assertEqual(primary_router, new_backup)
+        self.assertEqual(backup_router, new_primary)
+        self._assert_ipv6_accept_ra(new_primary, True)
+        self._assert_ipv6_forwarding(new_primary, True, True)
+        self._assert_ipv6_accept_ra(new_backup, False)
+        # after transition from primary -> backup, 'all' IPv6 forwarding should
         # be enabled
-        self._assert_ipv6_forwarding(new_slave, False, True)
+        self._assert_ipv6_forwarding(new_backup, False, True)
 
     def test_ha_router_lost_gw_connection(self):
         self.agent.conf.set_override(
@@ -460,18 +458,18 @@ class L3HATestFailover(framework.L3AgentTestFramework):
 
         router1, router2 = self.create_ha_routers()
 
-        master_router, slave_router = self._get_master_and_slave_routers(
+        primary_router, backup_router = self._get_primary_and_backup_routers(
             router1, router2)
 
-        self.fail_gw_router_port(master_router)
+        self.fail_gw_router_port(primary_router)
 
-        # NOTE: passing slave_router as first argument, because we expect
-        # that this router should be the master
-        new_master, new_slave = self._get_master_and_slave_routers(
-            slave_router, master_router)
+        # NOTE: passing backup_router as first argument, because we expect
+        # that this router should be the primary
+        new_primary, new_backup = self._get_primary_and_backup_routers(
+            backup_router, primary_router)
 
-        self.assertEqual(master_router, new_slave)
-        self.assertEqual(slave_router, new_master)
+        self.assertEqual(primary_router, new_backup)
+        self.assertEqual(backup_router, new_primary)
 
     def test_both_ha_router_lost_gw_connection(self):
         self.agent.conf.set_override(
@@ -481,24 +479,24 @@ class L3HATestFailover(framework.L3AgentTestFramework):
 
         router1, router2 = self.create_ha_routers()
 
-        master_router, slave_router = self._get_master_and_slave_routers(
+        primary_router, backup_router = self._get_primary_and_backup_routers(
             router1, router2)
 
-        self.fail_gw_router_port(master_router)
-        self.fail_gw_router_port(slave_router)
+        self.fail_gw_router_port(primary_router)
+        self.fail_gw_router_port(backup_router)
 
         common_utils.wait_until_true(
-            lambda: master_router.ha_state == 'master')
+            lambda: primary_router.ha_state == 'primary')
         common_utils.wait_until_true(
-            lambda: slave_router.ha_state == 'master')
+            lambda: backup_router.ha_state == 'primary')
 
-        self.restore_gw_router_port(master_router)
+        self.restore_gw_router_port(primary_router)
 
-        new_master, new_slave = self._get_master_and_slave_routers(
-            master_router, slave_router)
+        new_primary, new_backup = self._get_primary_and_backup_routers(
+            primary_router, backup_router)
 
-        self.assertEqual(master_router, new_master)
-        self.assertEqual(slave_router, new_slave)
+        self.assertEqual(primary_router, new_primary)
+        self.assertEqual(backup_router, new_backup)
 
 
 class LinuxBridgeL3HATestCase(L3HATestCase):
