@@ -15,10 +15,10 @@
 
 import copy
 import os.path
+from unittest import mock
 
 import eventlet
 import fixtures
-import mock
 import netaddr
 from neutron_lib import constants as lib_const
 from oslo_config import fixture as fixture_config
@@ -32,6 +32,7 @@ from neutron.agent.linux import external_process
 from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import utils
+from neutron.agent.metadata import driver as metadata_driver
 from neutron.common import utils as common_utils
 from neutron.conf.agent import common as config
 from neutron.tests.common import net_helpers
@@ -73,7 +74,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             'neutron.agent.linux.interface.OVSInterfaceDriver')
         self.conf.set_override('report_interval', 0, 'AGENT')
         br_int = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
-        self.conf.set_override('ovs_integration_bridge', br_int.br_name)
+        self.conf.set_override('integration_bridge', br_int.br_name, 'OVS')
 
         self.mock_plugin_api = mock.patch(
             'neutron.agent.dhcp.agent.DhcpPluginApi').start().return_value
@@ -84,7 +85,8 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
         self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
 
-    def network_dict_for_dhcp(self, dhcp_enabled=True, ip_version=4,
+    def network_dict_for_dhcp(self, dhcp_enabled=True,
+                              ip_version=lib_const.IP_VERSION_4,
                               prefix_override=None):
         net_id = uuidutils.generate_uuid()
         subnet_dict = self.create_subnet_dict(
@@ -99,41 +101,40 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             net_id, [subnet_dict], [port_dict])
         return net_dict
 
-    def create_subnet_dict(self, net_id, dhcp_enabled=True, ip_version=4,
+    def create_subnet_dict(self, net_id, dhcp_enabled=True,
+                           ip_version=lib_const.IP_VERSION_4,
                            prefix_override=None):
         cidr = self._IP_ADDRS[ip_version]['cidr']
         if prefix_override is not None:
             cidr = '/'.join((cidr.split('/')[0], str(prefix_override)))
-        sn_dict = dhcp.DictModel({
-            "id": uuidutils.generate_uuid(),
-            "network_id": net_id,
-            "ip_version": ip_version,
-            "cidr": cidr,
-            "gateway_ip": (self.
-                _IP_ADDRS[ip_version]['gateway']),
-            "enable_dhcp": dhcp_enabled,
-            "dns_nameservers": [],
-            "host_routes": [],
-            "ipv6_ra_mode": None,
-            "ipv6_address_mode": None})
-        if ip_version == 6:
+        sn_dict = dhcp.DictModel(
+            id=uuidutils.generate_uuid(),
+            network_id=net_id,
+            ip_version=ip_version,
+            cidr=cidr,
+            gateway_ip=self._IP_ADDRS[ip_version]['gateway'],
+            enable_dhcp=dhcp_enabled,
+            dns_nameservers=[],
+            host_routes=[],
+            ipv6_ra_mode=None,
+            ipv6_address_mode=None)
+        if ip_version == lib_const.IP_VERSION_6:
             sn_dict['ipv6_address_mode'] = lib_const.DHCPV6_STATEFUL
         return sn_dict
 
     def create_port_dict(self, network_id, subnet_id, mac_address,
-                         ip_version=4, ip_address=None):
+                         ip_version=lib_const.IP_VERSION_4, ip_address=None):
         ip_address = (self._IP_ADDRS[ip_version]['addr']
             if not ip_address else ip_address)
-        port_dict = dhcp.DictModel({
-            "id": uuidutils.generate_uuid(),
-            "name": "foo",
-            "mac_address": mac_address,
-            "network_id": network_id,
-            "admin_state_up": True,
-            "device_id": uuidutils.generate_uuid(),
-            "device_owner": "foo",
-            "fixed_ips": [{"subnet_id": subnet_id,
-                           "ip_address": ip_address}], })
+        port_dict = dhcp.DictModel(id=uuidutils.generate_uuid(),
+                                   name="foo",
+                                   mac_address=mac_address,
+                                   network_id=network_id,
+                                   admin_state_up=True,
+                                   device_id=uuidutils.generate_uuid(),
+                                   device_owner="foo",
+                                   fixed_ips=[{"subnet_id": subnet_id,
+                                               "ip_address": ip_address}])
         return port_dict
 
     def create_network_dict(self, net_id, subnets=None, ports=None,
@@ -141,13 +142,12 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
         subnets = [] if not subnets else subnets
         ports = [] if not ports else ports
         non_local_subnets = [] if not non_local_subnets else non_local_subnets
-        net_dict = dhcp.NetModel(d={
-            "id": net_id,
-            "subnets": subnets,
-            "non_local_subnets": non_local_subnets,
-            "ports": ports,
-            "admin_state_up": True,
-            "tenant_id": uuidutils.generate_uuid(), })
+        net_dict = dhcp.NetModel(id=net_id,
+                                 subnets=subnets,
+                                 non_local_subnets=non_local_subnets,
+                                 ports=ports,
+                                 admin_state_up=True,
+                                 tenant_id=uuidutils.generate_uuid())
         return net_dict
 
     def get_interface_name(self, network, port):
@@ -193,12 +193,12 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
         vif_name = self.get_interface_name(network.id, port)
 
         self.ovs_driver.plug(network.id, port.id, vif_name, port.mac_address,
-                             self.conf['ovs_integration_bridge'],
+                             self.conf.OVS.integration_bridge,
                              namespace=namespace)
 
     def _ip_list_for_vif(self, vif_name, namespace):
         ip_device = ip_lib.IPDevice(vif_name, namespace)
-        return ip_device.addr.list(ip_version=4)
+        return ip_device.addr.list(ip_version=lib_const.IP_VERSION_4)
 
     def _get_network_port_for_allocation_test(self):
         network = self.network_dict_for_dhcp()
@@ -254,7 +254,8 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
         return external_process.ProcessManager(
             self.conf,
             network.id,
-            network.namespace)
+            network.namespace,
+            service=metadata_driver.HAPROXY_SERVICE)
 
 
 class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
@@ -342,17 +343,26 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
             exception=RuntimeError("Stale metadata proxy didn't get killed"))
 
     def _test_metadata_proxy_spawn_kill_with_subnet_create_delete(self):
-        network = self.network_dict_for_dhcp(ip_version=6)
+        network = self.network_dict_for_dhcp(
+            ip_version=lib_const.IP_VERSION_6,
+            dhcp_enabled=False)
         self.configure_dhcp_for_network(network=network)
         pm = self._get_metadata_proxy_process(network)
 
-        # A newly created network with ipv6 subnet will not have metadata proxy
         self.assertFalse(pm.active)
 
         new_network = copy.deepcopy(network)
         dhcp_enabled_ipv4_subnet = self.create_subnet_dict(network.id)
         new_network.subnets.append(dhcp_enabled_ipv4_subnet)
+
         self.mock_plugin_api.get_network_info.return_value = new_network
+        dhcp_port_mock = self.create_port_dict(
+            network.id, dhcp_enabled_ipv4_subnet.id,
+            mac_address=str(self._DHCP_PORT_MAC_ADDRESS))
+        self.mock_plugin_api.create_dhcp_port.return_value = dhcp_port_mock
+        network.ports = []
+        new_network.ports = []
+
         self.agent.refresh_dhcp_helper(network.id)
         # Metadata proxy should be spawned for the newly added subnet
         common_utils.wait_until_true(
@@ -397,3 +407,43 @@ class DHCPAgentOVSTestCase(DHCPAgentOVSTestFramework):
             exception=RuntimeError("'dhcp_ready_on_ports' not be called"))
         self.mock_plugin_api.dhcp_ready_on_ports.assert_called_with(
             ports_to_send)
+
+    def test_dhcp_processing_pool_size(self):
+        mock.patch.object(self.agent, 'call_driver').start().return_value = (
+            True)
+        self.agent.update_isolated_metadata_proxy = mock.Mock()
+        self.agent.disable_isolated_metadata_proxy = mock.Mock()
+
+        network_info_1 = self.network_dict_for_dhcp()
+        self.configure_dhcp_for_network(network=network_info_1)
+        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
+                         self.agent._pool.size)
+
+        network_info_2 = self.network_dict_for_dhcp()
+        self.configure_dhcp_for_network(network=network_info_2)
+        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
+                         self.agent._pool.size)
+
+        network_info_list = [network_info_1, network_info_2]
+        for _i in range(agent.DHCP_PROCESS_GREENLET_MAX + 1):
+            ni = self.network_dict_for_dhcp()
+            self.configure_dhcp_for_network(network=ni)
+            network_info_list.append(ni)
+
+        self.assertEqual(agent.DHCP_PROCESS_GREENLET_MAX,
+                         self.agent._pool.size)
+
+        for network in network_info_list:
+            self.agent.disable_dhcp_helper(network.id)
+
+        agent_network_info_len = len(self.agent.cache.get_network_ids())
+        if agent_network_info_len < agent.DHCP_PROCESS_GREENLET_MIN:
+            self.assertEqual(agent.DHCP_PROCESS_GREENLET_MIN,
+                             self.agent._pool.size)
+        elif (agent.DHCP_PROCESS_GREENLET_MIN <= agent_network_info_len <=
+              agent.DHCP_PROCESS_GREENLET_MAX):
+            self.assertEqual(agent_network_info_len,
+                             self.agent._pool.size)
+        else:
+            self.assertEqual(agent.DHCP_PROCESS_GREENLET_MAX,
+                             self.agent._pool.size)
