@@ -24,17 +24,18 @@ import netaddr
 from neutron_lib import constants
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 import testscenarios
 
-from neutron.agent import firewall
 from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import openvswitch_firewall
+from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.cmd.sanity import checks
 from neutron.conf.agent import securitygroups_rpc as security_config
 from neutron.tests.common import conn_testers
 from neutron.tests.common import helpers
+from neutron.tests.functional.agent.l2 import base as l2_base
 from neutron.tests.functional.agent.linux import base as linux_base
-from neutron.tests.functional import base
 from neutron.tests.functional import constants as test_constants
 
 LOG = logging.getLogger(__name__)
@@ -74,8 +75,8 @@ def _add_rule(sg_rules, base, port_range_min=None, port_range_max=None):
     sg_rules.append(rule)
 
 
-class BaseFirewallTestCase(base.BaseSudoTestCase):
-    FAKE_SECURITY_GROUP_ID = 'fake_sg_id'
+class BaseFirewallTestCase(linux_base.BaseOVSLinuxTestCase):
+    FAKE_SECURITY_GROUP_ID = uuidutils.generate_uuid()
     MAC_SPOOFED = "fa:16:3e:9a:2f:48"
     scenarios_iptables = testscenarios.multiply_scenarios(
         [('IptablesFirewallDriver', {'initialize': 'initialize_iptables',
@@ -85,16 +86,16 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
 
     scenarios_ovs_fw_interfaces = testscenarios.multiply_scenarios(
         [('OVS Firewall Driver', {'initialize': 'initialize_ovs',
-                                  'firewall_name': 'openvswitch'})],
-        linux_base.BaseOVSLinuxTestCase.scenarios)
+                                  'firewall_name': 'openvswitch'})])
 
     scenarios = scenarios_iptables + scenarios_ovs_fw_interfaces
 
     ip_cidr = None
-    vlan_range = set(range(test_constants.VLAN_COUNT))
+    vlan_range = set(range(1, test_constants.VLAN_COUNT - 1))
 
     def setUp(self):
         security_config.register_securitygroups_opts()
+        self.net_id = uuidutils.generate_uuid()
         super(BaseFirewallTestCase, self).setUp()
         self.tester, self.firewall = getattr(self, self.initialize)()
         if self.firewall_name == "openvswitch":
@@ -103,22 +104,26 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
             self.tester.vm_port_id,
             [self.tester.vm_ip_address],
             self.tester.vm_mac_address,
-            [self.FAKE_SECURITY_GROUP_ID])
+            [self.FAKE_SECURITY_GROUP_ID],
+            self.net_id)
         # FIXME(jlibosva): We should consider to call prepare_port_filter with
         # deferred bridge depending on its performance
         self.firewall.prepare_port_filter(self.src_port_desc)
+        # Traffic coming from patch-port is always VLAN tagged
+        self.tester.set_peer_port_as_patch_port()
 
     def initialize_iptables(self):
         cfg.CONF.set_override('enable_ipset', self.enable_ipset,
                               'SECURITYGROUP')
+        br_name = ('brq' + self.net_id)[:constants.LINUX_DEV_LEN]
         tester = self.useFixture(
-            conn_testers.LinuxBridgeConnectionTester(self.ip_cidr))
+            conn_testers.LinuxBridgeConnectionTester(self.ip_cidr,
+                                                     bridge_name=br_name))
         firewall_drv = iptables_firewall.IptablesFirewallDriver(
             namespace=tester.bridge_namespace)
         return tester, firewall_drv
 
     def initialize_ovs(self):
-        self.config(group='OVS', ovsdb_interface=self.ovsdb_interface)
         # Tests for ovs requires kernel >= 4.3 and OVS >= 2.5
         if not checks.ovs_conntrack_supported():
             self.skipTest("Open vSwitch with conntrack is not installed "
@@ -127,8 +132,12 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
                           "OVS>=2.5). More info at "
                           "https://github.com/openvswitch/ovs/blob/master/"
                           "FAQ.md")
+        self.of_helper = l2_base.OVSOFControllerHelper()
+        self.of_helper.addCleanup = self.addCleanup
+        self.of_helper.start_of_controller(cfg.CONF)
         tester = self.useFixture(
-            conn_testers.OVSConnectionTester(self.ip_cidr))
+            conn_testers.OVSConnectionTester(self.ip_cidr,
+                                             self.of_helper.br_int_cls))
         firewall_drv = openvswitch_firewall.OVSFirewallDriver(tester.bridge)
         return tester, firewall_drv
 
@@ -140,7 +149,8 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
         self.tester.set_peer_tag(vlan)
 
     @staticmethod
-    def _create_port_description(port_id, ip_addresses, mac_address, sg_ids):
+    def _create_port_description(port_id, ip_addresses, mac_address, sg_ids,
+                                 net_id):
         return {'admin_state_up': True,
                 'device': port_id,
                 'device_owner': DEVICE_OWNER_COMPUTE,
@@ -148,7 +158,8 @@ class BaseFirewallTestCase(base.BaseSudoTestCase):
                 'mac_address': mac_address,
                 'port_security_enabled': True,
                 'security_groups': sg_ids,
-                'status': 'ACTIVE'}
+                'status': 'ACTIVE',
+                'network_id': net_id}
 
     def _apply_security_group_rules(self, sg_id, sg_rules):
         with self.firewall.defer_apply():
@@ -269,15 +280,15 @@ class FirewallTestCase(BaseFirewallTestCase):
                                (outgoing_rule_pref, i['port_range_min']))
                    for i in sg_rules]
         # all indexes should be in order with no unexpected rules in between
-        self.assertEqual(range(indexes[0], indexes[-1] + 1), indexes)
+        self.assertEqual(list(range(indexes[0], indexes[-1] + 1)), indexes)
 
     def test_ingress_icmp_secgroup(self):
         # update the sg_group to make ping pass
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP},
                     {'ethertype': constants.IPv4,
-                     'direction': firewall.EGRESS_DIRECTION}]
+                     'direction': constants.EGRESS_DIRECTION}]
 
         self.tester.assert_no_connection(protocol=self.tester.ICMP,
                                          direction=self.tester.INGRESS)
@@ -287,10 +298,10 @@ class FirewallTestCase(BaseFirewallTestCase):
 
     def test_mac_spoofing(self):
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP},
                     {'ethertype': constants.IPv4,
-                     'direction': firewall.EGRESS_DIRECTION}]
+                     'direction': constants.EGRESS_DIRECTION}]
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
 
         self.tester.assert_connection(protocol=self.tester.ICMP,
@@ -340,7 +351,7 @@ class FirewallTestCase(BaseFirewallTestCase):
 
     def test_ip_spoofing(self):
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP}]
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
         not_allowed_ip = "%s/24" % (
@@ -363,7 +374,7 @@ class FirewallTestCase(BaseFirewallTestCase):
         self.firewall.update_port_filter(self.src_port_desc)
 
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP}]
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
         not_allowed_ip = "%s/24" % (
@@ -379,29 +390,60 @@ class FirewallTestCase(BaseFirewallTestCase):
 
     def test_allowed_address_pairs(self):
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP},
                     {'ethertype': constants.IPv4,
-                     'direction': firewall.EGRESS_DIRECTION}]
+                     'direction': constants.EGRESS_DIRECTION}]
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
 
         port_mac = self.tester.vm_mac_address
         allowed_ip = netaddr.IPAddress(self.tester.vm_ip_address) + 1
         not_allowed_ip = "%s/24" % (allowed_ip + 1)
+        allowed_mac = 'fa:16:3e:8c:84:13'
+        not_allowed_mac = 'fa:16:3e:8c:84:14'
         self.src_port_desc['allowed_address_pairs'] = [
             {'mac_address': port_mac,
-             'ip_address': allowed_ip}]
+             'ip_address': "%s/32" % allowed_ip},
+            {'mac_address': allowed_mac,
+             'ip_address': "%s/32" % allowed_ip}]
         allowed_ip = "%s/24" % allowed_ip
 
         self.firewall.update_port_filter(self.src_port_desc)
         self.tester.assert_connection(protocol=self.tester.ICMP,
                                       direction=self.tester.INGRESS)
+        self.tester.assert_connection(protocol=self.tester.ICMP,
+                                      direction=self.tester.EGRESS)
         self.tester.vm_ip_cidr = allowed_ip
         self.tester.assert_connection(protocol=self.tester.ICMP,
                                       direction=self.tester.INGRESS)
+        self.tester.assert_connection(protocol=self.tester.ICMP,
+                                      direction=self.tester.EGRESS)
         self.tester.vm_ip_cidr = not_allowed_ip
         self.tester.assert_no_connection(protocol=self.tester.ICMP,
                                          direction=self.tester.INGRESS)
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.EGRESS)
+        self.tester.vm_mac_address = allowed_mac
+        self.tester.vm_ip_cidr = allowed_ip
+        self.tester.flush_arp_tables()
+        self.tester.assert_connection(protocol=self.tester.ICMP,
+                                      direction=self.tester.INGRESS)
+        self.tester.assert_connection(protocol=self.tester.ICMP,
+                                      direction=self.tester.EGRESS)
+        self.tester.vm_mac_address = allowed_mac
+        self.tester.vm_ip_cidr = not_allowed_ip
+        self.tester.flush_arp_tables()
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.INGRESS)
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.EGRESS)
+        self.tester.vm_mac_address = not_allowed_mac
+        self.tester.vm_ip_cidr = allowed_ip
+        self.tester.flush_arp_tables()
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.INGRESS)
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.EGRESS)
 
     def test_arp_is_allowed(self):
         self.tester.assert_connection(protocol=self.tester.ARP,
@@ -459,7 +501,7 @@ class FirewallTestCase(BaseFirewallTestCase):
         port_min = 12345
         port_max = 12346
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_TCP,
                      'port_range_min': port_min,
                      'port_range_max': port_max}]
@@ -482,7 +524,7 @@ class FirewallTestCase(BaseFirewallTestCase):
         source_port_min = 12345
         source_port_max = 12346
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.EGRESS_DIRECTION,
+                     'direction': constants.EGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_TCP,
                      'source_port_range_min': source_port_min,
                      'source_port_range_max': source_port_max}]
@@ -505,7 +547,7 @@ class FirewallTestCase(BaseFirewallTestCase):
     def test_established_connection_is_cut(self):
         port = 12345
         sg_rules = [{'ethertype': constants.IPv4,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_TCP,
                      'port_range_min': port,
                      'port_range_max': port}]
@@ -518,7 +560,6 @@ class FirewallTestCase(BaseFirewallTestCase):
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, list())
         self.tester.assert_no_established_connection(**connection)
 
-    @skip_if_firewall('openvswitch')
     def test_preventing_firewall_blink(self):
         direction = self.tester.INGRESS
         sg_rules = [{'ethertype': 'IPv4', 'direction': 'ingress',
@@ -534,14 +575,18 @@ class FirewallTestCase(BaseFirewallTestCase):
         self.assertEqual(packets_received, 0)
 
     def test_remote_security_groups(self):
+        self.tester.set_peer_port_as_vm_port()
+
         remote_sg_id = 'remote_sg_id'
         peer_port_desc = self._create_port_description(
             self.tester.peer_port_id,
             [self.tester.peer_ip_address],
             self.tester.peer_mac_address,
-            [remote_sg_id])
+            [remote_sg_id],
+            self.net_id)
 
-        vm_sg_members = {'IPv4': [self.tester.peer_ip_address]}
+        vm_sg_members = {'IPv4': [
+            (self.tester.peer_ip_address, self.tester.peer_mac_address)]}
         peer_sg_rules = [{'ethertype': 'IPv4', 'direction': 'egress',
                           'protocol': 'icmp'}]
         self.firewall.update_security_group_rules(remote_sg_id, peer_sg_rules)
@@ -573,7 +618,7 @@ class FirewallTestCase(BaseFirewallTestCase):
 
         """
         # Enable ip forwarding on the interface in order to reply with
-        # destionation net unreachable
+        # destination net unreachable
         self.tester._peer.execute([
             'sysctl', '-w', 'net.ipv4.conf.%s.forwarding=1' %
             self.tester._peer.port.name])
@@ -585,6 +630,31 @@ class FirewallTestCase(BaseFirewallTestCase):
 
         self.tester.assert_net_unreachable(self.tester.EGRESS, '1.2.3.4')
 
+    @skip_if_firewall('iptables')
+    def test_tracked_connection(self):
+        # put an openflow rule to perform a CT lookup and hence packet will
+        # carry conntrack information
+        self.tester.bridge.add_flow(
+            table=0,
+            priority=200,
+            dl_type="0x0800",
+            ct_state=ovsfw_consts.OF_STATE_NOT_TRACKED,
+            actions="ct(table=0)"
+        )
+
+        # update the sg_group to make ping pass
+        sg_rules = [{'ethertype': constants.IPv4,
+                     'direction': constants.INGRESS_DIRECTION,
+                     'protocol': constants.PROTO_NAME_ICMP},
+                    {'ethertype': constants.IPv4,
+                     'direction': constants.EGRESS_DIRECTION}]
+
+        self.tester.assert_no_connection(protocol=self.tester.ICMP,
+                                         direction=self.tester.INGRESS)
+        self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
+        self.tester.assert_connection(protocol=self.tester.ICMP,
+                                      direction=self.tester.INGRESS)
+
 
 class FirewallTestCaseIPv6(BaseFirewallTestCase):
     scenarios = BaseFirewallTestCase.scenarios_ovs_fw_interfaces
@@ -592,7 +662,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_icmp_from_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP,
                      'source_ip_prefix': self.tester.peer_ip_address}]
 
@@ -604,7 +674,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_icmp_to_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.EGRESS_DIRECTION,
+                     'direction': constants.EGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP,
                      'destination_ip_prefix': self.tester.peer_ip_address}]
 
@@ -616,7 +686,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_tcp_from_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_TCP,
                      'source_ip_prefix': self.tester.peer_ip_address}]
 
@@ -632,7 +702,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_tcp_to_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.EGRESS_DIRECTION,
+                     'direction': constants.EGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_TCP,
                      'destination_ip_prefix': self.tester.peer_ip_address}]
 
@@ -648,7 +718,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_udp_from_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_UDP,
                      'source_ip_prefix': self.tester.peer_ip_address}]
 
@@ -664,7 +734,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
 
     def test_udp_to_specific_address(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.EGRESS_DIRECTION,
+                     'direction': constants.EGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_UDP,
                      'destination_ip_prefix': self.tester.peer_ip_address}]
 
@@ -681,7 +751,7 @@ class FirewallTestCaseIPv6(BaseFirewallTestCase):
     @skip_if_firewall('openvswitch')
     def test_ip_spoofing(self):
         sg_rules = [{'ethertype': constants.IPv6,
-                     'direction': firewall.INGRESS_DIRECTION,
+                     'direction': constants.INGRESS_DIRECTION,
                      'protocol': constants.PROTO_NAME_ICMP}]
         self._apply_security_group_rules(self.FAKE_SECURITY_GROUP_ID, sg_rules)
         not_allowed_ip = "%s/64" % (
