@@ -15,8 +15,11 @@
 #    under the License.
 
 import time
+from unittest import mock
 
-from eventlet.timeout import Timeout
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 
 from neutron.common import utils
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
@@ -37,8 +40,10 @@ class TestOVSAgent(base.OVSAgentTestFramework):
 
     def test_no_stale_flows_after_port_delete(self):
         def find_drop_flow(ofport, flows):
-            for flow in flows.split("\n"):
-                if "in_port=%d" % ofport in flow and "actions=drop" in flow:
+            for flow in flows:
+                # flow.instruction == [] means actions=drop
+                if (not flow.instructions and
+                        ('in_port', ofport) in flow.match.items()):
                     return True
             return False
 
@@ -60,7 +65,7 @@ class TestOVSAgent(base.OVSAgentTestFramework):
         ofports = [port.ofport for port in self.agent.int_br.get_vif_ports()
                    if port.port_name in portnames]
 
-        #wait until ports are marked dead, with drop flow
+        # wait until ports are marked dead, with drop flow
         utils.wait_until_true(
             lambda: num_ports_with_drop_flows(
                 ofports,
@@ -68,12 +73,12 @@ class TestOVSAgent(base.OVSAgentTestFramework):
                     constants.LOCAL_SWITCHING
                 )) == len(ofports))
 
-        #delete the ports on bridge
+        # delete the ports on bridge
         for port in self.ports:
             self.agent.int_br.delete_port(port['vif_name'])
         self.wait_until_ports_state(self.ports, up=False)
 
-        #verify no stale drop flows
+        # verify no stale drop flows
         self.assertEqual(0,
             num_ports_with_drop_flows(
                 ofports,
@@ -93,6 +98,7 @@ class TestOVSAgent(base.OVSAgentTestFramework):
                         ('br_int', 'br_tun', 'br_phys')):
             actual = self.ovs.db_get_val('Bridge', br_name, 'datapath_type')
             self.assertEqual(expected, actual)
+        self.stop_agent(agent, self.agent_thread)
 
     def test_datapath_type_change(self):
         self._check_datapath_type_netdev('system')
@@ -240,6 +246,7 @@ class TestOVSAgent(base.OVSAgentTestFramework):
         patch_int_ofport_before = self.agent.patch_int_ofport
         patch_tun_ofport_before = self.agent.patch_tun_ofport
 
+        self.stop_agent(self.agent, self.agent_thread)
         self.setup_agent_and_ports(port_dicts=[], create_tunnels=True)
         self.assertEqual(patch_int_ofport_before, self.agent.patch_int_ofport)
         self.assertEqual(patch_tun_ofport_before, self.agent.patch_tun_ofport)
@@ -251,6 +258,7 @@ class TestOVSAgent(base.OVSAgentTestFramework):
         patch_int_ofport_before = self.agent.int_ofports['physnet']
         patch_phys_ofport_before = self.agent.phys_ofports['physnet']
 
+        self.stop_agent(self.agent, self.agent_thread)
         self.setup_agent_and_ports(port_dicts=[])
         self.assertEqual(patch_int_ofport_before,
                          self.agent.int_ofports['physnet'])
@@ -319,8 +327,26 @@ class TestOVSAgent(base.OVSAgentTestFramework):
                          unplug_ports=[self.ports[1]])
         self.wait_until_ports_state([self.ports[0]], up=True)
         self.assertRaises(
-            Timeout, self.wait_until_ports_state, [self.ports[1]], up=True,
-            timeout=10)
+            utils.WaitTimeout, self.wait_until_ports_state, [self.ports[1]],
+            up=True, timeout=10)
+
+    def test_ovs_restarted_event(self):
+        callback = mock.Mock()
+
+        self.setup_agent_and_ports(
+            port_dicts=self.create_test_ports())
+
+        registry.subscribe(callback,
+                           resources.AGENT,
+                           events.OVS_RESTARTED)
+
+        self.agent.check_ovs_status.return_value = constants.OVS_RESTARTED
+
+        utils.wait_until_true(lambda: callback.call_count, timeout=10)
+
+        callback.assert_called_with(resources.AGENT,
+                                    events.OVS_RESTARTED,
+                                    mock.ANY, payload=None)
 
 
 class TestOVSAgentExtensionConfig(base.OVSAgentTestFramework):
