@@ -13,17 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import eventlet
-import fixtures
-import mock
-import testscenarios
-
 from neutron_lib import constants as n_const
 from oslo_config import cfg
 from oslo_serialization import jsonutils
-from oslo_utils import importutils
 from testtools.content import text_content
 
+from neutron.agent.common import ovs_lib
 from neutron.agent.common import utils
 from neutron.agent.linux import ip_lib
 from neutron.cmd.sanity import checks
@@ -32,12 +27,13 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.plugins.ml2.drivers.openvswitch.agent \
     import ovs_neutron_agent as ovsagt
 from neutron.tests.common import base as common_base
+from neutron.tests.common import helpers
 from neutron.tests.common import net_helpers
+from neutron.tests.functional.agent.l2 import base as l2_base
 from neutron.tests.functional.agent import test_ovs_lib
 from neutron.tests.functional import base
 from neutron.tests import tools
 
-
 OVS_TRACE_FINAL_FLOW = 'Final flow'
 OVS_TRACE_DATAPATH_ACTIONS = 'Datapath actions'
 
@@ -46,53 +42,17 @@ cfg.CONF.import_group('OVS', 'neutron.plugins.ml2.drivers.openvswitch.agent.'
 
 
 class OVSAgentTestBase(test_ovs_lib.OVSBridgeTestBase,
-                       base.BaseSudoTestCase):
-    scenarios = testscenarios.multiply_scenarios([
-        ('ofctl', {'main_module': ('neutron.plugins.ml2.drivers.openvswitch.'
-                                  'agent.openflow.ovs_ofctl.main')}),
-        ('native', {'main_module': ('neutron.plugins.ml2.drivers.openvswitch.'
-                                  'agent.openflow.native.main')})],
-        test_ovs_lib.OVSBridgeTestBase.scenarios)
+                       base.BaseSudoTestCase,
+                       l2_base.OVSOFControllerHelper):
 
     def setUp(self):
         super(OVSAgentTestBase, self).setUp()
         self.br = self.useFixture(net_helpers.OVSBridgeFixture()).bridge
-        self.of_interface_mod = importutils.import_module(self.main_module)
-        self.br_int_cls = None
-        self.br_tun_cls = None
-        self.br_phys_cls = None
-        self.br_int = None
-        self.init_done = False
-        self.init_done_ev = eventlet.event.Event()
-        self.main_ev = eventlet.event.Event()
-        self.addCleanup(self._kill_main)
-        retry_count = 3
-        while True:
-            cfg.CONF.set_override('of_listen_port',
-                                  net_helpers.get_free_namespace_port(
-                                      n_const.PROTO_NAME_TCP),
-                                  group='OVS')
-            self.of_interface_mod.init_config()
-            self._main_thread = eventlet.spawn(self._kick_main)
-
-            # Wait for _kick_main -> of_interface main -> _agent_main
-            # NOTE(yamamoto): This complexity came from how "native"
-            # of_interface runs its openflow controller.  "native"
-            # of_interface's main routine blocks while running the
-            # embedded openflow controller.  In that case, the agent
-            # rpc_loop runs in another thread.  However, for FT we
-            # need to run setUp() and test_xxx() in the same thread.
-            # So I made this run of_interface's main in a separate
-            # thread instead.
-            try:
-                while not self.init_done:
-                    self.init_done_ev.wait()
-                break
-            except fixtures.TimeoutException:
-                self._kill_main()
-            retry_count -= 1
-            if retry_count < 0:
-                raise Exception('port allocation failed')
+        self.start_of_controller(cfg.CONF)
+        self.br_int = self.br_int_cls(self.br.br_name)
+        self.br_int.set_secure_mode()
+        self.br_int.setup_controllers(cfg.CONF)
+        self.br_int.setup_default_table()
 
     def _run_trace(self, brname, spec):
         required_keys = [OVS_TRACE_FINAL_FLOW, OVS_TRACE_DATAPATH_ACTIONS]
@@ -112,36 +72,12 @@ class OVSAgentTestBase(test_ovs_lib.OVSBridgeTestBase,
 
         return trace
 
-    def _kick_main(self):
-        with mock.patch.object(ovsagt, 'main', self._agent_main):
-            self.of_interface_mod.main()
-
-    def _kill_main(self):
-        self.main_ev.send()
-        self._main_thread.wait()
-
-    def _agent_main(self, bridge_classes):
-        self.br_int_cls = bridge_classes['br_int']
-        self.br_phys_cls = bridge_classes['br_phys']
-        self.br_tun_cls = bridge_classes['br_tun']
-        self.br_int = self.br_int_cls(self.br.br_name)
-        self.br_int.set_secure_mode()
-        self.br_int.setup_controllers(cfg.CONF)
-        self.br_int.setup_default_table()
-
-        # signal to setUp()
-        self.init_done = True
-        self.init_done_ev.send()
-
-        self.main_ev.wait()
-
 
 class ARPSpoofTestCase(OVSAgentTestBase):
     def setUp(self):
         # NOTE(kevinbenton): it would be way cooler to use scapy for
         # these but scapy requires the python process to be running as
         # root to bind to the ports.
-        self.addOnException(self.collect_flows_and_ports)
         super(ARPSpoofTestCase, self).setUp()
         self.skip_without_arp_support()
         self.src_addr = '192.168.0.1'
@@ -156,6 +92,7 @@ class ARPSpoofTestCase(OVSAgentTestBase):
             net_helpers.OVSPortFixture(self.br, self.dst_namespace)).port
         # wait to add IPs until after anti-spoof rules to ensure ARP doesn't
         # happen before
+        self.addOnException(self.collect_flows_and_ports)
 
     def collect_flows_and_ports(self, exc_info):
         nicevif = lambda x: ['%s=%s' % (k, getattr(x, k))
@@ -310,7 +247,7 @@ class ARPSpoofTestCase(OVSAgentTestBase):
 
 class CanaryTableTestCase(OVSAgentTestBase):
     def test_canary_table(self):
-        self.br_int.delete_flows()
+        self.br_int.uninstall_flows(cookie=ovs_lib.COOKIE_ANY)
         self.assertEqual(constants.OVS_RESTARTED,
                          self.br_int.check_canary_table())
         self.br_int.setup_canary_table()
@@ -318,12 +255,46 @@ class CanaryTableTestCase(OVSAgentTestBase):
                          self.br_int.check_canary_table())
 
 
+class DeleteFlowsTestCase(OVSAgentTestBase):
+
+    def test_delete_flows_bridge_cookie_only(self):
+        PORT = 1
+
+        self.br_int.add_flow(in_port=PORT, ip=True, nw_dst="1.1.1.1",
+                             actions="output:11")
+        self.br_int.add_flow(in_port=PORT, ip=True, nw_dst="2.2.2.2",
+                             cookie=42, actions="output:42")
+
+        # delete (should only delete flows with the bridge cookie)
+        self.br_int.delete_flows(in_port=PORT)
+        flows = self.br_int.dump_flows_for(in_port=PORT,
+                                           cookie=self.br_int._default_cookie)
+        flows42 = self.br_int.dump_flows_for(in_port=PORT, cookie=42)
+
+        # check that only flows with cookie 42 remain
+        self.assertFalse(flows)
+        self.assertTrue(flows42)
+
+    def test_delete_flows_all(self):
+        PORT = 1
+
+        self.br_int.add_flow(in_port=PORT, ip=True, nw_dst="1.1.1.1",
+                             actions="output:11")
+        self.br_int.add_flow(in_port=PORT, ip=True, nw_dst="2.2.2.2",
+                             cookie=42, actions="output:42")
+
+        # delete both flows
+        self.br_int.delete_flows(in_port=PORT, cookie=ovs_lib.COOKIE_ANY)
+
+        # check that no flow remains
+        flows = self.br_int.dump_flows_for(in_port=PORT)
+        self.assertFalse(flows)
+
+
 class OVSFlowTestCase(OVSAgentTestBase):
     """Tests defined in this class use ovs-appctl ofproto/trace commands,
     which simulate processing of imaginary packets, to check desired actions
-    are correctly set up by OVS flows.  In this way, subtle variations in
-    flows between of_interface drivers are absorbed and the same tests work
-    against those drivers.
+    are correctly set up by OVS flows.
     """
 
     def setUp(self):
@@ -346,8 +317,10 @@ class OVSFlowTestCase(OVSAgentTestBase):
         self.br_tun.set_secure_mode()
         self.br_tun.setup_controllers(cfg.CONF)
         self.tun_p = self.br_tun.add_patch_port(
-            cfg.CONF.OVS.tun_peer_patch_port,
-            cfg.CONF.OVS.int_peer_patch_port)
+            common_utils.get_rand_device_name(
+                prefix=cfg.CONF.OVS.tun_peer_patch_port),
+            common_utils.get_rand_device_name(
+                prefix=cfg.CONF.OVS.int_peer_patch_port))
         self.br_tun.setup_default_table(self.tun_p, True)
 
     def test_provision_local_vlan(self):
@@ -360,8 +333,8 @@ class OVSFlowTestCase(OVSAgentTestBase):
                                 "nw_proto=1,nw_tos=0,nw_ttl=128,"
                                 "icmp_type=8,icmp_code=0,dl_vlan=%(lvid)d"
                                 % kwargs)
-        self.assertTrue(("dl_vlan=%(segmentation_id)d" % kwargs) in
-                        trace["Final flow"])
+        self.assertIn("dl_vlan=%(segmentation_id)d" % kwargs,
+                      trace["Final flow"])
 
     def test_install_dvr_to_src_mac(self):
         other_dvr_mac = 'fa:16:3f:01:de:ad'
@@ -371,7 +344,8 @@ class OVSFlowTestCase(OVSAgentTestBase):
                   'dst_mac': '12:34:56:78:cc:dd',
                   'dst_port': 123}
         self.br_int.install_dvr_to_src_mac(network_type='vlan', **kwargs)
-        self.br_int.add_dvr_mac_vlan(mac=other_dvr_mac, port=other_dvr_port)
+        self.br_int.add_dvr_mac_physical(mac=other_dvr_mac,
+                                         port=other_dvr_port)
 
         trace = self._run_trace(self.br.br_name,
                                 "in_port=%d," % other_dvr_port +
@@ -381,14 +355,15 @@ class OVSFlowTestCase(OVSAgentTestBase):
                                 "nw_proto=1,nw_tos=0,nw_ttl=128,"
                                 "icmp_type=8,icmp_code=0,"
                                 "dl_vlan=%(vlan_tag)d" % kwargs)
-        self.assertTrue("vlan_tci=0x0000" in trace["Final flow"])
-        self.assertTrue(("dl_src=%(gateway_mac)s" % kwargs) in
-                        trace["Final flow"])
+        self.assertIn("vlan_tci=0x0000", trace["Final flow"])
+        self.assertIn(("dl_src=%(gateway_mac)s" % kwargs),
+                      trace["Final flow"])
 
+    @helpers.skip_if_ovs_older_than("2.5.1")
     def test_install_flood_to_tun(self):
         attrs = {
-            'remote_ip': '192.0.2.1',  # RFC 5737 TEST-NET-1
-            'local_ip': '198.51.100.1',  # RFC 5737 TEST-NET-2
+            'remote_ip': self.get_test_net_address(1),
+            'local_ip': self.get_test_net_address(2),
         }
         kwargs = {'vlan': 777, 'tun_id': 888}
         port_name = common_utils.get_rand_device_name(net_helpers.PORT_PREFIX)
@@ -401,12 +376,44 @@ class OVSFlowTestCase(OVSAgentTestBase):
                        "nw_tos=0,nw_ttl=128,icmp_type=8,icmp_code=0,"
                        "dl_vlan=%(vlan)d,dl_vlan_pcp=0" % kwargs)
         trace = self._run_trace(self.tun_br.br_name, test_packet)
-        self.assertTrue(("tun_id=0x%(tun_id)x" % kwargs) in
-                        trace["Final flow"])
-        self.assertTrue("vlan_tci=0x0000," in trace["Final flow"])
+        self.assertIn(("tun_id=0x%(tun_id)x" % kwargs), trace["Final flow"])
+        self.assertIn("vlan_tci=0x0000,", trace["Final flow"])
 
         self.br_tun.delete_flood_to_tun(kwargs['vlan'])
 
         trace = self._run_trace(self.tun_br.br_name, test_packet)
         self.assertEqual(" unchanged", trace["Final flow"])
-        self.assertTrue("drop" in trace["Datapath actions"])
+        self.assertIn("drop", trace["Datapath actions"])
+
+    def test_install_instructions_str(self):
+        kwargs = {'in_port': 345, 'vlan_tci': 0x1123}
+        dst_p = self.useFixture(
+            net_helpers.OVSPortFixture(self.br_tun, self.namespace)).port
+        dst_ofp = self.br_tun.get_port_ofport(dst_p.name)
+        self.br_tun.install_instructions("pop_vlan,output:%d" % dst_ofp,
+                                         priority=10, **kwargs)
+        trace = self._run_trace(self.br_tun.br_name,
+                                "in_port=%(in_port)d,dl_src=12:34:56:78:aa:bb,"
+                                "dl_dst=24:12:56:78:aa:bb,dl_type=0x0800,"
+                                "nw_src=192.168.0.1,nw_dst=192.168.0.2,"
+                                "nw_proto=1,nw_tos=0,nw_ttl=128,"
+                                "icmp_type=8,icmp_code=0,vlan_tci=%(vlan_tci)d"
+                                % kwargs)
+        self.assertIn("pop_vlan,", trace["Datapath actions"])
+
+    def test_bundled_install(self):
+        kwargs = {'in_port': 345, 'vlan_tci': 0x1321}
+        dst_p = self.useFixture(
+            net_helpers.OVSPortFixture(self.br_tun, self.namespace)).port
+        dst_ofp = self.br_tun.get_port_ofport(dst_p.name)
+        with self.br_tun.bundled() as br:
+            br.install_instructions("pop_vlan,output:%d" % dst_ofp,
+                                    priority=10, **kwargs)
+        trace = self._run_trace(self.br_tun.br_name,
+                                "in_port=%(in_port)d,dl_src=12:34:56:78:aa:bb,"
+                                "dl_dst=24:12:56:78:aa:bb,dl_type=0x0800,"
+                                "nw_src=192.168.0.1,nw_dst=192.168.0.2,"
+                                "nw_proto=1,nw_tos=0,nw_ttl=128,"
+                                "icmp_type=8,icmp_code=0,vlan_tci=%(vlan_tci)d"
+                                % kwargs)
+        self.assertIn("pop_vlan,", trace["Datapath actions"])
