@@ -13,24 +13,26 @@
 #    under the License.
 
 import collections
+from contextlib import contextmanager
+import subprocess
 
 from alembic.ddl import base as alembic_ddl
 from alembic import script as alembic_script
-from contextlib import contextmanager
 from oslo_config import cfg
 from oslo_config import fixture as config_fixture
 from oslo_db.sqlalchemy import test_migrations
+from oslo_log import log as logging
 from oslotest import base as oslotest_base
-import six
 import sqlalchemy
-from sqlalchemy import event
+from sqlalchemy import event  # noqa
 from sqlalchemy.sql import ddl as sqla_ddl
-import subprocess
 
+from neutron.db import migration as migration_root
 from neutron.db.migration.alembic_migrations import external
 from neutron.db.migration import cli as migration
 from neutron.db.migration.models import head as head_models
 from neutron.tests import base as test_base
+from neutron.tests.functional import base as functional_base
 from neutron.tests.unit import testlib_api
 
 cfg.CONF.import_opt('core_plugin', 'neutron.conf.common')
@@ -53,6 +55,14 @@ DROP_OPERATIONS = {
                 )
 }
 
+LOG = logging.getLogger(__name__)
+
+# NOTE(slaweq): replace alembic_util logging functions used normally with
+# olso_log logger to limit output on stdout
+migration.log_error = LOG.error
+migration.log_warning = LOG.warning
+migration.log_info = LOG.info
+
 
 def upgrade(engine, alembic_config, branch_name='heads'):
     cfg.CONF.set_override('connection', engine.url, group='database')
@@ -152,9 +162,9 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
         return head_models.get_metadata()
 
     def include_object(self, object_, name, type_, reflected, compare_to):
-        if type_ == 'table' and (name == 'alembic_version'
-                                 or name in external.TABLES):
-                return False
+        if type_ == 'table' and (name == 'alembic_version' or
+                                 name in external.TABLES):
+            return False
 
         return super(_TestModelsMigrations, self).include_object(
             object_, name, type_, reflected, compare_to)
@@ -290,10 +300,19 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
         find_migration_exceptions()
         engine = self.engine
         cfg.CONF.set_override('connection', engine.url, group='database')
+
         with engine.begin() as connection:
             self.alembic_config.attributes['connection'] = connection
-            migration.do_alembic_command(self.alembic_config, 'upgrade',
-                                         'kilo')
+
+            # upgrade to latest release first; --expand users are expected to
+            # apply all alembic scripts from previous releases before applying
+            # the new ones
+            for release in migration_root.NEUTRON_MILESTONES:
+                release_revisions = migration._find_milestone_revisions(
+                    self.alembic_config, release)
+                for rev in release_revisions:
+                    migration.do_alembic_command(
+                        self.alembic_config, 'upgrade', rev[0])
 
             with self._listener(engine, check_expand_branch):
                 migration.do_alembic_command(
@@ -305,25 +324,32 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                     self.alembic_config, 'upgrade',
                     '%s@head' % migration.CONTRACT_BRANCH)
 
-    def _test_has_offline_migrations(self, revision, expected):
+    # NOTE(ihrachys): if this test fails for you, it probably means that you
+    # attempt to add an unsafe contract migration script, that is in
+    # contradiction to blueprint online-upgrades
+    def test_forbid_offline_migrations_starting_newton(self):
         engine = self.get_engine()
         cfg.CONF.set_override('connection', engine.url, group='database')
-        migration.do_alembic_command(self.alembic_config, 'upgrade', revision)
-        self.assertEqual(expected,
-                         migration.has_offline_migrations(self.alembic_config,
-                                                          'unused'))
-
-    def test_has_offline_migrations_pending_contract_scripts(self):
-        self._test_has_offline_migrations('kilo', True)
-
-    def test_has_offline_migrations_all_heads_upgraded(self):
-        self._test_has_offline_migrations('heads', False)
+        # the following revisions are Newton heads
+        for revision in ('5cd92597d11d', '5c85685d616d'):
+            migration.do_alembic_command(
+                self.alembic_config, 'upgrade', revision)
+        self.assertFalse(migration.has_offline_migrations(
+            self.alembic_config, 'unused'),
+            msg='Offline contract migration scripts are forbidden for Ocata+')
 
 
 class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
                                 _TestModelsMigrations,
-                                testlib_api.SqlTestCaseLight):
+                                testlib_api.SqlTestCaseLight,
+                                functional_base.BaseLoggingTestCase):
 
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_forbid_offline_migrations_starting_newton(self):
+        super(TestModelsMigrationsMysql,
+              self).test_forbid_offline_migrations_starting_newton()
+
+    @test_base.skip_if_timeout("bug 1687027")
     def test_check_mysql_engine(self):
         engine = self.get_engine()
         cfg.CONF.set_override('connection', engine.url, group='database')
@@ -334,13 +360,30 @@ class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
             insp = sqlalchemy.engine.reflection.Inspector.from_engine(engine)
             # Test that table creation on MySQL only builds InnoDB tables
             tables = insp.get_table_names()
-            self.assertTrue(len(tables) > 0,
-                            "No tables found. Wrong schema?")
+            self.assertGreater(len(tables), 0,
+                               "No tables found. Wrong schema?")
             res = [table for table in tables if
-                   insp.get_table_options(table)['mysql_engine'] != 'InnoDB'
-                   and table != 'alembic_version']
+                   insp.get_table_options(table)['mysql_engine'] !=
+                   'InnoDB' and
+                   table != 'alembic_version']
             self.assertEqual(0, len(res), "%s non InnoDB tables created" % res)
 
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_upgrade_expand_branch(self):
+        super(TestModelsMigrationsMysql, self).test_upgrade_expand_branch()
+
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_upgrade_contract_branch(self):
+        super(TestModelsMigrationsMysql, self).test_upgrade_contract_branch()
+
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_branches(self):
+        super(TestModelsMigrationsMysql, self).test_branches()
+
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_models_sync(self):
+        super(TestModelsMigrationsMysql, self).test_models_sync()
+
 
 class TestModelsMigrationsPsql(testlib_api.PostgreSQLTestCaseMixin,
                                _TestModelsMigrations,
@@ -356,6 +399,10 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
         self.alembic_config = migration.get_neutron_config()
         self.alembic_config.neutron_config = cfg.CONF
 
+    def _drop_table(self, table):
+        with self.engine.begin() as conn:
+            table.drop(conn)
+
     def test_check_sanity_1df244e556f5(self):
         ha_router_agent_port_bindings = sqlalchemy.Table(
             'ha_router_agent_port_bindings', sqlalchemy.MetaData(),
@@ -365,6 +412,10 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
 
         with self.engine.connect() as conn:
             ha_router_agent_port_bindings.create(conn)
+            self.addCleanup(self._drop_table, ha_router_agent_port_bindings)
+            # NOTE(haleyb): without this disabled, pylint complains
+            # about a missing 'dml' argument.
+            # pylint: disable=no-value-for-parameter
             conn.execute(ha_router_agent_port_bindings.insert(), [
                 {'port_id': '1234', 'router_id': '12345',
                  'l3_agent_id': '123'},
@@ -386,6 +437,10 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
 
         with self.engine.connect() as conn:
             routerports.create(conn)
+            self.addCleanup(self._drop_table, routerports)
+            # NOTE(haleyb): without this disabled, pylint complains
+            # about a missing 'dml' argument.
+            # pylint: disable=no-value-for-parameter
             conn.execute(routerports.insert(), [
                 {'router_id': '1234', 'port_id': '12345',
                  'port_type': '123'},
@@ -407,6 +462,10 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
 
         with self.engine.connect() as conn:
             floatingips.create(conn)
+            self.addCleanup(self._drop_table, floatingips)
+            # NOTE(haleyb): without this disabled, pylint complains
+            # about a missing 'dml' argument.
+            # pylint: disable=no-value-for-parameter
             conn.execute(floatingips.insert(), [
                 {'floating_network_id': '12345',
                  'fixed_port_id': '1234567',
@@ -430,6 +489,10 @@ class TestSanityCheck(testlib_api.SqlTestCaseLight):
 
         with self.engine.connect() as conn:
             floatingips.create(conn)
+            self.addCleanup(self._drop_table, floatingips)
+            # NOTE(haleyb): without this disabled, pylint complains
+            # about a missing 'dml' argument.
+            # pylint: disable=no-value-for-parameter
             conn.execute(floatingips.insert(), [
                 {'floating_network_id': '12345',
                  'fixed_port_id': '1234567',
@@ -467,7 +530,7 @@ class TestWalkDowngrade(oslotest_base.BaseTestCase):
 
 
 class _TestWalkMigrations(object):
-    '''This will add framework for testing schema migarations
+    '''This will add framework for testing schema migration
        for different backends.
 
     '''
@@ -486,7 +549,7 @@ class _TestWalkMigrations(object):
         self.script_dir = alembic_script.ScriptDirectory.from_config(db_config)
         db_config.neutron_config = cfg.CONF
         db_config.neutron_config.set_override('connection',
-                                              six.text_type(uri),
+                                              str(uri),
                                               group='database')
         return db_config
 
@@ -503,18 +566,18 @@ class _TestWalkMigrations(object):
             # Destination, current
             yield rev.revision, rev.down_revision
 
-    def _migrate_up(self, config, engine, dest, curr, with_data=False):
-        if with_data:
-            data = None
-            pre_upgrade = getattr(
-                self, "_pre_upgrade_%s" % dest, None)
-            if pre_upgrade:
-                data = pre_upgrade(engine)
-        migration.do_alembic_command(config, 'upgrade', dest)
-        if with_data:
-            check = getattr(self, "_check_%s" % dest, None)
-            if check and data:
-                check(engine, data)
+    def _migrate_up(self, config, engine, dest, curr):
+        data = None
+        check = getattr(self, "_check_%s" % dest, None)
+        pre_upgrade = getattr(self, "_pre_upgrade_%s" % dest, None)
+        if pre_upgrade:
+            if curr:
+                migration.do_alembic_command(config, 'upgrade', curr)
+            data = pre_upgrade(engine)
+
+        if check and data:
+            migration.do_alembic_command(config, 'upgrade', dest)
+            check(engine, data)
 
     def test_walk_versions(self):
         """Test migrations ability to upgrade and downgrade.
@@ -524,13 +587,28 @@ class _TestWalkMigrations(object):
         config = self._get_alembic_config(engine.url)
         revisions = self._revisions()
         for dest, curr in revisions:
-            self._migrate_up(config, engine, dest, curr, with_data=True)
+            self._migrate_up(config, engine, dest, curr)
+
+        if dest:
+            migration.do_alembic_command(config, 'upgrade', dest)
 
 
 class TestWalkMigrationsMysql(testlib_api.MySQLTestCaseMixin,
                               _TestWalkMigrations,
                               testlib_api.SqlTestCaseLight):
-    pass
+
+    # NOTE(slaweq): this workaround is taken from Manila patch:
+    # https://review.opendev.org/#/c/291397/
+    # Set 5 minutes timeout for case of running it on
+    # very slow nodes/VMs. Note, that this test becomes slower with each
+    # addition of new DB migration. On fast nodes it can take about 5-10
+    # secs having Mitaka set of migrations. 'pymysql' works much slower
+    # on slow nodes than 'psycopg2' and because of that this increased
+    # timeout is required only when for testing with 'mysql' backend.
+    @test_base.set_timeout(600)
+    @test_base.skip_if_timeout("bug 1687027")
+    def test_walk_versions(self):
+        super(TestWalkMigrationsMysql, self).test_walk_versions()
 
 
 class TestWalkMigrationsPsql(testlib_api.PostgreSQLTestCaseMixin,
