@@ -13,18 +13,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
+from neutron_lib.api import attributes
+from neutron_lib.callbacks import events
+from neutron_lib import context
+from neutron_lib.db import constants as db_const
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_policy import policy as oslo_policy
 from oslo_serialization import jsonutils
-from oslo_utils import uuidutils
 
-from neutron.api.v2 import attributes
-from neutron import context
 from neutron.db.quota import driver as quota_driver
 from neutron import manager
 from neutron.pecan_wsgi.controllers import resource
-from neutron.pecan_wsgi.hooks import policy_enforcement as pe
 from neutron import policy
 from neutron.tests.functional.pecan_wsgi import test_functional
 
@@ -45,6 +47,51 @@ class TestOwnershipHook(test_functional.PecanFunctionalTest):
         self.assertEqual(201, port_response.status_int)
 
 
+class TestQueryParametersHook(test_functional.PecanFunctionalTest):
+
+    def test_if_match_on_update(self):
+        net_response = jsonutils.loads(self.app.post_json(
+            '/v2.0/networks.json',
+            params={'network': {'name': 'meh'}},
+            headers={'X-Project-Id': 'tenid'}).body)
+        network_id = net_response['network']['id']
+        response = self.app.put_json('/v2.0/networks/%s.json' % network_id,
+                                     params={'network': {'name': 'cat'}},
+                                     headers={'X-Project-Id': 'tenid',
+                                              'If-Match': 'revision_number=0'},
+                                     expect_errors=True)
+        # revision plugin not supported by default, so badrequest
+        self.assertEqual(400, response.status_int)
+
+
+class TestQueryParametersHookWithRevision(test_functional.PecanFunctionalTest):
+
+    def setUp(self):
+        cfg.CONF.set_override('service_plugins', ['revisions'])
+        super(TestQueryParametersHookWithRevision, self).setUp()
+
+    def test_if_match_on_update(self):
+        net_response = jsonutils.loads(self.app.post_json(
+            '/v2.0/networks.json',
+            params={'network': {'name': 'meh'}},
+            headers={'X-Project-Id': 'tenid'}).body)
+        network_id = net_response['network']['id']
+        rev = net_response['network']['revision_number']
+        stale = rev - 1
+
+        response = self.app.put_json(
+            '/v2.0/networks/%s.json' % network_id,
+            params={'network': {'name': 'cat'}},
+            headers={'X-Project-Id': 'tenid',
+                     'If-Match': 'revision_number=%s' % stale},
+            expect_errors=True)
+        self.assertEqual(412, response.status_int)
+        self.app.put_json('/v2.0/networks/%s.json' % network_id,
+                          params={'network': {'name': 'cat'}},
+                          headers={'X-Project-Id': 'tenid',
+                                   'If-Match': 'revision_number=%s' % rev})
+
+
 class TestQuotaEnforcementHook(test_functional.PecanFunctionalTest):
 
     def test_quota_enforcement_single(self):
@@ -101,7 +148,7 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
             'tenant_id': {'allow_post': True, 'allow_put': False,
                           'required_by_policy': True,
                           'validate': {'type:string':
-                                       attributes.TENANT_ID_MAX_LEN},
+                                       db_const.PROJECT_ID_FIELD_SIZE},
                           'is_visible': True}
         }
     }
@@ -113,9 +160,9 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
         # tests, or at least I hope so)
         super(TestPolicyEnforcementHook, self).setUp()
         self.mock_plugin = mock.Mock()
-        attributes.RESOURCE_ATTRIBUTE_MAP.update(self.FAKE_RESOURCE)
-        attributes.PLURALS['mehs'] = 'meh'
-        manager.NeutronManager.set_plugin_for_resource('meh', self.mock_plugin)
+        attributes.RESOURCES.update(self.FAKE_RESOURCE)
+        manager.NeutronManager.set_plugin_for_resource('mehs',
+                                                       self.mock_plugin)
         fake_controller = resource.CollectionsController('mehs', 'meh')
         manager.NeutronManager.set_controller_for_resource(
             'mehs', fake_controller)
@@ -163,6 +210,23 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
         self.assertEqual(1, self.mock_plugin.get_meh.call_count)
         self.assertEqual(0, self.mock_plugin.update_meh.call_count)
 
+    def test_before_on_put_not_found_when_not_authorized_to_get(self):
+        # the user won't even have permission to view this resource
+        # so the error on unauthorized updates should be translated into
+        # a 404
+        self.mock_plugin.get_meh.return_value = {
+            'id': 'yyy',
+            'attr': 'meh',
+            'restricted_attr': '',
+            'tenant_id': 'tenid'}
+        response = self.app.put_json('/v2.0/mehs/yyy.json',
+                                     params={'meh': {'attr': 'meh'}},
+                                     headers={'X-Project-Id': 'tenid'},
+                                     expect_errors=True)
+        self.assertEqual(404, response.status_int)
+        self.assertEqual(1, self.mock_plugin.get_meh.call_count)
+        self.assertEqual(0, self.mock_plugin.update_meh.call_count)
+
     def test_before_on_delete_not_authorized(self):
         # The policy hook here should load the resource, and therefore we must
         # mock a get response
@@ -181,9 +245,10 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
         self.assertEqual(1, self.mock_plugin.get_meh.call_count)
         self.assertEqual(0, self.mock_plugin.delete_meh.call_count)
 
-    def test_after_on_get_not_authorized(self):
+    def test_after_on_get_not_found(self):
         # The GET test policy will deny access to anything whose id is not
-        # 'xxx', so the following request should be forbidden
+        # 'xxx', so the following request should be forbidden and presented
+        # to the user as an HTTPNotFound
         self.mock_plugin.get_meh.return_value = {
             'id': 'yyy',
             'attr': 'meh',
@@ -194,7 +259,7 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
         response = self.app.get('/v2.0/mehs/yyy.json',
                                 headers={'X-Project-Id': 'tenid'},
                                 expect_errors=True)
-        self.assertEqual(403, response.status_int)
+        self.assertEqual(404, response.status_int)
         self.assertEqual(1, self.mock_plugin.get_meh.call_count)
 
     def test_after_on_get_excludes_admin_attribute(self):
@@ -221,173 +286,16 @@ class TestPolicyEnforcementHook(test_functional.PecanFunctionalTest):
         json_response = jsonutils.loads(response.body)
         self.assertNotIn('restricted_attr', json_response['mehs'][0])
 
-
-class DHCPNotifierTestBase(test_functional.PecanFunctionalTest):
-
-    def setUp(self):
-        # the DHCP notifier needs to be mocked so that correct operations can
-        # be easily validated. For the purpose of this test it is indeed not
-        # necessary that the notification is actually received and processed by
-        # the agent
-        patcher = mock.patch('neutron.api.rpc.agentnotifiers.'
-                             'dhcp_rpc_agent_api.DhcpAgentNotifyAPI.notify')
-        self.mock_notifier = patcher.start()
-        super(DHCPNotifierTestBase, self).setUp()
-
-
-class TestDHCPNotifierHookNegative(DHCPNotifierTestBase):
-
-    def setUp(self):
-        cfg.CONF.set_override('dhcp_agent_notification', False)
-        super(TestDHCPNotifierHookNegative, self).setUp()
-
-    def test_dhcp_notifications_disabled(self):
-        self.app.post_json(
-            '/v2.0/networks.json',
-            params={'network': {'name': 'meh'}},
-            headers={'X-Project-Id': 'tenid'})
-        self.assertEqual(0, self.mock_notifier.call_count)
-
-
-class TestDHCPNotifierHook(DHCPNotifierTestBase):
-
-    def test_get_does_not_trigger_notification(self):
-        self.do_request('/v2.0/networks', tenant_id='tenid')
-        self.assertEqual(0, self.mock_notifier.call_count)
-
-    def test_post_put_delete_triggers_notification(self):
-        ctx = context.get_admin_context()
-        plugin = manager.NeutronManager.get_plugin()
-
-        req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
-        response = self.app.post_json(
-            '/v2.0/networks.json',
-            params={'network': {'name': 'meh'}}, headers=req_headers)
-        self.assertEqual(201, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        net = {'network': plugin.get_network(ctx, json_body['network']['id'])}
-        self.assertEqual(1, self.mock_notifier.call_count)
-        self.assertEqual(mock.call(mock.ANY, net, 'network.create.end'),
-                         self.mock_notifier.mock_calls[-1])
-        network_id = json_body['network']['id']
-
-        response = self.app.put_json(
-            '/v2.0/networks/%s.json' % network_id,
-            params={'network': {'name': 'meh-2'}},
-            headers=req_headers)
+    def test_after_inits_policy(self):
+        self.mock_plugin.get_mehs.return_value = [{
+            'id': 'xxx',
+            'attr': 'meh',
+            'restricted_attr': '',
+            'tenant_id': 'tenid'}]
+        policy.reset()
+        response = self.app.get('/v2.0/mehs',
+                                headers={'X-Project-Id': 'tenid'})
         self.assertEqual(200, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        net = {'network': plugin.get_network(ctx, json_body['network']['id'])}
-        self.assertEqual(2, self.mock_notifier.call_count)
-        self.assertEqual(mock.call(mock.ANY, net, 'network.update.end'),
-                         self.mock_notifier.mock_calls[-1])
-
-        response = self.app.delete(
-            '/v2.0/networks/%s.json' % network_id, headers=req_headers)
-        self.assertEqual(204, response.status_int)
-        self.assertEqual(3, self.mock_notifier.call_count)
-        # No need to validate data content sent to the notifier as it's just
-        # going to load the object from the database
-        self.assertEqual(mock.call(mock.ANY, mock.ANY, 'network.delete.end'),
-                         self.mock_notifier.mock_calls[-1])
-
-    def test_bulk_create_triggers_notifications(self):
-        req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
-        response = self.app.post_json(
-            '/v2.0/networks.json',
-            params={'networks': [{'name': 'meh_1'},
-                                 {'name': 'meh_2'}]},
-            headers=req_headers)
-        self.assertEqual(201, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        item_1 = json_body['networks'][0]
-        item_2 = json_body['networks'][1]
-        self.assertEqual(2, self.mock_notifier.call_count)
-        self.mock_notifier.assert_has_calls(
-            [mock.call(mock.ANY, {'network': item_1}, 'network.create.end'),
-             mock.call(mock.ANY, {'network': item_2}, 'network.create.end')])
-
-
-class TestNovaNotifierHook(test_functional.PecanFunctionalTest):
-
-    def setUp(self):
-        patcher = mock.patch('neutron.pecan_wsgi.hooks.notifier.NotifierHook.'
-                             '_nova_notify')
-        self.mock_notifier = patcher.start()
-        super(TestNovaNotifierHook, self).setUp()
-
-    def test_nova_notification_skips_on_failure(self):
-        req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
-        response = self.app.put_json(
-            '/v2.0/networks/%s.json' % uuidutils.generate_uuid(),
-            params={'network': {'name': 'meh-2'}},
-            headers=req_headers,
-            expect_errors=True)
-        self.assertEqual(404, response.status_int)
-        self.assertFalse(self.mock_notifier.called)
-
-    def test_nova_notifications_disabled(self):
-        cfg.CONF.set_override('notify_nova_on_port_data_changes', False)
-        self.app.post_json(
-            '/v2.0/networks.json',
-            params={'network': {'name': 'meh'}},
-            headers={'X-Project-Id': 'tenid'})
-        self.assertFalse(self.mock_notifier.called)
-
-    def test_post_put_delete_triggers_notification(self):
-        req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
-        response = self.app.post_json(
-            '/v2.0/networks.json',
-            params={'network': {'name': 'meh'}}, headers=req_headers)
-        self.assertEqual(201, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        self.mock_notifier.assert_called_once_with('create', 'network', {},
-                                                   json_body)
-        self.mock_notifier.reset_mock()
-        network_id = json_body['network']['id']
-
-        # NOTE(kevinbenton): the original passed into the notifier does
-        # not contain all of the fields of the object. Only those required
-        # by the policy engine are included.
-        controller = manager.NeutronManager.get_controller_for_resource(
-            'networks')
-        orig = pe.fetch_resource(context.get_admin_context(), controller,
-                                 'network', network_id)
-        response = self.app.put_json(
-            '/v2.0/networks/%s.json' % network_id,
-            params={'network': {'name': 'meh-2'}},
-            headers=req_headers)
-        self.assertEqual(200, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        self.mock_notifier.assert_called_once_with('update', 'network',
-                                                   orig, json_body)
-        self.mock_notifier.reset_mock()
-
-        orig = pe.fetch_resource(context.get_admin_context(), controller,
-                                 'network', network_id)
-        response = self.app.delete(
-            '/v2.0/networks/%s.json' % network_id, headers=req_headers)
-        self.assertEqual(204, response.status_int)
-        # No need to validate data content sent to the notifier as it's just
-        # going to load the object from the database
-        self.mock_notifier.assert_called_once_with('delete', 'network', {},
-                                                   {'network': orig})
-
-    def test_bulk_create_triggers_notifications(self):
-        req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
-        response = self.app.post_json(
-            '/v2.0/networks.json',
-            params={'networks': [{'name': 'meh_1'},
-                                 {'name': 'meh_2'}]},
-            headers=req_headers)
-        self.assertEqual(201, response.status_int)
-        json_body = jsonutils.loads(response.body)
-        item_1 = json_body['networks'][0]
-        item_2 = json_body['networks'][1]
-        self.assertEqual(
-            [mock.call('create', 'network', {}, {'network': item_1}),
-             mock.call('create', 'network', {}, {'network': item_2})],
-            self.mock_notifier.mock_calls)
 
 
 class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
@@ -427,13 +335,16 @@ class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
             self.mock_notifier.mock_calls)
         self.mock_notifier.reset_mock()
 
+        before_payload = {'network_id': network_id}
+        after_payload = before_payload.copy()
+        after_payload['network'] = directory.get_plugin().get_network(
+            context.get_admin_context(), network_id)
         response = self.app.delete(
             '/v2.0/networks/%s.json' % network_id, headers=req_headers)
         self.assertEqual(204, response.status_int)
-        payload = {'network_id': network_id}
         self.assertEqual(
-            [mock.call(mock.ANY, 'network.delete.start', payload),
-             mock.call(mock.ANY, 'network.delete.end', payload)],
+            [mock.call(mock.ANY, 'network.delete.start', before_payload),
+             mock.call(mock.ANY, 'network.delete.end', after_payload)],
             self.mock_notifier.mock_calls)
 
     def test_bulk_create_triggers_notification(self):
@@ -453,7 +364,7 @@ class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
     def test_bad_create_doesnt_emit_end(self):
         req_headers = {'X-Project-Id': 'tenid', 'X-Roles': 'admin'}
         payload = {'network': {'name': 'meh'}}
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         with mock.patch.object(plugin, 'create_network',
                                side_effect=ValueError):
             response = self.app.post_json(
@@ -475,7 +386,7 @@ class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
         self.assertEqual(201, response.status_int)
         json_body = jsonutils.loads(response.body)
         self.mock_notifier.reset_mock()
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         with mock.patch.object(plugin, 'update_network',
                                side_effect=ValueError):
             response = self.app.put_json(
@@ -497,7 +408,7 @@ class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
         self.assertEqual(201, response.status_int)
         json_body = jsonutils.loads(response.body)
         self.mock_notifier.reset_mock()
-        plugin = manager.NeutronManager.get_plugin()
+        plugin = directory.get_plugin()
         with mock.patch.object(plugin, 'delete_network',
                                side_effect=ValueError):
             response = self.app.delete(
@@ -507,3 +418,84 @@ class TestMetricsNotifierHook(test_functional.PecanFunctionalTest):
         self.assertEqual(
             [mock.call(mock.ANY, 'network.delete.start', mock.ANY)],
             self.mock_notifier.mock_calls)
+
+
+class TestCallbackRegistryNotifier(test_functional.PecanFunctionalTest):
+
+    def setUp(self):
+        super(TestCallbackRegistryNotifier, self).setUp()
+        patcher = mock.patch('neutron.pecan_wsgi.hooks.notifier.registry')
+        self.mock_notifier = patcher.start().publish
+
+    def _create(self, bulk=False):
+        if bulk:
+            body = {'networks': [{'name': 'meh-1'}, {'name': 'meh-2'}]}
+        else:
+            body = {'network': {'name': 'meh-1'}}
+        response = self.app.post_json(
+            '/v2.0/networks.json',
+            params=body, headers={'X-Project-Id': 'tenid'})
+        return response.json
+
+    def test_create(self):
+        self._create()
+        self.mock_notifier.assert_called_once_with(
+            'network', events.BEFORE_RESPONSE, mock.ANY, payload=mock.ANY)
+
+        payload = self.mock_notifier.call_args[1]['payload']
+        self.assertEqual('network.create.end', payload.method_name)
+        self.assertEqual('create_network', payload.action)
+        self.assertEqual('networks', payload.collection_name)
+
+        actual = payload.latest_state
+        self.assertEqual('meh-1', actual['network']['name'])
+
+    def test_create_bulk(self):
+        self._create(bulk=True)
+        self.mock_notifier.assert_called_once_with(
+            'network', events.BEFORE_RESPONSE, mock.ANY, payload=mock.ANY)
+
+        payload = self.mock_notifier.call_args[1]['payload']
+        self.assertEqual('network.create.end', payload.method_name)
+        self.assertEqual('create_network', payload.action)
+        self.assertEqual('networks', payload.collection_name)
+        actual = payload.latest_state
+        self.assertEqual(2, len(actual['networks']))
+        self.assertEqual('meh-1', actual['networks'][0]['name'])
+        self.assertEqual('meh-2', actual['networks'][1]['name'])
+
+    def test_update(self):
+        network_id = self._create()['network']['id']
+        self.mock_notifier.reset_mock()
+        self.app.put_json('/v2.0/networks/%s.json' % network_id,
+                          params={'network': {'name': 'new-meh'}},
+                          headers={'X-Project-Id': 'tenid'})
+        self.mock_notifier.assert_called_once_with(
+            'network', events.BEFORE_RESPONSE, mock.ANY, payload=mock.ANY)
+
+        payload = self.mock_notifier.call_args[1]['payload']
+        self.assertEqual('network.update.end', payload.method_name)
+        self.assertEqual('update_network', payload.action)
+        self.assertEqual('networks', payload.collection_name)
+
+        actual_new = payload.latest_state
+        self.assertEqual('new-meh', actual_new['network']['name'])
+        actual_original = payload.states[0]
+        self.assertEqual(network_id, actual_original['id'])
+
+    def test_delete(self):
+        network_id = self._create()['network']['id']
+        self.mock_notifier.reset_mock()
+        self.app.delete(
+            '/v2.0/networks/%s.json' % network_id,
+            headers={'X-Project-Id': 'tenid'})
+        self.mock_notifier.assert_called_once_with(
+            'network', events.BEFORE_RESPONSE, mock.ANY, payload=mock.ANY)
+
+        payload = self.mock_notifier.call_args[1]['payload']
+        self.assertEqual('network.delete.end', payload.method_name)
+        self.assertEqual('delete_network', payload.action)
+        self.assertEqual('networks', payload.collection_name)
+
+        actual = payload.latest_state
+        self.assertEqual(network_id, actual['network']['id'])
