@@ -16,16 +16,15 @@
 import collections
 from operator import attrgetter
 
+from neutron_lib.api.definitions import provider_net as providernet
 from neutron_lib import constants
-import six
+from neutron_lib import context
+from oslo_utils import uuidutils
 import testscenarios
 
-from neutron import context
 from neutron.db import agents_db
 from neutron.db import agentschedulers_db
-from neutron.db import common_db_mixin
-from neutron.db.network_dhcp_agent_binding import models as ndab_model
-from neutron.extensions import providernet
+from neutron.objects import network
 from neutron.scheduler import dhcp_agent_scheduler
 from neutron.tests.common import helpers
 from neutron.tests.unit.plugins.ml2 import test_plugin
@@ -106,7 +105,6 @@ class BaseTestScheduleNetwork(object):
 class TestChanceScheduleNetwork(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
                                 agentschedulers_db.DhcpAgentSchedulerDbMixin,
                                 agents_db.AgentDbMixin,
-                                common_db_mixin.CommonDbMixin,
                                 BaseTestScheduleNetwork):
     """Test various scenarios for ChanceScheduler.schedule."""
 
@@ -144,7 +142,6 @@ class TestChanceScheduleNetwork(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
 class TestWeightScheduleNetwork(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
                                 agentschedulers_db.DhcpAgentSchedulerDbMixin,
                                 agents_db.AgentDbMixin,
-                                common_db_mixin.CommonDbMixin,
                                 BaseTestScheduleNetwork):
     """Test various scenarios for WeightScheduler.schedule."""
 
@@ -190,8 +187,7 @@ class TestWeightScheduleNetwork(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
 
 class TestAutoSchedule(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
                        agentschedulers_db.DhcpAgentSchedulerDbMixin,
-                       agents_db.AgentDbMixin,
-                       common_db_mixin.CommonDbMixin):
+                       agents_db.AgentDbMixin):
     """Test various scenarios for ChanceScheduler.auto_schedule_networks.
 
         Below is the brief description of the scenario variables
@@ -287,15 +283,12 @@ class TestAutoSchedule(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
                                                     'network-3'],
                                         'agent-1': ['network-0',
                                                     'network-1',
-                                                    'network-2',
                                                     'network-3'],
                                         'agent-2': ['network-1',
                                                     'network-2',
                                                     'network-3'],
                                         'agent-3': ['network-0',
-                                                    'network-1',
-                                                    'network-2',
-                                                    'network-3']})),
+                                                    'network-2']})),
 
         ('No agents scheduled if networks already hosted and'
          ' max_agents_per_network reached',
@@ -344,26 +337,19 @@ class TestAutoSchedule(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
               no_network_with_az_match=True)),
     ]
 
-    def _strip_host_index(self, name):
-        """Strips the host index.
-
-        Eg. if name = '2-agent-3', then 'agent-3' is returned.
-        """
-        return name[name.find('-') + 1:]
-
     def _extract_index(self, name):
         """Extracts the index number and returns.
 
-        Eg. if name = '2-agent-3', then 3 is returned
+        Eg. if name = 'agent-3', then 3 is returned
         """
         return int(name.split('-')[-1])
 
     def get_subnets(self, context, fields=None):
         subnets = []
-        for net_id in self._networks:
-            enable_dhcp = (not self._strip_host_index(net_id) in
-                           self.networks_with_dhcp_disabled)
-            subnets.append({'network_id': net_id,
+        for net in self._networks:
+            enable_dhcp = (net['name'] not in
+              self.networks_with_dhcp_disabled)
+            subnets.append({'network_id': net.id,
                             'enable_dhcp': enable_dhcp,
                             'segment_id': None})
         return subnets
@@ -375,62 +361,61 @@ class TestAutoSchedule(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
         return {'availability_zone_hints': az_hints}
 
     def _get_hosted_networks_on_dhcp_agent(self, agent_id):
-        query = self.ctx.session.query(
-            ndab_model.NetworkDhcpAgentBinding.network_id)
-        query = query.filter(
-            ndab_model.NetworkDhcpAgentBinding.dhcp_agent_id ==
-            agent_id)
+        binding_objs = network.NetworkDhcpAgentBinding.get_objects(
+            self.ctx, dhcp_agent_id=agent_id)
+        return [item.network_id for item in binding_objs]
 
-        return [item[0] for item in query]
-
-    def _test_auto_schedule(self, host_index):
+    def test_auto_schedule(self):
         self.config(dhcp_agents_per_network=self.max_agents_per_network)
         scheduler = dhcp_agent_scheduler.ChanceScheduler()
         self.ctx = context.get_admin_context()
-        msg = 'host_index = %s' % host_index
 
         # create dhcp agents
-        hosts = ['%s-agent-%s' % (host_index, i)
-                 for i in range(self.agent_count)]
+        hosts = ['agent-%s' % i for i in range(self.agent_count)]
         dhcp_agents = self._create_and_set_agents_down(hosts)
 
         # create networks
-        self._networks = ['%s-network-%s' % (host_index, i)
-                          for i in range(self.network_count)]
-        self._save_networks(self._networks)
+        self._networks = [
+            network.Network(
+                self.ctx,
+                id=uuidutils.generate_uuid(),
+                name='network-%s' % i)
+            for i in range(self.network_count)
+        ]
+        for i in range(len(self._networks)):
+            self._networks[i].create()
+        network_ids = [net.id for net in self._networks]
 
         # pre schedule the networks to the agents defined in
         # self.hosted_networks before calling auto_schedule_network
-        for agent, networks in six.iteritems(self.hosted_networks):
+        for agent, networks in self.hosted_networks.items():
             agent_index = self._extract_index(agent)
             for net in networks:
                 net_index = self._extract_index(net)
                 scheduler.resource_filter.bind(self.ctx,
                                                [dhcp_agents[agent_index]],
-                                               self._networks[net_index])
+                                               network_ids[net_index])
+        for host_index in range(self.agent_count):
+            msg = 'host_index = %s' % host_index
+            retval = scheduler.auto_schedule_networks(self, self.ctx,
+                                                      hosts[host_index])
+            self.assertEqual(self.expected_auto_schedule_return_value, retval,
+                             message=msg)
 
-        retval = scheduler.auto_schedule_networks(self, self.ctx,
-                                                  hosts[host_index])
-        self.assertEqual(self.expected_auto_schedule_return_value, retval,
-                         message=msg)
-
-        agent_id = dhcp_agents[host_index].id
-        hosted_networks = self._get_hosted_networks_on_dhcp_agent(agent_id)
-        hosted_net_ids = [self._strip_host_index(net)
-                          for net in hosted_networks]
-        expected_hosted_networks = self.expected_hosted_networks['agent-%s' %
-                                                                 host_index]
-        self.assertItemsEqual(hosted_net_ids, expected_hosted_networks, msg)
-
-    def test_auto_schedule(self):
-        for i in range(self.agent_count):
-            self._test_auto_schedule(i)
+            agent_id = dhcp_agents[host_index].id
+            hosted_net_ids = self._get_hosted_networks_on_dhcp_agent(agent_id)
+            hosted_net_names = [
+                net['name'] for net in
+                network.Network.get_objects(self.ctx, id=hosted_net_ids)]
+            expected_hosted_networks = self.expected_hosted_networks[
+                'agent-%s' % host_index]
+            self.assertItemsEqual(
+                hosted_net_names, expected_hosted_networks, msg)
 
 
 class TestAZAwareWeightScheduler(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
                                  agentschedulers_db.DhcpAgentSchedulerDbMixin,
-                                 agents_db.AgentDbMixin,
-                                 common_db_mixin.CommonDbMixin):
+                                 agents_db.AgentDbMixin):
     """Test various scenarios for AZAwareWeightScheduler.schedule.
 
         az_count
@@ -569,7 +554,7 @@ class TestAZAwareWeightScheduler(test_dhcp_sch.TestDhcpSchedulerBaseTestCase,
 
 
 class TestDHCPSchedulerWithNetworkAccessibility(
-    test_plugin.Ml2PluginV2TestCase):
+        test_plugin.Ml2PluginV2TestCase):
 
     _mechanism_drivers = ['openvswitch']
 
@@ -599,7 +584,7 @@ class TestDHCPSchedulerWithNetworkAccessibility(
             admin_context,
             {'subnet':
                 {'name': 'name',
-                 'ip_version': 4,
+                 'ip_version': constants.IP_VERSION_4,
                  'network_id': net['id'],
                  'cidr': '10.0.0.0/24',
                  'gateway_ip': constants.ATTR_NOT_SPECIFIED,
