@@ -16,14 +16,23 @@
 import collections
 import random
 
+from neutron_lib import constants
+from neutron_lib import context
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from oslo_utils import uuidutils
 import testscenarios
 
-from neutron import context
+from neutron.objects import network as net_obj
 from neutron.scheduler import l3_agent_scheduler
 from neutron.services.l3_router import l3_router_plugin
 from neutron.tests.common import helpers
 from neutron.tests.unit.db import test_db_base_plugin_v2
 
+_uuid = uuidutils.generate_uuid
+
+PLUGIN_NAME = 'neutron.plugins.ml2.plugin.Ml2Plugin'
+
 # Required to generate tests from scenarios. Not compatible with nose.
 load_tests = testscenarios.load_tests_apply_scenarios
 
@@ -35,14 +44,15 @@ class L3SchedulerBaseTest(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
     """
 
     def setUp(self):
-        super(L3SchedulerBaseTest, self).setUp()
+        super(L3SchedulerBaseTest, self).setUp(PLUGIN_NAME)
 
         self.l3_plugin = l3_router_plugin.L3RouterPlugin()
+        directory.add_plugin(plugin_constants.L3, self.l3_plugin)
         self.adminContext = context.get_admin_context()
-        self.adminContext.tenant_id = '_func_test_tenant_'
+        self.adminContext.tenant_id = _uuid()
 
-    def _create_l3_agent(self, host, context, agent_mode='legacy', plugin=None,
-                         state=True):
+    def _create_l3_agent(self, host, context, agent_mode='legacy',
+                         state=True, ext_net_id=''):
         agent = helpers.register_l3_agent(host, agent_mode)
         helpers.set_agent_admin_state(agent.id, state)
         return agent
@@ -58,7 +68,7 @@ class L3SchedulerBaseTest(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         #  down agent count.
         self.hosts = ['host-%s' % i for i in range(agent_count)]
         self.l3_agents = [self._create_l3_agent(self.hosts[i],
-               self.adminContext, 'legacy', self.l3_plugin,
+               self.adminContext, 'legacy',
                (i >= down_agent_count)) for i in range(agent_count)]
 
     def _create_routers(self, scheduled_router_count,
@@ -81,29 +91,33 @@ class L3SchedulerBaseTest(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         for i in range(count):
             router = self.routers[i]
             agent = random.choice(self.l3_agents)
-            scheduler.bind_router(self.adminContext, router['id'], agent)
+            scheduler.bind_router(self.l3_plugin, self.adminContext,
+                                  router['id'], agent.id)
             hosting_agents.append(agent)
         return hosting_agents
 
     def _test_auto_schedule(self, expected_count):
         router_ids = [rtr['id'] for rtr in self.routers]
 
-        did_it_schedule = False
+        hosting_before = self.l3_plugin.get_l3_agents_hosting_routers(
+            self.adminContext, router_ids)
 
         # Try scheduling on each host
         for host in self.hosts:
-            did_it_schedule = self.scheduler.auto_schedule_routers(
+            self.scheduler.auto_schedule_routers(
                 self.l3_plugin,
                 self.adminContext,
-                host,
-                router_ids)
-            if did_it_schedule:
-                break
+                host)
+
+        hosting_after = self.l3_plugin.get_l3_agents_hosting_routers(
+            self.adminContext, router_ids)
 
         if expected_count:
-            self.assertTrue(did_it_schedule, 'Failed to schedule agent')
+            self.assertNotEqual(hosting_before, hosting_after,
+                                'Failed to schedule agent')
         else:
-            self.assertFalse(did_it_schedule, 'Agent scheduled, not expected')
+            self.assertEqual(hosting_before, hosting_after,
+                             'Agent scheduled, not expected')
 
 
 class L3ChanceSchedulerTestCase(L3SchedulerBaseTest):
@@ -280,11 +294,12 @@ class L3LeastRoutersSchedulerTestCase(L3SchedulerBaseTest):
 class L3AZSchedulerBaseTest(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
 
     def setUp(self):
-        core_plugin = 'neutron.plugins.ml2.plugin.Ml2Plugin'
-        super(L3AZSchedulerBaseTest, self).setUp(plugin=core_plugin)
+        super(L3AZSchedulerBaseTest, self).setUp(plugin='ml2')
 
         self.l3_plugin = l3_router_plugin.L3RouterPlugin()
+        directory.add_plugin(plugin_constants.L3, self.l3_plugin)
         self.l3_plugin.router_scheduler = None
+        directory.add_plugin(plugin_constants.L3, self.l3_plugin)
         self.adminContext = context.get_admin_context()
         self.adminContext.tenant_id = '_func_test_tenant_'
 
@@ -332,10 +347,6 @@ class L3AZLeastRoutersSchedulerTestCase(L3AZSchedulerBaseTest):
             Maximum number of agents on which a router will be scheduled.
             0 means test for regular router.
 
-        min_l3_agents_per_router
-            Minimum number of agents on which a router will be scheduled.
-            N/A for regular router test.
-
         down_agent_count[each az]
             Number of l3 agents which are down.
 
@@ -349,7 +360,6 @@ class L3AZLeastRoutersSchedulerTestCase(L3AZSchedulerBaseTest):
               router_az_hints=1,
               agent_count=[1, 1],
               max_l3_agents_per_router=0,
-              min_l3_agents_per_router=0,
               down_agent_count=[0, 0],
               expected_scheduled_agent_count=[1, 0])),
 
@@ -358,7 +368,6 @@ class L3AZLeastRoutersSchedulerTestCase(L3AZSchedulerBaseTest):
               router_az_hints=2,
               agent_count=[1, 1, 1],
               max_l3_agents_per_router=2,
-              min_l3_agents_per_router=2,
               down_agent_count=[0, 0, 0],
               expected_scheduled_agent_count=[1, 1, 0])),
 
@@ -367,26 +376,19 @@ class L3AZLeastRoutersSchedulerTestCase(L3AZSchedulerBaseTest):
               router_az_hints=2,
               agent_count=[2, 1],
               max_l3_agents_per_router=3,
-              min_l3_agents_per_router=2,
               down_agent_count=[0, 0],
               expected_scheduled_agent_count=[2, 1])),
-
-        ('HA router, not enough agents',
-         dict(az_count=3,
-              router_az_hints=2,
-              agent_count=[2, 2, 2],
-              max_l3_agents_per_router=3,
-              min_l3_agents_per_router=2,
-              down_agent_count=[1, 1, 0],
-              expected_scheduled_agent_count=[1, 1, 0])),
     ]
 
+    def setUp(self):
+        super(L3AZLeastRoutersSchedulerTestCase, self).setUp()
+        self.scheduler = l3_agent_scheduler.AZLeastRoutersScheduler()
+        self.l3_plugin.router_scheduler = self.scheduler
+
     def test_schedule_router(self):
-        scheduler = l3_agent_scheduler.AZLeastRoutersScheduler()
         ha = False
         if self.max_l3_agents_per_router:
             self.config(max_l3_agents_per_router=self.max_l3_agents_per_router)
-            self.config(min_l3_agents_per_router=self.min_l3_agents_per_router)
             ha = True
 
         # create l3 agents
@@ -400,7 +402,8 @@ class L3AZLeastRoutersSchedulerTestCase(L3AZSchedulerBaseTest):
         az_hints = ['az%s' % i for i in range(self.router_az_hints)]
         router = self._create_router(az_hints, ha)
 
-        scheduler.schedule(self.l3_plugin, self.adminContext, router['id'])
+        self.scheduler.schedule(self.l3_plugin, self.adminContext,
+                                router['id'])
         # schedule returns only one agent. so get all agents scheduled.
         scheduled_agents = self.l3_plugin.get_l3_agents_hosting_routers(
             self.adminContext, [router['id']])
@@ -434,10 +437,6 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
             Maximum number of agents on which a router will be scheduled.
             0 means test for regular router.
 
-        min_l3_agents_per_router
-            Minimum number of agents on which a router will be scheduled.
-            N/A for regular router test.
-
         down_agent_count[each az]
             Number of l3 agents which are down.
 
@@ -455,7 +454,6 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
               agent_az='az0',
               agent_count=[1, 1],
               max_l3_agents_per_router=0,
-              min_l3_agents_per_router=0,
               down_agent_count=[1, 1],
               scheduled_agent_count=[0, 0],
               expected_scheduled_agent_count=[1, 0])),
@@ -466,7 +464,6 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
               agent_az='az1',
               agent_count=[1, 1],
               max_l3_agents_per_router=0,
-              min_l3_agents_per_router=0,
               down_agent_count=[1, 1],
               scheduled_agent_count=[0, 0],
               expected_scheduled_agent_count=[0, 0])),
@@ -477,7 +474,6 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
               agent_az='az1',
               agent_count=[1, 1, 1],
               max_l3_agents_per_router=2,
-              min_l3_agents_per_router=2,
               down_agent_count=[0, 1, 0],
               scheduled_agent_count=[0, 0, 0],
               expected_scheduled_agent_count=[0, 1, 0])),
@@ -488,21 +484,10 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
               agent_az='az2',
               agent_count=[1, 1, 1],
               max_l3_agents_per_router=2,
-              min_l3_agents_per_router=2,
               down_agent_count=[0, 0, 1],
               scheduled_agent_count=[0, 0, 0],
               expected_scheduled_agent_count=[0, 0, 0])),
 
-        ('HA router, partial scheduled, agent in specified AZ activated',
-         dict(az_count=3,
-              router_az_hints=2,
-              agent_az='az1',
-              agent_count=[1, 1, 1],
-              max_l3_agents_per_router=2,
-              min_l3_agents_per_router=2,
-              down_agent_count=[0, 1, 0],
-              scheduled_agent_count=[1, 0, 0],
-              expected_scheduled_agent_count=[1, 1, 0])),
     ]
 
     def test_auto_schedule_router(self):
@@ -510,7 +495,6 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
         ha = False
         if self.max_l3_agents_per_router:
             self.config(max_l3_agents_per_router=self.max_l3_agents_per_router)
-            self.config(min_l3_agents_per_router=self.min_l3_agents_per_router)
             ha = True
 
         # create l3 agents
@@ -530,7 +514,8 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
             az = 'az%s' % i
             for j in range(self.scheduled_agent_count[i]):
                 agent = l3_agents[az][j + self.down_agent_count[i]]
-                scheduler.bind_router(self.adminContext, router['id'], agent)
+                scheduler.bind_router(self.l3_plugin, self.adminContext,
+                                      router['id'], agent.id)
 
         # activate down agent and call auto_schedule_routers
         activate_agent = l3_agents[self.agent_az][0]
@@ -538,7 +523,7 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
                                       admin_state_up=True)
 
         scheduler.auto_schedule_routers(self.l3_plugin, self.adminContext,
-                                        activate_agent['host'], None)
+                                        activate_agent['host'])
 
         scheduled_agents = self.l3_plugin.get_l3_agents_hosting_routers(
             self.adminContext, [router['id']])
@@ -550,3 +535,278 @@ class L3AZAutoScheduleTestCaseBase(L3AZSchedulerBaseTest):
         for i in range(self.az_count):
             self.assertEqual(self.expected_scheduled_agent_count[i],
                              scheduled_azs.get('az%s' % i, 0))
+
+
+class L3DVRSchedulerBaseTest(L3SchedulerBaseTest):
+
+    """Base class for functional test of DVR L3 schedulers.
+       Provides basic setup and utility functions.
+    """
+
+    def setUp(self):
+        super(L3DVRSchedulerBaseTest, self).setUp()
+
+        self.default_ext_net_id = _uuid()
+        self.default_ext_subnet_id = _uuid()
+
+        self.router_ext_net_id = _uuid()
+        self.router_ext_subnet_id = _uuid()
+
+    def _create_router(self, name, distributed, ext_net_id=None):
+        router = {'name': name, 'admin_state_up': True,
+                  'tenant_id': self.adminContext.tenant_id,
+                  'distributed': distributed}
+
+        if ext_net_id:
+            router['external_gateway_info'] = {'network_id': ext_net_id}
+
+        return self.l3_plugin.create_router(self.adminContext,
+                                            {'router': router})
+
+    def _create_network(self, net_id, name=None, external=False):
+        network_dict = {'tenant_id': self.adminContext.tenant_id,
+                        'id': net_id,
+                        'name': name,
+                        'admin_state_up': True,
+                        'shared': False,
+                        'status': constants.NET_STATUS_ACTIVE}
+        network = self.plugin.create_network(self.adminContext,
+                                             {'network': network_dict})
+        if external:
+            network = net_obj.ExternalNetwork(
+                self.adminContext, network_id=net_id)
+            network.create()
+
+        return network
+
+    def _create_subnet(self, sub_id, network_id, cidr, gw_ip, name='test_sub'):
+        subnet = {'tenant_id': self.adminContext.tenant_id,
+                  'id': sub_id,
+                  'name': name,
+                  'network_id': network_id,
+                  'ip_version': constants.IP_VERSION_4,
+                  'cidr': cidr,
+                  'enable_dhcp': False,
+                  'gateway_ip': gw_ip,
+                  'shared': False,
+                  'allocation_pools': constants.ATTR_NOT_SPECIFIED,
+                  'dns_nameservers': constants.ATTR_NOT_SPECIFIED,
+                  'host_routes': constants.ATTR_NOT_SPECIFIED}
+
+        return self.plugin.create_subnet(self.adminContext, {'subnet': subnet})
+
+
+class L3DVRSchedulerTestCase(L3DVRSchedulerBaseTest):
+
+    """Test various scenarios for L3 DVR schedulers:
+
+        agent_mode
+            L3 agent mode.
+
+        second_agent_mode
+            Second L3 agent mode for scenarios with two agents.
+
+        agent_has_ext_network
+            Is there external network on the host.
+
+        router_is_distributed
+            Is router distributed.
+
+        router_already_hosted
+            Is router already hosted.
+
+        router_has_ext_gw
+            Does router have external gateway.
+
+        router_agent_have_same_ext_net
+            Do router and agent have the same external network.
+
+        expected_router_scheduled
+            To verify do we expect router to get scheduled.
+    """
+
+    def get_scenario(agent_mode=constants.L3_AGENT_MODE_DVR_SNAT,
+                     second_agent_mode=None,
+                     agent_has_ext_network=False,
+                     router_is_distributed=False,
+                     router_already_hosted=False,
+                     router_has_ext_gw=False,
+                     router_agent_have_same_ext_net=False,
+                     expected_router_scheduled=False):
+        return dict(agent_mode=agent_mode,
+            second_agent_mode=second_agent_mode,
+            agent_has_ext_network=agent_has_ext_network,
+            router_is_distributed=router_is_distributed,
+            router_already_hosted=router_already_hosted,
+            router_has_ext_gw=router_has_ext_gw,
+            router_agent_have_same_ext_net=router_agent_have_same_ext_net,
+            expected_router_scheduled=expected_router_scheduled)
+
+    scenarios = [
+        ('Legacy router not scheduled on dvr agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_DVR)),
+
+        ('Legacy router scheduled on dvr_snat agent',
+         get_scenario(expected_router_scheduled=True)),
+
+        ('Distributed router not scheduled on legacy agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_LEGACY,
+                      router_is_distributed=True)),
+
+        ('Distributed router not scheduled on dvr agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_DVR,
+                      router_is_distributed=True)),
+
+        ('Distributed router scheduled on dvr_snat agent',
+         get_scenario(router_is_distributed=True,
+                      expected_router_scheduled=True)),
+
+        ('Already hosted legacy router not scheduled on dvr agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_DVR,
+                      router_already_hosted=True)),
+
+        ('Already hosted legacy router not scheduled on dvr_snat agent',
+         get_scenario(router_already_hosted=True)),
+
+        ('Already hosted distributed router not scheduled on legacy agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_LEGACY,
+                      router_already_hosted=True,
+                      router_is_distributed=True)),
+
+        ('Already hosted distributed router not scheduled on dvr agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_DVR,
+                      router_is_distributed=True,
+                      router_already_hosted=True)),
+
+        ('Already hosted distributed router not scheduled on dvr_snat agent',
+         get_scenario(router_is_distributed=True,
+                      router_already_hosted=True)),
+
+        ('Already hosted legacy router not scheduled on additional dvr agent',
+         get_scenario(agent_mode=constants.L3_AGENT_MODE_LEGACY,
+                      second_agent_mode=constants.L3_AGENT_MODE_DVR_SNAT,
+                      router_already_hosted=True)),
+
+        ('Distributed router not scheduled if it is on a different '
+         'external network than the dvr_snat agent',
+         get_scenario(agent_has_ext_network=True,
+                      router_is_distributed=True,
+                      router_has_ext_gw=True,
+                      router_agent_have_same_ext_net=False)),
+    ]
+
+    def setUp(self):
+        super(L3DVRSchedulerTestCase, self).setUp()
+
+        agent_cnt = 2 if self.second_agent_mode else 1
+
+        # create hosts for each agent
+        self.hosts = ['host-%s' % i for i in range(agent_cnt)]
+
+        # create default external network
+        self._create_network(self.default_ext_net_id,
+                             name='_test-ext-net', external=True)
+        self._create_subnet(self.default_ext_subnet_id,
+                            self.default_ext_net_id,
+                            '10.10.9.0/24', '10.10.9.1',
+                            '_test-ext-net-subnet')
+
+        if self.router_has_ext_gw and not self.router_agent_have_same_ext_net:
+            # for the test cases in which router and agent are not on same
+            # external network, we create an external network for router
+            self._create_network(self.router_ext_net_id,
+                                 name='_test-ext-net2', external=True)
+            self._create_subnet(self.router_ext_subnet_id,
+                                self.router_ext_net_id,
+                                '10.10.8.0/24', '10.10.8.1',
+                                '_test-ext-net2-subnet')
+        # create agents:
+        self.l3_agents = [self._create_l3_agent(self.hosts[0],
+            self.adminContext, self.agent_mode, True,
+            self.default_ext_net_id if self.agent_has_ext_network else '')]
+        if self.second_agent_mode:
+            self.l3_agents.append(self._create_l3_agent(self.hosts[1],
+                self.adminContext, self.second_agent_mode, True,
+                self.default_ext_net_id if self.agent_has_ext_network else ''))
+
+        # The router to schedule:
+        self.router_to_schedule = self._create_router_to_schedule()
+
+    def _create_router_to_schedule(self):
+        router_to_schedule = None
+
+        if self.router_has_ext_gw:
+            if self.router_agent_have_same_ext_net:
+                router_to_schedule = self._create_router('schd_rtr',
+                    self.router_is_distributed,
+                    self.default_ext_net_id)
+            else:
+                router_to_schedule = self._create_router('schd_rtr',
+                    self.router_is_distributed,
+                    self.router_ext_net_id)
+        else:
+            router_to_schedule = self._create_router('schd_rtr',
+                self.router_is_distributed)
+
+        return router_to_schedule
+
+    def _test_schedule_router(self):
+        if self.router_already_hosted:
+            self.scheduler.bind_router(self.l3_plugin,
+                                       self.adminContext,
+                                       self.router_to_schedule['id'],
+                                       self.l3_agents[0].id)
+
+        # schedule:
+        actual_scheduled_agent = self.scheduler.schedule(
+                                     self.l3_plugin,
+                                     self.adminContext,
+                                     self.router_to_schedule['id'])
+
+        # check for router scheduling:
+        self.assertEqual(self.expected_router_scheduled,
+                         bool(actual_scheduled_agent),
+                         message='Failed to schedule agent')
+
+    def _test_auto_schedule_routers(self):
+        if self.router_already_hosted:
+            self.scheduler.bind_router(self.l3_plugin,
+                                       self.adminContext,
+                                       self.router_to_schedule['id'],
+                                       self.l3_agents[0].id)
+        # schedule:
+        hosting_before = self.l3_plugin.get_l3_agents_hosting_routers(
+            self.adminContext, [self.router_to_schedule['id']])
+
+        for host in self.hosts:
+            self.scheduler.auto_schedule_routers(
+                self.l3_plugin, self.adminContext, host)
+
+        hosting_after = self.l3_plugin.get_l3_agents_hosting_routers(
+            self.adminContext, [self.router_to_schedule['id']])
+
+        if self.router_already_hosted:
+            self.assertEqual(hosting_before, hosting_after,
+                             'Agent pre scheduled, yet no binding found!')
+        elif self.expected_router_scheduled:
+            self.assertNotEqual(hosting_before, hosting_after,
+                                'Agent not scheduled, not expected')
+        else:
+            self.assertEqual(hosting_before, hosting_after,
+                             'Agent scheduled, not expected')
+
+    def test_least_routers_schedule_router(self):
+        self.scheduler = l3_agent_scheduler.LeastRoutersScheduler()
+        self._test_schedule_router()
+
+    def test_least_routers_auto_schedule_routers(self):
+        self.scheduler = l3_agent_scheduler.LeastRoutersScheduler()
+        self._test_auto_schedule_routers()
+
+    def test_chance_schedule_router(self):
+        self.scheduler = l3_agent_scheduler.ChanceScheduler()
+        self._test_schedule_router()
+
+    def test_chance_auto_schedule_routers(self):
+        self.scheduler = l3_agent_scheduler.ChanceScheduler()
+        self._test_auto_schedule_routers()
