@@ -13,39 +13,45 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import multiprocessing
 import os
 import signal
 import socket
 import time
 import traceback
+from unittest import mock
 
 import httplib2
-import mock
+from neutron_lib import worker as neutron_worker
 from oslo_config import cfg
+from oslo_log import log
 import psutil
 
 from neutron.common import utils
+from neutron import manager
 from neutron import service
-from neutron.tests import base
-from neutron import worker as neutron_worker
+from neutron.tests.functional import base
 from neutron import wsgi
 
 
+LOG = log.getLogger(__name__)
+
 CONF = cfg.CONF
 
-# This message will be written to temporary file each time
-# start method is called.
-FAKE_START_MSG = b"start"
+# Those messages will be written to temporary file each time
+# start/reset methods are called.
+FAKE_START_MSG = 'start'
+FAKE_RESET_MSG = 'reset'
 
 TARGET_PLUGIN = 'neutron.plugins.ml2.plugin.Ml2Plugin'
 
 
-class TestNeutronServer(base.BaseTestCase):
+class TestNeutronServer(base.BaseLoggingTestCase):
     def setUp(self):
         super(TestNeutronServer, self).setUp()
         self.service_pid = None
         self.workers = None
-        self.temp_file = self.get_temp_file_path("test_server.tmp")
+        self._mp_queue = multiprocessing.Queue()
         self.health_checker = self._check_active
         self.pipein, self.pipeout = os.pipe()
         self.addCleanup(self._destroy_workers)
@@ -109,7 +115,7 @@ class TestNeutronServer(base.BaseTestCase):
 
         def safe_ppid(proc):
             try:
-                return proc.ppid
+                return proc.ppid()
             except psutil.NoSuchProcess:
                 return None
 
@@ -126,8 +132,10 @@ class TestNeutronServer(base.BaseTestCase):
         return True
 
     def _fake_start(self):
-        with open(self.temp_file, 'a') as f:
-            f.write(FAKE_START_MSG)
+        self._mp_queue.put(FAKE_START_MSG)
+
+    def _fake_reset(self):
+        self._mp_queue.put(FAKE_RESET_MSG)
 
     def _test_restart_service_on_sighup(self, service, workers=1):
         """Test that a service correctly (re)starts on receiving SIGHUP.
@@ -140,29 +148,36 @@ class TestNeutronServer(base.BaseTestCase):
         self._start_server(callback=service, workers=workers)
         os.kill(self.service_pid, signal.SIGHUP)
 
-        expected_msg = FAKE_START_MSG * workers * 2
+        # After sending SIGHUP it is expected that there will be as many
+        # FAKE_RESET_MSG as number of workers + one additional for main
+        # process
+        expected_msg = (
+            FAKE_START_MSG * workers + FAKE_RESET_MSG * (workers + 1))
 
         # Wait for temp file to be created and its size reaching the expected
         # value
         expected_size = len(expected_msg)
-        condition = lambda: (os.path.isfile(self.temp_file)
-                             and os.stat(self.temp_file).st_size ==
-                             expected_size)
+        ret_msg = ''
 
-        utils.wait_until_true(
-            condition, timeout=5, sleep=0.1,
-            exception=RuntimeError(
-                "Timed out waiting for file %(filename)s to be created and "
-                "its size become equal to %(size)s." %
-                {'filename': self.temp_file,
-                 'size': expected_size}))
+        def is_ret_buffer_ok():
+            nonlocal ret_msg
+            LOG.debug('Checking returned buffer size')
+            while not self._mp_queue.empty():
+                ret_msg += self._mp_queue.get()
+            LOG.debug('Size of buffer is %s. Expected size: %s',
+                      len(ret_msg), expected_size)
+            return len(ret_msg) == expected_size
+
+        try:
+            utils.wait_until_true(is_ret_buffer_ok, timeout=5, sleep=1)
+        except utils.WaitTimeout:
+            raise RuntimeError('Expected buffer size: %s, current size: %s' %
+                               (len(ret_msg), expected_size))
 
         # Verify that start has been called twice for each worker (one for
         # initial start, and the second one on SIGHUP after children were
         # terminated).
-        with open(self.temp_file, 'r') as f:
-            res = f.readline()
-            self.assertEqual(expected_msg, res)
+        self.assertEqual(expected_msg, ret_msg)
 
 
 class TestWsgiServer(TestNeutronServer):
@@ -200,8 +215,10 @@ class TestWsgiServer(TestNeutronServer):
 
         # Mock start method to check that children are started again on
         # receiving SIGHUP.
-        with mock.patch("neutron.wsgi.WorkerService.start") as start_method:
+        with mock.patch("neutron.wsgi.WorkerService.start") as start_method,\
+                mock.patch("neutron.wsgi.WorkerService.reset") as reset_method:
             start_method.side_effect = self._fake_start
+            reset_method.side_effect = self._fake_reset
 
             server = wsgi.Server("Test")
             server.start(self.application, 0, "0.0.0.0",
@@ -209,7 +226,7 @@ class TestWsgiServer(TestNeutronServer):
 
             # Memorize a port that was chosen for the service
             self.port = server.port
-            os.write(self.pipeout, str(self.port))
+            os.write(self.pipeout, bytes(str(self.port), 'utf-8'))
 
             server.wait()
 
@@ -223,7 +240,7 @@ class TestRPCServer(TestNeutronServer):
 
     def setUp(self):
         super(TestRPCServer, self).setUp()
-        self.setup_coreplugin('ml2')
+        self.setup_coreplugin('ml2', load_plugins=False)
         self._plugin_patcher = mock.patch(TARGET_PLUGIN, autospec=True)
         self.plugin = self._plugin_patcher.start()
         self.plugin.return_value.rpc_workers_supported = True
@@ -233,19 +250,21 @@ class TestRPCServer(TestNeutronServer):
 
         # Mock start method to check that children are started again on
         # receiving SIGHUP.
-        with mock.patch("neutron.service.RpcWorker.start") as start_method:
-            with mock.patch(
-                    "neutron.manager.NeutronManager.get_plugin"
-            ) as get_plugin:
-                start_method.side_effect = self._fake_start
-                get_plugin.return_value = self.plugin
+        with mock.patch("neutron.service.RpcWorker.start") as start_method,\
+                mock.patch(
+                    "neutron.service.RpcWorker.reset") as reset_method,\
+                mock.patch(
+                    "neutron_lib.plugins.directory.get_plugin") as get_plugin:
+            start_method.side_effect = self._fake_start
+            reset_method.side_effect = self._fake_reset
+            get_plugin.return_value = self.plugin
 
-                CONF.set_override("rpc_workers", workers)
-                # not interested in state report workers specifically
-                CONF.set_override("rpc_state_report_workers", 0)
+            CONF.set_override("rpc_workers", workers)
+            # not interested in state report workers specifically
+            CONF.set_override("rpc_state_report_workers", 0)
 
-                rpc_workers_launcher = service.start_rpc_workers()
-                rpc_workers_launcher.wait()
+            rpc_workers_launcher = service.start_rpc_workers()
+            rpc_workers_launcher.wait()
 
     def test_restart_rpc_on_sighup_multiple_workers(self):
         self._test_restart_service_on_sighup(service=self._serve_rpc,
@@ -257,18 +276,19 @@ class TestPluginWorker(TestNeutronServer):
 
     def setUp(self):
         super(TestPluginWorker, self).setUp()
-        self.setup_coreplugin('ml2')
+        self.setup_coreplugin('ml2', load_plugins=False)
         self._plugin_patcher = mock.patch(TARGET_PLUGIN, autospec=True)
         self.plugin = self._plugin_patcher.start()
+        manager.init()
 
     def _start_plugin(self, workers=1):
-        with mock.patch('neutron.manager.NeutronManager.get_plugin') as gp:
+        with mock.patch('neutron_lib.plugins.directory.get_plugin') as gp:
             gp.return_value = self.plugin
             plugin_workers_launcher = service.start_plugins_workers()
             plugin_workers_launcher.wait()
 
     def test_start(self):
-        class FakeWorker(neutron_worker.NeutronWorker):
+        class FakeWorker(neutron_worker.BaseWorker):
             def start(self):
                 pass
 
@@ -283,6 +303,7 @@ class TestPluginWorker(TestNeutronServer):
 
         # Make both ABC happy and ensure 'self' is correct
         FakeWorker.start = self._fake_start
+        FakeWorker.reset = self._fake_reset
         workers = [FakeWorker()]
         self.plugin.return_value.get_workers.return_value = workers
         self._test_restart_service_on_sighup(service=self._start_plugin,
