@@ -13,8 +13,8 @@
 #    under the License.
 
 import collections
+from unittest import mock
 
-import mock
 from neutron_lib import exceptions
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
@@ -471,7 +471,8 @@ class OVS_Lib_Test(base.BaseTestCase):
         ) as port_exists_mock:
             self.br.delete_egress_bw_limit_for_port("test_port")
             port_exists_mock.assert_called_once_with("test_port")
-            set_egress_mock.assert_called_once_with("test_port", 0, 0)
+            set_egress_mock.assert_called_once_with("test_port", 0, 0,
+                                                    check_error=False)
 
     def test_delete_egress_bw_limit_for_port_port_not_exists(self):
         with mock.patch.object(
@@ -497,9 +498,9 @@ class OVS_Lib_Test(base.BaseTestCase):
         self.br.ovsdb.list_ports.return_value.execute.return_value = [
             'qvo1', 'qvo2', 'qvo4']
         by_id = self.br.get_vifs_by_ids(['pid1', 'pid2', 'pid3', 'pid4'])
-        # pid3 isn't on bridge and pid4 doesn't have a valid ofport
+        # pid3 isn't on bridge
         self.assertIsNone(by_id['pid3'])
-        self.assertIsNone(by_id['pid4'])
+        self.assertEqual(-1, by_id['pid4'].ofport)
         self.assertEqual('pid1', by_id['pid1'].vif_id)
         self.assertEqual('qvo1', by_id['pid1'].port_name)
         self.assertEqual(1, by_id['pid1'].ofport)
@@ -511,6 +512,8 @@ class OVS_Lib_Test(base.BaseTestCase):
                        if_exists=True)])
 
     def test_get_port_ofport_retry(self):
+        # Increase this value to avoid a timeout during the test execution
+        self.br.ovsdb.ovsdb_connection.timeout = 10
         with mock.patch.object(
                 self.br, 'db_get_val',
                 side_effect=[[], [], [], [], 1]):
@@ -518,7 +521,7 @@ class OVS_Lib_Test(base.BaseTestCase):
 
     def test_get_port_ofport_retry_fails(self):
         # reduce timeout for faster execution
-        self.br.ovsdb_timeout = 1
+        self.br.ovsdb.ovsdb_connection.timeout = 1
         # after 7 calls the retry will timeout and raise
         with mock.patch.object(
                 self.br, 'db_get_val',
@@ -526,26 +529,6 @@ class OVS_Lib_Test(base.BaseTestCase):
             self.assertRaises(tenacity.RetryError,
                               self.br._get_port_val, '1', 'ofport')
 
-    def test_get_port_external_ids_retry(self):
-        external_ids = [["iface-id", "tap99id"],
-                        ["iface-status", "active"],
-                        ["attached-mac", "de:ad:be:ef:13:37"]]
-        with mock.patch.object(
-                self.br, 'db_get_val',
-                side_effect=[[], [], [], [], external_ids]):
-            self.assertEqual(external_ids,
-                             self.br._get_port_val('1', 'external_ids'))
-
-    def test_get_port_external_ids_retry_fails(self):
-        # reduce timeout for faster execution
-        self.br.ovsdb_timeout = 1
-        # after 7 calls the retry will timeout and raise
-        with mock.patch.object(
-                self.br, 'db_get_val',
-                side_effect=[[] for _ in range(7)]):
-            self.assertRaises(tenacity.RetryError,
-                              self.br._get_port_val, '1', 'external_ids')
-
     def test_set_controller_rate_limit(self):
         with mock.patch.object(
                 self.br, "set_controller_field"
@@ -578,6 +561,27 @@ class OVS_Lib_Test(base.BaseTestCase):
             set_ctrl_field_mock.assert_called_once_with(
                 'controller_burst_limit', ovs_lib.CTRL_BURST_LIMIT_MIN)
 
+    def test_hw_offload_enabled_false(self):
+        config_mock1 = mock.PropertyMock(return_value={"other_config": {}})
+        config_mock2 = mock.PropertyMock(
+            return_value={"other_config": {"hw-offload": "false"}})
+        config_mock3 = mock.PropertyMock(
+            return_value={"other_config": {"hw-offload": "False"}})
+        for config_mock in (config_mock1, config_mock2, config_mock3):
+            with mock.patch("neutron.agent.common.ovs_lib.OVSBridge.config",
+                            new_callable=config_mock):
+                self.assertFalse(self.br.is_hw_offload_enabled)
+
+    def test_hw_offload_enabled_true(self):
+        config_mock1 = mock.PropertyMock(
+            return_value={"other_config": {"hw-offload": "true"}})
+        config_mock2 = mock.PropertyMock(
+            return_value={"other_config": {"hw-offload": "True"}})
+        for config_mock in (config_mock1, config_mock2):
+            with mock.patch("neutron.agent.common.ovs_lib.OVSBridge.config",
+                            new_callable=config_mock):
+                self.assertTrue(self.br.is_hw_offload_enabled)
+
 
 class TestDeferredOVSBridge(base.BaseTestCase):
 
