@@ -14,15 +14,15 @@
 #    under the License.
 
 import copy
+from unittest import mock
 
-import mock
 from neutron_lib import constants
+from neutron_lib.utils import helpers
 from oslo_config import cfg
-import six
 
 from neutron.agent.l2.extensions.fdb_population import (
         FdbPopulationAgentExtension)
-from neutron.common import utils as n_utils
+from neutron.agent.linux import ip_lib
 from neutron.plugins.ml2.drivers.linuxbridge.agent.common import (
      constants as linux_bridge_constants)
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
@@ -47,9 +47,9 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         self.DEVICE = self._get_existing_device()
 
     def _get_existing_device(self):
-        device_mappings = n_utils.parse_mappings(
+        device_mappings = helpers.parse_mappings(
             cfg.CONF.FDB.shared_physical_device_mappings, unique_keys=False)
-        DEVICES = six.next(six.itervalues(device_mappings))
+        DEVICES = next(iter(device_mappings.values()))
         return DEVICES[0]
 
     def _get_fdb_extension(self, mock_execute, fdb_table):
@@ -58,25 +58,25 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         fdb_pop.initialize(None, ovs_constants.EXTENSION_DRIVER_TYPE)
         return fdb_pop
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch('neutron.agent.common.utils.execute')
     def test_initialize(self, mock_execute):
         fdb_extension = FdbPopulationAgentExtension()
         fdb_extension.initialize(None, ovs_constants.EXTENSION_DRIVER_TYPE)
         fdb_extension.initialize(None,
                                  linux_bridge_constants.EXTENSION_DRIVER_TYPE)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch('neutron.agent.common.utils.execute')
     def test_initialize_invalid_agent(self, mock_execute):
         fdb_extension = FdbPopulationAgentExtension()
         self.assertRaises(SystemExit, fdb_extension.initialize, None, 'sriov')
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_construct_empty_fdb_table(self, mock_execute):
         self._get_fdb_extension(mock_execute, fdb_table='')
         cmd = ['bridge', 'fdb', 'show', 'dev', self.DEVICE]
         mock_execute.assert_called_once_with(cmd, run_as_root=True)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_construct_existing_fdb_table(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute,
                                                 fdb_table=self.FDB_TABLE)
@@ -88,7 +88,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         for mac in macs:
             self.assertIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_update_port_add_rule(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, self.FDB_TABLE)
         mock_execute.reset_mock()
@@ -101,7 +101,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         mac = self.UPDATE_MSG['mac_address']
         self.assertIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_update_port_changed_mac(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, self.FDB_TABLE)
         mock_execute.reset_mock()
@@ -124,7 +124,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         self.assertIn(updated_mac, updated_macs_for_device)
         self.assertNotIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch('neutron.agent.common.utils.execute')
     def test_unpermitted_device_owner(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, '')
         mock_execute.reset_mock()
@@ -137,7 +137,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         mac = self.UPDATE_MSG['mac_address']
         self.assertNotIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch('neutron.agent.common.utils.execute')
     def test_catch_init_exception(self, mock_execute):
         mock_execute.side_effect = RuntimeError
         fdb_extension = self._get_fdb_extension(mock_execute, '')
@@ -145,7 +145,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
             fdb_extension.fdb_tracker.device_to_macs.get(self.DEVICE))
         self.assertIsNone(updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_catch_update_port_exception(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, '')
         mock_execute.side_effect = RuntimeError
@@ -155,7 +155,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         mac = self.UPDATE_MSG['mac_address']
         self.assertNotIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_catch_delete_port_exception(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, '')
         fdb_extension.handle_port(context=None, details=self.UPDATE_MSG)
@@ -166,7 +166,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
         mac = self.UPDATE_MSG['mac_address']
         self.assertIn(mac, updated_macs_for_device)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_delete_port(self, mock_execute):
         fdb_extension = self._get_fdb_extension(mock_execute, '')
         fdb_extension.handle_port(context=None, details=self.UPDATE_MSG)
@@ -176,7 +176,7 @@ class FdbPopulationExtensionTestCase(base.BaseTestCase):
                'dev', self.DEVICE]
         mock_execute.assert_called_once_with(cmd, run_as_root=True)
 
-    @mock.patch('neutron.agent.linux.utils.execute')
+    @mock.patch.object(ip_lib.IpNetnsCommand, 'execute')
     def test_multiple_devices(self, mock_execute):
         cfg.CONF.set_override('shared_physical_device_mappings',
                 ['physnet1:p1p1', 'physnet1:p2p2'], 'FDB')
