@@ -528,11 +528,19 @@ class TestDvrRouterOperations(base.BaseTestCase):
         ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         ports = ri.router.get(lib_constants.INTERFACE_KEY, [])
         subnet_id = l3_test_common.get_subnet_id(ports[0])
+        ri.router['_snat_router_interfaces'] = [{
+            'mac_address': 'fa:16:3e:80:8d:80',
+            'fixed_ips': [{'subnet_id': subnet_id,
+                           'ip_address': '1.2.3.10'}]}]
+
         test_ports = [{'mac_address': '00:11:22:33:44:55',
                        'device_owner': lib_constants.DEVICE_OWNER_DHCP,
                        'fixed_ips': [{'ip_address': '1.2.3.4',
                                       'prefixlen': 24,
-                                      'subnet_id': subnet_id}]},
+                                      'subnet_id': subnet_id}],
+                       'allowed_address_pairs': [
+                           {'ip_address': '10.20.30.40',
+                            'mac_address': '00:11:22:33:44:55'}]},
                       {'mac_address': '11:22:33:44:55:66',
                        'device_owner': lib_constants.DEVICE_OWNER_LOADBALANCER,
                        'fixed_ips': [{'ip_address': '1.2.3.5',
@@ -554,8 +562,10 @@ class TestDvrRouterOperations(base.BaseTestCase):
                                '_process_arp_cache_for_internal_port') as parp:
             ri._set_subnet_arp_info(subnet_id)
         self.assertEqual(1, parp.call_count)
-        self.mock_ip_dev.neigh.add.assert_called_once_with(
-            '1.2.3.4', '00:11:22:33:44:55')
+        self.mock_ip_dev.neigh.add.assert_has_calls([
+            mock.call('1.2.3.4', '00:11:22:33:44:55'),
+            mock.call('10.20.30.40', '00:11:22:33:44:55'),
+            mock.call('1.2.3.10', 'fa:16:3e:80:8d:80')])
 
         # Test negative case
         router['distributed'] = False
