@@ -13,15 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from neutron_lib import constants
-from oslo_log import versionutils
+from oslo_utils import excutils
+from pyroute2.netlink import exceptions as pyroute2_exc
 
-from neutron.agent.common import config
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
-from neutron.agent.linux import utils
+from neutron.common import utils
+from neutron.conf.agent import common as config
+from neutron.conf.plugins.ml2.drivers import ovs_conf
+from neutron.plugins.ml2.drivers.openvswitch.agent.common \
+    import constants as ovs_const
+from neutron.privileged.agent.linux import ethtool
 from neutron.tests import base
 
 
@@ -44,7 +50,7 @@ class FakeSubnet(object):
 class FakeAllocation(object):
     subnet = FakeSubnet()
     ip_address = '192.168.1.2'
-    ip_version = 4
+    ip_version = constants.IP_VERSION_4
 
 
 class FakePort(object):
@@ -55,18 +61,12 @@ class FakePort(object):
     network_id = network.id
 
 
-class FakeInterfaceDriverNoMtu(interface.LinuxInterfaceDriver):
-    # NOTE(ihrachys) this method intentially omit mtu= parameter, since that
-    # was the method signature before Mitaka. We should make sure the old
-    # signature still works.
-
-    def __init__(self, *args, **kwargs):
-        super(FakeInterfaceDriverNoMtu, self).__init__(*args, **kwargs)
-        self.plug_called = False
+class FakeLegacyInterfaceDriver(interface.LinuxInterfaceDriver):
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None):
-        self.plug_called = True
+                 bridge=None, namespace=None, prefix=None, mtu=None):
+        """This is legacy method which don't accepts link_up argument."""
+        pass
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         pass
@@ -76,26 +76,19 @@ class TestBase(base.BaseTestCase):
     def setUp(self):
         super(TestBase, self).setUp()
         self.conf = config.setup_conf()
-        self.conf.register_opts(interface.OPTS)
+        ovs_conf.register_ovs_opts(self.conf)
+        config.register_interface_opts(self.conf)
+        self.eth_offload_p = mock.patch.object(ethtool, 'offload')
+        self.eth_offload = self.eth_offload_p.start()
         self.ip_dev_p = mock.patch.object(ip_lib, 'IPDevice')
         self.ip_dev = self.ip_dev_p.start()
         self.ip_p = mock.patch.object(ip_lib, 'IPWrapper')
         self.ip = self.ip_p.start()
         self.device_exists_p = mock.patch.object(ip_lib, 'device_exists')
         self.device_exists = self.device_exists_p.start()
-
-
-class TestABCDriverNoMtu(TestBase):
-
-    def test_plug_with_no_mtu_works(self):
-        driver = FakeInterfaceDriverNoMtu(self.conf)
-        self.device_exists.return_value = False
-        with mock.patch.object(
-                versionutils, 'report_deprecated_feature') as report:
-            driver.plug(
-                mock.Mock(), mock.Mock(), mock.Mock(), mock.Mock(), mtu=9000)
-        self.assertTrue(driver.plug_called)
-        self.assertTrue(report.called)
+        self.get_devices_with_ip_p = mock.patch.object(ip_lib,
+                                                       'get_devices_with_ip')
+        self.get_devices_with_ip = self.get_devices_with_ip_p.start()
 
 
 class TestABCDriver(TestBase):
@@ -121,9 +114,9 @@ class TestABCDriver(TestBase):
                             extra_subnets=[{'cidr': '172.20.0.0/24'}])
         self.ip_dev.assert_has_calls(
             [mock.call('tap0', namespace=ns),
-             mock.call().addr.list(filters=['permanent']),
-             mock.call().addr.add('192.168.1.2/24'),
+             mock.call().addr.list(),
              mock.call().addr.delete('172.16.77.240/24'),
+             mock.call().addr.add('192.168.1.2/24'),
              mock.call('tap0', namespace=ns),
              mock.call().route.list_onlink_routes(constants.IP_VERSION_4),
              mock.call().route.list_onlink_routes(constants.IP_VERSION_6),
@@ -155,7 +148,7 @@ class TestABCDriver(TestBase):
                    preserve_ips=['192.168.1.3/32'])
         self.ip_dev.assert_has_calls(
             [mock.call('tap0', namespace=ns),
-             mock.call().addr.list(filters=['permanent']),
+             mock.call().addr.list(),
              mock.call().addr.add('192.168.1.2/24')])
         self.assertFalse(self.ip_dev().addr.delete.called)
         self.assertFalse(self.ip_dev().delete_addr_and_conntrack_state.called)
@@ -198,9 +191,9 @@ class TestABCDriver(TestBase):
         bc.init_router_port('tap0', [new_cidr], **kwargs)
         expected_calls = (
             [mock.call('tap0', namespace=ns),
-             mock.call().addr.list(filters=['permanent']),
-             mock.call().addr.add('2001:db8:a::124/64'),
-             mock.call().addr.delete('2001:db8:a::123/64')])
+             mock.call().addr.list(),
+             mock.call().addr.delete('2001:db8:a::123/64'),
+             mock.call().addr.add('2001:db8:a::124/64')])
         expected_calls += (
              [mock.call('tap0', namespace=ns),
               mock.call().route.list_onlink_routes(constants.IP_VERSION_4),
@@ -209,9 +202,9 @@ class TestABCDriver(TestBase):
         self.ip_dev.assert_has_calls(expected_calls)
 
     def test_init_router_port_ext_gw_with_dual_stack(self):
-        old_addrs = [dict(ip_version=4, scope='global',
+        old_addrs = [dict(ip_version=constants.IP_VERSION_4, scope='global',
                           dynamic=False, cidr='172.16.77.240/24'),
-                     dict(ip_version=6, scope='global',
+                     dict(ip_version=constants.IP_VERSION_6, scope='global',
                           dynamic=False, cidr='2001:db8:a::123/64')]
         self.ip_dev().addr.list = mock.Mock(return_value=old_addrs)
         self.ip_dev().route.list_onlink_routes.return_value = []
@@ -222,7 +215,7 @@ class TestABCDriver(TestBase):
             extra_subnets=[{'cidr': '172.20.0.0/24'}])
         self.ip_dev.assert_has_calls(
             [mock.call('tap0', namespace=ns),
-             mock.call().addr.list(filters=['permanent']),
+             mock.call().addr.list(),
              mock.call().addr.add('192.168.1.2/24'),
              mock.call().addr.add('2001:db8:a::124/64'),
              mock.call().addr.delete('172.16.77.240/24'),
@@ -269,6 +262,51 @@ class TestABCDriver(TestBase):
                    namespace=ns)
         self.assertFalse(self.ip_dev().addr.add.called)
 
+    def test_l3_init_with_duplicated_ipv6_dynamic(self):
+        device_name = 'tap0'
+        cidr = '2001:db8:a::123/64'
+        ns = '12345678-1234-5678-90ab-ba0987654321'
+        addresses = [dict(scope='global',
+                          dynamic=True,
+                          cidr=cidr)]
+        self.ip_dev().addr.list = mock.Mock(return_value=addresses)
+        bc = BaseChild(self.conf)
+        bc.init_l3(device_name, [cidr], namespace=ns)
+        self.ip_dev.assert_has_calls(
+            [mock.call(device_name, namespace=ns),
+             mock.call().addr.list(),
+             mock.call().addr.delete(cidr),
+             mock.call().addr.add(cidr)])
+
+    def test_l3_init_with_duplicated_ipv6_lla(self):
+        device_name = 'tap0'
+        cidr = 'fe80::a8bb:ccff:fedd:eeff/64'
+        ns = '12345678-1234-5678-90ab-ba0987654321'
+        addresses = [dict(scope='link',
+                          dynamic=False,
+                          cidr=cidr)]
+        self.ip_dev().addr.list = mock.Mock(return_value=addresses)
+        bc = BaseChild(self.conf)
+        bc.init_l3(device_name, [cidr], namespace=ns)
+        self.ip_dev.assert_has_calls(
+            [mock.call(device_name, namespace=ns),
+             mock.call().addr.list()])
+        # The above assert won't verify there were no extra calls right
+        # after list()
+        self.assertFalse(self.ip_dev().addr.add.called)
+
+    def test_l3_init_with_not_present_ipv6_lla(self):
+        device_name = 'tap0'
+        cidr = 'fe80::a8bb:ccff:fedd:eeff/64'
+        ns = '12345678-1234-5678-90ab-ba0987654321'
+        self.ip_dev().addr.list = mock.Mock(return_value=[])
+        bc = BaseChild(self.conf)
+        bc.init_l3(device_name, [cidr], namespace=ns)
+        self.ip_dev.assert_has_calls(
+            [mock.call(device_name, namespace=ns),
+             mock.call().addr.list(),
+             mock.call().addr.add(cidr)])
+
     def test_add_ipv6_addr(self):
         device_name = 'tap0'
         cidr = '2001:db8::/64'
@@ -337,16 +375,24 @@ class TestABCDriver(TestBase):
         addresses = [dict(scope='link',
                           dynamic=False,
                           cidr='fe80:cafe::/64')]
-        self.ip_dev().addr.list = mock.Mock(return_value=addresses)
+        self.get_devices_with_ip.return_value = addresses
         device_name = self.ip_dev().name
         bc = BaseChild(self.conf)
 
         llas = bc.get_ipv6_llas(device_name, ns)
 
         self.assertEqual(addresses, llas)
-        self.ip_dev.assert_has_calls(
-            [mock.call(device_name, namespace=ns),
-             mock.call().addr.list(scope='link', ip_version=6)])
+        kwargs = {'family': utils.get_socket_address_family(
+                                constants.IP_VERSION_6),
+                  'scope': 'link'}
+        self.get_devices_with_ip.assert_called_with(
+            ns, name=device_name, **kwargs)
+
+    def test_set_mtu_logs_once(self):
+        bc = BaseChild(self.conf)
+        with mock.patch('neutron.agent.linux.interface.LOG.warning') as log:
+            bc.set_mtu('dev', 9999)
+            log.assert_called_once_with(mock.ANY)
 
 
 class TestOVSInterfaceDriver(TestBase):
@@ -368,8 +414,8 @@ class TestOVSInterfaceDriver(TestBase):
     def test_plug_configured_bridge(self):
         br = 'br-v'
         self.conf.set_override('ovs_use_veth', False)
-        self.conf.set_override('ovs_integration_bridge', br)
-        self.assertEqual(self.conf.ovs_integration_bridge, br)
+        self.conf.set_override('integration_bridge', br, 'OVS')
+        self.assertEqual(self.conf.OVS.integration_bridge, br)
 
         def device_exists(dev, namespace=None):
             return dev == br
@@ -391,8 +437,7 @@ class TestOVSInterfaceDriver(TestBase):
                                          internal=True)
 
     def _test_plug(self, bridge=None, namespace=None):
-        with mock.patch('neutron.agent.ovsdb.native.connection.'
-                        'Connection.start'):
+        with mock.patch('neutron.agent.ovsdb.impl_idl._connection'):
             if not bridge:
                 bridge = 'br-int'
 
@@ -431,19 +476,47 @@ class TestOVSInterfaceDriver(TestBase):
                      mock.call().ensure_namespace().add_device_to_namespace(
                          mock.ANY)])
             expected.extend([
+                mock.call(namespace=namespace),
+                mock.call().device('tap0'),
                 mock.call().device().link.set_mtu(9000),
                 mock.call().device().link.set_up(),
             ])
 
             self.ip.assert_has_calls(expected)
 
-    def test_unplug(self, bridge=None):
-        if not bridge:
+    def test_plug_new(self):
+        with mock.patch('neutron.agent.ovsdb.impl_idl._connection'):
             bridge = 'br-int'
+            namespace = '01234567-1234-1234-99'
+            with mock.patch.object(ovs_lib.OVSBridge,
+                                   'delete_port') as delete_port:
+                with mock.patch.object(ovs_lib.OVSBridge, 'replace_port'):
+                    ovs = interface.OVSInterfaceDriver(self.conf)
+                    reraise = mock.patch.object(
+                        excutils, 'save_and_reraise_exception')
+                    reraise.start()
+                    ip_wrapper = mock.Mock()
+                    for exception in (OSError(),
+                                      pyroute2_exc.NetlinkError(22)):
+                        ip_wrapper.ensure_namespace.side_effect = exception
+                        self.ip.return_value = ip_wrapper
+                        delete_port.reset_mock()
+                        ovs.plug_new(
+                            '01234567-1234-1234-99',
+                            'port-1234',
+                            'tap0',
+                            'aa:bb:cc:dd:ee:ff',
+                            bridge=bridge,
+                            namespace=namespace,
+                            prefix='veth',
+                            mtu=9000)
+                        delete_port.assert_called_once_with('tap0')
+
+    def test_unplug(self):
         with mock.patch('neutron.agent.common.ovs_lib.OVSBridge') as ovs_br:
             ovs = interface.OVSInterfaceDriver(self.conf)
             ovs.unplug('tap0')
-            ovs_br.assert_has_calls([mock.call(bridge),
+            ovs_br.assert_has_calls([mock.call('br-int'),
                                      mock.call().delete_port('tap0')])
 
 
@@ -451,7 +524,12 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
 
     def setUp(self):
         super(TestOVSInterfaceDriverWithVeth, self).setUp()
+        ovs_conf.register_ovs_agent_opts(self.conf)
         self.conf.set_override('ovs_use_veth', True)
+        self.conf.set_override(
+            'datapath_type',
+            ovs_const.OVS_DATAPATH_NETDEV,
+            group='OVS')
 
     def test_get_device_name(self):
         br = interface.OVSInterfaceDriver(self.conf)
@@ -463,8 +541,7 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
 
     def _test_plug(self, devname=None, bridge=None, namespace=None,
                    prefix=None):
-        with mock.patch('neutron.agent.ovsdb.native.connection.'
-                        'Connection.start'):
+        with mock.patch('neutron.agent.ovsdb.impl_idl._connection'):
 
             if not devname:
                 devname = 'ns-0'
@@ -480,6 +557,11 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
             root_dev = mock.Mock()
             ns_dev = mock.Mock()
             self.ip().add_veth = mock.Mock(return_value=(root_dev, ns_dev))
+            mock.patch.object(
+                interface, '_get_veth',
+                return_value=(root_dev, ns_dev)).start()
+            ns_dev.name = devname
+
             expected = [mock.call(),
                         mock.call().add_veth('tap0', devname,
                                              namespace2=namespace)]
@@ -509,10 +591,17 @@ class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
             self.ip.assert_has_calls(expected)
             root_dev.assert_has_calls([mock.call.link.set_up()])
             ns_dev.assert_has_calls([mock.call.link.set_up()])
+            self.eth_offload.assert_has_calls(
+                [mock.call(devname, rx=False, tx=False, namespace=namespace)])
 
-    def test_unplug(self, bridge=None):
-        if not bridge:
-            bridge = 'br-int'
+    def test_plug_new(self):
+        # The purpose of test_plug_new in parent class(TestOVSInterfaceDriver)
+        # is to test exception(exceptions.ProcessExecutionError), method here
+        # would not go through that code, So just pass
+        pass
+
+    def test_unplug(self):
+        bridge = 'br-int'
         with mock.patch('neutron.agent.common.ovs_lib.OVSBridge') as ovs_br:
             ovs = interface.OVSInterfaceDriver(self.conf)
             ovs.unplug('ns-0', bridge=bridge)
@@ -542,6 +631,9 @@ class TestBridgeInterfaceDriver(TestBase):
         ns_veth = mock.Mock()
 
         self.ip().add_veth = mock.Mock(return_value=(root_veth, ns_veth))
+        mock.patch.object(
+            interface, '_get_veth',
+            return_value=(root_veth, ns_veth)).start()
 
         self.device_exists.side_effect = device_exists
         br = interface.BridgeInterfaceDriver(self.conf)
@@ -595,72 +687,18 @@ class TestBridgeInterfaceDriver(TestBase):
                                       mock.call().link.delete()])
 
 
-class TestIVSInterfaceDriver(TestBase):
+class TestLegacyDriver(TestBase):
 
-    def setUp(self):
-        super(TestIVSInterfaceDriver, self).setUp()
-
-    def test_get_device_name(self):
-        br = interface.IVSInterfaceDriver(self.conf)
-        device_name = br.get_device_name(FakePort())
-        self.assertEqual('ns-abcdef01-12', device_name)
-
-    def test_plug_with_prefix(self):
-        self._test_plug(devname='qr-0', prefix='qr-')
-
-    def _test_plug(self, devname=None, namespace=None, prefix=None):
-
-        if not devname:
-            devname = 'ns-0'
-
-        def device_exists(dev, namespace=None):
-            return dev == 'indigo'
-
-        ivs = interface.IVSInterfaceDriver(self.conf)
-        self.device_exists.side_effect = device_exists
-
-        root_dev = mock.Mock()
-        _ns_dev = mock.Mock()
-        ns_dev = mock.Mock()
-        self.ip().add_veth = mock.Mock(return_value=(root_dev, _ns_dev))
-        self.ip().device = mock.Mock(return_value=(ns_dev))
-        expected = [mock.call(), mock.call().add_veth('tap0', devname),
-                    mock.call().device(devname)]
-
-        ivsctl_cmd = ['ivs-ctl', 'add-port', 'tap0']
-
-        with mock.patch.object(utils, 'execute') as execute:
-            ivs.plug('01234567-1234-1234-99',
-                     'port-1234',
-                     devname,
-                     'aa:bb:cc:dd:ee:ff',
-                     namespace=namespace,
-                     prefix=prefix,
-                     mtu=9000)
-            execute.assert_called_once_with(ivsctl_cmd, run_as_root=True)
-
-        ns_dev.assert_has_calls(
-            [mock.call.link.set_address('aa:bb:cc:dd:ee:ff')])
-        ns_dev.assert_has_calls([mock.call.link.set_mtu(9000)])
-        root_dev.assert_has_calls([mock.call.link.set_mtu(9000)])
-        if namespace:
-            expected.extend(
-                [mock.call().ensure_namespace(namespace),
-                 mock.call().ensure_namespace().add_device_to_namespace(
-                     mock.ANY)])
-
-        self.ip.assert_has_calls(expected)
-        root_dev.assert_has_calls([mock.call.link.set_up()])
-        ns_dev.assert_has_calls([mock.call.link.set_up()])
-
-    def test_plug_namespace(self):
-        self._test_plug(namespace='mynamespace')
-
-    def test_unplug(self):
-        ivs = interface.IVSInterfaceDriver(self.conf)
-        ivsctl_cmd = ['ivs-ctl', 'del-port', 'tap0']
-        with mock.patch.object(utils, 'execute') as execute:
-            ivs.unplug('ns-0')
-            execute.assert_called_once_with(ivsctl_cmd, run_as_root=True)
-            self.ip_dev.assert_has_calls([mock.call('ns-0', namespace=None),
-                                          mock.call().link.delete()])
+    def test_plug(self):
+        self.device_exists.return_value = False
+        with mock.patch('neutron.agent.linux.interface.LOG.warning') as log:
+            driver = FakeLegacyInterfaceDriver(self.conf)
+            try:
+                driver.plug(
+                    '01234567-1234-1234-99', 'port-1234', 'tap0',
+                    'aa:bb:cc:dd:ee:ff')
+            except TypeError:
+                self.fail("LinuxInterfaceDriver class can not call properly "
+                          "plug_new method from the legacy drivers that "
+                          "do not accept 'link_up' parameter.")
+            log.assert_called_once()
