@@ -15,19 +15,17 @@
 import copy
 import signal
 import socket
+from unittest import mock
 
-import mock
-import six
 import testtools
 
 from neutron_lib import exceptions
+from neutron_lib import fixture as lib_fixtures
 from oslo_config import cfg
 import oslo_i18n
 
 from neutron.agent.linux import utils
 from neutron.tests import base
-from neutron.tests.common import helpers
-from neutron.tests import tools
 
 
 _marker = object()
@@ -155,16 +153,11 @@ class AgentUtilsExecuteTest(base.BaseTestCase):
     def test_encode_process_input(self):
         str_idata = "%s\n" % self.test_file[:-1]
         str_odata = "%s\n" % self.test_file
-        if six.PY3:
-            bytes_idata = str_idata.encode(encoding='utf-8')
-            bytes_odata = str_odata.encode(encoding='utf-8')
-            self.mock_popen.return_value = [bytes_odata, b'']
-            result = utils.execute(['cat'], process_input=str_idata)
-            self.mock_popen.assert_called_once_with(bytes_idata)
-        else:
-            self.mock_popen.return_value = [str_odata, '']
-            result = utils.execute(['cat'], process_input=str_idata)
-            self.mock_popen.assert_called_once_with(str_idata)
+        bytes_idata = str_idata.encode(encoding='utf-8')
+        bytes_odata = str_odata.encode(encoding='utf-8')
+        self.mock_popen.return_value = [bytes_odata, b'']
+        result = utils.execute(['cat'], process_input=str_idata)
+        self.mock_popen.assert_called_once_with(bytes_idata)
         self.assertEqual(str_odata, result)
 
     def test_return_str_data(self):
@@ -173,7 +166,6 @@ class AgentUtilsExecuteTest(base.BaseTestCase):
         result = utils.execute(['ls', self.test_file], return_stderr=True)
         self.assertEqual((str_data, ''), result)
 
-    @helpers.requires_py3
     def test_surrogateescape_in_decoding_out_data(self):
         bytes_err_data = b'\xed\xa0\xbd'
         err_data = bytes_err_data.decode('utf-8', 'surrogateescape')
@@ -306,7 +298,7 @@ class TestGetCmdlineFromPid(base.BaseTestCase):
     def _test_cmdline(self, process, expected_cmd):
         self.process_is_running_mock.return_value = True
         mock_open = self.useFixture(
-            tools.OpenFixture('/proc/%s/cmdline' % self.pid, process)
+            lib_fixtures.OpenFixture('/proc/%s/cmdline' % self.pid, process)
         ).mock_open
         cmdline = utils.get_cmdline_from_pid(self.pid)
         mock_open.assert_called_once_with('/proc/%s/cmdline' % self.pid, 'r')
@@ -322,15 +314,30 @@ class TestGetCmdlineFromPid(base.BaseTestCase):
         expected_cmdline = ["python3", "test-binary", "test", "option"]
         self._test_cmdline(process_cmd, expected_cmdline)
 
+    def test_cmdline_with_single_argument(self):
+        process_cmd = "test-binary\0"
+        expected_cmdline = ["test-binary"]
+        self._test_cmdline(process_cmd, expected_cmdline)
+
     def test_no_process_running(self):
         self.process_is_running_mock.return_value = False
         mock_open = self.useFixture(
-            tools.OpenFixture('/proc/%s/cmdline' % self.pid)
+            lib_fixtures.OpenFixture('/proc/%s/cmdline' % self.pid)
         ).mock_open
         cmdline = utils.get_cmdline_from_pid(self.pid)
         mock_open.assert_not_called()
         self.assertEqual([], cmdline)
 
+    def test_cmdline_process_disappearing(self):
+        self.process_is_running_mock.return_value = True
+        mock_open = self.useFixture(
+            lib_fixtures.OpenFixture('/proc/%s/cmdline' % self.pid, 'process')
+        ).mock_open
+        mock_open.side_effect = IOError()
+        cmdline = utils.get_cmdline_from_pid(self.pid)
+        mock_open.assert_called_once_with('/proc/%s/cmdline' % self.pid, 'r')
+        self.assertEqual([], cmdline)
+
 
 class TestFindChildPids(base.BaseTestCase):
 
@@ -421,14 +428,29 @@ class TestPathUtilities(base.BaseTestCase):
 
     def test_cmd_matches_expected_matches_abs_path(self):
         cmd = ['/bar/../foo']
-        self.assertTrue(utils.cmd_matches_expected(cmd, cmd))
+        self.assertTrue(utils.cmd_matches_expected(cmd, cmd, None))
 
     def test_cmd_matches_expected_matches_script(self):
         self.assertTrue(utils.cmd_matches_expected(['python', 'script'],
-                                                   ['script']))
+                                                   ['script'], None))
 
     def test_cmd_matches_expected_doesnt_match(self):
-        self.assertFalse(utils.cmd_matches_expected('foo', 'bar'))
+        self.assertFalse(utils.cmd_matches_expected('foo', 'bar', None))
+
+    def test_cmd_matches_expected_matches_script_with_procname(self):
+        self.assertTrue(utils.cmd_matches_expected(
+            ['proc_name', '(python', 'script)'], ['script'], 'proc_name'))
+
+    def test_cmd_matches_expected_matches_abs_path_script_with_procname(self):
+        self.assertTrue(utils.cmd_matches_expected(
+            ['proc_name', '(python', '/bar/../foo)'], ['/bar/../foo'],
+            'proc_name'))
+        self.assertTrue(utils.cmd_matches_expected(
+            ['proc_name', '(python', '/bar/../foo', 'input_param)'],
+            ['/bar/../foo', 'input_param'], 'proc_name'))
+        self.assertTrue(utils.cmd_matches_expected(
+            ['proc_name', '(/bar/../foo', 'input_param)'],
+            ['/bar/../foo', 'input_param'], 'proc_name'))
 
 
 class FakeUser(object):
