@@ -13,17 +13,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import abc
 import copy
+from unittest import mock
 
 import fixtures
-import mock
+from neutron_lib import exceptions
+from neutron_lib.plugins import constants as lib_const
+from neutron_lib.plugins import directory
+from neutron_lib.services import base as service_base
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_service import wsgi as base_wsgi
 import routes
-import six
 import testtools
 import webob
 import webob.exc as webexc
@@ -31,14 +33,12 @@ import webtest
 
 import neutron
 from neutron.api import extensions
-from neutron.api.v2 import attributes
 from neutron.common import config
-from neutron.common import exceptions
-from neutron import manager
 from neutron.plugins.common import constants
 from neutron import quota
 from neutron.tests import base
 from neutron.tests.unit.api.v2 import test_base
+from neutron.tests.unit import dummy_plugin
 from neutron.tests.unit import extension_stubs as ext_stubs
 import neutron.tests.unit.extensions
 from neutron.tests.unit.extensions import extendedattribute as extattr
@@ -84,7 +84,7 @@ class ExtensionsTestApp(base_wsgi.Router):
         super(ExtensionsTestApp, self).__init__(mapper)
 
 
-class FakePluginWithExtension(object):
+class FakePluginWithExtension(service_base.ServicePluginBase):
     """A fake plugin used only for extension testing in this file."""
 
     supported_extension_aliases = ["FOXNSOX"]
@@ -92,6 +92,12 @@ class FakePluginWithExtension(object):
     def method_to_support_foxnsox_extension(self, context):
         self._log("method_to_support_foxnsox_extension", context)
 
+    def get_plugin_type(self):
+        pass
+
+    def get_plugin_description(self):
+        pass
+
 
 class ExtensionPathTest(base.BaseTestCase):
 
@@ -100,8 +106,10 @@ class ExtensionPathTest(base.BaseTestCase):
         super(ExtensionPathTest, self).setUp()
 
     def test_get_extensions_path_with_plugins(self):
+        cfg.CONF.set_override('api_extensions_path',
+                              'neutron/tests/unit/extensions')
         path = extensions.get_extensions_path(
-            {constants.CORE: FakePluginWithExtension()})
+            {lib_const.CORE: FakePluginWithExtension()})
         self.assertEqual(path,
                          '%s:neutron/tests/unit/extensions' % self.base_path)
 
@@ -127,47 +135,6 @@ class ExtensionPathTest(base.BaseTestCase):
         self.assertEqual(path, '%s:path1' % self.base_path)
 
 
-class PluginInterfaceTest(base.BaseTestCase):
-    def test_issubclass_hook(self):
-        class A(object):
-            def f(self):
-                pass
-
-        class B(extensions.PluginInterface):
-            @abc.abstractmethod
-            def f(self):
-                pass
-
-        self.assertTrue(issubclass(A, B))
-
-    def test_issubclass_hook_class_without_abstract_methods(self):
-        class A(object):
-            def f(self):
-                pass
-
-        class B(extensions.PluginInterface):
-            def f(self):
-                pass
-
-        self.assertFalse(issubclass(A, B))
-
-    def test_issubclass_hook_not_all_methods_implemented(self):
-        class A(object):
-            def f(self):
-                pass
-
-        class B(extensions.PluginInterface):
-            @abc.abstractmethod
-            def f(self):
-                pass
-
-            @abc.abstractmethod
-            def g(self):
-                pass
-
-        self.assertFalse(issubclass(A, B))
-
-
 class ResourceExtensionTest(base.BaseTestCase):
 
     class ResourceExtensionController(wsgi.Controller):
@@ -191,21 +158,21 @@ class ResourceExtensionTest(base.BaseTestCase):
             return {'collection': 'value'}
 
     class DummySvcPlugin(wsgi.Controller):
-            @classmethod
-            def get_plugin_type(cls):
-                return constants.DUMMY
+        @classmethod
+        def get_plugin_type(cls):
+            return dummy_plugin.DUMMY_SERVICE_TYPE
 
-            def index(self, request, **kwargs):
-                return "resource index"
+        def index(self, request, **kwargs):
+            return "resource index"
 
-            def custom_member_action(self, request, **kwargs):
-                return {'member_action': 'value'}
+        def custom_member_action(self, request, **kwargs):
+            return {'member_action': 'value'}
 
-            def collection_action(self, request, **kwargs):
-                return {'collection': 'value'}
+        def collection_action(self, request, **kwargs):
+            return {'collection': 'value'}
 
-            def show(self, request, id):
-                return {'data': {'id': id}}
+        def show(self, request, id):
+            return {'data': {'id': id}}
 
     def test_exceptions_notimplemented(self):
         controller = self.ResourceExtensionController()
@@ -220,7 +187,7 @@ class ResourceExtensionTest(base.BaseTestCase):
         try:
             test_app.get("/tweedles/some_id/notimplemented_function")
             # Shouldn't be reached
-            self.assertTrue(False)
+            self.fail()
         except webtest.AppError as e:
             self.assertIn('501', str(e))
 
@@ -242,7 +209,7 @@ class ResourceExtensionTest(base.BaseTestCase):
 
             @classmethod
             def get_plugin_type(cls):
-                return constants.DUMMY
+                return dummy_plugin.DUMMY_SERVICE_TYPE
 
         res_ext = extensions.ResourceExtension(
             'tweedles', DummySvcPlugin(), path_prefix="/dummy_svc")
@@ -702,12 +669,73 @@ class ExtensionManagerTest(base.BaseTestCase):
         self.assertNotIn('ext1-attr', attr_map['ext2'])
         self.assertNotIn('ext2-attr', attr_map['ext1'])
 
+    def test_extension_extends_sub_resource(self):
+        """Unit test for bug 1722842
+
+        Check that an extension can extend a sub-resource
+        """
+        RESOURCE = "test_resource"
+        SUB_RESOURCE_NAME = "test_sub_resource"
+        INITIAL_PARAM = "dummy_param1"
+        ADDITIONAL_PARAM = "dummy_param2"
+
+        SUB_RESOURCE = {
+            'parent': {'member_name': RESOURCE},
+            'parameters': {
+                INITIAL_PARAM: {'allow_post': False,
+                                'allow_put': False,
+                                'validate': {'type:uuid': None},
+                                'is_visible': True}
+            }
+        }
+
+        class BaseExtension(ext_stubs.StubExtension):
+
+            def get_extended_resources(self, version):
+                return {
+                     SUB_RESOURCE_NAME: SUB_RESOURCE
+                }
+
+        class ExtensionExtendingASubresource(ext_stubs.StubExtension):
+
+            def get_extended_resources(self, version):
+                return {
+                    SUB_RESOURCE_NAME: {
+                        'parameters': {
+                            ADDITIONAL_PARAM: {'allow_post': False,
+                                               'allow_put': False,
+                                               'validate': {'type:uuid': None},
+                                               'is_visible': True}
+                        }
+                    }
+                }
+
+            def get_required_extensions(self):
+                return ['base_extension']
+
+        ext_mgr = extensions.ExtensionManager('')
+        attr_map = {}
+        ext_mgr.add_extension(BaseExtension('base_extension'))
+        ext_mgr.add_extension(
+            ExtensionExtendingASubresource())
+        ext_mgr.extend_resources("2.0", attr_map)
+
+        # check that the parent descriptor is untouched
+        self.assertEqual(SUB_RESOURCE['parent'],
+                         attr_map[SUB_RESOURCE_NAME]['parent'])
+        # check that the initial attribute is still here
+        self.assertIn(INITIAL_PARAM,
+                      attr_map[SUB_RESOURCE_NAME]['parameters'])
+        # check that the new attribute is here as well
+        self.assertIn(ADDITIONAL_PARAM,
+                      attr_map[SUB_RESOURCE_NAME]['parameters'])
+
 
 class PluginAwareExtensionManagerTest(base.BaseTestCase):
 
     def test_unsupported_extensions_are_not_loaded(self):
         stub_plugin = ext_stubs.StubPlugin(supported_extensions=["e1", "e3"])
-        plugin_info = {constants.CORE: stub_plugin}
+        plugin_info = {lib_const.CORE: stub_plugin}
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
             ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
@@ -728,7 +756,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
             """
             pass
 
-        plugin_info = {constants.CORE: ExtensionUnawarePlugin()}
+        plugin_info = {lib_const.CORE: ExtensionUnawarePlugin()}
         ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
         ext_mgr.add_extension(ext_stubs.StubExtension("e1"))
 
@@ -740,7 +768,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
             """Does not implement get_foo method as expected by extension."""
             supported_extension_aliases = ["supported_extension"]
 
-        plugin_info = {constants.CORE: PluginWithoutExpectedIface()}
+        plugin_info = {lib_const.CORE: PluginWithoutExpectedIface()}
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
             ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
@@ -751,14 +779,20 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
 
     def test_extensions_are_loaded_for_plugin_with_expected_interface(self):
 
-        class PluginWithExpectedInterface(object):
+        class PluginWithExpectedInterface(service_base.ServicePluginBase):
             """Implements get_foo method as expected by extension."""
             supported_extension_aliases = ["supported_extension"]
 
             def get_foo(self, bar=None):
                 pass
 
-        plugin_info = {constants.CORE: PluginWithExpectedInterface()}
+            def get_plugin_type(self):
+                pass
+
+            def get_plugin_description(self):
+                pass
+
+        plugin_info = {lib_const.CORE: PluginWithExpectedInterface()}
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
             ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
@@ -775,7 +809,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
             """
             pass
         stub_plugin = ext_stubs.StubPlugin(supported_extensions=["e1"])
-        plugin_info = {constants.CORE: stub_plugin}
+        plugin_info = {lib_const.CORE: stub_plugin}
 
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
@@ -794,7 +828,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
                 return None
 
         stub_plugin = ext_stubs.StubPlugin(supported_extensions=["e1"])
-        plugin_info = {constants.CORE: stub_plugin}
+        plugin_info = {lib_const.CORE: stub_plugin}
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
             ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
@@ -808,7 +842,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
                 return None
 
         stub_plugin = ext_stubs.StubPlugin(supported_extensions=["e1"])
-        plugin_info = {constants.DUMMY: stub_plugin}
+        plugin_info = {dummy_plugin.DUMMY_SERVICE_TYPE: stub_plugin}
         with mock.patch("neutron.api.extensions.PluginAwareExtensionManager."
                         "check_if_plugin_extensions_loaded"):
             ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
@@ -819,7 +853,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
     def test_unloaded_supported_extensions_raises_exception(self):
         stub_plugin = ext_stubs.StubPlugin(
             supported_extensions=["unloaded_extension"])
-        plugin_info = {constants.CORE: stub_plugin}
+        plugin_info = {lib_const.CORE: stub_plugin}
         self.assertRaises(exceptions.ExtensionsNotFound,
                           extensions.PluginAwareExtensionManager,
                           '', plugin_info)
@@ -837,7 +871,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
 
         ext = FakeExtension()
 
-        plugin_info = {constants.CORE: FakePlugin()}
+        plugin_info = {lib_const.CORE: FakePlugin()}
         ext_mgr = extensions.PluginAwareExtensionManager('', plugin_info)
         ext_mgr.add_extension(ext)
         self.assertIn("stub_extension", ext_mgr.extensions)
@@ -860,7 +894,7 @@ class PluginAwareExtensionManagerTest(base.BaseTestCase):
                 'stub_plugin_extension', lambda: True, plugin_agnostic=False
             )
 
-        plugin_info = {constants.CORE: FakePlugin()}
+        plugin_info = {lib_const.CORE: FakePlugin()}
         self.assertRaises(
             exceptions.ExtensionsNotFound,
             extensions.PluginAwareExtensionManager, '', plugin_info)
@@ -908,7 +942,7 @@ def setup_extensions_middleware(extension_manager=None):
     extension_manager = (extension_manager or
                          extensions.PluginAwareExtensionManager(
                              extensions_path,
-                             {constants.CORE: FakePluginWithExtension()}))
+                             {lib_const.CORE: FakePluginWithExtension()}))
     base.BaseTestCase.config_parse()
     app = config.load_paste_app('extensions_test_app')
     return extensions.ExtensionMiddleware(app, ext_mgr=extension_manager)
@@ -985,7 +1019,7 @@ class ExtensionExtendedAttributeTestCase(base.BaseTestCase):
 
         ext_mgr = extensions.PluginAwareExtensionManager(
             extensions_path,
-            {constants.CORE: ExtensionExtendedAttributeTestPlugin()}
+            {lib_const.CORE: ExtensionExtendedAttributeTestPlugin()}
         )
         ext_mgr.extend_resources("2.0", {})
         extensions.PluginAwareExtensionManager._instance = ext_mgr
@@ -994,27 +1028,13 @@ class ExtensionExtendedAttributeTestCase(base.BaseTestCase):
         self._api = extensions.ExtensionMiddleware(app, ext_mgr=ext_mgr)
 
         self._tenant_id = "8c70909f-b081-452d-872b-df48e6c355d1"
-        # Save the global RESOURCE_ATTRIBUTE_MAP
-        self.saved_attr_map = {}
-        for res, attrs in six.iteritems(attributes.RESOURCE_ATTRIBUTE_MAP):
-            self.saved_attr_map[res] = attrs.copy()
-        # Add the resources to the global attribute map
-        # This is done here as the setup process won't
-        # initialize the main API router which extends
-        # the global attribute map
-        attributes.RESOURCE_ATTRIBUTE_MAP.update(
-            extattr.EXTENDED_ATTRIBUTES_2_0)
-        self.agentscheduler_dbMinxin = manager.NeutronManager.get_plugin()
-        self.addCleanup(self.restore_attribute_map)
+
+        self.agentscheduler_dbMinxin = directory.get_plugin()
 
         quota.QUOTAS._driver = None
         cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
                               group='QUOTAS')
 
-    def restore_attribute_map(self):
-        # Restore the original RESOURCE_ATTRIBUTE_MAP
-        attributes.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map
-
     def _do_request(self, method, path, data=None, params=None, action=None):
         content_type = 'application/json'
         body = None
