@@ -15,19 +15,24 @@
 
 import copy
 import os
+import re
 import sys
 import textwrap
+from unittest import mock
 
 from alembic.autogenerate import api as alembic_ag_api
 from alembic import config as alembic_config
 from alembic.operations import ops as alembic_ops
 from alembic import script as alembic_script
 import fixtures
-import mock
+from neutron_lib import fixture as lib_fixtures
+from neutron_lib.utils import helpers
 from oslo_utils import fileutils
 import pkg_resources
 import sqlalchemy as sa
+from testtools import matchers
 
+from neutron.conf.db import migration_cli
 from neutron.db import migration
 from neutron.db.migration import autogen
 from neutron.db.migration import cli
@@ -49,7 +54,7 @@ class FakeRevision(object):
         self.branch_labels = labels
         self.down_revision = down_revision
         self.is_branch_point = is_branch_point
-        self.revision = tools.get_random_string()
+        self.revision = helpers.get_random_string(10)
         self.module = mock.MagicMock()
 
 
@@ -60,12 +65,12 @@ class MigrationEntrypointsMemento(fixtures.Fixture):
 
     def _setUp(self):
         self.ep_backup = {}
-        for proj, ep in cli.migration_entrypoints.items():
+        for proj, ep in migration_cli.migration_entrypoints.items():
             self.ep_backup[proj] = copy.copy(ep)
         self.addCleanup(self.restore)
 
     def restore(self):
-        cli.migration_entrypoints = self.ep_backup
+        migration_cli.migration_entrypoints = self.ep_backup
 
 
 class TestDbMigration(base.BaseTestCase):
@@ -118,8 +123,8 @@ class TestCli(base.BaseTestCase):
         super(TestCli, self).setUp()
         self.do_alembic_cmd_p = mock.patch.object(cli, 'do_alembic_command')
         self.do_alembic_cmd = self.do_alembic_cmd_p.start()
-        self.mock_alembic_err = mock.patch('alembic.util.err').start()
-        self.mock_alembic_warn = mock.patch('alembic.util.warn').start()
+        self.mock_alembic_err = mock.patch.object(cli, "log_error").start()
+        self.mock_alembic_warn = mock.patch.object(cli, "log_warning").start()
         self.mock_alembic_err.side_effect = SystemExit
 
         def mocked_root_dir(cfg):
@@ -127,14 +132,14 @@ class TestCli(base.BaseTestCase):
         mock_root = mock.patch.object(cli, '_get_package_root_dir').start()
         mock_root.side_effect = mocked_root_dir
         # Avoid creating fake directories
-        mock.patch('neutron.common.utils.ensure_dir').start()
+        mock.patch('oslo_utils.fileutils.ensure_tree').start()
 
         # Set up some configs and entrypoints for tests to chew on
         self.configs = []
         self.projects = ('neutron', 'networking-foo', 'neutron-fwaas')
         ini = os.path.join(os.path.dirname(cli.__file__), 'alembic.ini')
         self.useFixture(MigrationEntrypointsMemento())
-        cli.migration_entrypoints = {}
+        migration_cli.migration_entrypoints = {}
         for project in self.projects:
             config = alembic_config.Config(ini)
             config.set_main_option('neutron_project', project)
@@ -146,7 +151,7 @@ class TestCli(base.BaseTestCase):
             entrypoint = pkg_resources.EntryPoint(project,
                                                   module_name,
                                                   attrs=attrs)
-            cli.migration_entrypoints[project] = entrypoint
+            migration_cli.migration_entrypoints[project] = entrypoint
 
     def _main_test_helper(self, argv, func_name, exp_kwargs=[{}]):
         with mock.patch.object(sys, 'argv', argv),\
@@ -405,10 +410,10 @@ class TestCli(base.BaseTestCase):
                     heads[1]: FakeRevision(labels=cli.EXPAND_BRANCH)}
             fc.return_value.get_revision.side_effect = revs.__getitem__
             mock_open_con = self.useFixture(
-                tools.OpenFixture(cli._get_contract_head_file_path(
+                lib_fixtures.OpenFixture(cli._get_contract_head_file_path(
                     fake_config), contract_head + '\n')).mock_open
             mock_open_ex = self.useFixture(
-                tools.OpenFixture(cli._get_expand_head_file_path(
+                lib_fixtures.OpenFixture(cli._get_expand_head_file_path(
                     fake_config), expand_head + '\n')).mock_open
 
             if contract_head in heads and expand_head in heads:
@@ -448,10 +453,10 @@ class TestCli(base.BaseTestCase):
     def test_update_head_files_success(self, *mocks):
         heads = ['a', 'b']
         mock_open_con = self.useFixture(
-                    tools.OpenFixture(cli._get_contract_head_file_path(
+                    lib_fixtures.OpenFixture(cli._get_contract_head_file_path(
                         self.configs[0]))).mock_open
         mock_open_ex = self.useFixture(
-            tools.OpenFixture(cli._get_expand_head_file_path(
+            lib_fixtures.OpenFixture(cli._get_expand_head_file_path(
                 self.configs[0]))).mock_open
         with mock.patch('alembic.script.ScriptDirectory.from_config') as fc:
             fc.return_value.get_heads.return_value = heads
@@ -512,7 +517,7 @@ class TestCli(base.BaseTestCase):
 
     @mock.patch.object(cli, '_compare_labels')
     def test__validate_single_revision_labels_branchless_fail_different_labels(
-        self, compare_mock):
+            self, compare_mock):
 
         fake_down_revision = FakeRevision()
         fake_revision = FakeRevision(down_revision=fake_down_revision)
@@ -530,7 +535,7 @@ class TestCli(base.BaseTestCase):
 
     @mock.patch.object(cli, '_compare_labels')
     def test__validate_single_revision_labels_branches_fail_different_labels(
-        self, compare_mock):
+            self, compare_mock):
 
         fake_down_revision = FakeRevision()
         fake_revision = FakeRevision(down_revision=fake_down_revision)
@@ -558,7 +563,7 @@ class TestCli(base.BaseTestCase):
 
     @mock.patch.object(cli, '_validate_single_revision_labels')
     def test__validate_revision_validates_branchless_migrations(
-        self, validate_mock):
+            self, validate_mock):
 
         script_dir = mock.Mock()
         fake_revision = FakeRevision()
@@ -568,7 +573,7 @@ class TestCli(base.BaseTestCase):
     @mock.patch.object(cli, '_validate_revision')
     @mock.patch('alembic.script.ScriptDirectory.walk_revisions')
     def test_validate_revisions_walks_thru_all_revisions(
-        self, walk_mock, validate_mock):
+            self, walk_mock, validate_mock):
 
         revisions = [FakeRevision() for i in range(10)]
         walk_mock.return_value = revisions
@@ -580,7 +585,7 @@ class TestCli(base.BaseTestCase):
     @mock.patch.object(cli, '_validate_revision')
     @mock.patch('alembic.script.ScriptDirectory.walk_revisions')
     def test_validate_revisions_fails_on_multiple_branch_points(
-        self, walk_mock, validate_mock):
+            self, walk_mock, validate_mock):
 
         revisions = [FakeRevision(is_branch_point=True) for i in range(2)]
         walk_mock.return_value = revisions
@@ -674,8 +679,13 @@ class TestCli(base.BaseTestCase):
         self.assertTrue(expand.downgrade_ops.is_empty())
         self.assertTrue(contract.downgrade_ops.is_empty())
 
-        self.assertEqual(
-            textwrap.dedent("""\
+        def _get_regex(s):
+            s = textwrap.dedent(s)
+            s = re.escape(s)
+            # alembic 0.8.9 added additional leading '# ' before comments
+            return s.replace('\\#\\#\\#\\ ', '(# )?### ')
+
+        expected_regex = ("""\
             ### commands auto generated by Alembic - please adjust! ###
                 op.create_table('organization',
                 sa.Column('id', sa.Integer(), nullable=False),
@@ -686,17 +696,19 @@ class TestCli(base.BaseTestCase):
                 """sa.Column('organization_id', sa.Integer(), nullable=True))
                 op.create_foreign_key('org_fk', 'user', """
                 """'organization', ['organization_id'], ['id'])
-                ### end Alembic commands ###"""),
-            alembic_ag_api.render_python_code(expand.upgrade_ops)
-        )
-        self.assertEqual(
-            textwrap.dedent("""\
+                ### end Alembic commands ###""")
+        self.assertThat(
+            alembic_ag_api.render_python_code(expand.upgrade_ops),
+            matchers.MatchesRegex(_get_regex(expected_regex)))
+
+        expected_regex = ("""\
             ### commands auto generated by Alembic - please adjust! ###
                 op.drop_constraint('user', 'uq_user_org', type_=None)
                 op.drop_column('user', 'organization_name')
-                ### end Alembic commands ###"""),
-            alembic_ag_api.render_python_code(contract.upgrade_ops)
-        )
+                ### end Alembic commands ###""")
+        self.assertThat(
+            alembic_ag_api.render_python_code(contract.upgrade_ops),
+            matchers.MatchesRegex(_get_regex(expected_regex)))
 
     @mock.patch('alembic.script.ScriptDirectory.walk_revisions')
     def test__find_milestone_revisions_one_branch(self, walk_mock):
