@@ -13,17 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
+from neutron_lib.api.definitions import servicetype as svctype_apidef
+from neutron_lib import context
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import constants
 from oslo_config import cfg
+from oslo_utils import uuidutils
 import webob.exc as webexc
 import webtest
 
 from neutron.api import extensions
-from neutron import context
 from neutron.db import servicetype_db as st_db
 from neutron.extensions import servicetype
-from neutron.plugins.common import constants
+from neutron.objects import servicetype as servicetype_obj
 from neutron.services import provider_configuration as provconf
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.api.v2 import test_base
@@ -35,6 +39,8 @@ from neutron.tests.unit import testlib_api
 _uuid = test_base._uuid
 _get_path = test_base._get_path
 
+PLUGIN_NAME = 'neutron.db.db_base_plugin_v2.NeutronDbPluginV2'
+
 
 class ServiceTypeManagerTestCase(testlib_api.SqlTestCase):
     def setUp(self):
@@ -42,36 +48,42 @@ class ServiceTypeManagerTestCase(testlib_api.SqlTestCase):
             provconf.NeutronModule, 'service_providers').start()
         super(ServiceTypeManagerTestCase, self).setUp()
         self.ctx = context.get_admin_context()
+        self.setup_coreplugin(PLUGIN_NAME)
 
     def _set_override(self, service_providers):
         self.service_providers.return_value = service_providers
         st_db.ServiceTypeManager._instance = None
         self.manager = st_db.ServiceTypeManager.get_instance()
         for provider in service_providers:
+            service_type = provider.split(':')[0]
             self.manager.add_provider_configuration(
-                provider.split(':')[0], provconf.ProviderConfiguration())
+                service_type, provconf.ProviderConfiguration(
+                    svc_type=service_type))
 
     def test_service_provider_driver_not_unique(self):
-        self._set_override([constants.LOADBALANCER + ':lbaas:driver'])
-        prov = {'service_type': constants.LOADBALANCER,
+        self._set_override([constants.FIREWALL + ':fwaas:driver'])
+        prov = {'service_type': constants.FIREWALL,
                 'name': 'name2',
                 'driver': 'driver',
                 'default': False}
         self.assertRaises(
             n_exc.Invalid,
-            self.manager.config['LOADBALANCER'].add_provider, prov)
+            self.manager.config['FIREWALL'].add_provider, prov)
 
     def test_get_service_providers(self):
         """Test that get_service_providers filters correctly."""
         self._set_override(
-            [constants.LOADBALANCER +
-             ':lbaas:driver_path1',
+            [constants.VPN +
+             ':vpnaas:driver_path1',
              constants.FIREWALL +
              ':fwaas:driver_path2'])
         ctx = context.get_admin_context()
+        res = self.manager.get_service_providers(ctx)
+        self.assertEqual(2, len(res))
+
         res = self.manager.get_service_providers(
             ctx,
-            filters=dict(service_type=[constants.LOADBALANCER])
+            filters=dict(service_type=[constants.FIREWALL])
         )
         self.assertEqual(1, len(res))
 
@@ -85,66 +97,76 @@ class ServiceTypeManagerTestCase(testlib_api.SqlTestCase):
         self.assertRaises(
             n_exc.Invalid,
             self._set_override,
-            [constants.LOADBALANCER +
-            ':lbaas1:driver_path:default',
-            constants.LOADBALANCER +
-            ':lbaas2:driver_path:default'])
+            [constants.FIREWALL +
+            ':fwaas1:driver_path:default',
+            constants.FIREWALL +
+            ':fwaas2:driver_path:default'])
 
     def test_get_default_provider(self):
-        self._set_override([constants.LOADBALANCER +
-                            ':lbaas1:driver_path:default',
-                            constants.DUMMY +
-                            ':lbaas2:driver_path2'])
+        self._set_override([constants.FIREWALL +
+                            ':fwaas1:driver_path:default',
+                            dp.DUMMY_SERVICE_TYPE +
+                            ':fwaas2:driver_path2'])
         # can pass None as a context
         p = self.manager.get_default_service_provider(None,
-                                                      constants.LOADBALANCER)
-        self.assertEqual({'service_type': constants.LOADBALANCER,
-                          'name': 'lbaas1',
+                                                      constants.FIREWALL)
+        self.assertEqual({'service_type': constants.FIREWALL,
+                          'name': 'fwaas1',
                           'driver': 'driver_path',
                           'default': True}, p)
 
         self.assertRaises(
             provconf.DefaultServiceProviderNotFound,
             self.manager.get_default_service_provider,
-            None, constants.DUMMY
+            None, dp.DUMMY_SERVICE_TYPE
         )
 
     def test_get_provider_names_by_resource_ids(self):
-        self._set_override([constants.DUMMY + ':dummy1:driver_path',
-                            constants.DUMMY + ':dummy2:driver_path2'])
+        self._set_override([dp.DUMMY_SERVICE_TYPE +
+                            ':dummy1:driver_path',
+                            dp.DUMMY_SERVICE_TYPE +
+                            ':dummy2:driver_path2'])
         ctx = context.get_admin_context()
-        self.manager.add_resource_association(ctx, constants.DUMMY,
-                                              'dummy1', '1')
-        self.manager.add_resource_association(ctx, constants.DUMMY,
-                                              'dummy1', '2')
-        self.manager.add_resource_association(ctx, constants.DUMMY,
-                                              'dummy2', '3')
+        test_data = [{'provider_name': 'dummy1',
+                      'resource_id': uuidutils.generate_uuid()},
+                     {'provider_name': 'dummy1',
+                      'resource_id': uuidutils.generate_uuid()},
+                     {'provider_name': 'dummy2',
+                      'resource_id': uuidutils.generate_uuid()}]
+        self.manager.add_resource_association(ctx,
+                                              dp.DUMMY_SERVICE_TYPE,
+                                              **test_data[0])
+        self.manager.add_resource_association(ctx,
+                                              dp.DUMMY_SERVICE_TYPE,
+                                              **test_data[1])
+        self.manager.add_resource_association(ctx,
+                                              dp.DUMMY_SERVICE_TYPE,
+                                              **test_data[2])
         names_by_id = self.manager.get_provider_names_by_resource_ids(
-            ctx, ['1', '2', '3', '4'])
+            ctx, [td['resource_id'] for td in test_data])
         # unmatched IDs will be excluded from the result
-        self.assertEqual({'1': 'dummy1', '2': 'dummy1', '3': 'dummy2'},
-                         names_by_id)
+        self.assertEqual({td['resource_id']: td['provider_name']
+                          for td in test_data}, names_by_id)
 
     def test_add_resource_association(self):
-        self._set_override([constants.LOADBALANCER +
-                            ':lbaas1:driver_path:default',
-                            constants.DUMMY +
-                            ':lbaas2:driver_path2'])
+        self._set_override([constants.FIREWALL +
+                            ':fwaas1:driver_path:default',
+                            dp.DUMMY_SERVICE_TYPE +
+                            ':fwaas2:driver_path2'])
         ctx = context.get_admin_context()
         self.manager.add_resource_association(ctx,
-                                              constants.LOADBALANCER,
-                                              'lbaas1', '123-123')
-        self.assertEqual(ctx.session.
-                         query(st_db.ProviderResourceAssociation).count(),
-                         1)
-        assoc = ctx.session.query(st_db.ProviderResourceAssociation).one()
-        ctx.session.delete(assoc)
+                                              constants.FIREWALL,
+                                              'fwaas1',
+                                              uuidutils.generate_uuid())
+        self.assertEqual(
+            1, servicetype_obj.ProviderResourceAssociation.count(ctx))
+        servicetype_obj.ProviderResourceAssociation.delete_objects(ctx)
 
     def test_invalid_resource_association(self):
-        self._set_override([constants.LOADBALANCER +
-                            ':lbaas1:driver_path:default',
-                            constants.DUMMY +
-                            ':lbaas2:driver_path2'])
+        self._set_override([constants.FIREWALL +
+                            ':fwaas1:driver_path:default',
+                            dp.DUMMY_SERVICE_TYPE +
+                            ':fwaas2:driver_path2'])
         ctx = context.get_admin_context()
         self.assertRaises(provconf.ServiceProviderNotFound,
                           self.manager.add_resource_association,
@@ -180,7 +202,7 @@ class ServiceTypeExtensionTestCaseBase(testlib_api.WebTestCase):
         ext_mgr = TestServiceTypeExtensionManager()
         self.ext_mdw = test_extensions.setup_extensions_middleware(ext_mgr)
         self.api = webtest.TestApp(self.ext_mdw)
-        self.resource_name = servicetype.RESOURCE_NAME.replace('-', '_')
+        self.resource_name = svctype_apidef.RESOURCE_NAME.replace('-', '_')
         super(ServiceTypeExtensionTestCaseBase, self).setUp()
 
 
@@ -211,16 +233,18 @@ class ServiceTypeManagerExtTestCase(ServiceTypeExtensionTestCaseBase):
         self.service_providers = mock.patch.object(
             provconf.NeutronModule, 'service_providers').start()
         service_providers = [
-            constants.LOADBALANCER + ':lbaas:driver_path',
-            constants.DUMMY + ':dummy:dummy_dr'
+            constants.FIREWALL + ':fwaas:driver_path',
+            dp.DUMMY_SERVICE_TYPE + ':dummy:dummy_dr'
         ]
         self.service_providers.return_value = service_providers
         # Blank out service type manager instance
         st_db.ServiceTypeManager._instance = None
         self.manager = st_db.ServiceTypeManager.get_instance()
         for provider in service_providers:
+            service_type = provider.split(':')[0]
             self.manager.add_provider_configuration(
-                provider.split(':')[0], provconf.ProviderConfiguration())
+                service_type, provconf.ProviderConfiguration(
+                    svc_type=service_type))
         super(ServiceTypeManagerExtTestCase, self).setUp()
 
     def _list_service_providers(self):
