@@ -13,17 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
 
-import mock
 from neutron_lib import constants as n_const
+from neutron_lib import context as n_ctx
 from neutron_lib import exceptions as n_exc
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from novaclient import api_versions
 from novaclient import exceptions as nova_exceptions
 from oslo_config import cfg
 from oslo_utils import uuidutils
 from sqlalchemy.orm import attributes as sql_attr
 
-from neutron.db import models_v2
 from neutron.notifiers import nova
+from neutron.objects import ports as port_obj
 from neutron.tests import base
 
 DEVICE_OWNER_COMPUTE = n_const.DEVICE_OWNER_COMPUTE_PREFIX + 'fake'
@@ -33,15 +37,18 @@ DEVICE_OWNER_BAREMETAL = n_const.DEVICE_OWNER_BAREMETAL_PREFIX + 'fake'
 class TestNovaNotify(base.BaseTestCase):
     def setUp(self, plugin=None):
         super(TestNovaNotify, self).setUp()
+        self.ctx = n_ctx.get_admin_context()
+        self.port_uuid = uuidutils.generate_uuid()
 
         class FakePlugin(object):
             def get_port(self, context, port_id):
                 device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
                 return {'device_id': device_id,
-                        'device_owner': DEVICE_OWNER_COMPUTE}
+                        'device_owner': DEVICE_OWNER_COMPUTE,
+                        'id': port_id}
 
         self.nova_notifier = nova.Notifier()
-        self.nova_notifier._plugin_ref = FakePlugin()
+        directory.add_plugin(plugin_constants.CORE, FakePlugin())
 
     def test_notify_port_status_all_values(self):
         states = [n_const.PORT_STATUS_ACTIVE, n_const.PORT_STATUS_DOWN,
@@ -52,51 +59,45 @@ class TestNovaNotify(base.BaseTestCase):
         for previous_port_status in states:
             for current_port_status in states:
 
-                port = models_v2.Port(id='port-uuid', device_id=device_id,
-                                      device_owner=DEVICE_OWNER_COMPUTE,
-                                      status=current_port_status)
+                port = port_obj.Port(self.ctx, id=self.port_uuid,
+                                     device_id=device_id,
+                                     device_owner=DEVICE_OWNER_COMPUTE,
+                                     status=current_port_status)
                 self._record_port_status_changed_helper(current_port_status,
                                                         previous_port_status,
                                                         port)
 
     def test_port_without_uuid_device_id_no_notify(self):
-        port = models_v2.Port(id='port-uuid', device_id='compute_probe:',
-                              device_owner=DEVICE_OWNER_COMPUTE,
-                              status=n_const.PORT_STATUS_ACTIVE)
+        port = port_obj.Port(self.ctx, id=self.port_uuid,
+                             device_id='compute_probe:',
+                             device_owner=DEVICE_OWNER_COMPUTE,
+                             status=n_const.PORT_STATUS_ACTIVE)
         self._record_port_status_changed_helper(n_const.PORT_STATUS_ACTIVE,
                                                 sql_attr.NO_VALUE,
                                                 port)
 
     def test_port_without_device_owner_no_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        port = models_v2.Port(id='port-uuid', device_id=device_id,
-                              status=n_const.PORT_STATUS_ACTIVE)
+        port = port_obj.Port(self.ctx, id=self.port_uuid, device_id=device_id,
+                             device_owner="",
+                             status=n_const.PORT_STATUS_ACTIVE)
         self._record_port_status_changed_helper(n_const.PORT_STATUS_ACTIVE,
                                                 sql_attr.NO_VALUE,
                                                 port)
 
     def test_port_without_device_id_no_notify(self):
-        port = models_v2.Port(id='port-uuid',
-                              device_owner=n_const.DEVICE_OWNER_DHCP,
-                              status=n_const.PORT_STATUS_ACTIVE)
-        self._record_port_status_changed_helper(n_const.PORT_STATUS_ACTIVE,
-                                                sql_attr.NO_VALUE,
-                                                port)
-
-    def test_port_without_id_no_notify(self):
-        device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        port = models_v2.Port(device_id=device_id,
-                              device_owner=DEVICE_OWNER_COMPUTE,
-                              status=n_const.PORT_STATUS_ACTIVE)
+        port = port_obj.Port(self.ctx, id=self.port_uuid, device_id="",
+                             device_owner=n_const.DEVICE_OWNER_DHCP,
+                             status=n_const.PORT_STATUS_ACTIVE)
         self._record_port_status_changed_helper(n_const.PORT_STATUS_ACTIVE,
                                                 sql_attr.NO_VALUE,
                                                 port)
 
     def test_non_compute_instances_no_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        port = models_v2.Port(id='port-uuid', device_id=device_id,
-                              device_owner=n_const.DEVICE_OWNER_DHCP,
-                              status=n_const.PORT_STATUS_ACTIVE)
+        port = port_obj.Port(self.ctx, id=self.port_uuid, device_id=device_id,
+                             device_owner=n_const.DEVICE_OWNER_DHCP,
+                             status=n_const.PORT_STATUS_ACTIVE)
         self._record_port_status_changed_helper(n_const.PORT_STATUS_ACTIVE,
                                                 sql_attr.NO_VALUE,
                                                 port)
@@ -116,9 +117,9 @@ class TestNovaNotify(base.BaseTestCase):
 
         elif (previous_port_status in [sql_attr.NO_VALUE,
                                        n_const.PORT_STATUS_DOWN,
-                                       n_const.PORT_STATUS_BUILD]
-              and current_port_status in [n_const.PORT_STATUS_ACTIVE,
-                                          n_const.PORT_STATUS_ERROR]):
+                                       n_const.PORT_STATUS_BUILD] and
+              current_port_status in [n_const.PORT_STATUS_ACTIVE,
+                                      n_const.PORT_STATUS_ERROR]):
             event_name = nova.VIF_PLUGGED
 
         else:
@@ -131,7 +132,7 @@ class TestNovaNotify(base.BaseTestCase):
                                                       None)
 
         event = {'server_uuid': port.device_id, 'status': status,
-                 'name': event_name, 'tag': 'port-uuid'}
+                 'name': event_name, 'tag': self.port_uuid}
         self.assertEqual(event, port._notify_event)
 
     def test_update_fixed_ip_changed(self):
@@ -142,7 +143,8 @@ class TestNovaNotify(base.BaseTestCase):
                          'device_id': device_id}}
 
         expected_event = {'server_uuid': device_id,
-                          'name': 'network-changed'}
+                          'name': 'network-changed',
+                          'tag': returned_obj['port']['id']}
         event = self.nova_notifier.create_port_changed_event('update_port',
                                                              {}, returned_obj)
         self.assertEqual(event, expected_event)
@@ -153,7 +155,8 @@ class TestNovaNotify(base.BaseTestCase):
                         {'port_id': u'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
 
         expected_event = {'server_uuid': device_id,
-                          'name': 'network-changed'}
+                          'name': 'network-changed',
+                          'tag': returned_obj['floatingip']['port_id']}
         event = self.nova_notifier.create_port_changed_event(
             'create_floatingip', {}, returned_obj)
         self.assertEqual(event, expected_event)
@@ -164,7 +167,7 @@ class TestNovaNotify(base.BaseTestCase):
 
         event = self.nova_notifier.create_port_changed_event(
             'create_floatingip', {}, returned_obj)
-        self.assertFalse(event, None)
+        self.assertIsNone(event)
 
     def test_delete_floatingip_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
@@ -172,16 +175,16 @@ class TestNovaNotify(base.BaseTestCase):
                         {'port_id': u'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
 
         expected_event = {'server_uuid': device_id,
-                          'name': 'network-changed'}
+                          'name': 'network-changed',
+                          'tag': returned_obj['floatingip']['port_id']}
         event = self.nova_notifier.create_port_changed_event(
             'delete_floatingip', {}, returned_obj)
         self.assertEqual(expected_event, event)
 
     def test_delete_floatingip_deleted_port_no_notify(self):
         port_id = 'bee50827-bcee-4cc8-91c1-a27b0ce54222'
-        with mock.patch.object(
-            self.nova_notifier._plugin_ref, 'get_port',
-            side_effect=n_exc.PortNotFound(port_id=port_id)):
+        with mock.patch.object(directory.get_plugin(), 'get_port',
+                side_effect=n_exc.PortNotFound(port_id=port_id)):
             returned_obj = {'floatingip':
                             {'port_id': port_id}}
             event = self.nova_notifier.create_port_changed_event(
@@ -203,7 +206,8 @@ class TestNovaNotify(base.BaseTestCase):
         original_obj = {'port_id': None}
 
         expected_event = {'server_uuid': device_id,
-                          'name': 'network-changed'}
+                          'name': 'network-changed',
+                          'tag': returned_obj['floatingip']['port_id']}
         event = self.nova_notifier.create_port_changed_event(
             'update_floatingip', original_obj, returned_obj)
         self.assertEqual(expected_event, event)
@@ -214,7 +218,8 @@ class TestNovaNotify(base.BaseTestCase):
         original_obj = {'port_id': '5a39def4-3d3f-473d-9ff4-8e90064b9cc1'}
 
         expected_event = {'server_uuid': device_id,
-                          'name': 'network-changed'}
+                          'name': 'network-changed',
+                          'tag': original_obj['port_id']}
 
         event = self.nova_notifier.create_port_changed_event(
             'update_floatingip', original_obj, returned_obj)
@@ -227,65 +232,58 @@ class TestNovaNotify(base.BaseTestCase):
                                'send_events') as send_events:
             self.nova_notifier.send_network_change('update_floatingip',
                                                    {}, {})
-            self.assertFalse(send_events.called, False)
+            self.assertFalse(send_events.called)
 
-    def test_nova_send_events_returns_bad_list(self):
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.return_value = 'i am a string!'
-            self.nova_notifier.send_events([])
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_returns_bad_list(self, mock_client):
+        mock_client.server_external_events.create.return_value = (
+            'i am a string!')
+        self.nova_notifier.send_events([])
 
-    def test_nova_send_event_rasies_404(self):
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.side_effect = nova_exceptions.NotFound
-            self.nova_notifier.send_events([])
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_event_rasies_404(self, mock_client):
+        mock_client.server_external_events.create.return_value = (
+            nova_exceptions.NotFound)
+        self.nova_notifier.send_events([])
 
-    def test_nova_send_events_raises(self):
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.side_effect = Exception
-            self.nova_notifier.send_events([])
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_raises(self, mock_client):
+        mock_client.server_external_events.create.return_value = Exception
+        self.nova_notifier.send_events([])
 
-    def test_nova_send_events_returns_non_200(self):
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_returns_non_200(self, mock_client):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.return_value = [{'code': 404,
-                                            'name': 'network-changed',
-                                            'server_uuid': device_id}]
-            self.nova_notifier.send_events(
-                [{'name': 'network-changed', 'server_uuid': device_id}])
+        mock_client.server_external_events.create.return_value = [
+            {'code': 404,
+             'name': 'network-changed',
+             'server_uuid': device_id}]
+        self.nova_notifier.send_events([{'name': 'network-changed',
+                                         'server_uuid': device_id}])
 
-    def test_nova_send_events_return_200(self):
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_return_200(self, mock_client):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.return_value = [{'code': 200,
-                                            'name': 'network-changed',
-                                            'server_uuid': device_id}]
-            self.nova_notifier.send_events(
-                [{'name': 'network-changed', 'server_uuid': device_id}])
+        mock_client.server_external_events.create.return_value = [
+            {'code': 200,
+             'name': 'network-changed',
+             'server_uuid': device_id}]
+        self.nova_notifier.send_events([{'name': 'network-changed',
+                                         'server_uuid': device_id}])
 
-    def test_nova_send_events_multiple(self):
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_multiple(self, mock_create):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        with mock.patch.object(
-            self.nova_notifier.nclient.server_external_events,
-                'create') as nclient_create:
-            nclient_create.return_value = [{'code': 200,
-                                            'name': 'network-changed',
-                                            'server_uuid': device_id},
-                                           {'code': 200,
-                                            'name': 'network-changed',
-                                            'server_uuid': device_id}]
-            self.nova_notifier.send_events([
-                {'name': 'network-changed', 'server_uuid': device_id},
-                {'name': 'network-changed', 'server_uuid': device_id}])
+        mock_create.server_external_events.create.return_value = [
+            {'code': 200,
+             'name': 'network-changed',
+             'server_uuid': device_id},
+            {'code': 200,
+             'name': 'network-changed',
+             'server_uuid': device_id}]
+        self.nova_notifier.send_events([
+            {'name': 'network-changed', 'server_uuid': device_id},
+            {'name': 'network-changed', 'server_uuid': device_id}])
 
     def test_reassociate_floatingip_without_disassociate_event(self):
         returned_obj = {'floatingip':
@@ -295,7 +293,7 @@ class TestNovaNotify(base.BaseTestCase):
         self.nova_notifier.send_network_change(
             'update_floatingip', original_obj, returned_obj)
         self.assertEqual(
-            2, len(self.nova_notifier.batch_notifier.pending_events))
+            2, len(self.nova_notifier.batch_notifier._pending_events.queue))
 
         returned_obj_non = {'floatingip': {'port_id': None}}
         event_dis = self.nova_notifier.create_port_changed_event(
@@ -303,9 +301,10 @@ class TestNovaNotify(base.BaseTestCase):
         event_assoc = self.nova_notifier.create_port_changed_event(
             'update_floatingip', original_obj, returned_obj)
         self.assertEqual(
-            self.nova_notifier.batch_notifier.pending_events[0], event_dis)
+            self.nova_notifier.batch_notifier._pending_events.get(), event_dis)
         self.assertEqual(
-            self.nova_notifier.batch_notifier.pending_events[1], event_assoc)
+            self.nova_notifier.batch_notifier._pending_events.get(),
+            event_assoc)
 
     def test_delete_port_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
@@ -322,46 +321,45 @@ class TestNovaNotify(base.BaseTestCase):
                                                              {}, returned_obj)
         self.assertEqual(expected_event, event)
 
-    def test_delete_baremetal_port_notify(self):
-        device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        port_id = 'bee50827-bcee-4cc8-91c1-a27b0ce54222'
-        returned_obj = {'port':
-                        {'device_owner': DEVICE_OWNER_BAREMETAL,
-                         'id': port_id,
-                         'device_id': device_id}}
-
-        expected_event = {'server_uuid': device_id,
-                          'name': nova.VIF_DELETED,
-                          'tag': port_id}
-        event = self.nova_notifier.create_port_changed_event('delete_port',
-                                                             {}, returned_obj)
-        self.assertEqual(expected_event, event)
-
     @mock.patch('novaclient.client.Client')
     def test_endpoint_types(self, mock_client):
-        nova.Notifier()
+        device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
+        batched_events = [{'code': 200,
+                           'name': 'network-changed',
+                           'server_uuid': device_id}]
+        response = [{'name': 'network-changed',
+                     'server_uuid': device_id}]
+        mock_client.server_external_events.create.return_value = (
+            batched_events)
+        self.nova_notifier.send_events(response)
         mock_client.assert_called_once_with(
-                                        nova.NOVA_API_VERSION,
-                                        session=mock.ANY,
-                                        region_name=cfg.CONF.nova.region_name,
-                                        endpoint_type='public',
-                                        extensions=mock.ANY)
+            api_versions.APIVersion(nova.NOVA_API_VERSION),
+            connect_retries=3,
+            session=mock.ANY,
+            region_name=cfg.CONF.nova.region_name,
+            endpoint_type='public',
+            extensions=mock.ANY,
+            global_request_id=mock.ANY)
 
         mock_client.reset_mock()
         cfg.CONF.set_override('endpoint_type', 'internal', 'nova')
-        nova.Notifier()
+        mock_client.server_external_events.create.return_value = (
+            batched_events)
+        self.nova_notifier.send_events(response)
         mock_client.assert_called_once_with(
-                                        nova.NOVA_API_VERSION,
-                                        session=mock.ANY,
-                                        region_name=cfg.CONF.nova.region_name,
-                                        endpoint_type='internal',
-                                        extensions=mock.ANY)
+            api_versions.APIVersion(nova.NOVA_API_VERSION),
+            connect_retries=3,
+            session=mock.ANY,
+            region_name=cfg.CONF.nova.region_name,
+            endpoint_type='internal',
+            extensions=mock.ANY,
+            global_request_id=mock.ANY)
 
     def test_notify_port_active_direct(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
         port_id = 'bee50827-bcee-4cc8-91c1-a27b0ce54222'
-        port = models_v2.Port(id=port_id, device_id=device_id,
-                              device_owner=DEVICE_OWNER_COMPUTE)
+        port = port_obj.Port(self.ctx, id=port_id, device_id=device_id,
+                             device_owner=DEVICE_OWNER_COMPUTE)
         expected_event = {'server_uuid': device_id,
                           'name': nova.VIF_PLUGGED,
                           'status': 'completed',
@@ -369,6 +367,7 @@ class TestNovaNotify(base.BaseTestCase):
         self.nova_notifier.notify_port_active_direct(port)
 
         self.assertEqual(
-            1, len(self.nova_notifier.batch_notifier.pending_events))
-        self.assertEqual(expected_event,
-                         self.nova_notifier.batch_notifier.pending_events[0])
+            1, len(self.nova_notifier.batch_notifier._pending_events.queue))
+        self.assertEqual(
+            expected_event,
+            self.nova_notifier.batch_notifier._pending_events.get())
