@@ -14,7 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
 import neutron.plugins.ml2.drivers.openvswitch.agent.common.constants \
     as ovs_const
@@ -32,11 +32,11 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
 
     def setUp(self):
         conn_patcher = mock.patch(
-            'neutron.agent.ovsdb.native.connection.Connection.start')
+            'neutron.agent.ovsdb.impl_idl._connection')
         conn_patcher.start()
         super(OVSTunnelBridgeTest, self).setUp()
         # NOTE(ivasilevskaya) The behaviour of oslotest.base.addCleanup()
-        # according to https://review.openstack.org/#/c/119201/4 guarantees
+        # according to https://review.opendev.org/#/c/119201/4 guarantees
         # that all started mocks will be stopped even without direct call to
         # patcher.stop().
         # If any individual mocks should be stopped by other than default
@@ -59,41 +59,48 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=2)],
                 match=ofpp.OFPMatch(in_port=patch_int_ofport),
-                priority=1, table_id=0)),
+                priority=1, table_id=0),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=0)),
+                priority=0, table_id=0),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=20)],
                 match=ofpp.OFPMatch(
                     eth_dst=('00:00:00:00:00:00', '01:00:00:00:00:00')),
                 priority=0,
-                table_id=2)),
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
                 match=ofpp.OFPMatch(
                     eth_dst=('01:00:00:00:00:00', '01:00:00:00:00:00')),
                 priority=0,
-                table_id=2)),
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=3)),
+                priority=0, table_id=3),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=4)),
+                priority=0, table_id=4),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=6)),
+                priority=0, table_id=6),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -130,19 +137,22 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(),
                 priority=1,
-                table_id=10)),
+                table_id=10),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=20)),
+                table_id=20),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=22))
+                table_id=22),
+                           active_bundle=None)
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -157,12 +167,14 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=2)],
                 match=ofpp.OFPMatch(in_port=patch_int_ofport),
-                priority=1, table_id=0)),
+                priority=1, table_id=0),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=0)),
+                priority=0, table_id=0),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=21)],
@@ -170,36 +182,42 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                     eth_dst='ff:ff:ff:ff:ff:ff',
                     eth_type=self.ether_types.ETH_TYPE_ARP),
                 priority=1,
-                table_id=2)),
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=20)],
                 match=ofpp.OFPMatch(
                     eth_dst=('00:00:00:00:00:00', '01:00:00:00:00:00')),
                 priority=0,
-                table_id=2)),
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
                 match=ofpp.OFPMatch(
                     eth_dst=('01:00:00:00:00:00', '01:00:00:00:00:00')),
                 priority=0,
-                table_id=2)),
+                table_id=2),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=3)),
+                priority=0, table_id=3),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=4)),
+                priority=0, table_id=4),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
-                priority=0, table_id=6)),
+                priority=0, table_id=6),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -236,25 +254,29 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(),
                 priority=1,
-                table_id=10)),
+                table_id=10),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=20)),
+                table_id=20),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[ofpp.OFPInstructionGotoTable(table_id=22)],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=21)),
+                table_id=21),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
                 match=ofpp.OFPMatch(),
                 priority=0,
-                table_id=22))
+                table_id=22),
+                           active_bundle=None)
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -280,7 +302,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(tunnel_id=segmentation_id),
                 priority=1,
-                table_id=4)),
+                table_id=4),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -291,7 +314,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                                    segmentation_id=segmentation_id)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(
+            call.uninstall_flows(
                 table_id=4,
                 match=ofpp.OFPMatch(tunnel_id=segmentation_id)),
         ]
@@ -316,7 +339,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(vlan_vid=vlan | ofp.OFPVID_PRESENT),
                 priority=1,
-                table_id=22)),
+                table_id=22),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -325,7 +349,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.delete_flood_to_tun(vlan=vlan)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(table_id=22,
+            call.uninstall_flows(table_id=22,
                 match=ofpp.OFPMatch(vlan_vid=vlan | ofp.OFPVID_PRESENT)),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
@@ -353,7 +377,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 match=ofpp.OFPMatch(
                     eth_dst=mac, vlan_vid=vlan | ofp.OFPVID_PRESENT),
                 priority=2,
-                table_id=20)),
+                table_id=20),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -363,7 +388,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.delete_unicast_to_tun(vlan=vlan, mac=mac)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(table_id=20,
+            call.uninstall_flows(table_id=20,
                 match=ofpp.OFPMatch(
                     eth_dst=mac, vlan_vid=vlan | ofp.OFPVID_PRESENT)),
         ]
@@ -375,7 +400,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.delete_unicast_to_tun(vlan=vlan, mac=mac)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(table_id=20,
+            call.uninstall_flows(table_id=20,
                 match=ofpp.OFPMatch(vlan_vid=vlan | ofp.OFPVID_PRESENT)),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
@@ -414,7 +439,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                     arp_tpa=ip,
                     vlan_vid=vlan | ofp.OFPVID_PRESENT),
                 priority=1,
-                table_id=21)),
+                table_id=21),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -424,7 +450,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.delete_arp_responder(vlan=vlan, ip=ip)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(
+            call.uninstall_flows(
                 match=ofpp.OFPMatch(
                     eth_type=self.ether_types.ETH_TYPE_ARP,
                     arp_tpa=ip,
@@ -439,7 +465,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.delete_arp_responder(vlan=vlan, ip=ip)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(
+            call.uninstall_flows(
                 match=ofpp.OFPMatch(
                     eth_type=self.ether_types.ETH_TYPE_ARP,
                     vlan_vid=vlan | ofp.OFPVID_PRESENT),
@@ -460,7 +486,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(in_port=port),
                 priority=1,
-                table_id=0)),
+                table_id=0),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -469,7 +496,7 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.cleanup_tunnel_port(port=port)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(in_port=port),
+            call.uninstall_flows(in_port=port),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -488,7 +515,8 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                 ],
                 match=ofpp.OFPMatch(eth_src=mac),
                 priority=1,
-                table_id=9)),
+                table_id=9),
+                           active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
 
@@ -497,6 +525,6 @@ class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
         self.br.remove_dvr_mac_tun(mac=mac)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
-            call.delete_flows(eth_src=mac, table_id=9),
+            call.uninstall_flows(eth_src=mac, table_id=9),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
