@@ -229,6 +229,8 @@ class TunnelTest(object):
             mock.call.port_exists('int-%s' % self.MAP_TUN_BRIDGE),
             mock.call.add_patch_port('int-%s' % self.MAP_TUN_BRIDGE,
                                      constants.NONEXISTENT_PEER),
+            mock.call.set_igmp_snooping_flood('int-%s' % self.MAP_TUN_BRIDGE,
+                                              igmp_snooping),
         ]
 
         self.mock_int_bridge_expected += [
@@ -258,6 +260,7 @@ class TunnelTest(object):
         self.mock_int_bridge_expected += [
             mock.call.port_exists('patch-tun'),
             mock.call.add_patch_port('patch-tun', 'patch-int'),
+            mock.call.set_igmp_snooping_flood('patch-tun', igmp_snooping),
         ]
         self.mock_int_bridge_expected += [
             mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
@@ -710,7 +713,9 @@ class TunnelTestUseVethInterco(TunnelTest):
         self.mock_int_bridge_expected += [
             mock.call.db_get_val('Interface', 'int-%s' % self.MAP_TUN_BRIDGE,
                                  'type', log_errors=False),
-            mock.call.add_port('int-%s' % self.MAP_TUN_BRIDGE)
+            mock.call.add_port('int-%s' % self.MAP_TUN_BRIDGE),
+            mock.call.set_igmp_snooping_flood('int-%s' % self.MAP_TUN_BRIDGE,
+                                              igmp_snooping),
         ]
 
         self.mock_int_bridge_expected += [
@@ -733,7 +738,8 @@ class TunnelTestUseVethInterco(TunnelTest):
         ]
         self.mock_int_bridge_expected += [
             mock.call.port_exists('patch-tun'),
-            mock.call.add_patch_port('patch-tun', 'patch-int')
+            mock.call.add_patch_port('patch-tun', 'patch-int'),
+            mock.call.set_igmp_snooping_flood('patch-tun', igmp_snooping),
         ]
         self.mock_int_bridge_expected += [
             mock.call.get_vif_ports((ovs_lib.INVALID_OFPORT,
