@@ -13,20 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import datetime
+from operator import attrgetter
 import random
+from unittest import mock
 
-import mock
 from neutron_lib import constants
+from neutron_lib import context
+from neutron_lib.exceptions import dhcpagentscheduler as das_exc
 from oslo_config import cfg
 from oslo_utils import importutils
+from oslo_utils import uuidutils
 import testscenarios
 
-from neutron import context
 from neutron.db import agentschedulers_db as sched_db
-from neutron.db import common_db_mixin
-from neutron.db import models_v2
-from neutron.db.network_dhcp_agent_binding import models as ndab_model
-from neutron.extensions import dhcpagentscheduler
+from neutron.objects import agent
+from neutron.objects import network as network_obj
 from neutron.scheduler import dhcp_agent_scheduler
 from neutron.services.segments import db as segments_service_db
 from neutron.tests.common import helpers
@@ -41,12 +43,14 @@ HOST_D = 'host-d'
 
 
 class TestDhcpSchedulerBaseTestCase(testlib_api.SqlTestCase):
+    CORE_PLUGIN = 'neutron.db.db_base_plugin_v2.NeutronDbPluginV2'
 
     def setUp(self):
         super(TestDhcpSchedulerBaseTestCase, self).setUp()
+        self.setup_coreplugin(self.CORE_PLUGIN)
         self.ctx = context.get_admin_context()
-        self.network = {'id': 'foo_network_id'}
-        self.network_id = 'foo_network_id'
+        self.network = {'id': uuidutils.generate_uuid()}
+        self.network_id = self.network['id']
         self._save_networks([self.network_id])
 
     def _create_and_set_agents_down(self, hosts, down_agent_count=0,
@@ -64,17 +68,16 @@ class TestDhcpSchedulerBaseTestCase(testlib_api.SqlTestCase):
 
     def _save_networks(self, networks):
         for network_id in networks:
-            with self.ctx.session.begin(subtransactions=True):
-                self.ctx.session.add(models_v2.Network(id=network_id))
+            network_obj.Network(self.ctx, id=network_id).create()
 
     def _test_schedule_bind_network(self, agents, network_id):
+        cfg.CONF.set_override('dhcp_agents_per_network', len(agents))
         scheduler = dhcp_agent_scheduler.ChanceScheduler()
         scheduler.resource_filter.bind(self.ctx, agents, network_id)
-        results = self.ctx.session.query(
-            ndab_model.NetworkDhcpAgentBinding).filter_by(
-            network_id=network_id).all()
-        self.assertEqual(len(agents), len(results))
-        for result in results:
+        binding_objs = network_obj.NetworkDhcpAgentBinding.get_objects(
+            self.ctx, network_id=network_id)
+        self.assertEqual(len(agents), len(binding_objs))
+        for result in binding_objs:
             self.assertEqual(network_id, result.network_id)
 
 
@@ -91,7 +94,7 @@ class TestDhcpScheduler(TestDhcpSchedulerBaseTestCase):
     def test_schedule_bind_network_multi_agent_fail_one(self):
         agents = self._create_and_set_agents_down(['host-a'])
         self._test_schedule_bind_network(agents, self.network_id)
-        with mock.patch.object(dhcp_agent_scheduler.LOG, 'info') as fake_log:
+        with mock.patch.object(dhcp_agent_scheduler.LOG, 'debug') as fake_log:
             self._test_schedule_bind_network(agents, self.network_id)
             self.assertEqual(1, fake_log.call_count)
 
@@ -112,7 +115,7 @@ class TestDhcpScheduler(TestDhcpSchedulerBaseTestCase):
         plugin = mock.Mock()
         plugin.get_subnets.return_value = [{"network_id": self.network_id,
                                             "enable_dhcp": True}]
-        plugin.get_agents_db.return_value = dead_agent + alive_agent
+        plugin.get_agent_objects.return_value = dead_agent + alive_agent
         plugin.filter_hosts_with_network_access.side_effect = (
             lambda context, network_id, hosts: hosts)
         if active_hosts_only:
@@ -133,12 +136,10 @@ class TestDhcpScheduler(TestDhcpSchedulerBaseTestCase):
         self._test_reschedule_vs_network_on_dead_agent(False)
 
     def _get_agent_binding_from_db(self, agent):
-        return self.ctx.session.query(
-            ndab_model.NetworkDhcpAgentBinding
-        ).filter_by(dhcp_agent_id=agent[0].id).all()
+        return network_obj.NetworkDhcpAgentBinding.get_objects(
+            self.ctx, dhcp_agent_id=agent[0].id)
 
-    def _test_auto_reschedule_vs_network_on_dead_agent(self,
-                                                       active_hosts_only):
+    def test_auto_reschedule_vs_network_on_dead_agent(self):
         dead_agent, alive_agent, scheduler = (
             self._test_get_agents_and_scheduler_for_dead_agent())
         plugin = mock.Mock()
@@ -146,10 +147,7 @@ class TestDhcpScheduler(TestDhcpSchedulerBaseTestCase):
                                             "enable_dhcp": True,
                                             "segment_id": None}]
         plugin.get_network.return_value = self.network
-        if active_hosts_only:
-            plugin.get_dhcp_agents_hosting_networks.return_value = []
-        else:
-            plugin.get_dhcp_agents_hosting_networks.return_value = dead_agent
+        plugin.get_dhcp_agents_hosting_networks.return_value = dead_agent
         network_assigned_to_dead_agent = (
             self._get_agent_binding_from_db(dead_agent))
         self.assertEqual(1, len(network_assigned_to_dead_agent))
@@ -161,16 +159,9 @@ class TestDhcpScheduler(TestDhcpSchedulerBaseTestCase):
         network_assigned_to_alive_agent = (
             self._get_agent_binding_from_db(alive_agent))
         self.assertEqual(1, len(network_assigned_to_dead_agent))
-        if active_hosts_only:
-            self.assertEqual(1, len(network_assigned_to_alive_agent))
-        else:
-            self.assertEqual(0, len(network_assigned_to_alive_agent))
-
-    def test_network_auto_rescheduled_when_db_returns_active_hosts(self):
-        self._test_auto_reschedule_vs_network_on_dead_agent(True)
-
-    def test_network_not_auto_rescheduled_when_db_returns_all_hosts(self):
-        self._test_auto_reschedule_vs_network_on_dead_agent(False)
+        # network won't be scheduled to new agent unless removed from
+        # dead agent
+        self.assertEqual(0, len(network_assigned_to_alive_agent))
 
 
 class TestAutoScheduleNetworks(TestDhcpSchedulerBaseTestCase):
@@ -281,20 +272,21 @@ class TestAutoScheduleNetworks(TestDhcpSchedulerBaseTestCase):
         expected_hosted_agents = (1 if expected_result and
                                   self.valid_host else 0)
         if (self.az_hints and
-            agents[0]['availability_zone'] not in self.az_hints):
+                agents[0]['availability_zone'] not in self.az_hints):
             expected_hosted_agents = 0
         host = "host-a" if self.valid_host else "host-b"
         observed_ret_value = scheduler.auto_schedule_networks(
             plugin, self.ctx, host)
         self.assertEqual(expected_result, observed_ret_value)
-        hosted_agents = self.ctx.session.query(
-            ndab_model.NetworkDhcpAgentBinding).all()
-        self.assertEqual(expected_hosted_agents, len(hosted_agents))
+        count_hosted_agents = network_obj.NetworkDhcpAgentBinding.count(
+            self.ctx)
+        self.assertEqual(expected_hosted_agents, count_hosted_agents)
 
 
 class TestAutoScheduleSegments(test_plugin.Ml2PluginV2TestCase,
                                TestDhcpSchedulerBaseTestCase):
     """Unit test scenarios for ChanceScheduler"""
+    CORE_PLUGIN = 'neutron.plugins.ml2.plugin.Ml2Plugin'
 
     def setUp(self):
         super(TestAutoScheduleSegments, self).setUp()
@@ -302,6 +294,9 @@ class TestAutoScheduleSegments(test_plugin.Ml2PluginV2TestCase,
         self.segments_plugin = importutils.import_object(
             'neutron.services.segments.plugin.Plugin')
         self.ctx = context.get_admin_context()
+        # Remove MissingAuthPlugin exception from logs
+        mock.patch(
+            'neutron.notifiers.batch_notifier.BatchNotifier._notify').start()
 
     def _create_network(self):
         net = self.plugin.create_network(
@@ -326,7 +321,7 @@ class TestAutoScheduleSegments(test_plugin.Ml2PluginV2TestCase,
         subnet = self.plugin.create_subnet(
             self.ctx,
             {'subnet': {'name': 'name',
-                        'ip_version': 4,
+                        'ip_version': constants.IP_VERSION_4,
                         'network_id': network_id,
                         'cidr': cidr,
                         'gateway_ip': constants.ATTR_NOT_SPECIFIED,
@@ -400,13 +395,34 @@ class TestAutoScheduleSegments(test_plugin.Ml2PluginV2TestCase,
 
 
 class TestNetworksFailover(TestDhcpSchedulerBaseTestCase,
-                           sched_db.DhcpAgentSchedulerDbMixin,
-                           common_db_mixin.CommonDbMixin):
+                           sched_db.DhcpAgentSchedulerDbMixin):
+    def test_auto_schedule_network_excess_agents(self):
+        plugin = mock.MagicMock()
+        plugin.get_subnets.return_value = (
+            [{"network_id": self.network_id, "enable_dhcp": True}])
+        plugin.get_network.return_value = {'availability_zone_hints': ['nova']}
+        scheduler = dhcp_agent_scheduler.ChanceScheduler()
+        dhcpfilter = 'neutron.scheduler.dhcp_agent_scheduler.DhcpFilter'
+        self._create_and_set_agents_down(['host-a', 'host-b'])
+        expected_hosted_agents = 1
+        binding_index = 1
+        scheduler.auto_schedule_networks(plugin, self.ctx, 'host-a')
+        with mock.patch(
+                dhcpfilter + '.get_vacant_network_dhcp_agent_binding_index',
+                context=self.ctx, network_id=self.network_id) as ndab:
+            ndab.return_value = binding_index
+            scheduler.auto_schedule_networks(plugin, self.ctx, 'host-b')
+            self.assertTrue(ndab.called)
+        num_hosted_agents = network_obj.NetworkDhcpAgentBinding.count(
+            self.ctx, network_id=self.network_id)
+        self.assertEqual(expected_hosted_agents, num_hosted_agents)
+
     def test_reschedule_network_from_down_agent(self):
+        net_id = uuidutils.generate_uuid()
         agents = self._create_and_set_agents_down(['host-a', 'host-b'], 1)
         self._test_schedule_bind_network([agents[0]], self.network_id)
-        self._save_networks(["foo-network-2"])
-        self._test_schedule_bind_network([agents[1]], "foo-network-2")
+        self._save_networks([net_id])
+        self._test_schedule_bind_network([agents[1]], net_id)
         with mock.patch.object(self, 'remove_network_from_dhcp_agent') as rn,\
                 mock.patch.object(self,
                                   'schedule_network',
@@ -450,39 +466,85 @@ class TestNetworksFailover(TestDhcpSchedulerBaseTestCase,
 
     def test_reschedule_network_from_down_agent_concurrent_removal(self):
         self._test_failed_rescheduling(
-            rn_side_effect=dhcpagentscheduler.NetworkNotHostedByDhcpAgent(
+            rn_side_effect=das_exc.NetworkNotHostedByDhcpAgent(
                 network_id='foo', agent_id='bar'))
 
+    def _create_test_networks(self, num_net=0):
+        networks = [network_obj.Network(
+                       self.ctx,
+                       id=uuidutils.generate_uuid(),
+                       name='network-%s' % (i))
+                    for i in range(num_net)]
+        for net in networks:
+            net.create()
+        return [net.id for net in networks]
+
+    def _create_dhcp_agents(self):
+        timestamp = datetime.datetime.now()
+        dhcp_agent_ids = [uuidutils.generate_uuid() for x in range(2)]
+        dhcp_agent_1 = agent.Agent(self.ctx, id=dhcp_agent_ids[0],
+                                   agent_type='DHCP Agent',
+                                   topic='fake_topic',
+                                   host='fake_host',
+                                   binary='fake_binary',
+                                   created_at=timestamp,
+                                   started_at=timestamp,
+                                   heartbeat_timestamp=timestamp,
+                                   configurations={},
+                                   load=0)
+        dhcp_agent_1.create()
+
+        dhcp_agent_2 = agent.Agent(self.ctx, id=dhcp_agent_ids[1],
+                                   agent_type='DHCP Agent',
+                                   topic='fake_topic',
+                                   host='fake_host_1',
+                                   binary='fake_binary',
+                                   created_at=timestamp,
+                                   started_at=timestamp,
+                                   heartbeat_timestamp=timestamp,
+                                   configurations={},
+                                   load=0)
+        dhcp_agent_2.create()
+        return [dhcp_agent_1.id, dhcp_agent_2.id]
+
     def test_filter_bindings(self):
-        bindings = [
-            ndab_model.NetworkDhcpAgentBinding(network_id='foo1',
-                                               dhcp_agent={'id': 'id1'}),
-            ndab_model.NetworkDhcpAgentBinding(network_id='foo2',
-                                               dhcp_agent={'id': 'id1'}),
-            ndab_model.NetworkDhcpAgentBinding(network_id='foo3',
-                                               dhcp_agent={'id': 'id2'}),
-            ndab_model.NetworkDhcpAgentBinding(network_id='foo4',
-                                               dhcp_agent={'id': 'id2'})]
+        self.ctx = context.get_admin_context()
+        dhcp_agt_ids = self._create_dhcp_agents()
+        network_ids = sorted(self._create_test_networks(num_net=4))
+        ndab_obj1 = network_obj.NetworkDhcpAgentBinding(self.ctx,
+            network_id=network_ids[0], dhcp_agent_id=dhcp_agt_ids[0])
+        ndab_obj1.create()
+        ndab_obj2 = network_obj.NetworkDhcpAgentBinding(self.ctx,
+            network_id=network_ids[1], dhcp_agent_id=dhcp_agt_ids[0])
+        ndab_obj2.create()
+        ndab_obj3 = network_obj.NetworkDhcpAgentBinding(self.ctx,
+            network_id=network_ids[2], dhcp_agent_id=dhcp_agt_ids[1])
+        ndab_obj3.create()
+        ndab_obj4 = network_obj.NetworkDhcpAgentBinding(self.ctx,
+            network_id=network_ids[3], dhcp_agent_id=dhcp_agt_ids[1])
+        ndab_obj4.create()
+        bindings_objs = sorted(network_obj.NetworkDhcpAgentBinding.get_objects(
+            self.ctx), key=attrgetter('network_id'))
+
         with mock.patch.object(self, 'agent_starting_up',
                                side_effect=[True, False]):
-            res = [b for b in self._filter_bindings(None, bindings)]
+            res = [b for b in self._filter_bindings(None, bindings_objs)]
             # once per each agent id1 and id2
             self.assertEqual(2, len(res))
             res_ids = [b.network_id for b in res]
-            self.assertIn('foo3', res_ids)
-            self.assertIn('foo4', res_ids)
+            self.assertIn(network_ids[2], res_ids)
+            self.assertIn(network_ids[3], res_ids)
 
     def test_reschedule_network_from_down_agent_failed_on_unexpected(self):
         agents = self._create_and_set_agents_down(['host-a'], 1)
         self._test_schedule_bind_network([agents[0]], self.network_id)
-        with mock.patch.object(
-            self, '_filter_bindings',
-            side_effect=Exception()):
+        with mock.patch.object(self, '_filter_bindings',
+                               side_effect=Exception()):
             # just make sure that no exception is raised
             self.remove_networks_from_down_agents()
 
     def test_reschedule_network_catches_exceptions_on_fetching_bindings(self):
-        with mock.patch('neutron.context.get_admin_context') as get_ctx:
+        with mock.patch('neutron_lib.context.get_admin_context') as get_ctx:
             mock_ctx = mock.Mock()
             get_ctx.return_value = mock_ctx
             mock_ctx.session.query.side_effect = Exception()
@@ -492,8 +554,7 @@ class TestNetworksFailover(TestDhcpSchedulerBaseTestCase,
     def test_reschedule_doesnt_occur_if_no_agents(self):
         agents = self._create_and_set_agents_down(['host-a', 'host-b'], 2)
         self._test_schedule_bind_network([agents[0]], self.network_id)
-        with mock.patch.object(
-            self, 'remove_network_from_dhcp_agent') as rn:
+        with mock.patch.object(self, 'remove_network_from_dhcp_agent') as rn:
             self.remove_networks_from_down_agents()
             self.assertFalse(rn.called)
 
@@ -588,6 +649,15 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
         self.assertEqual('host-c', agent2[0]['host'])
         self.assertEqual('host-d', agent3[0]['host'])
 
+    def _get_network_with_candidate_hosts(self, net_id, seg_id):
+        # expire the session so that the segment is fully reloaded on fetch,
+        # including its new host mapping
+        self.ctx.session.expire_all()
+        net = self.plugin.get_network(self.ctx, net_id)
+        seg = self.segments_plugin.get_segment(self.ctx, seg_id)
+        net['candidate_hosts'] = seg['hosts']
+        return net
+
     def test_schedule_segment_one_hostable_agent(self):
         net_id = self._create_network()
         seg_id = self._create_segment(net_id)
@@ -595,9 +665,7 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
         helpers.register_dhcp_agent(HOST_D)
         segments_service_db.update_segment_host_mapping(
             self.ctx, HOST_C, {seg_id})
-        net = self.plugin.get_network(self.ctx, net_id)
-        seg = self.segments_plugin.get_segment(self.ctx, seg_id)
-        net['candidate_hosts'] = seg['hosts']
+        net = self._get_network_with_candidate_hosts(net_id, seg_id)
         agents = self.plugin.network_scheduler.schedule(
             self.plugin, self.ctx, net)
         self.assertEqual(1, len(agents))
@@ -612,9 +680,7 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
             self.ctx, HOST_C, {seg_id})
         segments_service_db.update_segment_host_mapping(
             self.ctx, HOST_D, {seg_id})
-        net = self.plugin.get_network(self.ctx, net_id)
-        seg = self.segments_plugin.get_segment(self.ctx, seg_id)
-        net['candidate_hosts'] = seg['hosts']
+        net = self._get_network_with_candidate_hosts(net_id, seg_id)
         agents = self.plugin.network_scheduler.schedule(
             self.plugin, self.ctx, net)
         self.assertEqual(1, len(agents))
@@ -642,9 +708,7 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
             self.ctx, HOST_C, {seg_id})
         segments_service_db.update_segment_host_mapping(
             self.ctx, HOST_D, {seg_id})
-        net = self.plugin.get_network(self.ctx, net_id)
-        seg = self.segments_plugin.get_segment(self.ctx, seg_id)
-        net['candidate_hosts'] = seg['hosts']
+        net = self._get_network_with_candidate_hosts(net_id, seg_id)
         agents = self.plugin.network_scheduler.schedule(
             self.plugin, self.ctx, net)
         self.assertEqual(2, len(agents))
@@ -659,9 +723,7 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
         helpers.register_dhcp_agent(HOST_D)
         segments_service_db.update_segment_host_mapping(
             self.ctx, HOST_C, {seg_id})
-        net = self.plugin.get_network(self.ctx, net_id)
-        seg = self.segments_plugin.get_segment(self.ctx, seg_id)
-        net['candidate_hosts'] = seg['hosts']
+        net = self._get_network_with_candidate_hosts(net_id, seg_id)
         agents = self.plugin.network_scheduler.schedule(
             self.plugin, self.ctx, net)
         self.assertEqual(1, len(agents))
@@ -671,6 +733,7 @@ class DHCPAgentWeightSchedulerTestCase(test_plugin.Ml2PluginV2TestCase):
 class TestDhcpSchedulerFilter(TestDhcpSchedulerBaseTestCase,
                               sched_db.DhcpAgentSchedulerDbMixin):
     def _test_get_dhcp_agents_hosting_networks(self, expected, **kwargs):
+        cfg.CONF.set_override('dhcp_agents_per_network', 4)
         agents = self._create_and_set_agents_down(['host-a', 'host-b'], 1)
         agents += self._create_and_set_agents_down(['host-c', 'host-d'], 1,
                                                    admin_state_up=False)
@@ -710,7 +773,7 @@ class TestDhcpSchedulerFilter(TestDhcpSchedulerBaseTestCase,
                                                     admin_state_up=False)
 
     def test_get_dhcp_agents_hosting_many_networks(self):
-        net_id = 'another-net-id'
+        net_id = uuidutils.generate_uuid()
         self._save_networks([net_id])
         networks = [net_id, self.network_id]
         self._test_get_dhcp_agents_hosting_networks({'host-a', 'host-b',
@@ -738,35 +801,38 @@ class DHCPAgentAZAwareWeightSchedulerTestCase(TestDhcpSchedulerBaseTestCase):
         cfg.CONF.set_override("dhcp_load_type", "networks")
 
     def test_az_scheduler_one_az_hints(self):
-        self._save_networks(['1111'])
+        net_id = uuidutils.generate_uuid()
+        self._save_networks([net_id])
         helpers.register_dhcp_agent('az1-host1', networks=1, az='az1')
         helpers.register_dhcp_agent('az1-host2', networks=2, az='az1')
         helpers.register_dhcp_agent('az2-host1', networks=3, az='az2')
         helpers.register_dhcp_agent('az2-host2', networks=4, az='az2')
         self.plugin.network_scheduler.schedule(self.plugin, self.ctx,
-            {'id': '1111', 'availability_zone_hints': ['az2']})
+            {'id': net_id, 'availability_zone_hints': ['az2']})
         agents = self.plugin.get_dhcp_agents_hosting_networks(self.ctx,
-                                                              ['1111'])
+                                                              [net_id])
         self.assertEqual(1, len(agents))
         self.assertEqual('az2-host1', agents[0]['host'])
 
     def test_az_scheduler_default_az_hints(self):
+        net_id = uuidutils.generate_uuid()
         cfg.CONF.set_override('default_availability_zones', ['az1'])
-        self._save_networks(['1111'])
+        self._save_networks([net_id])
         helpers.register_dhcp_agent('az1-host1', networks=1, az='az1')
         helpers.register_dhcp_agent('az1-host2', networks=2, az='az1')
         helpers.register_dhcp_agent('az2-host1', networks=3, az='az2')
         helpers.register_dhcp_agent('az2-host2', networks=4, az='az2')
         self.plugin.network_scheduler.schedule(self.plugin, self.ctx,
-            {'id': '1111', 'availability_zone_hints': []})
+            {'id': net_id, 'availability_zone_hints': []})
         agents = self.plugin.get_dhcp_agents_hosting_networks(self.ctx,
-                                                              ['1111'])
+                                                              [net_id])
         self.assertEqual(1, len(agents))
         self.assertEqual('az1-host1', agents[0]['host'])
 
     def test_az_scheduler_two_az_hints(self):
+        net_id = uuidutils.generate_uuid()
         cfg.CONF.set_override('dhcp_agents_per_network', 2)
-        self._save_networks(['1111'])
+        self._save_networks([net_id])
         helpers.register_dhcp_agent('az1-host1', networks=1, az='az1')
         helpers.register_dhcp_agent('az1-host2', networks=2, az='az1')
         helpers.register_dhcp_agent('az2-host1', networks=3, az='az2')
@@ -774,17 +840,18 @@ class DHCPAgentAZAwareWeightSchedulerTestCase(TestDhcpSchedulerBaseTestCase):
         helpers.register_dhcp_agent('az3-host1', networks=5, az='az3')
         helpers.register_dhcp_agent('az3-host2', networks=6, az='az3')
         self.plugin.network_scheduler.schedule(self.plugin, self.ctx,
-            {'id': '1111', 'availability_zone_hints': ['az1', 'az3']})
+            {'id': net_id, 'availability_zone_hints': ['az1', 'az3']})
         agents = self.plugin.get_dhcp_agents_hosting_networks(self.ctx,
-                                                              ['1111'])
+                                                              [net_id])
         self.assertEqual(2, len(agents))
         expected_hosts = set(['az1-host1', 'az3-host1'])
         hosts = set([a['host'] for a in agents])
         self.assertEqual(expected_hosts, hosts)
 
     def test_az_scheduler_two_az_hints_one_available_az(self):
+        net_id = uuidutils.generate_uuid()
         cfg.CONF.set_override('dhcp_agents_per_network', 2)
-        self._save_networks(['1111'])
+        self._save_networks([net_id])
         helpers.register_dhcp_agent('az1-host1', networks=1, az='az1')
         helpers.register_dhcp_agent('az1-host2', networks=2, az='az1')
         helpers.register_dhcp_agent('az2-host1', networks=3, alive=False,
@@ -792,26 +859,27 @@ class DHCPAgentAZAwareWeightSchedulerTestCase(TestDhcpSchedulerBaseTestCase):
         helpers.register_dhcp_agent('az2-host2', networks=4,
                                     admin_state_up=False, az='az2')
         self.plugin.network_scheduler.schedule(self.plugin, self.ctx,
-            {'id': '1111', 'availability_zone_hints': ['az1', 'az2']})
+            {'id': net_id, 'availability_zone_hints': ['az1', 'az2']})
         agents = self.plugin.get_dhcp_agents_hosting_networks(self.ctx,
-                                                              ['1111'])
+                                                              [net_id])
         self.assertEqual(2, len(agents))
         expected_hosts = set(['az1-host1', 'az1-host2'])
         hosts = set([a['host'] for a in agents])
         self.assertEqual(expected_hosts, hosts)
 
     def _test_az_scheduler_no_az_hints(self, multiple_agent=False):
+        net_id = uuidutils.generate_uuid()
         num_agent = 2 if multiple_agent else 1
         cfg.CONF.set_override('dhcp_agents_per_network', num_agent)
-        self._save_networks(['1111'])
+        self._save_networks([net_id])
         helpers.register_dhcp_agent('az1-host1', networks=2, az='az1')
         helpers.register_dhcp_agent('az1-host2', networks=3, az='az1')
         helpers.register_dhcp_agent('az2-host1', networks=2, az='az2')
         helpers.register_dhcp_agent('az2-host2', networks=1, az='az2')
         self.plugin.network_scheduler.schedule(self.plugin, self.ctx,
-            {'id': '1111', 'availability_zone_hints': []})
+            {'id': net_id, 'availability_zone_hints': []})
         agents = self.plugin.get_dhcp_agents_hosting_networks(self.ctx,
-                                                              ['1111'])
+                                                              [net_id])
         self.assertEqual(num_agent, len(agents))
         if multiple_agent:
             expected_hosts = set(['az1-host1', 'az2-host2'])
@@ -827,7 +895,7 @@ class DHCPAgentAZAwareWeightSchedulerTestCase(TestDhcpSchedulerBaseTestCase):
         self._test_az_scheduler_no_az_hints()
 
     def test_az_scheduler_select_az_with_least_weight(self):
-        self._save_networks(['1111'])
+        self._save_networks([uuidutils.generate_uuid()])
         dhcp_agents = []
         # Register 6 dhcp agents in 3 AZs, every AZ will have 2 agents.
         dhcp_agents.append(
