@@ -11,16 +11,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
-import testtools
+from unittest import mock
 
 from neutron_lib.api.definitions import constants as api_const
+from neutron_lib import constants
 from neutron_lib import context
 from neutron_lib import exceptions as n_exc
 from oslo_db import exception as db_exc
 from oslo_utils import uuidutils
+import testtools
 
-from neutron.common import exceptions as c_exc
 from neutron.services.auto_allocate import db
 from neutron.services.auto_allocate import exceptions
 from neutron.tests.unit import testlib_api
@@ -215,11 +215,12 @@ class AutoAllocateTestCase(testlib_api.SqlTestCase):
         self.mixin._core_plugin.create_network.return_value = (
             {'id': network_id})
         self.mixin._core_plugin.create_subnet.side_effect = (
-            c_exc.SubnetAllocationError(reason='disaster'))
+            n_exc.SubnetAllocationError(reason='disaster'))
         with mock.patch.object(self.mixin, "_get_supported_subnetpools") as f,\
                 mock.patch.object(self.mixin, "_cleanup") as g:
             f.return_value = (
-                [{'ip_version': 4, "id": uuidutils.generate_uuid()}])
+                [{'ip_version': constants.IP_VERSION_4,
+                  'id': uuidutils.generate_uuid()}])
             self.assertRaises(exceptions.AutoAllocationFailure,
                               self.mixin._provision_tenant_private_network,
                               self.ctx, 'foo_tenant')
