@@ -128,10 +128,8 @@ class TestQosPlugin(base.BaseQosTestCase):
 
         if has_qos_policy:
             self.port_data['port']['qos_policy_id'] = self.policy.id
-            self.policy.rules = bw_rules
         elif has_net_qos_policy:
             self.port_data['port']['qos_network_policy_id'] = self.policy.id
-            self.policy.rules = bw_rules
 
         self.port = ports_object.Port(
             self.ctxt, **self.port_data['port'])
@@ -142,8 +140,10 @@ class TestQosPlugin(base.BaseQosTestCase):
 
         with mock.patch('neutron.objects.network.NetworkSegment.get_objects',
                         return_value=[segment_mock]), \
-                mock.patch('neutron.objects.qos.policy.QosPolicy.get_object',
-                           return_value=self.policy):
+                mock.patch(
+                    'neutron.objects.qos.rule.QosMinimumBandwidthRule.'
+                    'get_objects',
+                    return_value=bw_rules):
             return qos_plugin.QoSPlugin._extend_port_resource_request(
                 port_res, self.port)
 
@@ -184,7 +184,7 @@ class TestQosPlugin(base.BaseQosTestCase):
         )
 
     def test__extend_port_resource_request_non_min_bw_rule(self):
-        port = self._create_and_extend_port([self.rule])
+        port = self._create_and_extend_port([])
 
         self.assertIsNone(port.get('resource_request'))
 
@@ -202,7 +202,6 @@ class TestQosPlugin(base.BaseQosTestCase):
 
     def test__extend_port_resource_request_inherited_policy(self):
         self.min_rule.direction = lib_constants.EGRESS_DIRECTION
-        self.policy.rules = [self.min_rule]
         self.min_rule.qos_policy_id = self.policy.id
 
         port = self._create_and_extend_port([self.min_rule],
@@ -327,6 +326,8 @@ class TestQosPlugin(base.BaseQosTestCase):
             'neutron.objects.qos.policy.QosPolicy.get_object',
             return_value=policy_mock
         ) as get_policy, mock.patch.object(
+            self.qos_plugin, "validate_policy_for_network"
+        ) as validate_policy_for_network, mock.patch.object(
             self.qos_plugin, "validate_policy_for_ports"
         ) as validate_policy_for_ports, mock.patch.object(
             self.ctxt, "elevated", return_value=admin_ctxt
@@ -338,6 +339,7 @@ class TestQosPlugin(base.BaseQosTestCase):
                     states=(kwargs['original_network'],)))
             if policy_id is None or policy_id == original_policy_id:
                 get_policy.assert_not_called()
+                validate_policy_for_network.assert_not_called()
                 get_ports.assert_not_called()
                 validate_policy_for_ports.assert_not_called()
             else:
@@ -385,6 +387,20 @@ class TestQosPlugin(base.BaseQosTestCase):
             except qos_exc.QosRuleNotSupported:
                 self.fail("QosRuleNotSupported exception unexpectedly raised")
 
+    def test_validate_policy_for_network(self):
+        network = uuidutils.generate_uuid()
+        with mock.patch.object(
+            self.qos_plugin.driver_manager, "validate_rule_for_network",
+            return_value=True
+        ):
+            self.policy.rules = [self.rule]
+            try:
+                self.qos_plugin.validate_policy_for_network(
+                    self.ctxt, self.policy, network_id=network)
+            except qos_exc.QosRuleNotSupportedByNetwork:
+                self.fail("QosRuleNotSupportedByNetwork "
+                          "exception unexpectedly raised")
+
     def test_create_min_bw_rule_on_bound_port(self):
         policy = self._get_policy()
         policy.rules = [self.min_rule]
@@ -1237,6 +1253,35 @@ class TestQosPluginDB(base.BaseQosTestCase):
         network.create()
         return network
 
+    def _test_validate_create_network_callback(self, network_qos=False):
+        net_qos_obj = self._make_qos_policy()
+        net_qos_id = net_qos_obj.id if network_qos else None
+        network = self._make_network(qos_policy_id=net_qos_id)
+        kwargs = {"context": self.context,
+                  "network": network}
+
+        with mock.patch.object(self.qos_plugin,
+                               'validate_policy_for_network') \
+                as mock_validate_policy:
+            self.qos_plugin._validate_create_network_callback(
+                'NETWORK', 'precommit_create', 'test_plugin', **kwargs)
+
+        qos_policy = None
+        if network_qos:
+            qos_policy = net_qos_obj
+
+        if qos_policy:
+            mock_validate_policy.assert_called_once_with(
+                self.context, qos_policy, network.id)
+        else:
+            mock_validate_policy.assert_not_called()
+
+    def test_validate_create_network_callback(self):
+        self._test_validate_create_network_callback(network_qos=True)
+
+    def test_validate_create_network_callback_no_qos(self):
+        self._test_validate_create_network_callback(network_qos=False)
+
     def _test_validate_create_port_callback(self, port_qos=False,
                                             network_qos=False):
         net_qos_obj = self._make_qos_policy()
