@@ -11,13 +11,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import mock
+from unittest import mock
 
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
 from neutron_lib import constants
 from oslo_config import cfg
 
-from neutron.callbacks import events
-from neutron.callbacks import registry
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
     constants as agent_consts)
 from neutron.services.trunk.drivers.openvswitch import driver
@@ -60,7 +60,14 @@ class OVSDriverTestCase(base.BaseTestCase):
         driver.register()
         mock_gen_br_name.return_value = 'fake-trunk-br-name'
         test_trigger = mock.Mock()
-        registry.notify(agent_consts.OVS_BRIDGE_NAME, events.BEFORE_READ,
-                        test_trigger, **{'port': {'trunk_details':
-                                                  {'trunk_id': 'foo'}}})
+        registry.publish(agent_consts.OVS_BRIDGE_NAME, events.BEFORE_READ,
+                         test_trigger,
+                         payload=events.EventPayload(
+                             None, metadata={
+                                 'port': {
+                                     'trunk_details': {
+                                         'trunk_id': 'foo'
+                                     }
+                                 }
+                             }))
         test_trigger.assert_called_once_with('fake-trunk-br-name')
