@@ -13,21 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
 
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
+from neutron_lib.plugins import directory
+from neutron_lib.services.trunk import constants
 import testtools
 
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.extensions import portbindings
-from neutron import manager
 from neutron.objects import trunk as trunk_objects
 from neutron.services.trunk import callbacks
-from neutron.services.trunk import constants
 from neutron.services.trunk import drivers
 from neutron.services.trunk import exceptions as trunk_exc
 from neutron.services.trunk import plugin as trunk_plugin
+from neutron.services.trunk import rules
 from neutron.services.trunk.seg_types import validators
 from neutron.tests.unit.plugins.ml2 import test_plugin
 from neutron.tests.unit.services.trunk import fakes
@@ -76,7 +77,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
                                        exception):
         subport = create_subport_dict(child_port['port']['id'])
         self._create_test_trunk(parent_port, [subport])
-        core_plugin = manager.NeutronManager.get_plugin()
+        core_plugin = directory.get_plugin()
         self.assertRaises(exception, core_plugin.delete_port,
                           self.context, port_id)
 
@@ -94,23 +95,31 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
 
     def test_delete_trunk_raise_in_use(self):
         with self.port() as port:
+            fakes.FakeDriverCanTrunkBoundPort.create()
+            self.trunk_plugin = trunk_plugin.TrunkPlugin()
+            directory.add_plugin('trunk', self.trunk_plugin)
+
             trunk = self._create_test_trunk(port)
-            core_plugin = manager.NeutronManager.get_plugin()
+            core_plugin = directory.get_plugin()
             port['port']['binding:host_id'] = 'host'
             core_plugin.update_port(self.context, port['port']['id'], port)
-            self.assertRaises(trunk_exc.TrunkInUse,
+
+            trunk_port_validator = rules.TrunkPortValidator(trunk['port_id'])
+            if not trunk_port_validator.can_be_trunked_or_untrunked(
+                    self.context):
+                self.assertRaises(trunk_exc.TrunkInUse,
                               self.trunk_plugin.delete_trunk,
                               self.context, trunk['id'])
 
     def _test_trunk_create_notify(self, event):
         with self.port() as parent_port:
-            callback = register_mock_callback(constants.TRUNK, event)
+            callback = register_mock_callback(resources.TRUNK, event)
             trunk = self._create_test_trunk(parent_port)
             trunk_obj = self._get_trunk_obj(trunk['id'])
             payload = callbacks.TrunkPayload(self.context, trunk['id'],
                                              current_trunk=trunk_obj)
             callback.assert_called_once_with(
-                constants.TRUNK, event, self.trunk_plugin, payload=payload)
+                resources.TRUNK, event, self.trunk_plugin, payload=payload)
 
     def test_create_trunk_notify_after_create(self):
         self._test_trunk_create_notify(events.AFTER_CREATE)
@@ -120,7 +129,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
 
     def _test_trunk_update_notify(self, event):
         with self.port() as parent_port:
-            callback = register_mock_callback(constants.TRUNK, event)
+            callback = register_mock_callback(resources.TRUNK, event)
             trunk = self._create_test_trunk(parent_port)
             orig_trunk_obj = self._get_trunk_obj(trunk['id'])
             trunk_req = {'trunk': {'name': 'foo'}}
@@ -131,24 +140,40 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
                                              original_trunk=orig_trunk_obj,
                                              current_trunk=trunk_obj)
             callback.assert_called_once_with(
-                constants.TRUNK, event, self.trunk_plugin, payload=payload)
+                resources.TRUNK, event, self.trunk_plugin, payload=payload)
 
     def test_trunk_update_notify_after_update(self):
         self._test_trunk_update_notify(events.AFTER_UPDATE)
 
     def test_trunk_update_notify_precommit_update(self):
-        self._test_trunk_update_notify(events.PRECOMMIT_UPDATE)
+        # TODO(boden): refactor back into _test_trunk_update_notify
+        # once all code uses neutron-lib payloads
+        with self.port() as parent_port:
+            callback = register_mock_callback(
+                resources.TRUNK, events.PRECOMMIT_UPDATE)
+            trunk = self._create_test_trunk(parent_port)
+            orig_trunk_obj = self._get_trunk_obj(trunk['id'])
+            trunk_req = {'trunk': {'name': 'foo'}}
+            self.trunk_plugin.update_trunk(self.context, trunk['id'],
+                                           trunk_req)
+            trunk_obj = self._get_trunk_obj(trunk['id'])
+            callback.assert_called_once_with(
+                resources.TRUNK, events.PRECOMMIT_UPDATE,
+                self.trunk_plugin, payload=mock.ANY)
+            call_payload = callback.call_args[1]['payload']
+            self.assertEqual(orig_trunk_obj, call_payload.states[0])
+            self.assertEqual(trunk_obj, call_payload.desired_state)
 
     def _test_trunk_delete_notify(self, event):
         with self.port() as parent_port:
-            callback = register_mock_callback(constants.TRUNK, event)
+            callback = register_mock_callback(resources.TRUNK, event)
             trunk = self._create_test_trunk(parent_port)
             trunk_obj = self._get_trunk_obj(trunk['id'])
             self.trunk_plugin.delete_trunk(self.context, trunk['id'])
             payload = callbacks.TrunkPayload(self.context, trunk['id'],
                                              original_trunk=trunk_obj)
             callback.assert_called_once_with(
-                constants.TRUNK, event, self.trunk_plugin, payload=payload)
+                resources.TRUNK, event, self.trunk_plugin, payload=payload)
 
     def test_delete_trunk_notify_after_delete(self):
         self._test_trunk_delete_notify(events.AFTER_DELETE)
@@ -159,7 +184,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
     def _test_subport_action_empty_list_no_notify(self, event, subport_method):
         with self.port() as parent_port:
             trunk = self._create_test_trunk(parent_port)
-            callback = register_mock_callback(constants.SUBPORTS, event)
+            callback = register_mock_callback(resources.SUBPORTS, event)
             subport_method(self.context, trunk['id'], {'sub_ports': []})
             callback.assert_not_called()
 
@@ -188,7 +213,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
             trunk = self._create_test_trunk(parent_port)
             orig_trunk_obj = self._get_trunk_obj(trunk['id'])
             subport = create_subport_dict(child_port['port']['id'])
-            callback = register_mock_callback(constants.SUBPORTS, event)
+            callback = register_mock_callback(resources.SUBPORTS, event)
             self.trunk_plugin.add_subports(
                 self.context, trunk['id'], {'sub_ports': [subport]})
             trunk_obj = self._get_trunk_obj(trunk['id'])
@@ -198,7 +223,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
                                              original_trunk=orig_trunk_obj,
                                              subports=[subport_obj])
             callback.assert_called_once_with(
-                constants.SUBPORTS, event, self.trunk_plugin, payload=payload)
+                resources.SUBPORTS, event, self.trunk_plugin, payload=payload)
 
     def test_add_subports_notify_after_create(self):
         self._test_add_subports_notify(events.AFTER_CREATE)
@@ -211,7 +236,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
             subport = create_subport_dict(child_port['port']['id'])
             trunk = self._create_test_trunk(parent_port, [subport])
             orig_trunk_obj = self._get_trunk_obj(trunk['id'])
-            callback = register_mock_callback(constants.SUBPORTS, event)
+            callback = register_mock_callback(resources.SUBPORTS, event)
             subport_obj = self._get_subport_obj(subport['port_id'])
             self.trunk_plugin.remove_subports(
                 self.context, trunk['id'], {'sub_ports': [subport]})
@@ -221,7 +246,7 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
                                              original_trunk=orig_trunk_obj,
                                              subports=[subport_obj])
             callback.assert_called_once_with(
-                constants.SUBPORTS, event, self.trunk_plugin, payload=payload)
+                resources.SUBPORTS, event, self.trunk_plugin, payload=payload)
 
     def test_remove_subports_notify_after_delete(self):
         self._test_remove_subports_notify(events.AFTER_DELETE)
@@ -233,13 +258,13 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
         with self.port() as port:
             trunk = self._create_test_trunk(port)
             self.assertEqual(
-                constants.DOWN_STATUS, trunk['status'])
+                constants.TRUNK_DOWN_STATUS, trunk['status'])
 
     def test_add_subports_trunk_in_error_state_raises(self):
         with self.port() as port, self.port() as subport:
             trunk = self._create_test_trunk(port)
             trunk_obj = self._get_trunk_obj(trunk['id'])
-            trunk_obj.status = constants.ERROR_STATUS
+            trunk_obj.status = constants.TRUNK_ERROR_STATUS
             trunk_obj.update()
             s = create_subport_dict(subport['port']['id'])
             self.assertRaises(trunk_exc.TrunkInErrorState,
@@ -250,51 +275,57 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
         with self.port() as port, self.port() as subport:
             trunk = self._create_test_trunk(port)
             trunk_obj = self._get_trunk_obj(trunk['id'])
-            trunk_obj.status = constants.ACTIVE_STATUS
+            trunk_obj.status = constants.TRUNK_ACTIVE_STATUS
             trunk_obj.update()
             s = create_subport_dict(subport['port']['id'])
             trunk = self.trunk_plugin.add_subports(
                 self.context, trunk['id'], {'sub_ports': [s]})
-            self.assertEqual(constants.DOWN_STATUS, trunk['status'])
+            self.assertEqual(constants.TRUNK_DOWN_STATUS, trunk['status'])
 
     def test_remove_subports_trunk_goes_to_down(self):
         with self.port() as port, self.port() as subport:
             s = create_subport_dict(subport['port']['id'])
             trunk = self._create_test_trunk(port, [s])
             trunk_obj = self._get_trunk_obj(trunk['id'])
-            trunk_obj.status = constants.ACTIVE_STATUS
+            trunk_obj.status = constants.TRUNK_ACTIVE_STATUS
             trunk_obj.update()
             trunk = self.trunk_plugin.remove_subports(
                 self.context, trunk['id'],
                 {'sub_ports': [{'port_id': subport['port']['id']}]})
-            self.assertEqual(constants.DOWN_STATUS, trunk['status'])
+            self.assertEqual(constants.TRUNK_DOWN_STATUS, trunk['status'])
 
     def test__trigger_trunk_status_change_vif_type_changed_unbound(self):
+        callback = register_mock_callback(resources.TRUNK, events.AFTER_UPDATE)
         with self.port() as parent:
             parent[portbindings.VIF_TYPE] = portbindings.VIF_TYPE_UNBOUND
             original_port = {portbindings.VIF_TYPE: 'fakeviftype'}
-            self._test__trigger_trunk_status_change(parent,
-                                                    original_port,
-                                                    constants.ACTIVE_STATUS,
-                                                    constants.DOWN_STATUS)
+            original_trunk, current_trunk = (
+                self._test__trigger_trunk_status_change(
+                    parent, original_port,
+                    constants.TRUNK_ACTIVE_STATUS,
+                    constants.TRUNK_DOWN_STATUS))
+        payload = callbacks.TrunkPayload(self.context, original_trunk['id'],
+                                         original_trunk=original_trunk,
+                                         current_trunk=current_trunk)
+        callback.assert_called_once_with(
+            resources.TRUNK, events.AFTER_UPDATE,
+            self.trunk_plugin, payload=payload)
 
     def test__trigger_trunk_status_change_vif_type_unchanged(self):
         with self.port() as parent:
             parent[portbindings.VIF_TYPE] = 'fakeviftype'
             original_port = {portbindings.VIF_TYPE: 'fakeviftype'}
-            self._test__trigger_trunk_status_change(parent,
-                                                    original_port,
-                                                    constants.ACTIVE_STATUS,
-                                                    constants.ACTIVE_STATUS)
+            self._test__trigger_trunk_status_change(
+                parent, original_port, constants.TRUNK_ACTIVE_STATUS,
+                constants.TRUNK_ACTIVE_STATUS)
 
     def test__trigger_trunk_status_change_vif_type_changed(self):
         with self.port() as parent:
             parent[portbindings.VIF_TYPE] = 'realviftype'
             original_port = {portbindings.VIF_TYPE: 'fakeviftype'}
-            self._test__trigger_trunk_status_change(parent,
-                                                    original_port,
-                                                    constants.ACTIVE_STATUS,
-                                                    constants.ACTIVE_STATUS)
+            self._test__trigger_trunk_status_change(
+                parent, original_port, constants.TRUNK_ACTIVE_STATUS,
+                constants.TRUNK_ACTIVE_STATUS)
 
     def _test__trigger_trunk_status_change(self, new_parent,
                                            original_parent,
@@ -309,10 +340,11 @@ class TrunkPluginTestCase(test_plugin.Ml2PluginV2TestCase):
         kwargs = {'context': self.context, 'port': new_parent,
                   'original_port': original_parent}
         self.trunk_plugin._trigger_trunk_status_change(resources.PORT,
-                                                  events.AFTER_UPDATE,
-                                                  None, **kwargs)
-        trunk = self._get_trunk_obj(trunk.id)
-        self.assertEqual(final_trunk_status, trunk.status)
+                                                       events.AFTER_UPDATE,
+                                                       None, **kwargs)
+        current_trunk = self._get_trunk_obj(trunk.id)
+        self.assertEqual(final_trunk_status, current_trunk.status)
+        return trunk, current_trunk
 
 
 class TrunkPluginCompatDriversTestCase(test_plugin.Ml2PluginV2TestCase):
@@ -330,9 +362,9 @@ class TrunkPluginCompatDriversTestCase(test_plugin.Ml2PluginV2TestCase):
         fakes.FakeDriver.create()
         with mock.patch.object(
                 validators, 'get_validator', side_effect=KeyError), \
-            testtools.ExpectedException(
-                    trunk_exc.SegmentationTypeValidatorNotFound):
-                trunk_plugin.TrunkPlugin()
+                testtools.ExpectedException(
+                        trunk_exc.SegmentationTypeValidatorNotFound):
+            trunk_plugin.TrunkPlugin()
 
     def test_plugins_fails_to_start_conflicting_seg_types(self):
         fakes.FakeDriver.create()
