@@ -22,7 +22,8 @@ import sys
 import time
 
 import eventlet.wsgi
-from neutron.conf import wsgi as wsgi_config
+from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as exception
 from oslo_config import cfg
 import oslo_i18n
@@ -34,15 +35,12 @@ from oslo_service import systemd
 from oslo_service import wsgi
 from oslo_utils import encodeutils
 from oslo_utils import excutils
-import six
 import webob.dec
 import webob.exc
 
-from neutron._i18n import _, _LE, _LI
+from neutron._i18n import _
 from neutron.common import config
-from neutron.common import exceptions as n_exc
-from neutron import context
-from neutron.db import api
+from neutron.conf import wsgi as wsgi_config
 from neutron import worker as neutron_worker
 
 CONF = cfg.CONF
@@ -59,19 +57,20 @@ def encode_body(body):
     return encodeutils.to_utf8(body)
 
 
-class WorkerService(neutron_worker.NeutronWorker):
+class WorkerService(neutron_worker.NeutronBaseWorker):
     """Wraps a worker to be handled by ProcessLauncher"""
-    def __init__(self, service, application, disable_ssl=False,
+    def __init__(self, service, application, set_proctitle, disable_ssl=False,
                  worker_process_count=0):
-        super(WorkerService, self).__init__(worker_process_count)
+        super(WorkerService, self).__init__(worker_process_count,
+                                            set_proctitle)
 
         self._service = service
         self._application = application
         self._disable_ssl = disable_ssl
         self._server = None
 
-    def start(self):
-        super(WorkerService, self).start()
+    def start(self, desc=None):
+        super(WorkerService, self).start(desc=desc)
         # When api worker is stopped it kills the eventlet wsgi server which
         # internally closes the wsgi server socket object. This server socket
         # object becomes not usable which leads to "Bad file descriptor"
@@ -129,7 +128,7 @@ class Server(object):
             family = info[0]
             bind_addr = info[-1]
         except Exception:
-            LOG.exception(_LE("Unable to listen on %(host)s:%(port)s"),
+            LOG.exception("Unable to listen on %(host)s:%(port)s",
                           {'host': host, 'port': port})
             sys.exit(1)
 
@@ -163,7 +162,7 @@ class Server(object):
 
         return sock
 
-    def start(self, application, port, host='0.0.0.0', workers=0):
+    def start(self, application, port, host='0.0.0.0', workers=0, desc=None):
         """Run a WSGI server with the given application."""
         self._host = host
         self._port = port
@@ -175,25 +174,27 @@ class Server(object):
 
         self._launch(application, workers)
 
-    def _launch(self, application, workers=0):
-        service = WorkerService(self, application, self.disable_ssl, workers)
+    def _launch(self, application, workers=0, desc=None):
+        set_proctitle = "off" if desc is None else CONF.setproctitle
+        service = WorkerService(self, application, set_proctitle,
+                                self.disable_ssl, workers)
         if workers < 1:
             # The API service should run in the current process.
             self._server = service
             # Dump the initial option values
             cfg.CONF.log_opt_values(LOG, logging.DEBUG)
-            service.start()
+            service.start(desc=desc)
             systemd.notify_once()
         else:
             # dispose the whole pool before os.fork, otherwise there will
             # be shared DB connections in child processes which may cause
             # DB errors.
-            api.context_manager.dispose_pool()
+            db_api.get_context_manager().dispose_pool()
             # The API service runs in a number of child processes.
             # Minimize the cost of checking for child exit by extending the
             # wait interval past the default of 0.01s.
-            self._server = common_service.ProcessLauncher(cfg.CONF,
-                                                          wait_interval=1.0)
+            self._server = common_service.ProcessLauncher(
+                cfg.CONF, wait_interval=1.0, restart_method='mutate')
             self._server.launch_service(service,
                                         workers=service.worker_process_count)
 
@@ -221,8 +222,15 @@ class Server(object):
                              max_size=self.num_threads,
                              log=LOG,
                              keepalive=CONF.wsgi_keep_alive,
+                             log_format=CONF.wsgi_log_format,
                              socket_timeout=self.client_socket_timeout)
 
+    @property
+    def process_launcher(self):
+        if isinstance(self._server, common_service.ProcessLauncher):
+            return self._server
+        return None
+
 
 class Request(wsgi.Request):
 
@@ -241,15 +249,17 @@ class Request(wsgi.Request):
             if _format in ['json']:
                 return 'application/{0}'.format(_format)
 
-        #Then look up content header
+        # Then look up content header
         type_from_header = self.get_content_type()
         if type_from_header:
             return type_from_header
         ctypes = ['application/json']
 
-        #Finally search in Accept-* headers
-        bm = self.accept.best_match(ctypes)
-        return bm or 'application/json'
+        # Finally search in Accept-* headers
+        acceptable = self.accept.acceptable_offers(ctypes)
+        if acceptable:
+            return acceptable[0][0]
+        return 'application/json'
 
     def get_content_type(self):
         allowed_types = ("application/json",)
@@ -270,7 +280,11 @@ class Request(wsgi.Request):
         if not self.accept_language:
             return None
         all_languages = oslo_i18n.get_available_languages('neutron')
-        return self.accept_language.best_match(all_languages)
+        best_match = self.accept_language.lookup(all_languages,
+                                                 default='fake_LANG')
+        if best_match == 'fake_LANG':
+            best_match = None
+        return best_match
 
     @property
     def context(self):
@@ -307,7 +321,7 @@ class JSONDictSerializer(DictSerializer):
 
     def default(self, data):
         def sanitizer(obj):
-            return six.text_type(obj)
+            return str(obj)
         return encode_body(jsonutils.dumps(data, default=sanitizer))
 
 
@@ -378,7 +392,7 @@ class JSONDeserializer(TextDeserializer):
             return jsonutils.loads(datastring)
         except ValueError:
             msg = _("Cannot understand JSON")
-            raise n_exc.MalformedRequestBody(reason=msg)
+            raise exception.MalformedRequestBody(reason=msg)
 
     def default(self, datastring):
         return {'body': self._from_json(datastring)}
@@ -410,7 +424,7 @@ class RequestDeserializer(object):
         """Extract necessary pieces of the request.
 
         :param request: Request object
-        :returns tuple of expected controller action name, dictionary of
+        :returns: tuple of expected controller action name, dictionary of
                  keyword arguments to pass to the controller, the expected
                  content type of the response
 
@@ -582,27 +596,27 @@ class Resource(Application):
     def __call__(self, request):
         """WSGI method that controls (de)serialization and method dispatch."""
 
-        LOG.info(_LI("%(method)s %(url)s"),
+        LOG.info("%(method)s %(url)s",
                  {"method": request.method, "url": request.url})
 
         try:
             action, args, accept = self.deserializer.deserialize(request)
         except exception.InvalidContentType:
             msg = _("Unsupported Content-Type")
-            LOG.exception(_LE("InvalidContentType: %s"), msg)
+            LOG.exception("InvalidContentType: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
-        except n_exc.MalformedRequestBody:
+        except exception.MalformedRequestBody:
             msg = _("Malformed request body")
-            LOG.exception(_LE("MalformedRequestBody: %s"), msg)
+            LOG.exception("MalformedRequestBody: %s", msg)
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
 
         try:
             action_result = self.dispatch(request, action, args)
         except webob.exc.HTTPException as ex:
-            LOG.info(_LI("HTTP exception thrown: %s"), ex)
+            LOG.info("HTTP exception thrown: %s", ex)
             action_result = Fault(ex, self._fault_body_function)
         except Exception:
-            LOG.exception(_LE("Internal error"))
+            LOG.exception("Internal error")
             # Do not include the traceback to avoid returning it to clients.
             action_result = Fault(webob.exc.HTTPServerError(),
                                   self._fault_body_function)
@@ -615,10 +629,10 @@ class Resource(Application):
             response = action_result
 
         try:
-            LOG.info(_LI("%(url)s returned with HTTP %(status)d"),
+            LOG.info("%(url)s returned with HTTP %(status)d",
                      dict(url=request.url, status=response.status_int))
         except AttributeError as e:
-            LOG.info(_LI("%(url)s returned a fault: %(exception)s"),
+            LOG.info("%(url)s returned a fault: %(exception)s",
                      dict(url=request.url, exception=e))
 
         return response
@@ -628,11 +642,11 @@ class Resource(Application):
 
         controller_method = getattr(self.controller, action)
         try:
-            #NOTE(salvatore-orlando): the controller method must have
+            # NOTE(salvatore-orlando): the controller method must have
             # an argument whose name is 'request'
             return controller_method(request=request, **action_args)
         except TypeError:
-            LOG.exception(_LE('Invalid request'))
+            LOG.exception('Invalid request')
             return Fault(webob.exc.HTTPBadRequest())
 
 
