@@ -32,7 +32,11 @@ Configure a host to run Neutron's functional test suite.
 
 Warning: This script relies on devstack to perform extensive
 modification to the underlying host.  It is recommended that it be
-invoked only on a throw-away VM."
+invoked only on a throw-away VM.
+
+NOTE: Default values in this file, such as passwords, have been taken
+from the devstack samples/local.conf file, but can be over-ridden by
+setting them in your environment if necessary."
     exit 1
 fi
 
@@ -54,9 +58,13 @@ VENV=${VENV:-dsvm-functional}
 DEVSTACK_PATH=${DEVSTACK_PATH:-$1}
 PROJECT_NAME=${PROJECT_NAME:-neutron}
 REPO_BASE=${GATE_DEST:-$(cd $(dirname "$0")/../.. && pwd)}
+NEUTRON_DIR=${NEUTRON_DIR:=$REPO_BASE/$PROJECT_NAME}
 INSTALL_MYSQL_ONLY=${INSTALL_MYSQL_ONLY:-False}
 # The gate should automatically install dependencies.
 INSTALL_BASE_DEPENDENCIES=${INSTALL_BASE_DEPENDENCIES:-$IS_GATE}
+BUILD_OVS_FROM_SOURCE=${BUILD_OVS_FROM_SOURCE:-True}
+OVN_BRANCH=${OVN_BRANCH:-master}
+OVS_BRANCH=${OVS_BRANCH:-master}
 
 
 if [ ! -f "$DEVSTACK_PATH/stack.sh" ]; then
@@ -74,6 +82,10 @@ function _init {
     FILES=$DEVSTACK_PATH/files
     TOP_DIR=$DEVSTACK_PATH
 
+    if [ -f $DEVSTACK_PATH/local.conf ]; then
+        source $DEVSTACK_PATH/local.conf 2> /dev/null || true
+    fi
+
     source $DEVSTACK_PATH/stackrc
 
     # Allow the gate to override values set by stackrc.
@@ -82,20 +94,36 @@ function _init {
 
     GetDistro
     source $DEVSTACK_PATH/tools/fixup_stuff.sh
-    fixup_uca
+    fixup_ubuntu
 }
 
 function _install_base_deps {
     echo_summary "Installing base dependencies"
 
     INSTALL_TESTONLY_PACKAGES=True
-    PACKAGES=$(get_packages general,neutron,q-agt,q-l3,openvswitch)
-    # Do not install 'python-' prefixed packages other than
-    # python-dev*.  Neutron's functional testing relies on deployment
-    # to a tox env so there is no point in installing python
-    # dependencies system-wide.
-    PACKAGES=$(echo $PACKAGES | perl -pe 's|python-(?!dev)[^ ]*||g')
-    install_package $PACKAGES
+    if [[ "$BUILD_OVS_FROM_SOURCE" == "True" ]]; then
+        PACKAGES=$(get_packages general,neutron,q-agt,q-l3)
+        # Do not install 'python-' prefixed packages other than
+        # python-dev*.  Neutron's functional testing relies on deployment
+        # to a tox env so there is no point in installing python
+        # dependencies system-wide.
+        PACKAGES=$(echo $PACKAGES | perl -pe 's|python-(?!dev)[^ ]*||g')
+        install_package $PACKAGES
+
+        source $NEUTRON_DIR/devstack/lib/ovs
+        source $DEVSTACK_PATH/lib/neutron_plugins/ovn_agent
+        echo_summary "OVN_BRANCH: ${OVN_BRANCH} OVS_BRANCH: ${OVS_BRANCH}"
+        compile_ovs False /usr /var
+        compile_ovn False /usr /var
+    else
+        PACKAGES=$(get_packages general,neutron,q-agt,q-l3,openvswitch)
+        PACKAGES=$(echo $PACKAGES | perl -pe 's|python-(?!dev)[^ ]*||g')
+        install_package $PACKAGES
+    fi
+
+    if is_ubuntu && [[ "$DISTRO" != "bionic" ]]; then
+        install_package "ncat"
+    fi
 }
 
 
@@ -104,7 +132,7 @@ function _install_rpc_backend {
 
     RABBIT_USERID=${RABBIT_USERID:-stackrabbit}
     RABBIT_HOST=${RABBIT_HOST:-$SERVICE_HOST}
-    RABBIT_PASSWORD=${RABBIT_HOST:-secretrabbit}
+    RABBIT_PASSWORD=${RABBIT_PASSWORD:-stackqueue}
 
     source $DEVSTACK_PATH/lib/rpc_backend
 
@@ -127,8 +155,8 @@ function _install_databases {
         return 0
     fi
 
-    MYSQL_PASSWORD=${MYSQL_PASSWORD:-secretmysql}
-    DATABASE_PASSWORD=${DATABASE_PASSWORD:-secretdatabase}
+    MYSQL_PASSWORD=${MYSQL_PASSWORD:-stackdb}
+    DATABASE_PASSWORD=${DATABASE_PASSWORD:-stackdb}
 
     source $DEVSTACK_PATH/lib/database
 
@@ -156,7 +184,7 @@ GRANT ALL PRIVILEGES ON *.* TO 'openstack_citest'@'localhost';
 GRANT ALL PRIVILEGES ON *.* TO 'openstack_citest';
 FLUSH PRIVILEGES;
 EOF
-    /usr/bin/mysql -u root < $tmp_dir/mysql.sql
+    /usr/bin/mysql -u root -p"$MYSQL_PASSWORD" < $tmp_dir/mysql.sql
 
     if [[ "$install_pg" == "True" ]]; then
         cat << EOF > $tmp_dir/postgresql.sql
@@ -194,7 +222,7 @@ function _install_rootwrap_sudoers {
 
     SECURE_PATH="$PROJECT_VENV/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
     if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
-        SECURE_PATH="$REPO_BASE/$PROJECT_NAME/neutron/tests/fullstack/cmd:$SECURE_PATH"
+        SECURE_PATH="$REPO_BASE/$PROJECT_NAME/neutron/tests/fullstack/agents:$SECURE_PATH"
     fi
 
     cat << EOF > $TEMPFILE
@@ -247,16 +275,6 @@ function _install_post_devstack {
         exit_distro_not_supported "installing dhclient and ncat packages"
     fi
 
-    # Installing python-openvswitch from packages is a stop-gap while
-    # python-openvswitch remains unavailable from pypi.  This also
-    # requires that sitepackages=True be set in tox.ini to allow the
-    # venv to use the installed package.  Once python-openvswitch
-    # becomes available on pypi, this will no longer be required.
-    #
-    # NOTE: the package name 'python-openvswitch' is common across
-    # supported distros.
-    install_package python-openvswitch
-
     enable_kernel_bridge_firewall
 }
 
@@ -270,6 +288,11 @@ function _configure_iptables_rules {
 }
 
 
+function _enable_ipv6 {
+    sudo sysctl -w net.ipv6.conf.all.disable_ipv6=0
+}
+
+
 function configure_host_for_func_testing {
     echo_summary "Configuring host for functional testing"
 
@@ -297,8 +320,10 @@ if [[ "$IS_GATE" != "True" ]]; then
 fi
 
 if [[ "$VENV" =~ "dsvm-fullstack" ]]; then
+    _enable_ipv6
     _configure_iptables_rules
-    sudo modprobe ip_conntrack_proto_sctp
+    # This module only exists on older kernels, built-in otherwise
+    modinfo ip_conntrack_proto_sctp 1> /dev/null 2>&1 && sudo modprobe ip_conntrack_proto_sctp
 fi
 
 echo "Phew, we're done!"
